include <imhdr.h>

bool procedure err_exist(ptr,mode,fname,must_exist)

#--------------------------------------------------------------------------------------
#      ERR_EXIST -- Find whether a specified error file exists
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) ptr:        (pointer) Pointer to data file associated with error file
#      (>) mode:       (int) Read/write mode of error array
#      (>) fname:      (char) Input name of error array
#      (>) must_exist: (bool) TRUE if the error file must exist beforehand.
#
#      Returns:
#         ERR_EXIST = true: if the error file exists and can be accessed in the
#                           manner specified by mode.
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:      21-OCT-1993 -- Wrong name in data file header is no longer
#                                  an error. Also added `must_exist' parameter
#                                  (JRL)
#
#--------------------------------------------------------------------------------------

# Parameters

pointer ptr
char fname[ARB]
int mode
bool must_exist

# Local variables

pointer sp,cp1,eptr
bool exist,clobber

# Functions

pointer immap()

errchk immap,rt_fstat

begin

      # Get some workspace

      call smark(sp)
      call salloc(cp1,SZ_FNAME,TY_CHAR)

      # Check to see if you can access the file.  If you can't then
      # signal an error.  If you can, then write the name of the 
      # error file into the header of the data file and the name of
      # the data file into the header of the error file

      call rt_fstat(fname,mode,must_exist,exist,clobber)
      if (exist) {      
         call imastr(ptr,"ERR_FILE",fname,SZ_FNAME)
         eptr = immap(fname,READ_WRITE,0)
         call imastr(eptr,"DAT_FILE",IM_HDRFILE(ptr),SZ_FNAME)
         call imunmap(eptr) 
      }

      # Release workspace

      call sfree(sp)

      # Return parameter

      return(exist)
end
