include <imhdr.h>
include <imio.h>

pointer procedure err_open(ptr,mode,must_exist)

#------------------------------------------------------------------------------
#      ERR_OPEN -- Open an error file associated with a data file.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) ptr:        (pointer) Pointer to data file associated with error file
#      (>) mode:       (int) Read/write mode of opened file
#      (>) must_exist  (bool) TRUE if the error file must exist beforehand
#
#      Returns:
#         ERR_OPEN = eptr: The pointer to the opened error file
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:      30-MAR-1992 -- Files now created in routine rt_crfile (JRL)
#	
#
#------------------------------------------------------------------------------

# Parameters

pointer ptr
int mode
bool must_exist

# Local variables

pointer sp,cp1,cp2,cp3,eptr,ptr2,wpt1
int ip,ndim,i
bool exist

# Functions

int imaccf()
bool err_exist()
pointer immap(),rt_crfile()

# Check for errors

errchk err_exist,imdelf,immap,imastr,imunmap,rt_crfile

begin

      # Start by getting some workspace

      call smark(sp)
      call salloc(cp1,SZ_FNAME,TY_CHAR)
      call salloc(cp2,SZ_LINE,TY_CHAR)
      call salloc(cp3,SZ_FNAME,TY_CHAR)

      # If the mode is for a new file, then delete any reference to an error
      # file in the data header

      if ((mode == NEW_COPY || mode == NEW_FILE) && 
         imaccf(ptr,"ERR_FILE") == YES) {
         call imdelf(ptr,"ERR_FILE")
      }

      # Map original data file without sections...

      ptr2 = immap(IM_HDRFILE(ptr),READ_ONLY,0)
      
      # Create the error file name

      call err_fname(ptr,Memc[cp1])

      # Does this file exist?

      exist = err_exist(ptr,mode,Memc[cp1],must_exist)

      # Open the error file.  If it is a new file, then set the data size. If it
      # is an old file, then check that it has the same size as the data file
      
      if (exist) { 
         eptr = immap(Memc[cp1],mode,0)
         for (ip = 1; ip <= IM_MAXDIM && (IM_LEN(ptr2,ip) == IM_LEN(eptr,ip));
            ip = ip + 1)
            ;
         if (ip <= IM_MAXDIM) {
            call sprintf(Memc[cp2],SZ_LINE,
               "Error file: %s has different dimensions from data file: %s")
               call pargstr(Memc[cp1])
               call pargstr(IM_HDRFILE(ptr))
            call error(1,Memc[cp2])
         }
         if (IM_PIXTYPE(eptr) != IM_PIXTYPE(ptr2)) {
            call sprintf(Memc[cp2],SZ_LINE,
               "Error file: %s has different data type from data file: %s")
               call pargstr(Memc[cp1])
               call pargstr(IM_HDRFILE(ptr))
            call error(1,Memc[cp2])
         }
      }
      else {
         ndim = IM_NDIM(ptr2)
         call malloc(wpt1,ndim,TY_INT)
         do i = 1,ndim
            Memi[wpt1+i-1] = IM_LEN(ptr2,i)
         eptr = rt_crfile(Memc[cp1],NEW_COPY,ptr2,ndim,Memi[wpt1],
            IM_PIXTYPE(ptr2))
         call mfree(wpt1,TY_INT)
         call imastr(ptr,"ERR_FILE",Memc[cp1])
         call imflush(ptr)
         call imastr(eptr,"DAT_FILE",IM_HDRFILE(ptr))
         call imflush(eptr)
      }

      # If an image section has been opened in data file, then reopen the error
      # file with the same image section

      call imgsection(IM_NAME(ptr),Memc[cp3],SZ_FNAME)
      if (Memc[cp3] != EOS) {
         call imunmap(eptr)
         call strcat(Memc[cp3],Memc[cp1],SZ_FNAME)
         if (exist) {
            eptr = immap(Memc[cp1],mode,0) }
         else {
            eptr = immap(Memc[cp1],READ_WRITE,0)
         }
      }     

      # Close the local mapping of the data file

      call imunmap(ptr2)

      # Free workspace

      call sfree(sp)

      # Return parameter

      return(eptr)
end
