include <imhdr.h>
include <fset.h>
include <gset.h>
include <mwset.h>
include "cspe.h"
include <pkg/gtools.h>

define HELPFIL      "rgo$ndprogs/curfils/cubespe.key"
define PROMPT       "cubespe options"

procedure t_cubespe()

# Local variables

pointer sp,ifil2d,ifil3d,iptr3,trcode,iptr2,mw,ct,wcs
int ndim3,idims3[3],ndim2,idims2[2],box,i,nplots

# Functions

int fstati(),imaccf()
bool streq()
pointer immap(),mw_openim(),mw_sctran()

begin

      # Mark and allocate stack workspace

      call smark(sp)
      call salloc(ifil2d,SZ_FNAME,TY_CHAR)
      call salloc(ifil3d,SZ_FNAME,TY_CHAR)
      call salloc(trcode,SZ_LINE,TY_CHAR)
      call salloc(wcs,SZ_LINE,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Open the input cube. Check its dimensionality. Open its WCS

      call clgstr("input3d",Memc[ifil3d],SZ_FNAME)
      iferr(iptr3 = immap(Memc[ifil3d],READ_ONLY,0)) call error(0,
         "Couldn't open input cube")
      ndim3 = IM_NDIM(iptr3)
      if (ndim3 != 3) call error(0,"This file must have exactly 3 dimensions")
      do i = 1,ndim3
         idims3[i] = IM_LEN(iptr3,i)
      mw = mw_openim(iptr3)
      call mw_seti(mw,MW_USEAXMAP,NO)

      # If the cube has been sorted by 'ndtrans' then is it in ZXY order?

      if (imaccf(iptr3,"trcode") == YES) {
         call imgstr(iptr3,"trcode",Memc[trcode],SZ_LINE)
         if (! streq(Memc[trcode],"312"))  { 
            call eprintf("The transform code is \"%s\". This cube may need")
               call pargstr(Memc[trcode])
            call eprintf("to be transformed using \"ndtrans\".\n")
         }
      } 

      # Now open the input frame and check its dimensionality

      call clgstr("input2d",Memc[ifil2d],SZ_FNAME)
      iferr(iptr2 = immap(Memc[ifil2d],READ_ONLY,0)) call error(0,
         "Couldn't open input frame")
      ndim2 = IM_NDIM(iptr2)
      if (ndim2 != 2) call error(0,"This file must have exactly 3 dimensions")
      do i = 1,ndim2
         idims2[i] = IM_LEN(iptr2,i)

      # Check that the XY dimensions of the two files match

      if (idims2[1] != idims3[2] || idims2[2] != idims3[3]) 
         call error(0,"The XY dimensions of the cube and image don't match")
     
      # Get some other parameters now

      call clgstr("wcs",Memc[wcs],SZ_LINE)

      # Set up mwcs to make transformation between logical and the 
      # requested coordinates for the X axis of the plots...

      ct = mw_sctran(mw,"logical",Memc[wcs],0)
                           
      # Now go to the main processing routine

      call sub_cubespe(iptr3,idims3,iptr2,idims2,box,nplots,wcs,mw,ct)

      # Close up some files

      call imunmap(iptr2)
      call imunmap(iptr3)
      call mw_ctfree(ct)
      call mw_close(mw)
      call sfree(sp)
end

procedure sub_cubespe(iptr3,idims3,iptr2,idims2,box,nplots,wcs,mw,ct)

# Parameters

pointer iptr3,iptr2,wcs,mw,ct
int idims3[3],idims2[2],box,nplots

# Local variables

int key,nlam,nst,wcsdim,i,txq,id
real x,y,low,high
pointer sp,cmd,xplot,yplot,gp,title,csp,gt,xin,xout,xlabel,format,
   str1,str2,tqual,tqdict
bool quit,newplot,erase,common

# Functions

pointer gopen(),cspe_open(),gt_init()
int clgcur(),clgeti(),mw_stati(),strdic()
real clgetr()
bool clgetb(),streq()

begin

      # Get stack workspace
                                                   
      call smark(sp)
      call salloc(cmd,SZ_LINE,TY_CHAR)
      call salloc(title,SZ_LINE,TY_CHAR)
      call salloc(xin,IM_MAXDIM,TY_REAL)
      call salloc(xout,IM_MAXDIM,TY_REAL)
      call salloc(xlabel,SZ_LINE,TY_CHAR)
      call salloc(format,SZ_LINE,TY_CHAR)
      call salloc(str1,SZ_LINE,TY_CHAR)
      call salloc(str2,SZ_LINE,TY_CHAR)
      call salloc(tqual,SZ_LINE,TY_CHAR)
      call salloc(tqdict,SZ_LINE,TY_CHAR)

      # Get some workspace for the plots

      nlam = idims3[1]
      call malloc(xplot,nlam,TY_REAL)
      call malloc(yplot,nlam,TY_REAL)

      # Set some defaults

      nplots = 3
      box = 1
      txq = GT_NORMAL

      # Create X axis array for plots

      wcsdim = mw_stati(mw,MW_NDIM)
      do i = 1,idims3[1] {
         Memr[xin] = real(i)
         call mw_ctranr(ct,Memr[xin],Memr[xout],wcsdim)
         Memr[xplot+i-1] = Memr[xout]
      }

      # Create an X axis label

      Memc[format] = EOS
      if (streq(Memc[wcs],"logical") || streq(Memc[wcs],"physical")) {
         call strcpy("pixels",Memc[xlabel],SZ_LINE)
      } else {
         Memc[xlabel] = EOS
         ifnoerr (call mw_gwattrs(mw,1,"label",Memc[str1],SZ_LINE)) {
            ifnoerr (call mw_gwattrs(mw,1,"units",Memc[str2],SZ_LINE)) {
               call sprintf(Memc[xlabel],SZ_LINE,"%s (%s)")
                  call pargstr(Memc[str1])
                  call pargstr(Memc[str2])
            } else {
               call sprintf(Memc[xlabel],SZ_LINE,"%s")
                  call pargstr(Memc[str1])
            }
         }
         ifnoerr (call mw_gwattrs(mw,1,"format",Memc[str1],SZ_LINE))
            call strcpy(Memc[str1],Memc[format],SZ_LINE)
      }

      # Open the plot device and gtools

      gp = gopen("stdgraph",NEW_FILE,STDGRAPH)
      gt = gt_init()

      # Sort out the parameters for the plot panel

      csp = cspe_open()
      call cspe_init(csp,nplots)

      # Do initial setup of plots

      call plinit(gp,gt,nplots,format,xlabel,tqual)

      # Now go into main loop

      quit = false
      while (! quit) {
         nst = clgcur("imagecur",x,y,wcs,key,Memc[cmd],SZ_LINE)

         # Switch for all the various key commands...

         switch (key) {
         case '?':
            call gpagefile(gp,HELPFIL,PROMPT)
         case 'b':
            box = clgeti("boxsize")
            erase = true
         case 'd':
            newplot = true
         case 'k':
            x = clgetr("xpos")
            y = clgetr("ypos")
            newplot = true
         case 'n':
            nplots = clgeti("nplots")
            call cspe_init(csp,nplots)
            erase = true
         case 'q':
            quit = true
            call gclose(gp)
         case 's':
            common = clgetb("common")
            low = clgetr("low_val")
            high = clgetr("high_val")
            erase = true
         case 't':
            call clgstr("txtqual.p_min",Memc[tqdict],SZ_LINE)
            call clgwrd("txtqual",Memc[tqual],SZ_LINE,Memc[tqdict])
            id = strdic(Memc[tqual],Memc[tqual],SZ_LINE,Memc[tqdict])
            switch (id) {
               case 1:
                  txq = GT_NORMAL
               case 2:
                  txq = GT_LOW
               case 3:
                  txq = GT_MEDIUM
               case 4:
                  txq = GT_HIGH
            }
            erase = true
         case 'I':
            call fatal(0,"Interrupt")
         }
  
         # If you have requested a plot, then do it here
    
         if (newplot) {
            call cubeplot(gp,gt,csp,iptr3,idims3,box,nplots,erase,x,y,
               nlam,Memr[xplot],Memr[yplot],low,high,common,title,
               xlabel,format,txq)
            newplot = false
         }
      }

      # Free workspace
 
      call sfree(sp)
      call mfree(xplot,TY_REAL)
      call mfree(yplot,TY_REAL)
      call cspe_close(csp)
      call gt_free(gt)
end

procedure cubeplot(gp,gt,csp,iptr3,idims3,box,nplots,erase,x,y,nlam,
   xplot,yplot,low,high,common,title,xlabel,format,txq)

# Parameters

pointer gp,iptr3,csp,title,gt,xlabel,format
int idims3[3],box,nplots,nlam,txq
real x,y,xplot[nlam],yplot[nlam],low,high
bool erase,common

# Local variables

int iminus,ix,iy,ix1,ix2,iy1,iy2,box2,nx1,ny1
pointer ptr1

# Functions

pointer imgs3r()

begin

      # Sort out the bit of the cube you want

      if (mod(box,2) == 0)
         iminus = -1
      else
         iminus = 0
      box2 = box/2
      ix = nint(x)
      iy = nint(y)
      ix1 = max(1,ix-box2)
      ix2 = min(idims3[2],ix+box2+iminus)
      iy1 = max(1,iy-box2)
      iy2 = min(idims3[3],iy+box2+iminus)

      # Get the section now...

      ptr1 = imgs3r(iptr3,1,nlam,ix1,ix2,iy1,iy2)
      nx1 = ix2 - ix1 + 1
      ny1 = iy2 - iy1 + 1

      # Block average this into the y workspace

      if (box > 1) 
         call cubeblock(nlam,nx1,ny1,Memr[ptr1],yplot)
      else
         call amovr(Memr[ptr1],yplot,nlam)

      # Now plot the spectrum

      call drawspe(gp,gt,csp,nlam,xplot,yplot,nplots,erase,low,high,
         common,ix,iy,title,xlabel,format,txq)
end
  

procedure cubeblock(nl,nx,ny,array,output)

# Parameters

int nl,nx,ny
real array[nl,nx,ny],output[nl]

# Local variables

int i,j,k
real rns

begin

      # Clear the output array first...

      call aclrr(output,nl)

      # Now sum all the spectra together

      do j = 1,ny {
         do i = 1,nx {
            do k = 1,nl 
               output[k] = output[k] + array[k,i,j]
         }
      }

      # Now normalise by the number of spectra

      rns = real(nx*ny)
      call adivkr(output,rns,output,nl)
end

procedure drawspe(gp,gt,csp,nlam,xplot,yplot,nplots,erase,low,high,
   common,ix,iy,title,xlabel,format,txq)

# Parameters

int nlam,nplots,ix,iy,txq
bool erase
real xplot[nlam],yplot[nlam],low,high
bool common
pointer gp,csp,title,gt,format,xlabel

# Local variables

int ndx,ndy
real x1,x2,y1,y2

begin

      # Clear the screen if that has been requested

      if (erase) {
         call gclear(gp)
         call cspe_init(csp,nplots)
         call plinit(gp,gt,nplots,format,xlabel,txq)
         erase = false
      }

      # Check the plot counter and see if you have filled up the panel

      if (NDONE(csp) == nplots*nplots) {
         call gclear(gp)
         call plinit(gp,gt,nplots,format,xlabel,txq)
         NDONE(csp) = 0
      }

      # First define the place where the plot will be done

      ndx = mod(NDONE(csp),nplots) + 1
      ndy = NDONE(csp)/nplots + 1
      x1 = (2*ndx - 1)*XOFFSET(csp) + (ndx - 1)*XLEN(csp)
      x2 = x1 + XLEN(csp)
      y1 = 1.0 - ((2*ndy - 1)*YOFFSET(csp) + ndy*YLEN(csp))
      y2 = y1 + YLEN(csp)
      call gsview(gp,x1,x2,y1,y2)

      # Set up scale 

      call gascale(gp,xplot,nlam,1)
      call gascale(gp,yplot,nlam,2)
      if (common) {
         call gt_setr(gt,GTYMIN,low)
         call gt_setr(gt,GTYMAX,high)
      }
      call gt_swind(gp,gt)

      # Set up axes and labels

      call sprintf(Memc[title],SZ_LINE,"(%d,%d)")
         call pargi(ix)
         call pargi(iy)
      call gt_sets(gt,GTTITLE,Memc[title])
      call gt_labax(gp,gt)

      # Plot the graph

      call gt_plot(gp,gt,xplot,yplot,nlam)

      # Increment the plot counter...

      NDONE(csp) = NDONE(csp) + 1      
end

procedure plinit(gp,gt,nplots,format,xlabel,txq)

# Parameters

pointer gp,gt,format,xlabel
int nplots,txq

# Local variables

real csize,tsize

# Functions

real ggetr()

begin

      # Scale the size of axes and labels by the number of plots on the panel

      csize = 1.0/real(nplots)
      call gsetr(gp,G_AXISLABELSIZE,csize)
      call gsetr(gp,G_TICKLABELSIZE,csize)
      call gsetr(gp,G_TITLESIZE,csize)

      # No minor ticks and set the major tick length to be scaled by the
      # number of plots on the panel

      call gseti(gp,G_NMINOR,0)
      tsize = ggetr(gp,"ch")*0.6*csize
      call gsetr(gp,G_MAJORLENGTH,tsize)
      call gsets(gp,G_XTICKFORMAT,Memc[format])

      # The plot type will be lines...

      call gt_sets(gt,GTTYPE,"line")
      call gt_seti(gt,GTSYSID,NO)

      # Set up the X,Y axis labels

      call gt_sets(gt,GTXLABEL,Memc[xlabel])
      call gt_sets(gt,GTYLABEL,"Intensity")

      # Set up text quality

      call gseti(gp,G_TXQUALITY,txq)

end
           
pointer procedure cspe_open()

# Parameters

pointer csp

begin

      # Open the space for the structure...

      call calloc(csp,CS_LEN,TY_STRUCT)
      return(csp)
end

procedure cspe_init(csp,nplots)

# Parameters

pointer csp
int nplots

begin

      # Initialise the number of plots already done on the panel

      NDONE(csp) = 0

      # Initialise the amount of extra space allowed for titles and
      # axis labels for each plot.  This is expressed as a fraction
      # of the axis length. E.g, the total linear dimension allowed
      # for each plot is (1.0 + 2*EXTRA)*AXIS_LENGTH

      EXTRA(csp) = 0.15

      # Define the length of each axis in NDC.  Because we define the
      # lengths to be the same in NDC, it preserves the aspect ratio of
      # the output device.

      XLEN(csp) = 1.0/((1.0+2.0*EXTRA(csp))*real(nplots))
      YLEN(csp) = XLEN(csp)

      # Define the offset of each axis in NDC

      XOFFSET(csp) = EXTRA(csp)*XLEN(csp)
      YOFFSET(csp) = EXTRA(csp)*YLEN(csp)
end

procedure cspe_close(csp)

# Parameters

pointer csp

begin

      # Free the workspace

      call mfree(csp,TY_STRUCT)
end
