include "rstruct.h"
include <ctype.h>

procedure fillpars(rp,rps)

# Parameters

pointer rp,rps

# Local variables

pointer sp,rsrpar,redpar,sval
int i,ival
bool bval
real rval

# Functions

bool clgetb()
real clgetr()
int clgeti(),ps_search()

begin

      # Get some workspace

      call smark(sp)
      call salloc(rsrpar,SZ_LINE,TY_CHAR)
      call salloc(redpar,SZ_LINE,TY_CHAR)
      call salloc(sval,SZ_LINE,TY_CHAR)

      # Loop for all of the parameters in the standard parameter structure

      do i = 1,NPARS(rp) {

         # If this parameter matches with one from the standard rspecred set
         # then continue.  Otherwise skip it.

         if (ps_search(rps,Memc[PNAME(rp,i)]) <= 0)
            next

         # Create the name of the rspecred parameter

         call sprintf(Memc[rsrpar],SZ_LINE,"rspecred.%s")
            call pargstr(Memc[PNAME(rp,i)])

         # Create the name of the current parameter with p_value appended

         call sprintf(Memc[redpar],SZ_LINE,"%s.p_value")
            call pargstr(Memc[PNAME(rp,i)])

         # Now switch for the parameter type and get the current value.
         # Deposit this value into the rspecred parameter

         switch (Memc[PTYPE(rp,i)]) {
            case 's','f':
               call clgstr(Memc[redpar],Memc[sval],SZ_LINE)
               call clpstr(Memc[rsrpar],Memc[sval])
            case 'b':
               bval = clgetb(Memc[redpar])
               call clputb(Memc[rsrpar],bval)
            case 'i':
               ival = clgeti(Memc[redpar])
               call clputi(Memc[rsrpar],ival)
            case 'r':
               rval = clgetr(Memc[redpar])
               call clputr(Memc[rsrpar],rval)
         }
      }

      # Ditch workspace and exit

      call sfree(sp)
end

procedure getparam(gp,pp1,pp2,rp1,rp2,strval)

# Parameters

pointer gp,pp1,pp2,rp1,rp2
char strval[ARB]

# Local variables

pointer sp,param,gparam,tmp,rp,pp,tmp2
int nps,ival,imin,imax,ip,ip2,maxch,index
real rval,rmin,rmax
bool err,ok
char msg[SZ_LINE]

# Functions

int ps_search(),clgpseti(),strlen(),ps_geti(),isblankstr(),stridx(),strdic()
real clgpsetr(),ps_getr()
bool clgpsetb(),streq(),strne()

define  exit_  99
begin

      # Get some workspace

      call smark(sp)
      call salloc(param,SZ_LINE,TY_CHAR)
      call salloc(gparam,SZ_LINE,TY_CHAR)
      call salloc(tmp,SZ_LINE,TY_CHAR)
      call salloc(tmp2,SZ_LINE,TY_CHAR)

      # Get the first word from the input string.  This should be the
      # parameter name...

      call sscan(strval)
         call gargwrd(Memc[gparam],SZ_LINE)

      # If the first to letters are 'rp' then we know this is an rpars pset
      # parameter.

      if (Memc[gparam] == 'r' && Memc[gparam+1] == 'p') {
         rp = rp2
         pp = pp2
         call strcpy(Memc[gparam+2],Memc[param],SZ_LINE)
      } else {
         rp = rp1
         pp = pp1
         call strcpy(Memc[gparam],Memc[param],SZ_LINE)
      }

      # Make sure this isn't blank

      if (isblankstr(Memc[param]) == YES || isblankstr(Memc[gparam]))
         goto exit_

      # Locate the parameter in the relevant structure

      nps = ps_search(rp,Memc[param])
      if (nps == 0) {
         call sprintf(msg,SZ_LINE,"Parameter \"%s\" doesn't exist in par structure")
            call pargstr(Memc[param])
         call gmsg(gp,"errrmsg",msg)
         goto exit_
      }

      # Now switch for the parameter type.  For integer and real types,
      # you need to test the min and max.  For boolean it has to either be
      # 'yes' or 'no' (case independent).  For strings and files, it doesn't
      # matter.

      switch (Memc[PTYPE(rp,nps)]) {
         case 'i':
            iferr(call gargi(ival))
               err = true
            else {
               imin = ps_geti(Memc[PMIN(rp,nps)])
               imax = ps_geti(Memc[PMAX(rp,nps)])
               if (ival < imin && !IS_INDEFI(imin)) {
                  err = true
               } else if (ival > imax && !IS_INDEFI(imax)) {
                  err = true
               } else {
                  err = false
               }
            }
            if (err) {
               call sprintf(Memc[tmp],SZ_LINE,
                  "Parameter out of range (%s:%s) -- Try again\n")
                  call pargstr(Memc[PMIN(rp,nps)])
                  call pargstr(Memc[PMAX(rp,nps)])
               call gmsg(gp,"errmsg",Memc[tmp])
               call sprintf(Memc[tmp],SZ_LINE,"%s.p_value")
                  call pargstr(Memc[param])
               call gmsgi(gp,"wrong",1)
               call gmsgi(gp,Memc[gparam],clgpseti(pp,Memc[tmp]))
            } else {
               call gmsgi(gp,"wrong",0)
               call gmsgi(gp,Memc[gparam],ival)
               call clppseti(pp,Memc[param],ival)
            }
         case 'r':
            iferr(call gargr(rval))
               err = true
            else {
               rmin = ps_getr(Memc[PMIN(rp,nps)])
               rmax = ps_getr(Memc[PMAX(rp,nps)])
               if (rval < rmin && !IS_INDEFR(rmin))
                  err = true
               else if (rval > rmax && !IS_INDEFR(rmax))
                  err = true
               else
                  err = false
            }
            if (err) {
               call sprintf(Memc[tmp],SZ_LINE,
                  "Parameter out of range (%s:%s) -- Try again\n")
                  call pargstr(Memc[PMIN(rp,nps)])
                  call pargstr(Memc[PMAX(rp,nps)])
               call gmsg(gp,"errmsg",Memc[tmp])
               call sprintf(Memc[tmp],SZ_LINE,"%s.p_value")
                  call pargstr(Memc[param])
               call gmsgi(gp,"wrong",1)
               call gmsgr(gp,Memc[gparam],clgpsetr(pp,Memc[tmp]))
            } else {
               call gmsgi(gp,"wrong",0)
               call gmsgr(gp,Memc[gparam],rval)
               call clppsetr(pp,Memc[param],rval)
            }
         case 'b':
            call gargstr(Memc[tmp],SZ_LINE)
            call strlwr(Memc[tmp])
            for (ip = 1; ip <= strlen(Memc[tmp]) && IS_WHITE(Memc[tmp+ip-1]);
               ip = ip + 1)
               ;
            if (streq(Memc[tmp+ip-1],"yes")) {
               call gmsgi(gp,"wrong",0)
               call gmsgb(gp,Memc[gparam],true)
               call clppsetb(pp,Memc[param],true)
            } else if (streq(Memc[tmp+ip-1],"no")) {
               call gmsgi(gp,"wrong",0)
               call gmsgb(gp,Memc[gparam],false)
               call clppsetb(pp,Memc[param],false)
            } else {
               call sprintf(Memc[tmp],SZ_LINE,"Parameter out of range (yes:no) -- Try again\n")
               call gmsg(gp,"errmsg",Memc[tmp])
               call sprintf(Memc[tmp],SZ_LINE,"%s.p_value")
                  call pargstr(Memc[param])
               call gmsgi(gp,"wrong",1)
               call gmsgb(gp,Memc[gparam],clgpsetb(pp,Memc[tmp]))
            }
         case 's','f':
            call gargstr(Memc[tmp2],SZ_LINE)
            for (ip = 1; ip <= strlen(Memc[tmp2]) && IS_WHITE(Memc[tmp2+ip-1]);
               ip = ip + 1)
               ;
            for (ip2 = strlen(Memc[tmp2]); ip2 >= 1 && IS_WHITE(Memc[tmp2+ip2-1]);
               ip2 = ip2 + 1)
               ;
            maxch = ip2 - ip + 1
            call strcpy(Memc[tmp2+ip-1],Memc[tmp],maxch)
            ok = true
            if (stridx("|",Memc[PMIN(rp,nps)]) > 0) {
               index = strdic(Memc[tmp],Memc[tmp2],SZ_LINE,Memc[PMIN(rp,nps)])
               if (index == 0 || strne(Memc[tmp],Memc[tmp2])) {
                  call sprintf(Memc[tmp],SZ_LINE,
                     "Parameter must be one of: %s -- Try again\n")
                     call pargstr(Memc[PMIN(rp,nps)])
                  call gmsg(gp,"errmsg",Memc[tmp])
                  call sprintf(Memc[tmp],SZ_LINE,"%s.p_value")
                     call pargstr(Memc[param])
                  call clgpseta(pp,Memc[tmp],Memc[tmp2],SZ_LINE)
                  call gmsg(gp,"wrong",1)
                  call gmsg(gp,Memc[gparam],Memc[tmp2])
                  ok = false
               }
            }
            if (ok) {
               call gmsgi(gp,"wrong",0)
               call gmsg(gp,Memc[gparam],Memc[tmp])
               call clppseta(pp,Memc[param],Memc[tmp])
            }
      }

      # Ditch workspace

exit_
      call sfree(sp)
end

procedure setpars(gp,pp,rp,mm)

# Parameters 

pointer gp,rp,pp
int mm

# Local variables

pointer sval,sp,gpar
int i,ival
real rval
bool bval

# Functions

real clgpsetr()
int clgpseti()
bool clgpsetb()

begin

      # Get some workspace

      call smark(sp)
      call salloc(gpar,SZ_LINE,TY_CHAR)
      call salloc(sval,SZ_LINE,TY_CHAR)

      # Loop for each of the parameters...

      do i = 1,NPARS(rp) {

         # Form the gui parameter name

         if (mm == 1) {
            call strcpy(Memc[PNAME(rp,i)],Memc[gpar],SZ_LINE)
         } else {
            call sprintf(Memc[gpar],SZ_LINE,"rp%s")
               call pargstr(Memc[PNAME(rp,i)])
         }

         # Get the value according to type and send it to the gui...

         switch (Memc[PTYPE(rp,i)]) {
            case 's','f':
               call clgpseta(pp,Memc[PNAME(rp,i)],Memc[sval],SZ_LINE)
               call gmsg(gp,Memc[gpar],Memc[sval])
            case 'r':
               rval = clgpsetr(pp,Memc[PNAME(rp,i)])
               call gmsgr(gp,Memc[gpar],rval)
            case 'i':
               ival = clgpseti(pp,Memc[PNAME(rp,i)])
               call gmsgi(gp,Memc[gpar],ival)
            case 'b':
               bval = clgpsetb(pp,Memc[PNAME(rp,i)])
               call gmsgb(gp,Memc[gpar],bval)
         }
      }

      # Ditch some workspace...

      call sfree(sp)
end


int procedure isblankstr(instring) 

# Parameters 

char instring[ARB]

# Local variables

int ip

begin

      # Search for a non-white character in the string...

      ip = 1
      while (IS_WHITE(instring[ip]) && instring[ip] != EOS) 
         ip = ip + 1

      # Return the appropriate value

      if (instring[ip] == EOS)
         return(YES)
      else
         return(NO)
end




