include "rstruct.h"

pointer procedure ps_open(pfile)

# Parameters

char pfile[ARB]
pointer rp

# Local variables

pointer sp,msg
int fd,nalloc

# Functions

int open(),access()

common /fps/ nalloc

begin

      # Get some workspace

      call smark(sp)
      call salloc(msg,SZ_LINE,TY_CHAR)

      # Check that the parameter file exists. If it does then open it.

      if (access(pfile,0,0) == NO) {
         call sprintf(Memc[msg],SZ_LINE,"Can't find file: %s")
            call pargstr(pfile)
         call error(0,Memc[msg])
      } else {
         iferr(fd = open(pfile,READ_ONLY,TEXT_FILE)) {
            call sprintf(Memc[msg],SZ_LINE,"Error opening file: %s")
               call pargstr(pfile)
            call error(0,Memc[msg])
         }
      }

      # Get an initial allocation for the structure

      call malloc(rp,IALLOC(INITALLOC),TY_STRUCT)

      # Fill in some initial values...

      NPARS(rp) = 0
      FDESCR(rp) = fd
      nalloc = INITALLOC

      # Tidy and exit

      call sfree(sp)
      return(rp)
end

procedure ps_close(rp)      

# Parameters

pointer rp

# Local variables

int i

begin

      # Close up the param file

      call close(FDESCR(rp))

      # Release workspaces associated with input structure
 
      do i = 1,NPARS(rp)
         call ps_frstrsp(rp,i)
      call mfree(rp,TY_STRUCT)
end

procedure ps_getmore(rp)

# Parameters 

pointer rp

# Local variables

int nalloc,nall

common /fps/ nalloc

begin

      # Increment the total allocation by the initial buffer size and 
      # reallocate the total space.

      nalloc = nalloc + INITALLOC
      nall = IALLOC(nalloc)
      call realloc(rp,nall,TY_STRUCT)
end

procedure ps_trim(rp)

# Parameters

pointer rp

# Local variables

int nall

begin

      # Trim off the excess by reallocating on the total space you need

      nall = IALLOC(NPARS(rp))
      call realloc(rp,nall,TY_STRUCT)
end

procedure ps_gwrd(data,ip,temp)

# Parameters

char data[ARB],temp[ARB]
int ip

# Local variables

int op

begin

      # Go through the data line until you either find a comma or
      # the EOS.  Ignore any double quote characters.

      op = 0
      while (data[ip] != ',' && data[ip] != EOS) {
         if (data[ip] != '"') {
            op = op + 1
            temp[op] = data[ip]
         }
         ip = ip + 1
      }
      op = op + 1
      temp[op] = EOS
end

procedure ps_getstrsp(rp,i)

# Parameters

pointer rp
int i

begin

      # Allocate workspace for each of the parameter properties

      call malloc(PNAME(rp,i),LEN_FIELD,TY_CHAR)
      call malloc(PTYPE(rp,i),LEN_TYPE,TY_CHAR)
      call malloc(PDEF(rp,i),LEN_FIELD,TY_CHAR)
      call malloc(PMIN(rp,i),LEN_FIELD,TY_CHAR)
      call malloc(PMAX(rp,i),LEN_FIELD,TY_CHAR)
end

procedure ps_frstrsp(rp,i)

# Parameters

pointer rp
int i

begin

      # Free workspace for each of the parameter properties

      call mfree(PNAME(rp,i),TY_CHAR)
      call mfree(PTYPE(rp,i),TY_CHAR)
      call mfree(PDEF(rp,i),TY_CHAR)
      call mfree(PMIN(rp,i),TY_CHAR)
      call mfree(PMAX(rp,i),TY_CHAR)
end

procedure ps_fill(rp)

# Parameters 

pointer rp

# Local variables

pointer sp,data,tmp
int ip,nalloc

# Functions

int fscan(),stridx()
bool streq()

common /fps/ nalloc

begin

      # Get some workspace

      call smark(sp)
      call salloc(data,SZ_LINE,TY_CHAR)
      call salloc(tmp,SZ_LINE,TY_CHAR)

      # Scan the file until the EOF

      while (fscan(FDESCR(rp)) != EOF) {
         call gargstr(Memc[data],SZ_LINE)

         # If there isn't a comma on the line, then skip it
 
         if (stridx(",",Memc[data]) == 0)
            next

         # Increment the parameter number and check that you don't need
         # to up the structure workspace allocation.

         NPARS(rp) = NPARS(rp) + 1
         if (NPARS(rp) > nalloc)
            call ps_getmore(rp)
         call ps_getstrsp(rp,NPARS(rp))

         # Get the first word and strip off any quotation marks (param name)

         ip = 1
         call ps_gwrd(Memc[data],ip,Memc[tmp])
         call strcpy(Memc[tmp],Memc[PNAME(rp,NPARS(rp))],LEN_FIELD)

         # Skip the next character (a comma) and copy the following one. If 
         # this is an asterisk, then decrement the number of parameters and 
         # skip this entry.

         ip = ip + 1
         call strcpy(Memc[data+ip-1],Memc[PTYPE(rp,NPARS(rp))],1)
         if (streq(Memc[PTYPE(rp,NPARS(rp))],"*")) {
            call ps_frstrsp(rp,NPARS(rp))
            NPARS(rp) = NPARS(rp) - 1
            next
         }

         # Skip over the mode field and get the next word (default value)
         # Then get the min and max.

         ip = ip + 4
         call ps_gwrd(Memc[data],ip,Memc[tmp])
         call strcpy(Memc[tmp],Memc[PDEF(rp,NPARS(rp))],LEN_FIELD)
         ip = ip + 1
         call ps_gwrd(Memc[data],ip,Memc[tmp])
         call strcpy(Memc[tmp],Memc[PMIN(rp,NPARS(rp))],LEN_FIELD)
         ip = ip + 1
         call ps_gwrd(Memc[data],ip,Memc[tmp])
         call strcpy(Memc[tmp],Memc[PMAX(rp,NPARS(rp))],LEN_FIELD)
      }  

      # Trim down the structure so that it only has as much space as it needs

      call ps_trim(rp)

      # Tidy and exit

      call sfree(sp)
end

int procedure ps_search(rp,parname)

# Parameters

pointer rp
char parname[ARB]
int np

# Local variables

int i

# Functions

bool streq()

begin

      # Search the parameter list for a match

      np = 0
      for (i = 1; i <= NPARS(rp) ; i = i + 1) {
         if (streq(Memc[PNAME(rp,i)],parname)) {
            np = i
            break
         }
      }

      # Send back the matching index.

      return(np)
end

int procedure ps_geti(cval)

# Parameters

char cval[ARB]
int ival

# Functions

int nscan()

begin

      # Scan the string and get a value off it if there is one to be had
   
      call sscan(cval)
         call gargi(ival)
      if (nscan() == 0)
         ival = INDEFI

      # Return the value

      return(ival)
end

real procedure ps_getr(cval)

# Parameters

char cval[ARB]
real rval

# Functions

int nscan()

begin

      # Scan the string and get a value off it if there is one to be had
   
      call sscan(cval)
         call gargr(rval)
      if (nscan() == 0)
         rval = INDEF

      # Return the value

      return(rval)
end

bool procedure ps_getb(cval)

# Parameters

char cval[ARB]
bool bval

begin

      # Scan the string and get a value off it 
   
      call sscan(cval)
         call gargb(bval)

      # Return the value

      return(bval)
end
