include "apinfo.h"
include <imhdr.h>

pointer procedure api_open(imname,censum,sp_axis,iptr)

# Parameters

char imname[ARB]
int censum,sp_axis
pointer iptr,ap

# Local variables

pointer pp,sp,line
int imlen,i,dispaxis
real x,x1,x2

# Functions

pointer clopset()
int clgpseti(),ctor()
real clgpsetr()

begin

    # Get some workspace

    call smark(sp)
    call salloc(line,SZ_LINE,TY_CHAR)

    # First allocate the necessary workspace

    call calloc(ap,NAPINF,TY_STRUCT)
    call malloc(INNAME(ap),SZ_FNAME,TY_CHAR)
    call malloc(LOW(ap),SZ_LINE,TY_CHAR)
    call malloc(HIGH(ap),SZ_LINE,TY_CHAR)
    call malloc(BFUNC(ap),SZ_LINE,TY_CHAR)
    call malloc(BSAMP(ap),SZ_LINE,TY_CHAR)

    # Open the rpars pset and read some of the info...

    pp = clopset("rpars")
    call clgpseta(pp,"b_function",Memc[BFUNC(ap)],SZ_LINE)
    BORD(ap) = clgpseti(pp,"b_order")
    call clgpseta(pp,"b_sample",Memc[BSAMP(ap)],SZ_LINE)
    BNAVE(ap) = clgpseti(pp,"b_naverage")
    BNIT(ap) = clgpseti(pp,"b_niterate")
    BLREJ(ap) = clgpsetr(pp,"b_low_reject")
    BHREJ(ap) = clgpsetr(pp,"b_high_reject")
    BGROW(ap) = clgpsetr(pp,"b_grow")
    dispaxis = 3 - sp_axis
    imlen = IM_LEN(iptr,dispaxis)
    call strcpy(Memc[BSAMP(ap)],Memc[line],SZ_LINE)
    for (i = line; Memc[i] != EOS; i = i + 1)
       if (Memc[i] == ':') Memc[i] = ','
    for (i = 1; Memc[line+i-1] != EOS; i = i + 1) {
       if (Memc[line+i-1] == '*') {
          x1 = min(x1,real(-imlen))
          x2 = max(x2,real(imlen))
       } else if (ctor(Memc[line],i,x) > 0) {
          x1 = min(x1,x)
          x2 = max(x2,x)
          i = i - 1
       }
    }
    BXMIN(ap) = x1
    BXMAX(ap) = x2

    # Ok, fill in some more of the info...

    call strcpy(imname,Memc[INNAME(ap)],SZ_FNAME)
    if (sp_axis == 1) {
       YCEN(ap) = censum
    } else {
       XCEN(ap) = censum
    }
    NCURVE(ap) = NC
    CURVE(ap,1) = real(sp_axis)
    CURVE(ap,2) = 1.0
    CURVE(ap,3) = 1.0
    CURVE(ap,4) = imlen
    CURVE(ap,5) = 0.0
    LOWER(ap) = clgpsetr(pp,"lower")
    UPPER(ap) = clgpsetr(pp,"upper")
    SPAX(ap) = sp_axis
    IMLEN(ap) = imlen

    # Return all this.  The only things left to define are the x or y 
    # position of the centre, the aperture number and the low and high bands.

    call clcpset(pp)
    call sfree(sp)
    return (ap)
end

procedure api_close(ap)

# Parameters

pointer ap

begin

    # Ditch all workspace.

    call mfree(INNAME(ap),TY_CHAR)
    call mfree(LOW(ap),TY_CHAR)
    call mfree(HIGH(ap),TY_CHAR)
    call mfree(BFUNC(ap),TY_CHAR)
    call mfree(BSAMP(ap),TY_CHAR)
    call mfree(ap,TY_STRUCT)

end

procedure api_fill(ap,apno,cen)

# Parameters

pointer ap
int apno
real cen

# Local variables

real spval1,spval2

begin

      # Fill in the required values...

      APNO(ap) = apno
      if (SPAX(ap) == 1) {
         XCEN(ap) = cen
         spval1 = YCEN(ap)
      } else {
         YCEN(ap) = cen
         spval1 = XCEN(ap)
      }
      spval2 = 1.0 - spval1
      call sprintf(Memc[LOW(ap)],SZ_LINE,"%g %g")
      if (SPAX(ap) == 1) {
         call pargr(LOWER(ap))
         call pargr(spval2)
      } else {
         call pargr(spval2)
         call pargr(LOWER(ap))
      }
      spval2 = IMLEN(ap) - spval1
      call sprintf(Memc[HIGH(ap)],SZ_LINE,"%g %g")
      if (SPAX(ap) == 1) {
         call pargr(UPPER(ap))
         call pargr(spval2)
      } else {
         call pargr(spval2)
         call pargr(UPPER(ap))
      }
end

procedure api_wdb(ap)

# Parameters

pointer ap

# Local variables

int i
pointer database,rootname,dbname,line,sp,dt

begin

      # Get some workspace

      call smark(sp)
      call salloc(database,SZ_FNAME,TY_CHAR)
      call salloc(rootname,SZ_FNAME,TY_CHAR)
      call salloc(dbname,SZ_FNAME,TY_CHAR)
      call salloc(line,SZ_LINE,TY_CHAR)

      # Get the name of the database directory and form db file name

      call clgstr("database",Memc[database],SZ_FNAME)
      call xt_imroot(Memc[INNAME(ap)],Memc[rootname],SZ_FNAME)
      call sprintf(Memc[dbname],SZ_FNAME,"ap%s")
         call pargstr(Memc[rootname])
      call dtremap(dt,Memc[database],Memc[dbname],APPEND)

      # Write out the necessary info

      call dtptime(dt)
      call sprintf(Memc[line],SZ_LINE,"begin\taperture %s %d %g %g\n")
         call pargstr(Memc[INNAME(ap)])
         call pargi(APNO(ap))
         call pargr(XCEN(ap))
         call pargr(YCEN(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\timage\t%s\n")
         call pargstr(Memc[INNAME(ap)])
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\taperture\t%d\n")
         call pargi(APNO(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\tbeam\t%d\n")
         call pargi(APNO(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\tcenter\t%g %g\n")
         call pargr(XCEN(ap))
         call pargr(YCEN(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\tlow\t%s\n")
         call pargstr(Memc[LOW(ap)])
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\thigh\t%s\n")
         call pargstr(Memc[HIGH(ap)])
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\tbackground\n")
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\t\txmin %g\n\t\txmax %g\n")
         call pargr(BXMIN(ap))
         call pargr(BXMAX(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\t\tfunction %s\n\t\torder %d\n\t\tsample %s\n")
         call pargstr(Memc[BFUNC(ap)])
         call pargi(BORD(ap))
         call pargstr(Memc[BSAMP(ap)])
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\t\tnaverage %d\n\t\tniterate %d\n")
         call pargi(BNAVE(ap))
         call pargi(BNIT(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\t\tlow_reject %g\n\t\thigh_reject %g\n\t\tgrow %g\n")
         call pargr(BLREJ(ap))
         call pargr(BHREJ(ap))
         call pargr(BGROW(ap))
      call dtput(dt,Memc[line])
      call sprintf(Memc[line],SZ_LINE,"\taxis\t%d\n\tcurve\t%d\n")
         call pargi(SPAX(ap))
         call pargi(NCURVE(ap))
      call dtput(dt,Memc[line])
      do i = 1,NCURVE(ap) {
         call sprintf(Memc[line],SZ_LINE,"\t\t%g\n")
            call pargr(CURVE(ap,i))
         call dtput(dt,Memc[line])
      }
      call dtput(dt,"\n")

      # Close up and get out of here

      call dtunmap(dt)
      call sfree(sp)
end

