procedure crzap(inlist,outlist)

string   inlist      {prompt="Images for cosmic ray cleaning"}
string   outlist     {prompt="Output images"}
real     nsigma      {5,prompt="Number of sky sigma for zapping threshold"}
int      xboxsz      {1,prompt="X Box size for zapping"}
int      yboxsz      {1,prompt="Y Box size for zapping"}
real     nobjsigma   {2,prompt="Number of sky sigma for object identification"}
int      nrings      {0,prompt="Number of pixels to flag as buffer around CRs"}
bool     donegative  {no,prompt="Clean deviant negative pixels?"}
real     nnegsigma   {5,prompt="Number of sky sigma for negative zapping"}
string   imgsec      {prompt="Image section to use for computing sky sigma"}
bool     verbose     {yes,prompt="Verbose output?"}

struct   *inimglist
struct   *outimglist

begin

      file temp1,temp2,medimage,crimage,mask1image,mask2image,crmask,cronly
      file maskneg,input,output
      int n1,n2,junk,nbox,nbox2
      bool maskobj
      real skymean,skysig,crthresh,objthresh,negthresh

      # Create some temporary file names

      temp1 = mktemp("tmp$iraf")
      temp2 = mktemp("tmp$iraf")
      medimage = mktemp("tmp$iraf")
      crimage = mktemp("tmp$iraf")
      mask1image = mktemp("tmp$iraf")
      mask2image = mktemp("tmp$iraf")
      crmask = mktemp("tmp$iraf")
      cronly = mktemp("tmp$iraf")
      maskneg = mktemp("tmp$iraf")

      # Expand the input file lists into temporary files

      sections(inlist,option="fullname",>temp1)
      sections(outlist,option="fullname",>temp2)
      count(temp1) | scan(n1)
      count(temp2) | scan(n2)
      if (n1 != n2) 
         error(0,"Input and output file lists are different lengths!")
      inimglist = temp1
      outimglist = temp2

      # Are you going to attempt to mask out objects?

      if (nobjsigma == 0) 
         maskobj = no
      else
         maskobj = yes

      # Loop through all of the input files

      while (fscan(inimglist,input) != EOF) {
         strpextn(input) | scan(input)
         junk = fscan(outimglist,output)
         strpextn(output) | scan(output)
         if (verbose)
            print("Working on ",input," --> ",output)

         # Get sky mean using iterative clipping scheme

         if (verbose) print("   Computing statistics")
         iterstat(input//imgsec,nsigrej=5.0,maxiter=3,verbose=no,
            lower=INDEF,upper=INDEF) | 
            scanf("%s mean=%g rms=%g",junkstr,skymean,skysig)

         # Median filter over the input box size

         if (verbose) print("   Median filtering")
         median(input,medimage,xboxsz,yboxsz,zloreject=INDEF,zhireject=INDEF,
            boundary="reflect",constant=0.0,verbose=no)

         # Take a difference between the median filtered image and the
         # original image.  This should give a cosmic ray "unsharp" mask.

         imarith(input,"-",medimage,crimage,verbose=no,noact=no)
         imcopy(crimage,mask1image,verbose=no)

         # Threshold the mask image so that any any pixels above the threshold
         # skysig*nsigma are 1 and the rest are zeros.  Note the threshold
         # doesn't have a 'skymean' zeropoint since the mask image should have
         # roughly a zero mean (it's the original minus the meadian filtered
         # image).

         if (verbose) print("   Masking potential CR events")
         crthresh = nsigma*skysig
         imreplace(mask1image,0.0,imaginary=0.0,lower=INDEF,upper=crthresh)
         imreplace(mask1image,1.0,imaginary=0.0,lower=crthresh,upper=INDEF)
 
         # Make a pixel list format version to save space...

         imcopy(mask1image,mask1image//".pl",verbose=no)
         imdelete(mask1image,verify=no)

         # If you want to make an object mask, then do that here...

         if (maskobj) {
            if (verbose) print("   Creating object mask")

            # Rename the median filtered file

            imrename(medimage,mask2image,verbose=no)

            # Mask the median file by an object threshold defined as:
            # objthresh = nobjsigma*skysig + skymean.  Replace background
            # values with 1 and any pixels above the object threshold (objects)
            # with zeros. (You must do this in the correct order)

            objthresh = skymean + nobjsigma*skysig
            if (objthresh > 1.0) {
               imreplace(mask2image,1.0,imaginary=0.0,lower=INDEF,
                  upper=objthresh)
               imreplace(mask2image,0.0,imaginary=0.0,lower=objthresh,
                  upper=INDEF)
            } else if (objthresh < 0.0) {
               imreplace(mask2image,0.0,imaginary=0.0,lower=objthresh,
                  upper=INDEF)
               imreplace(mask2image,1.0,imaginary=0.0,lower=INDEF,
                  upper=objthresh)
            } else {
               imreplace(mask2image,1.0,imaginary=0.0,lower=objthresh,
                  upper=INDEF)
               imreplace(mask2image,2.0,imaginary=0.0,lower=INDEF,
                  upper=objthresh)
               imarith(mask2image,"-",1.0,mask2image,verbose=no,noact=no)
            }

            # Now make a .pl version of this mask...

            imcopy(mask2image,mask2image//".pl",verbose=no)
            imdelete(mask2image,verify=no)

            # The final CR mask will be the product of the object mask and
            # the CR mask done in the last section:

            imarith(mask1image//".pl","*",mask2image//".pl",crmask//".pl",
               verbose=no,noact=no)
            imdelete(mask1image//".pl",verify=no)
            imdelete(mask2image//".pl",verify=no)

         # If no object masking is done, then the final CR mask is just
         # the CR mask done in the last section:

         } else {
            imrename(mask1image//".pl",crmask//".pl",verbose=no)
            imdelete(medimage,verify=no)
         }

         # Ok, if you want to grow rings around the identified CR events,
         # then do that here.

         if (nrings > 0) {
            if (verbose) print("   Growing mask rings around CR hits")
            nbox = 2*nrings + 1
            nbox2 = nbox**2
            imarith(crmask//".pl","*",nbox2,crmask//".pl",verbose=no,noact=no)
            boxcar(crmask//".pl",crmask//".pl",xwindow=nbox,ywindow=nbox,
               boundary="nearest",constant=0.0)
            imreplace(crmask//".pl",1.0,imaginary=0.0,lower=1.0,upper=INDEF)
         }

         # Identify negative events if you want to.  There are no grow rings
         # for negative events.

         if (donegative) {
            if (verbose) print("   Masking deviant negative pixels")
            imcopy(crimage,maskneg,verbose=no)
            negthresh = -1.0*nnegsigma*skysig
            imreplace(maskneg,0.0,imaginary=0.0,lower=negthresh,upper=INDEF)
            imreplace(maskneg,1.0,imaginary=0.0,lower=INDEF,upper=negthresh)
            imcopy(maskneg,maskneg//".pl",verbose=no)
            imdelete(maskneg,verify=no)
            imarith(crmask//".pl","+",maskneg//".pl",crmask//".pl",noact=no,
               verbose=no)
            imreplace(crmask//".pl",1,lower=1,upper=INDEF)
            imdelete(maskneg//".pl",verify=no)
         }

         # Multiply the unsharp mask by the final cosmic ray mask.  This should
         # give an image which has the location of each CR event, and the
         # amount each one is over the local median.

         if (verbose) print("   Replacing CR hits with local median")
         imarith(crimage,"*",crmask//".pl",cronly,noact=no,verbose=no)
         imdelete(crimage,verify=no)
         imdelete(crmask//".pl",verify=no)

         # Now subtract this file from the original to give the cleaned image
 
         imarith(input,"-",cronly,output,noact=no,verbose=no)
         imdelete(cronly,verify=no)
         if (verbose) print("   Finished file ",input)
      }

      # Clean up some temporary files, send a helpful message and get out
      # of here.

      if (verbose) print("Done")
      delete(temp1,verify=no)
      delete(temp2,verify=no)
      inimglist = ""; outimglist = ""
end
