procedure mksens(input,fluxfile,sensfile,autonm)

file    input                  {prompt="Input file with standard star spectrum"}
file    fluxfile               {prompt="Output flux file"}
file    sensfile               {prompt="Output sensitivity function file"}
bool    autonm = yes           {prompt="Sort out star name automatically?"}
file    rootdir = "onedstds$"  {prompt="Root directory for calibration data"}
real    bandwidth = INDEF      {prompt="Band width for calibration data"}
real    bandsep = INDEF        {prompt="Band separation for calibration data"}
bool    ans                    {prompt="Is this ok?",mode="q"}
string  dname                  {prompt="Path for calibration file",mode="q"}
string  starname               {prompt="Star name",mode="q"}

struct  *fd
begin

      file in,temp,dirname,fname,stname,flx,sns
      string lponame,othername,egname,junkstr,msg
      int junk,nw,junki
      real w1,dw,w2
      bool ans2

      # Get input file name and make sure it's wavelength calibrated

      in = input
      flx = fluxfile
      sns = sensfile
      hselect(in,"dc-flag",yes) | scan(junkstr)
      if (nscan() == 0)
         error(0,"Input file isn't wavelength calibrated")

      # If using the auto name facility then find out what the name of the
      # object is from the header.

      if (autonm) {
         if (telescope == "WHT")
            hselect(in,"cat-name",yes) | scan(lponame)
         else
            hselect(in,"i_title",yes) | scan(lponame)
         othername = ""; egname = ""
         stdname(lponame) | scan(othername,egname)
         temp = mktemp("tmp$iraf")

         # Find the spectral range of the standard star spectrum

         slist(in,apertures="",long_header=no,>>temp)
         fd = temp
         junk = fscan(fd,junkstr,junki,junki,junki,junki,w1,dw,nw)
         w2 = w1 + real(nw - 1)*dw
         delete(temp,verify=no)

         # Find out which directory in the root directory has the best
         # calibration data for your star

         stddir(othername,egname,temp,rootdir,w1,w2)
         if (access(temp)) {
            print("\nResults of directory search:")
            type(temp,map_cc=yes,device="terminal")
            print("\n")
            fd = temp
            junk = fscan(fd,dirname,fname)
            printf("I'll use %s, %s\n",dirname,fname)
            ans2 = ans
            if (!ans2)
               dirname = dname
            delete(temp,verify=no)
         } else
            error(0,"No calibration data found for your star")
         stname = substr(fname,1,strlen(fname)-4)

      # If you're not using the automatic name cross-reference, then you have
      # to include a complete directory name and a star name.  The star name 
      # needs to be the same format as the standard star file names. (e.g.
      # G191-B2B has a calibration file g191b2b.dat, all lower case with no
      # spaces, minus signs etc...)

      } else {
         dirname = dname
         print(starname) | lcase | scan(stname)
         fname = stname//".dat"
         if (!access(dirname//fname)) {
            printf("Can't find file %s",dirname//fname) | scan(msg)
            error(0,msg)
         }
      }

      # Fit the flux and create the sensitivity function

      standard.answer=yes
      sensfunc.answer=yes
      standard(in,flx,samestar=yes,beam_switch=no,apertures="",
         bandwidth=bandwidth,bandsep=bandsep,fnuzero=3.68e-20,
         extinction=extfile,caldir=dirname,observatory=observatory,
         interact=yes,graphics="stdgraph",cursor="",star_name=stname)
      sensfunc(flx,sns,apertures="",ignoreaps=yes,logfile=logfile,
         extinction=extfile,newextinction="extinct.dat",
         observatory=observatory,function="spline3",order=6,interactive=yes,
         graphs="si",marks="plus cross box",colors="2 1 3 4",cursor="",
         device="stdgraph")
end
