procedure t_bdsect()

# Local variables

pointer sp,inlist,tel,fname,dsect,bsect,list,iptr,chipname
real dxs,dxe,dys,dye,axs,axe,ays,aye,xb,yb,xs,ys,x1,x2,y1,y2,len,bxs,bxe,
   bys,bye
char wmode[1]
bool wm

# Functions

pointer imtopen(),immap()
int imtgetim(),strmatch()
bool imaccf(),streq()
real imgetr()

begin

      # Get workspace

      call smark(sp)
      call salloc(inlist,SZ_LINE,TY_CHAR)
      call salloc(tel,SZ_LINE,TY_CHAR)
      call salloc(fname,SZ_FNAME,TY_CHAR)
      call salloc(dsect,SZ_LINE,TY_CHAR)
      call salloc(bsect,SZ_LINE,TY_CHAR)
      call salloc(chipname,SZ_LINE,TY_CHAR)

      # Get input file list and telescope name

      call clgstr("inlist",Memc[inlist],SZ_LINE)
      call clgstr("telescope",Memc[tel],SZ_LINE)

      # Expand input file list and begin loop

      list = imtopen(Memc[inlist])
      while (imtgetim(list,Memc[fname],SZ_FNAME) != EOF) {

         # Open the file

         iferr(iptr = immap(Memc[fname],READ_WRITE,0))
            next

         # Check and see if 'biassec' and 'trimsec' are already there...

         if (imaccf(iptr,"BIASSEC") == YES && imaccf(iptr,"TRIMSEC") == YES)
            next

         # Section for JKT and INT data

         if (streq(Memc[tel],"JKT") || streq(Memc[tel],"INT")) {

            # Read info on which part of the chip has been read out.

            iferr {
               dxs = imgetr(iptr,"dxstart")
               dxe = imgetr(iptr,"dxend")
               dys = imgetr(iptr,"dystart")
               dye = imgetr(iptr,"dyend")
            } then {
               call eprintf("Data window info not available in: %s\n")
                  call pargstr(Memc[fname])
               next
            }
 
            # Read info on which part of the chip is light sensitive

            iferr {
               axs = imgetr(iptr,"axstart")
               axe = imgetr(iptr,"axend")
               ays = imgetr(iptr,"aystart")
               aye = imgetr(iptr,"ayend")
            } then {
               call eprintf("Data window info not available in: %s\n")
                  call pargstr(Memc[fname])
               next
            }

            # Now read the binning info

            iferr {
               xb = imgetr(iptr,"xbin")
               yb = imgetr(iptr,"yb")
            } then {
               call eprintf("Data binning info not available in: %s\n")
                  call pargstr(Memc[fname])
               next
            }

         # Section for WHT files

         } else {

            # First check and see if windowing information is available.

            iferr (call imgstr(iptr,"ccdwmode",wmode,1))
               wm = true
            else 
               wm = streq(wmode,"T")

            # Define the light sensitive part of the chip

            iferr {
               axs = imgetr(iptr,"ccdximst")
               xs = imgetr(iptr,"ccdximsi")
               ays = imgetr(iptr,"ccdyimst")
               ys = imgetr(iptr,"ccdyimsi")
            } then {
               call eprintf("Data window info not available in: %s\n")
                  call pargstr(Memc[fname])
               next
            }
            axe = axs + xs - 1.0
            aye = ays + ys - 1.0

            # Define the section of the chip which was read out.  If windowing
            # was enabled, read the data from the windowing header info.

            if (wm) {
               iferr {
                  dxs = imgetr(iptr,"ccdwxo1")
                  xs = imgetr(iptr,"ccdwxs1")
                  dys = imgetr(iptr,"ccdwyo1")
                  ys = imgetr(iptr,"ccdwys1")
               } then {
                  call eprintf("Data window info not available in: %s\n")
                     call pargstr(Memc[fname])
                  next
               }
               dxe = dxs + xs
               dxs = dxs + 1.0
               dye = dys + ys
               dys = dys + 1.0
               iferr {
                  xb = imgetr(iptr,"ccdxbin")
                  yb = imgetr(iptr,"ccdybin")
               } then {
                  call eprintf("Data binning info not available in: %s\n")
                     call pargstr(Memc[fname])
                  next
               }
            } else {
               iferr {
                  dxe = imgetr(iptr,"ccdxsize")
                  dye = imgetr(iptr,"ccdysize")
               } then {
                  call eprintf("Data window info not available in: %s \n")
                     call pargstr(Memc[fname])
                  next
               }
               dxs = 1.0
               dys = 1.0
               xb = 1.0
               yb = 1.0
            }
         }

         # Calculate the ends of the data section

         x1 = (axs - dxs)/xb + 1.0
         x1 = max(1.0,x1)
         len = (min(axe,dxe) - max(axs,dxs) + 1.0)/xb
         x2 = x1 + len - 1.0
         y1 = (ays - dys)/yb + 1.0
         y1 = max(1.0,y1)
         len = (min(aye,dye) - max(ays,dys) + 1.0)/yb
         y2 = y1 + len - 1.0

         # Create data section string
     
         call sprintf(Memc[dsect],SZ_LINE,"[%d:%d,%d:%d]")
            call pargi(nint(x1))
            call pargi(nint(x2))
            call pargi(nint(y1))
            call pargi(nint(y2))

         # Get chip name...

         call imgstr(iptr,"ccdchip",Memc[chipname],SZ_LINE)

         # This section for bias strip if it isn't a Loral chip...

         if (strmatch(Memc[chipname],"LOR") != 4) {

            # Define the maximum boundaries for the bias strip

            bxs = 6.0
            bxe = axs - 6.0
            bys = 101.0
            bye = aye - 101.0

            # Check and make sure the bias strip hasn't been windowed out.

            if (dxs > bxe ) {
               call eprintf("The bias strip has been windowed out in: %s\n")
                  call pargstr(Memc[fname])
               next
            }

            # Now define the bias strip limits based on header info

            x1 = (bxs - dxs)/xb + 1.0
            x1 = max(1.0,x1)
            len = (min(bxe,dxe) - max(bxs,dxs) + 1.0)/xb
            x2 = x1 + len - 1.0
            y1 = (bys - dys)/yb + 1.0
            y1 = max(1.0,y1)
            len = (min(bye,dye) - max(bys,dys) + 1.0)/yb
            y2 = y1 + len - 1.0

            # Create bias section string
     
            call sprintf(Memc[bsect],SZ_LINE,"[%d:%d,%d:%d]")
               call pargi(nint(x1))
               call pargi(nint(x2))
               call pargi(nint(y1))
               call pargi(nint(y2))
         } else {
            call sprintf(Memc[bsect],SZ_LINE,"[2054:2100,1:600]")
         }

         # Now edit these sections into the header of the input file.

         call imastr(iptr,"biassec",Memc[bsect])
         call imastr(iptr,"trimsec",Memc[dsect])

         # Close the file

         call imunmap(iptr)
      }

      # Tidy and exit

      call imtclose(list)
      call sfree(sp)
end
