define  FOS2MAXORD    2
define  FOS2CW1       7290.0
define  FOS2DW1       8.75
define  FOS2CW2       4670.0
define  FOS2DW2       4.33
define  FOS2SIGN      -1

define  FOS1MAXORD    2
define  FOS1CW1       8330.0
define  FOS1DW1       10.72
define  FOS1CW2       4170.0
define  FOS1DW2       5.35
define  FOS1SIGN      -1

include <error.h>

procedure t_wave0()

# Local variables

pointer sp,infile,tel,instr,iptr
int dispaxis,order,is
real cw,dw,xpix

# Functions

pointer immap()
int clgeti()
real imgetr(),clgetr()
bool streq()

begin

      # Get some workspace

      call smark(sp)
      call salloc(infile,SZ_FNAME,TY_CHAR)
      call salloc(tel,SZ_LINE,TY_CHAR)
      call salloc(instr,SZ_LINE,TY_CHAR)

      # Get the input file name, the telescope and the instrument name.

      call clgstr("input",Memc[infile],SZ_FNAME)
      iferr (iptr = immap(Memc[infile],READ_ONLY,0)) {
         call eprintf("Can't open file %s \n")
            call pargstr(Memc[infile])
         call erract(EA_ERROR)
      }
      call clgstr("telescope",Memc[tel],SZ_LINE)
      call clgstr("instrument",Memc[instr],SZ_LINE)

      # Get the dispersion axis.

      dispaxis = clgeti("dispaxis")

      # Section for WHT spectrographs...

      if (streq(Memc[tel],"WHT")) {

         # If the instrument is FOS2, then just set the required values.

         if (streq(Memc[instr],"fos2")) {
            order = clgeti("order")
            if (order > FOS2MAXORD) 
               call error(0,"Order number too high")
            switch (order) {
               case 1:
                  cw = FOS2CW1
                  dw = FOS2DW1
               case 2:
                  cw = FOS2CW2
                  dw = FOS2DW2
            }
            is = FOS2SIGN

         # Otherwise read the relevant info from the WHT header

         } else {
            iferr {
               cw = imgetr(iptr,"cenwave")
               dw = imgetr(iptr,"dispersi")
               if (dispaxis == 1) 
                  xpix = imgetr(iptr,"ccdxpixe")
               else
                  xpix = imgetr(iptr,"ccdypixe")
               dw = 0.001*dw*xpix
               call whtsign(iptr,is)
            } then {
               call error(0,"Missing header information")
            }
         }

      # Section for INT spectrographs...

      } else if (streq(Memc[tel],"INT")) {
       
         # If the instrument is FOS1, then just set the required values.

         if (streq(Memc[instr],"fos1")) {
            order = clgeti("order")
            if (order > FOS1MAXORD) 
               call error(0,"Order number too high")
            switch (order) {
               case 1:
                  cw = FOS1CW1
                  dw = FOS1DW1
               case 2:
                  cw = FOS1CW2
                  dw = FOS1DW2
            }
            is = FOS1SIGN
            
         # Otherwise read the relevant info from the INT header

         } else {
            iferr {
               cw = imgetr(iptr,"cenwave")
               call idsdisp(iptr,dispaxis,dw,is)
            } then {
               call error(0,"Missing header information")
            }
         }
      
      # Section for JKT spectrographs

      } else if (streq(Memc[tel],"JKT")) {
         cw = clgetr("cenwave")
         dw = clgetr("disp")
         is = clgetr("isign")
      } else {
         call error(0,"Illegal telescope name")
      }

      # Print out the results

      call printf("%0.6g %0.6g %d\n")
         call pargr(cw)
         call pargr(dw)
         call pargi(is)

      # Tidy and exit

      call imunmap(iptr)
      call sfree(sp)
end


procedure whtsign(iptr,is)

# Parameters

pointer iptr
int is

# Local variables

pointer sp,isiarm,chip

# Functions

bool streq()

begin

      # Get workspace

      call smark(sp)
      call salloc(isiarm,SZ_LINE,TY_CHAR)
      call salloc(chip,SZ_LINE,TY_CHAR)

      # Get info from header on the ISIS arm and the chip name.  Convert 
      # each to upper case.

      call imgstr(iptr,"isiarm",Memc[isiarm],SZ_LINE)
      call strupr(Memc[isiarm])
      call imgstr(iptr,"ccdchip",Memc[chip],SZ_LINE)
      call strupr(Memc[chip])

      # Look at the current blue arm and chip combinations

      if (streq(Memc[isiarm],"BLUE ARM")) {
         if (streq(Memc[chip],"TEK2") || streq(Memc[chip],"EEV6") ||
            streq(Memc[chip],"TEK5"))
            is = -1
         else if (streq(Memc[chip],"TEK1") || streq(Memc[chip],"EEV3"))
            is = 1
         else
            is = 0
      
      # Look at the current red arm and chip combinations

      } else if (streq(Memc[isiarm],"RED ARM")) {
         if (streq(Memc[chip],"TEK1") || streq(Memc[chip],"EEV3"))
            is = -1
         else if (streq(Memc[chip],"TEK2") || streq(Memc[chip],"EEV6") ||
            streq(Memc[chip],"TEK5"))
            is = 1
         else
            is = 0
      } else {
         call error(0,"Illegal ISIS arm name in header")
      }

      # Tidy

      call sfree(sp)
end

procedure idsdisp(iptr,dispaxis,dw,is)

# Parameters

pointer iptr
int dispaxis,is
real dw

# Local variables

char cam[3]
int grating
real app,dx

# Functions

int imgeti()
real imgetr()
bool streq()

begin

      # Start by getting the first three characters of the camera name and
      # the grating rule.  Also, get the pixel size in the dispersion axis

      iferr {
         call imgstr(iptr,"camera",cam,3)
         grating = imgeti(iptr,"gratrule")
         if (dispaxis == 1) 
            dx = imgetr(iptr,"pixxsize")
         else
            dx = imgetr(iptr,"pixysize")
      } then {
         call error(0,"IDS header information missing")
      }

      # Find the correct camera/grating combination 

      if (streq(cam,"235")) {
         switch (grating) {
            case 150:
               app = 271.4
            case 300:
               app = 138.6
            case 400:
               app = 104.5
            case 600:
               app = 70.00
            case 632:
               app = 66.66
            case 831:
               app = 50.91
            case 900:
               app = 46.36
            case 1200:
               app = 35.45
            case 1800:
               app = 23.18
            case 2400:
               app = 17.27
            default:
               call error(0,"Illegal IDS grating in header")
         }
         is = 1
      } else if (streq(cam,"500")) {
         switch (grating) {
            case 150:
               app = 132.3
            case 300:
               app = 72.50
            case 400:
               app = 49.54
            case 600:
               app = 33.18
            case 632:
               app = 31.81
            case 831:
               app = 24.09
            case 900:
               app = 22.27
            case 1200:
               app = 16.81
            case 1800:
               app = 10.45
            case 2400:
               app = 7.27
            default:
               call error(0,"Illegal IDS grating in header")
         }
         is = -1
      } else {
         call error(0,"Illegal IDS camera in header")                        
      }

      # Scale the dispersion by the appropriate pixel size

      dw = app*1000.0*dx
end
