procedure preccd(biasfrs,flatfrs,illumfrs,mbias,mflat,millum,interfit)

string  biasfrs  {prompt="List of bias frames"}
string  flatfrs  {prompt="List of tungsten flat frames"}
string  illumfrs {prompt="List of sky flat frames"}
file    mbias    {prompt="Mean bias frame"}
file    mflat    {prompt="Mean tungsten flat frame"}
file    millum   {prompt="Mean sky flat frame"}
bool    interfit {prompt="Do fits interactively?"}

struct  *fd
begin

      string log1,log2,blist,flist,temp1,imtype,filstr,pm,ilist
      file meanbias,meanflat,meanillum
      int nbias,nflat,nillum
      bool procbias,procflat,procillum,ifit

      # Set up the logging...

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose)
         log2 = "STDOUT"
      else
         log2 = "dev$null"

      # Get bias and flat field name lists

      blist = biasfrs
      flist = flatfrs
      ilist = illumfrs

      # Set up temp file names

      temp1 = mktemp("tmp$iraf")

      # Get image type

      imtype = "."//envget("imtype")

      # Find out how many frames of bias and flat types you have

      nbias = 0
      sections(blist,option="fullname") | count("STDIN") | scan(nbias)
      nflat = 0
      sections(flist,option="fullname") | count("STDIN") | scan(nflat)
      nillum = 0
      sections(ilist,option="fullname") | count("STDIN") | scan(nillum)

      # Get a name for the mean bias frame Do the same for the mean flat frame.

      strpextn(mbias) | scan(meanbias)
      strpextn(mflat) | scan(meanflat)
      strpextn(millum) | scan(meanillum)

      # Set up instrument name 

      setinstrument(instrument,site=observatory,directory="rgo$ccddb/",rev=no)
 
      # Set up ccdred parameters

      ccdred.logfile = logfile
      ccdred.backup = backup

      # If the ccdred backup prefix is a directory, then create it if it 
      # doesn't already exist

      if (stridx("/$]",backup) != 0)
         if (!access(backup)) mkdir(backup)

      # If you are processing bias frames, then do that here.  

      procbias = (nbias > 0 && !access(meanbias//imtype) && meanbias != "")
      if (procbias) {
         print("Processing bias frames")
         zerocombine(blist,output=meanbias) | tee (log1, > log2)
         evalexpr("BIASSEC",meanbias)
         evalexpr("TRIMSEC",meanbias)
      }

      # If a mean bias frame is specified or created, then subtract off a
      # bias frame. Otherwise subtract a constant bias.

      if ((meanbias != ""  && access(meanbias//imtype)) || procbias) {
         ccdproc.zerocor = yes
         ccdproc.zero = meanbias
      } else
         ccdproc.zerocor = no

      # If you are processing flat frames, then do that here.  

      ifit = interfit
      procflat = (nflat > 0 && !access(meanflat//imtype) && meanflat != "")
      if (procflat) {
         print("Processing flat-field frames")
         flatcombine(flist,output=meanflat) | tee (log1, > log2)
         response(meanflat,meanflat,temp1,interactive=ifit,
            sample="*",naverage=1,function="spline3",order=30,low_reject=3,
            high_reject=3,niterate=1,grow=0,graphics="stdgraph",cursor="")
         imdelete(meanflat,verify=no)
         imrename(temp1,meanflat,verbose=no)
         evalexpr("BIASSEC",meanflat)
         evalexpr("TRIMSEC",meanflat)
      }

      # If a mean flat frame is specified or created, then signal for doing
      # the flat fielding.

      if ((meanflat != ""  && access(meanflat//imtype)) || procflat) {
         ccdproc.flatcor = yes
         ccdproc.flat = meanflat
      } else
         ccdproc.flatcor = no

      # If you are processing sky flats then do that here. Note that
      # flatcombine should debias and flat field ilist before combining.

      procillum = (nillum > 0 && !access(meanillum//imtype) && meanillum != "")
      if (procillum) {
         print("Processing illumination frames")
         flatcombine(ilist,output=meanillum) | tee (log1, > log2)
         hedit(meanillum,"dispaxis",dispaxis,add=yes,delete=no,verify=no,
            show=no,update=yes)
         illumination(meanillum,temp1,interactive=ifit,
            sample="*",naverage=1,function="spline3",order=2,low_reject=3.0,
            high_reject=3.0,niterate=1,grow=0,interpolator="poly3",
            graphics="stdgraph",cursor="")
         imdelete(meanillum,verify=no)
         imrename(temp1,meanillum,verbose=no)
         evalexpr("BIASSEC",meanillum)
         evalexpr("TRIMSEC",meanillum)
      }

      # If a mean sky flat frame is specified or created, then signal for doing
      # the flat fielding.

      if ((meanillum != ""  && access(meanillum//imtype)) || procillum) {
         ccdproc.illumcor = yes
         ccdproc.illum = meanillum
      } else
         ccdproc.illumcor = no

end
