procedure rspecred()

string     imlist = ""    {prompt="List of objects to be reduced"}
string     arclist = ""   {prompt="List of arc files"}
string     biases = ""    {prompt="List of bias frames"}
string     flats = ""     {prompt="List of flat field frames"}
file       mbias = ""     {prompt="Name for mean bias frame"}
file       mflat = ""     {prompt="Name for mean flat frame"}
bool       dispcor = yes  {prompt="Dispersion correct spectra?"}
bool       tippex = yes   {prompt="Tippex out sky line residuals?"}
file       tfile = ""     {prompt="Name of tippex file"}
bool       splotres = yes {prompt="Plot final spectrum?"}
int        autoid = 1     {prompt="What level of automation in arcline id?"}
bool       redo = no      {prompt="Redo operations if previously done?"}
bool       zapcosmic = no {prompt="Zap cosmic rays before tracing spectrum?"}
bool       dcorarcs = yes {prompt="Dispersion correct arcs?"}
bool       fluxcal = yes  {prompt="Do a flux calibration?"}
file       sensfunc = ""  {prompt="Sensitivity file for flux calibration"}
file       coorlist       {prompt="Name of arcline file"}

pset       *rpars = ""
struct     *fd
struct     *fd2

begin

      string objs,ars,bis,fls,log1,log2,extn,ra,dec,imtype,idname,fmtname,junk
      string dofind
      struct title,data
      file objfile,arcfile,biasfile,flatfile,temp1,temp2,ofile,outfile
      file arc1,arc2,aoname1,aoname2,arefs,rname,tf,sf
      int isign,i,nsp,nsc,nx,dc_flag,jst,jfn,isign2,junki
      real srange,cw_est,disp_est,cw_est2,dw_est2,aplow,aphigh
      bool reextract,splotr,dcor,tipp,reid,test1,test2,do_ext,do_dcor,do_tipp
      bool zapcrs,dcar,flxcal,do_flx

      # Get the lists of files to be processed

      objs = imlist
      ars = arclist
      bis = biases
      fls = flats

      # Make sure that there are some objects listed...

      if (objs == "")
         error(0,"No object files have been specified")

      # See if you want to redo the work on the object files if previously 
      # done. Also look and see if the user wants to splot the results,
      # zap cosmic rays before locating spectra and scrunch the arcs.

      reextract = redo
      splotr = splotres
      zapcrs = zapcosmic
      dcar = dcorarcs

      # Set up logging...

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose)
         log2 = "STDOUT"
      else
         log2 = "dev$null"

      # Set up some temporary files which will contain lists of files
      # to be processed

      objfile = mktemp("tmp$iraf")
      arcfile = mktemp("tmp$iraf")
      biasfile = mktemp("tmp$iraf")
      flatfile = mktemp("tmp$iraf")
      temp1 = mktemp("tmp$iraf")
      temp2 = mktemp("tmp$iraf")

      # Now fill the file lists

      filllist(objs,objfile)
      filllist(ars,arcfile)
      filllist(bis,biasfile)
      filllist(fls,flatfile)

      # Define multispec extension

      imtype = "."//envget("imtype")
      extn = ".ms"//imtype

      # Sort out a few switches.  If there are no arc files, then you can't
      # possibly dispersion correct.  Similarly, if there is no tippex file,
      # then you can't do tippexing...

      dcor = dispcor
      if (ars == "" && dcor) {
         dcor = no
         print("No arc files specified => no dispersion correction possible") |
            tee(log1, > log2)
      }
      tipp = tippex
      tf = tfile
      if (tipp && !access(tf)) {
         tipp = no
         print("Cannot find a tippex file => no tippexing will be done") |
            tee(log1, > log2)
      }
      flxcal = fluxcal
      sf = ""
      strpextn(sensfunc) | scan(sf)
      if (flxcal && !access(sf//imtype)) {
         flxcal = no
         print("Cannot find sensitivity file => no flux calib. will be done") |
            tee(log1, > log2)
      }

      # Sort out the data and bias sections for each of the non-bias frames.

      evalexpr("BIASSEC","@"//objfile)
      evalexpr("TRIMSEC","@"//objfile)
      if (dcor) {
         evalexpr("BIASSEC","@"//arcfile)
         evalexpr("TRIMSEC","@"//arcfile)
      }
      if (fls != "") {
         evalexpr("BIASSEC","@"//flatfile)
         evalexpr("TRIMSEC","@"//flatfile)
      }

      # Prepare the bias and flat frames and set up the ccd instrumental
      # parameters for ccdred.

      preccd("@"//biasfile,"@"//flatfile,"",mbias,mflat,"",rpars.interactive)

      # Now do the ccd processing on the object and arc frames

      ccdproc("@"//objfile)
      if (dcor)
         ccdproc("@"//arcfile)

      # Set some defaults in apscript

      apscript.find = yes
      apscript.trace = yes
      apscript.extract = yes
      apscript.subtract = no
      apscript.smooth = no
      apscript.fitscatter = no
      apscript.fitsmooth = no

      # Now set some defaults depending upon user choices

      if (rpars.recenter)
         apscript.ansrecenter = "YES"
      else
         apscript.ansrecenter = "NO"
      if (rpars.resize)
         apscript.ansresize = "YES"
      else
         apscript.ansresize = "NO"
      if (rpars.edit)
         apscript.ansedit = "yes"
      else
         apscript.ansedit = "NO"
      apscript.ansfittrace = "NO"
      apscript.ansreview = "NO"

      # Set the Julian dates for all of the arc files (if you have any)

      if (dcor) {
         junk = "@"//arcfile//",@"//objfile
         evalexpr("RA",junk)
         evalexpr("DEC",junk)
         evalexpr("EPOCH",junk)
         evalexpr("UT",junk)
         evalexpr("ST",junk)
         evalexpr("DATE_OBS",junk)
         evalexpr("EXPOSURE",junk)
         setjd("@"//arcfile,observatory=observatory,date="date_obs",
            time="ut",exposure="exposure",ra="ra",dec="dec",epoch="epoch",
            jd="jd",hjd="hjd",ljd="ljd",utdate=yes,uttime=yes,
            listonly=no) | tee (log1, > log2)
      }

      # Edit the header item "REFSPEC1" into the header of the of each
      # arc file so that you can fool 'refspec' into thinking that they have
      # already been used in 'identify' and are hence proper reference images

      hedit("@"//arcfile,"REFSPEC1","($I)",add=yes,delete=no,verify=no,show=no,
         update=yes) | tee(log1, >log2)

      # Now loop for the individual object files

      fd = objfile
      while (fscan(fd,ofile) != EOF) {

         # Get output file name and see if it exists.  If it does exist and
         # you want to redo this file, then delete the output file.  Check
         # what else can be done to this file.  The variables do_??? specify
         # what can be done _in principle_.  Remember, we can only do tippexing
         # on spectra which have either been wavelength calibrated or which
         # will be wavelength calibrated on this run. (Same with fluxing).

         print("Beginning work on ",ofile)
         do_ext = yes; do_dcor = yes; do_tipp = yes; do_flx = yes
         outfile = ofile//extn
         if (access(outfile) && reextract) {
            imdelete(outfile,verify=no)
            delete(database//"/ap"//ofile,verify=no)
         } else if (access(outfile) && !reextract) {
            do_ext = no
            dc_flag = -1
            hselect(outfile,"DCLOG1",yes) | scan(junk)
            if (nscan() == 1) {
               dc_flag = 0
               do_dcor = no
            }
            hselect(outfile,"@tp-flag",yes) | scan(junk)
            if (nscan() == 0) {
               if (dc_flag < 0 && !(do_dcor && dcor))
                  do_tipp = no
            } else {
               do_tipp = no
            }
            hselect(outfile,"@ca-flag",yes) | scan(junk)
            if (nscan() == 0) {
               if (dc_flag < 0 && !(do_dcor && dcor))
                  do_flx = no
            } else {
               do_flx = no
            }
         }
         if (!do_ext && !(do_dcor && dcor) && !(do_tipp && tipp) && !(do_flx && flxcal)) {
            printf("Output file %s already exists\n",outfile) | tee(log1,>log2)
            next
         }

         # If do an extraction then start by determining the number of spectra
         # in the frame and creating an apidtable.  

         if (do_ext) {
            dofind = "YES"
            if (rpars.nspec == 0) {
               nsp = 0
               profile(ofile,verbose=no) | scan(nsp)
               if (nsp == 0) {
                  print("profile algorithm failed -- will extract 1 spectrum") |
                  tee(log1, >log2)
                  nsp = 1
               } else 
                  dofind = "NO"
            } else
               nsp = rpars.nspec
            mkapid(ofile,nsp,temp1)
            apscript.apidtable = temp1

            # Extract the spectra. If you want to zap cosmic rays before
            # locating the spectra, then do so.

            if (!zapcrs) {
               apscript(ofile,output="",references="",ansfind=dofind,
                  anstrace="YES",ansextract="NO",nfind=nsp)
               if (rpars.b_sample == "")
                  backsamps(ofile,rpars.maxsky,rpars.bufsky)
               apscript(ofile,output=outfile,references="",ansfind="NO",
                  anstrace="NO",ansextract="YES",nfind=nsp)
            } else {
               crzap(ofile,temp2,nsigma=3.0,nobjsigma=0.0,nrings=0,
                  donegative=no,nnegsigma=5.0,imgsec="[*,*]",verbose=no)
               apscript(temp2,output="",references="",ansfind=dofind,
                  anstrace="YES",ansextract="NO",nfind=nsp)
               if (rpars.b_sample == "")
                  backsamps(temp2,rpars.maxsky,rpars.bufsky)
               apscript(ofile,output=outfile,references=temp2,ansfind="NO",
                  anstrace="NO",ansextract="YES",nfind=nsp)
               delete(database//"/ap"//temp2,verify=no)
               imdelete(temp2,verify=no)
            }

            # Delete aperture id table

            delete(temp1,verify=no)

            # Tell the user where things were found

            for (i = 1; i <= nsp; i += 1) {
               hselect(outfile,"APNUM"//i,yes) | scan(data)
               print(data) | scan(junki,junki,aplow,aphigh)
               printf("   Spectrum # %d found at spatial pixel %0.6g\n",
                  i,0.5*(aplow+aphigh))
            }
         }

         # Do this next block only if you are doing a dispersion correction.
         # Start by setting Julian date for output program frame

         if (do_dcor && dcor) {
            setjd(outfile,observatory=observatory,date="date_obs",
               time="ut",exposure="exposure",ra="ra",dec="dec",epoch="epoch",
               jd="jd",hjd="hjd",ljd="ljd",utdate=yes,uttime=yes,
               listonly=no) | tee (log1, > log2)
            setairmass(outfile,observatory=observatory,intype="beginning",
               outtype="effective",date="date_obs",exposure="exposure",
               airmass="airmass",utmiddle="utmiddle",show=no,update=yes,
               override=yes, >& "dev$null")

            # Decide which arc files are appropriate for the object file

            refspectra(outfile,references="@"//arcfile,apertures="",
               refaps="",ignoreaps=yes,select=rpars.select,sort=rpars.sort,
               time=rpars.time,timewrap=rpars.timewrap,override=yes,
               confirm=no,assign=yes,logfiles="STDOUT",verbose=yes) | tee(log1, > log2)
            arc1 = "" ; arc2 = ""
            hselect(outfile,"refspec1",yes) | scan(data)
            nsc = nscan()
            print(data) | scan(arc1)
            if (nsc == 0 || arc1 == "") {
               printf("No reference arcs for file: ",outfile) | tee(log1, >log2)
               next
            }
            hselect(outfile,"refspec2",yes) | scan(data)
            if (nscan() != 0)
               print(data) | scan(arc2)

            # Get an estimate of the central wavelength, the dispersion
            # and the dispersion direction for the first arc

            evalexpr("CENWAVE",arc1,ordno=1)
            evalexpr("DISP_APP",arc1,ordno=1)
            evalexpr("DFLIP",arc1,ordno=1)
            hselect(arc1,"cenwave,disp_app,dflip",yes) | scan(cw_est,disp_est,isign)

            # Extract the first reference arc. Give it an output name which
            # appends the object file.

            aoname1 = arc1//ofile//".ms"
            if (access(aoname1//imtype) && reextract) 
               imdelete(aoname1,verify=no)
            junk = database//"/id"//aoname1
            if (access(junk)) 
               delete(junk,verify=no)
            apscript(arc1,output=aoname1,references=ofile,ansfind="NO",
               anstrace="NO",ansextract="YES",background="none",clean=no,
               weights="none",ansresize="NO",ansrecenter="NO")

            # If it exist, extract the second arc...

            aoname2 = ""
            if (arc2 != "") {
               aoname2 = arc2//ofile//".ms"
               if (access(aoname2//imtype) && reextract)
                  imdelete(aoname2,verify=no)
               junk = database//"/id"//aoname2
               if (access(junk)) 
                  delete(junk,verify=no)
               apscript(arc2,output=aoname2,references=ofile,ansfind="NO",
                  anstrace="NO",ansextract="YES",background="none",clean=no,
                  weights="none",ansresize="NO",ansrecenter="NO")
            }

            # Reset the reference names in the object file header to be the
            # _extracted_ arcs...

            arefs = aoname1
            if (arc2 != "")
               arefs = arefs//","//aoname2
            hedit(outfile,"refspec?",value="",add=no,delete=yes,verify=no,
               show=no,update=yes)
            hedit(arefs,"REFSPEC1","($I)",add=yes,delete=no,verify=no,show=no,
               update=yes) | tee(log1, >log2)
            refspectra(outfile,references=arefs,apertures="",
               refaps="",ignoreaps=yes,select=rpars.select,sort=rpars.sort,
               time=rpars.time,timewrap=rpars.timewrap,override=yes,confirm=no,
               assign=yes,logfiles="STDOUT",verbose=yes) | tee(log1, > log2)

            # If you are using the first or second autoid levels, then see
            # if there are any existing arcs on which you can base the current
            # arc id.

            if (autoid <= 2) {

               # First check and see if either arc1 or arc2 have been
               # identified for another object file

               idname = database//"/id"//arc1//"*"
               fmtname = database//"/id"
               sections(idname,option="fullname", >> temp1)
               if (arc2 != "") {
                  idname = database//"/id"//arc2//"*"
                  sections(idname,option="fullname", >> temp1)
               }
               test1 = no
               fd2 = temp1
               while (fscanf(fd2,fmtname//"%s",rname) != EOF) {
                  if (access(rname//imtype)) {
                     test1 = yes
                     break
                  }
               }
               delete(temp1,verify=no)
               fd2 = ""

               # Now check the arc log and see if another arc with a
               # similar central wavelength and dispersion has been identified.

               test2 = no
               if (!test1) {
                  hselect(aoname1,"i_naxis1",yes) | scan(nx)
                  srange = nx*disp_est
                  rname = ""
                  sarclog(arc1,srange,cw_est,disp_est,0.05,0.05,"jd") | 
                     scan(rname)
                  if (rname != "") 
                     test2 = yes
               }
               reid = (test1 || test2)

            # If you've opted to identify all spectra by hand, then
            # Don't allow any re-identifications.

            } else
               reid = no

            # If you can reidentify the whole spectrum from a previous
            # reference spectrum, then do that now.  Otherwise, identify
            # the first arc using which ever method you have chosen.

            if (reid)
               i_reid(rname,aoname1,nsp,coordlist=coorlist) | tee(log1, >log2)
            else {
               i_initid(aoname1,1,autoid,coordlist=coorlist,cenwave=cw_est,
                  disper=disp_est,dflip=isign)

               # If there are more than one then check to see which instrument
               # was used.  If it was a multi-order instrument, then the second
               # spectrum will have a completely different spectral range, and
               # will have to be identified using the method you used for the
               # first spectrum (e.g. with the initial id algorithm).  If it
               # isn't a FOS spectrum, then you can just use the reidentify
               # algorithm

               if (nsp > 1) {
                  if (maxords > 1) {
                     evalexpr("CENWAVE",arc1,ordno=2)
                     evalexpr("DISP_APP",arc1,ordno=2)
                     evalexpr("DFLIP",arc1,ordno=2)
                     hselect(arc1,"cenwave,disp_app,dflip",yes) | scan(cw_est2,disp_est2,isign2)
                     i_initid(aoname1,2,autoid,coordlist=coorlist,
                        cenwave=cw_est2,disper=disp_est2,dflip=isign2)
                  } else
                     i_reid(aoname1,aoname1,1,coordlist=coorlist) | tee(log1, >log2)
               }
            }

            # Send some info on the quality of the arc id to the log

            printf("Stats on arc identification for: %s\n",aoname1, >> log1)
            slist(aoname1,apertures="",long_header=no, > temp1)
            arcstat(aoname1,temp1,outfile,1, >> log1)
            delete(temp1,verify=no)
            mkarclog(aoname1,cw_est,disp_est)

            # Scrunch the arcs

            if (dcar) {
               dispcor.log = rpars.log
               dispcor(aoname1,"",linearize=rpars.linearize,database=database,
                  table="",w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,
                  flux=rpars.flux,samedisp=no,global=no,ignoreaps=yes,
                  confirm=no,listonly=no,verbose=verbose,
                  logfile="STDOUT") | tee(log1,>log2)
            }

            # Ok...Now if there there is a second arc then reidentify
            # it with the first arc as a reference (if you have chosen the
            # correct level of automation).  If you want to do it manually,
            # then do so...

            if (arc2 != "") {
               if (autoid <= 2)
                  i_reid(aoname1,aoname2,nsp,coordlist=coorlist) | tee(log1, >log2)
               else {
                  i_initid(aoname2,1,autoid,coordlist=coorlist,cenwave=cw_est,
                     disper=disp_est,dflip=isign)
                  if (nsp > 1) {
                     if (maxords > 1) {
                        evalexpr("CENWAVE",arc2,ordno=2)
                        evalexpr("DISP_APP",arc2,ordno=2)
                        evalexpr("DFLIP",arc2,ordno=2)
                        hselect(arc1,"cenwave,disp_app,dflip",yes) | scan(cw_est2,disp_est2,isign2)
                        i_initid(aoname2,2,autoid,coordlist=coorlist,
                           cenwave=cw_est2,disper=disp_est2,dflip=isign2)
                     } else
                        i_reid(aoname2,aoname2,1,coordlist=coorlist) | tee(log1, >log2)
                  }
               }

               # Send some info on the quality of the arc id to the log

               printf("Stats on arc identification for: %s\n",aoname2, >> log1)
               slist(aoname2,apertures="",long_header=no, > temp1)
               arcstat(aoname2,temp1,outfile,2, >> log1)
               delete(temp1,verify=no)
               evalexpr("CENWAVE",arc2,ordno=1)
               evalexpr("DISP_APP",arc2,ordno=1)
               evalexpr("DFLIP",arc2,ordno=1)
               hselect(arc1,"cenwave,disp_app,dflip",yes) | scan(cw_estq,disp_est,isign)
               mkarclog(aoname2,cw_est,disp_est)

               # Scrunch the arcs

               if (dcar) {
                  dispcor.log = rpars.log
                  dispcor(aoname2,"",linearize=rpars.linearize,
                     database=database,table="",w1=INDEF,w2=INDEF,dw=INDEF,
                     nw=INDEF,flux=rpars.flux,samedisp=no,global=no,
                     ignoreaps=yes,confirm=no,listonly=no,verbose=verbose,
                     logfile="STDOUT") | tee(log1,>log2)
               }
            }

            # Finally dispersion correct the spectra

            dispcor.log = rpars.log
            dispcor(outfile,"",linearize=rpars.linearize,database=database,
               table="",w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,
               flux=rpars.flux,samedisp=no,global=no,ignoreaps=yes,confirm=no,
               listonly=no,verbose=verbose,logfile="STDOUT") | tee(log1,>log2)
         }

         # If tippexing, then do that now.

         if (do_tipp && tipp) {
            tippex(tf,outfile,temp1) | tee (log1, > log2)
            imdelete(outfile,verify=no)
            imrename(temp1,outfile,verbose=no)
         }

         # If doing flux calibration, do that now.

         if (do_flx && flxcal) {
            calibrate(outfile,outfile,extinct=yes,flux=yes,
               extinction=extfile,observatory=observatory,
               ignoreaps=yes,sensitivity=sf,fnu=no) | tee(log1,>log2)
         }

         # Add this file into the running log

         runlog(ofile,ired=1,instfile=ccdred.instrument)

         # If you wanted to view the results, then look at that now...

         if (splotr) {
            print("Plotting spectra\n  Type 'q' to exit\n  Type '?' for help\n")
            splot(outfile,1,1,save_file=log1,option="auto wreset zero")
         }
      }

      # Clean out temporary files

      delete(objfile,verify=no)
      delete(biasfile,verify=no)
      delete(arcfile,verify=no)
      delete(flatfile,verify=no)
end
