include <imhdr.h>
include <error.h>

define K0 0.399
define K1 0.242
define K2 5.40e-2
define K3 4.43e-3
define K4 1.34e-4

procedure t_profile()

# Local variables

pointer sp,iname,iptr,ptr,wptr1,wptr2,wptr3,wptr4,pp,ap
int line,nsum,i1,i2,nalloc,i,ninc,ndiff,minsep,npeaks,dispaxis,sp_axis,fedge
real mean,sigma,nsig,cut,kernel[9]
bool verbose

# Functions

pointer immap(),imgs2r(),clopset(),api_open()
int clgeti(),aravr(),find_peaks(),clgpseti()
real amedr(),clgpsetr()
bool clgetb()

begin

      # Get workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)

      # Get input file and open it

      call clgstr("input",Memc[iname],SZ_FNAME)
      iferr (iptr = immap(Memc[iname],READ_ONLY,0)) {
         call eprintf("Can't find %s\n")
            call pargstr(Memc[iname])
         call erract(EA_ERROR)
      }

      # Get the dispersion axis and the spatial axis

      dispaxis = clgeti("dispaxis")
      sp_axis = 3 - dispaxis

      # Set up the smoothing kernel

      kernel[5] = K0
      kernel[6] = K1
      kernel[7] = K2
      kernel[8] = K3
      kernel[9] = K4
      do i = 1,4 
         kernel[i] = kernel[9-i+1]

      # Get the location of where to centre the summation, the number of
      # pixels to median and define the median limits.

      pp = clopset("rpars")
      line = clgpseti(pp,"line")
      if (IS_INDEFI(line))
         line = IM_LEN(iptr,dispaxis)/2
      else
         line = max(1,min(IM_LEN(iptr,dispaxis),line))
      nsum = abs(clgpseti(pp,"nsum"))
      i1 = max(1,line - nsum/2)
      i2 = min(i1 + nsum - 1,IM_LEN(iptr,dispaxis))

      # Get minimum separation between pixels, and number of background sigma
      # for threshold.  

      minsep = clgpseti(pp,"minsep")
      nsig = clgpsetr(pp,"nsig")
      verbose = clgetb("verbose")
      fedge = nint(clgpsetr(pp,"fedge")*IM_LEN(iptr,sp_axis))
      call clcpset(pp)

      # Allocate some workspace for the profile

      nalloc = IM_LEN(iptr,sp_axis)
      call calloc(wptr1,nalloc,TY_REAL)
      call calloc(wptr2,nalloc,TY_REAL)
      call calloc(wptr3,nalloc+8,TY_REAL)
      call calloc(wptr4,nalloc+8,TY_REAL)

      # Set some aperture parameters

      ap = api_open(Memc[iname],line,sp_axis,iptr)

      # Form the profile now...

      for (i = 1; i <= nalloc; i = i + 1) {
         if (dispaxis == 1) 
            ptr = imgs2r(iptr,i1,i2,i,i)
         else
            ptr = imgs2r(iptr,i,i,i1,i2)
         Memr[wptr1+i-1] = amedr(Memr[ptr],nsum)
      }

      # Find the clipped mean and sigma

      ninc = aravr(Memr[wptr1],nalloc,mean,sigma,0.0)
      ndiff = nalloc - ninc
      if (verbose) {
         call printf("No. Pixels clipped out: %d\nMean: %0.6g\nSigma: %0.6g\n")
            call pargi(ndiff)
            call pargr(mean)
            call pargr(sigma)
      }
      cut = mean + nsig*sigma

      # Smooth the profile...

      call amovr(Memr[wptr1],Memr[wptr3+4],nalloc)
      call acnvrr(Memr[wptr3],Memr[wptr4],nalloc,kernel,9)
      call amovr(Memr[wptr4],Memr[wptr1],nalloc)

      # Find the peaks

      npeaks = find_peaks(Memr[wptr1],Memr[wptr2],nalloc,0.0,minsep,fedge,
         nalloc,cut,false)

      # For each peak fill in some info and write the database file

      for (i = 1; i <= npeaks; i = i + 1) {
         call api_fill(ap,i,Memr[wptr2+i-1])
         call api_wdb(ap)
      }

      # Print out results

      if (verbose) {
         call printf("No. peaks found: %d\n")
            call pargi(npeaks)
         if (npeaks > 0) {
            call printf("They are at: \n")
            do i = 1,npeaks {
               call printf("%0.6g\n")
                  call pargr(Memr[wptr2+i-1])
            }
         }
      } else { 
         call printf("%d\n")
            call pargi(npeaks)
      }

      # Close up

      call api_close(ap)
      call mfree(wptr1,TY_REAL)
      call mfree(wptr2,TY_REAL)
      call mfree(wptr3,TY_REAL)
      call mfree(wptr4,TY_REAL)
      call imunmap(iptr)
      call sfree(sp)
end
