include "arclog.h"
include <tbset.h>
include <error.h>
include <mach.h>

procedure t_sarclog()

# Local variables

pointer sp,adbname,rname,db,extn,testfile,tp,cptr[3],arcfile,jd,iptr,id
int nrows,i,irem
real srange,cw_est,dw_est,thresh1,thresh2,frac1,frac2,cw_file,dw_file
double jdarc,jdtest,diffjd,mindjd

# Functions

pointer tbtopn(),immap()
int tbpsta(),access()
real clgetr()
double imgetd()
bool streq()

# Exit point

define exit_ 10

begin

      # Get workspace 

      call smark(sp)
      call salloc(adbname,SZ_FNAME,TY_CHAR)
      call salloc(rname,SZ_FNAME,TY_CHAR)
      call salloc(db,SZ_FNAME,TY_CHAR)
      call salloc(extn,SZ_FNAME,TY_CHAR)
      call salloc(testfile,SZ_FNAME,TY_CHAR)
      call salloc(arcfile,SZ_FNAME,TY_CHAR)
      call salloc(jd,SZ_LINE,TY_CHAR)
      call salloc(id,SZ_FNAME,TY_CHAR)

      # Get the name of the database directory and create the arclog file name
  
      call clgstr("database",Memc[db],SZ_FNAME)
      call sprintf(Memc[adbname],SZ_FNAME,"%s/%s")
         call pargstr(Memc[db])
         call pargstr(AL_NAME)

      # See if the arclog exists.  If it does, then just open it.  If it
      # doesn't, then get out of here.

      iferr(tp = tbtopn(Memc[adbname],READ_ONLY,NULL)) {
         goto exit_
      } else {
         call tbcfnd(tp,COL1,cptr[1],1)
         call tbcfnd(tp,COL2,cptr[2],1)
         call tbcfnd(tp,COL3,cptr[3],1)
      } 

      # Get the number of rows in the table

      nrows = tbpsta(tp,TBL_NROWS)

      # Get the name of the input arc file. Then get the spectral range 
      # of the input arc file, its estimated central wavelength and its 
      # estimated dispersion.  Get the two thresholds.

      call clgstr("arcfile",Memc[arcfile],SZ_FNAME)
      srange = clgetr("srange")
      cw_est = clgetr("cw_est")
      dw_est = clgetr("dw_est")
      thresh1 = clgetr("thresh1")
      thresh2 = clgetr("thresh2")
      call clgstr("jd",Memc[jd],SZ_FNAME)

      # Open the arc file and get its julian date

      iferr(iptr = immap(Memc[arcfile],READ_ONLY,0)) {
         call eprintf("Can't open file %s\n")
            call pargstr(Memc[arcfile])
         call erract(EA_ERROR)
      }
      jdarc = imgetd(iptr,Memc[jd])
      call imunmap(iptr)

      # Get the filename extension

      call envgets("imtype",Memc[extn],SZ_FNAME)

      # Now search the table for an arcfile which has the same or similar
      # central wavelength and dispersion as the input arc.  Make sure it's
      # not the same file as the input arc file.  This is to make sure that
      # you don't try and identify w.r.t. a bad previous identification.

      mindjd = MAX_DOUBLE
      irem = 0
      for (i = 1; i <= nrows; i = i + 1) {
         call tbegtt(tp,cptr[1],i,Memc[rname],SZ_FNAME)
         if (!streq(Memc[rname],Memc[arcfile])) {
            call tbegtr(tp,cptr[2],i,cw_file)
            call tbegtr(tp,cptr[3],i,dw_file)
            frac1 = abs(dw_est - dw_file)/dw_est
            frac2 = abs(cw_est - cw_file)/srange

            # If you do find such a file, then see if you can access it. If you
            # can, then see if it is closer in time to the arc than any of the
            # preceding candidates. Also check that you can access the arc id
            # file...

            if (frac1 <= thresh1 && frac2 <= thresh2) {
               call sprintf(Memc[testfile],SZ_FNAME,"%s.%s")
                  call pargstr(Memc[rname])
                  call pargstr(Memc[extn])
               call sprintf(Memc[id],SZ_FNAME,"%s/id%s")
                  call pargstr(Memc[db])
                  call pargstr(Memc[rname])
               if (access(Memc[testfile],0,0) == YES && 
                  access(Memc[id],0,0) == YES) {
                  iptr = immap(Memc[rname],READ_ONLY,0)
                  jdtest = imgetd(iptr,Memc[jd])
                  call imunmap(iptr)
                  diffjd = abs(jdtest - jdarc)
                  if (diffjd < mindjd) {
                     mindjd = diffjd
                     irem = i
                  }
               }
            }
         }
      }

      # If you got a match, then say so now.

      if (irem > 0) {
         call tbegtt(tp,cptr[1],irem,Memc[rname],SZ_FNAME)
         call tbegtr(tp,cptr[2],irem,cw_file)
         call tbegtr(tp,cptr[3],irem,dw_file)
         call printf("%s %0.6g %0.6g\n")
            call pargstr(Memc[rname])
            call pargr(cw_file)
            call pargr(dw_file)
      }

      # Tidy and exit

      call tbtclo(tp)
exit_
      call sfree(sp)
end
