include <evvexpr.h>
include <math.h>

define SPFCNS "|idsdisp|idsdflip|fos1cw|fos1disp|fos2cw|fos2disp|whtdflip|bdsect|isisdisp|aatrgowave|aatrgodflip|aatrgobd|stime|"

define F_IDSDISP     1 
define F_IDSDFLIP    2
define F_FOS1CW      3
define F_FOS1DISP    4
define F_FOS2CW      5
define F_FOS2DISP    6
define F_WHTDFLIP    7
define F_BDSECT      8
define F_ISISDISP    9
define F_AATRGOWAVE  10
define F_AATRGODFLIP 11
define F_AATRGOBD    12
define F_STIME       13

procedure ufcn(data,fcn,args,nargs,op)

# Parameters

char fcn[ARB]
pointer args[ARB],op,data
int nargs

# Local variables

pointer sp,buf,dval,cam,iptr,wptr,pp
int opcode,v_nargs,oplen,optype,i,ip,iresult,ordno,mode,iord,junk,camera
int gratrule
double dresult,pixsize,sinphi,cosphi,cosz,arg,lst,cosdec
char msg[SZ_LINE]

# Functions

int strdic(),ctod(),strmatch(),clgpseti(),ctoi()
bool strne(),streq()
double clgpsetd()
pointer clopset()

# Common block for input file header pointer and order number

common /iptr/ iptr,ordno

# Extra Functions (in the function evaluation section)

double idsdisp(),isisdisp(),aatrgowave()
int aatrgodflip()

begin

      call smark(sp)
      call salloc(buf,SZ_FNAME,TY_CHAR)
      call salloc(dval,nargs,TY_DOUBLE)

      # Look up the function name.

      opcode = strdic(fcn,Memc[buf],SZ_FNAME,SPFCNS)
      if (opcode > 0 && strne(fcn,Memc[buf]))
         opcode = 0

      # Punt if this is an unknown function

      if (opcode <= 0)
         call xvv_error1("unknown function `%s' called",fcn)

      # Verify correct number of arguments

      switch (opcode) {
         case F_IDSDISP:
            v_nargs = 6
            oplen = 0
            optype = TY_REAL
         case F_IDSDFLIP:
            v_nargs = 1
            oplen = 0
            optype = TY_INT
         case F_FOS1CW:
            v_nargs = 0
            oplen = 0
            optype = TY_REAL
         case F_FOS1DISP:
            v_nargs = 0
            oplen = 0
            optype = TY_REAL
         case F_FOS2CW:
            v_nargs = 0
            oplen = 0
            optype = TY_REAL
         case F_FOS2DISP:
            v_nargs = 0
            oplen = 0
            optype = TY_REAL
         case F_WHTDFLIP:
            v_nargs = 2
            oplen = 0
            optype = TY_INT
         case F_BDSECT:
            v_nargs = 15
            oplen = SZ_LINE
            optype = TY_CHAR
         case F_ISISDISP:
            v_nargs = 5
            oplen = 0
            optype = TY_REAL
         case F_AATRGOWAVE:
            v_nargs = 2
            oplen = 0
            optype = TY_REAL
         case F_AATRGODFLIP:
            v_nargs = 0
            oplen = 0
            optype = TY_INT
         case F_AATRGOBD:
            v_nargs = 9
            oplen = SZ_LINE
            optype = TY_CHAR
         case F_STIME:
            v_nargs = 3
            oplen = SZ_LINE
            optype = TY_CHAR
      }
      if (nargs != v_nargs)
         call xvv_error2("function `%s' requires `%d' arguments",fcn,v_nargs)

      # Convert datatypes to double (if you can)

      do i = 1, nargs {
         switch (O_TYPE(args[i])) {
            case TY_CHAR:
               ip = 1
               if (ctod (O_VALC(args[i]), ip, Memd[dval+i-1]) == 0)
                  Memd[dval+i-1] = 0.
            case TY_INT:
               Memd[dval+i-1] = O_VALI(args[i])
            case TY_REAL:
               Memd[dval+i-1] = O_VALR(args[i])
            case TY_DOUBLE:
               Memd[dval+i-1] = O_VALD(args[i])
         }
      }

      # Now do the work for each defined function

      switch (opcode) {
         case F_IDSDISP:
            dresult = idsdisp(O_VALC(args[1]),Memd[dval+1],Memd[dval+2],Memd[dval+3],Memd[dval+4],Memd[dval+5])
         case F_IDSDFLIP:
            call malloc(cam,3,TY_CHAR)
            call strcpy(O_VALC(args[1]),Memc[cam],3)
            if (streq(Memc[cam],"235")) {
               iresult = 1
            } else if (streq(Memc[cam],"500")) {
               iresult = -1
            } else {
               call error(1,"Illegal camera in IDS header")
            }
            call mfree(cam,TY_CHAR)
         case F_FOS1CW:
            switch (ordno) {
               case 1:
                  dresult = 8330.0d0
               case 2:
                  dresult = 4170.0d0
               default:
                  call error(1,"Illegal order number")
            }
         case F_FOS1DISP:
            switch (ordno) {
               case 1:
                  dresult = 10.72d0
               case 2:
                  dresult = 5.35d0
               default:
                  call error(1,"Illegal order number")
            }
         case F_FOS2CW:
            switch (ordno) {
               case 1:
                  dresult = 7290.0d0
               case 2:
                  dresult = 4670.0d0
               default:
                  call error(1,"Illegal order number")
            }
         case F_FOS2DISP:
            switch (ordno) {
               case 1:
                  dresult = 8.75d0
               case 2:
                  dresult = 4.33d0
               default:
                  call error(1,"Illegal order number")
            }
         case F_WHTDFLIP:
            call strupr(O_VALC(args[1]))
            call strupr(O_VALC(args[2]))
            if (streq(O_VALC(args[1]),"BLUE ARM") > 0) {
               if (streq(O_VALC(args[2]),"TEK2") > 0 || 
                  streq(O_VALC(args[2]),"EEV6") > 0 || 
                  streq(O_VALC(args[2]),"TEK5") > 0)
                  iresult = -1
               else if (streq(O_VALC(args[2]),"TEK1") ||
                  streq(O_VALC(args[2]),"EEV3")) 
                  iresult = 1
               else
                  iresult = 0
            } else if (streq(O_VALC(args[1]),"RED ARM")) {
               if (streq(O_VALC(args[2]),"TEK2") || 
                  streq(O_VALC(args[2]),"EEV6") || 
                  streq(O_VALC(args[2]),"TEK5"))
                  iresult = 1
               else if (streq(O_VALC(args[2]),"TEK1") ||
                  streq(O_VALC(args[2]),"EEV3")) 
                  iresult = -1
               else
                  iresult = 0
            } else {
               iresult = 0
            }
         case F_BDSECT:
            call malloc(iresult,oplen,TY_CHAR)
            mode = nint(Memd[dval+13])
            if (strmatch(O_VALC(args[15]),"TEK") == 4) {
               call bdsecttek(Memd[dval],nargs,mode,Memc[iresult])
            } else {
               call sprintf(msg,SZ_LINE,"Bias and data section for %s not defined\n")
                  call pargstr(O_VALC(args[15]))
               call error(1,msg)
            }
         case F_ISISDISP:
            dresult = isisdisp(Memd[dval],Memd[dval+1],Memd[dval+2],Memd[dval+3],Memd[dval+4])
         case F_AATRGOWAVE:
            call malloc(wptr,SZ_LINE,TY_CHAR)
            call clgstr("instrument",Memc[wptr],SZ_LINE)
            pp = clopset(Memc[wptr])
            iord = clgpseti(pp,"gratorder")
            call clgpseta(pp,"gratid",Memc[wptr],SZ_LINE)
            ip = 1
            junk = ctoi(Memc[wptr],ip,gratrule)
            call clgpseta(pp,"camera",Memc[wptr],SZ_LINE)
            ip = 1
            junk = ctoi(Memc[wptr],ip,camera)
            pixsize = clgpsetd(pp,"pixsize")
            dresult = aatrgowave(nint(Memd[dval+1]),Memd[dval],iord,gratrule,
               camera,pixsize)
            call clcpset(pp)
            call mfree(wptr,TY_CHAR)
         case F_AATRGODFLIP:
            call malloc(wptr,SZ_LINE,TY_CHAR) 
            call clgstr("instrument",Memc[wptr],SZ_LINE)
            pp = clopset(Memc[wptr])
            call clgpseta(pp,"camera",Memc[wptr],SZ_LINE)
            ip = 1
            junk = ctoi(Memc[wptr],ip,camera)
            call clgpseta(pp,"grblzdir",Memc[wptr],SZ_LINE)
            iresult = aatrgodflip(camera,Memc[wptr])
            call clcpset(pp)
            call mfree(wptr,TY_CHAR)
         case F_AATRGOBD:
            mode = nint(Memd[dval+8])
            call malloc(iresult,oplen,TY_CHAR)
            call bdsectaat(Memd[dval],8,mode,Memc[iresult],oplen)
         case F_STIME:
            sinphi = 0.48111577d0
            cosphi = 0.87665707d0
            if (Memd[dval+2] < 1.0d0)
               cosz = 1.0d0
            else
               cosz = 1.0d0/Memc[dval+2]
            cosdec = cos(DEGTORAD(min(89.9d0,max(-89.9d0,Memd[dval+1]))))
            arg = (cosz - sin(DEGTORAD(Memd[dval+1]))*sinphi)/(cosdec*cosphi)
            arg = min(1.0d0,max(-1.0d0,arg))
            lst = RADTODEG(acos(arg))/15.0d0 + Memd[dval]
            lst = mod(lst,24.0d0)
            if (lst < 0.0d0) 
               lst = lst + 24.0d0
            call malloc(iresult,oplen,TY_CHAR)
            call sprintf(Memc[iresult],oplen,"%010.1h")
               call pargd(lst)
      }

      # Transfer the results over and free up some workspace

      call xvv_initop(op,oplen,optype)
      switch (optype) {
         case TY_REAL,TY_DOUBLE,TY_COMPLEX:
            O_VALR(op) = real(dresult)
         case TY_INT,TY_BOOL,TY_SHORT,TY_LONG:
            O_VALI(op) = iresult
         case TY_CHAR:
            call strcpy(Memc[iresult],O_VALC(op),oplen)
            call mfree(iresult,TY_CHAR)
      }
      do i = 1,nargs
         call xvv_freeop(args[i])
      call sfree(sp)
end

############Extra procedures################

double procedure idsdisp(camera,grating,xpix,ypix,xb,yb)

# Parameters

char camera[ARB]
double grating,xpix,ypix,xb,yb,dw

# Local variables

char cam[3]
int grat
double binsize,pixsize
real app

# Functions

int clgeti()
bool streq()

begin

      # Initialise a few things...

      grat = nint(grating)
      if (clgeti("dispaxis") == 1) {
         pixsize = xpix
         binsize = xb
      } else {
         pixsize = ypix
         binsize = yb
      }
      call strcpy(camera,cam,3)      

      # Find the correct camera/grating combination 

      if (streq(cam,"235")) {
         switch (grat) {
            case 150:
               app = 271.4
            case 300:
               app = 138.6
            case 400:
               app = 104.5
            case 600:
               app = 70.00
            case 632:
               app = 66.66
            case 831:
               app = 50.91
            case 900:
               app = 46.36
            case 1200:
               app = 35.45
            case 1800:
               app = 23.18
            case 2400:
               app = 17.27
            default:
               call error(0,"Illegal IDS grating in header")
         }
      } else if (streq(cam,"500")) {
         switch (grat) {
            case 150:
               app = 132.3
            case 300:
               app = 72.50
            case 400:
               app = 49.54
            case 600:
               app = 33.18
            case 632:
               app = 31.81
            case 831:
               app = 24.09
            case 900:
               app = 22.27
            case 1200:
               app = 16.81
            case 1800:
               app = 10.45
            case 2400:
               app = 7.27
            default:
               call error(0,"Illegal IDS grating in header")
         }
      } else {
         call error(0,"Illegal IDS camera in header")                        
      }

      # Scale the dispersion by the appropriate pixel size

      dw = dble(app)*1000.0d0*pixsize
      return (dw)
end

double procedure isisdisp(dispmm,xpixel,ypixel,xb,yb) 

# Parameters

double dispmm,xpixel,ypixel,xb,yb,disp_app

# Local variables

double pixsize,binsize

# Functions

int clgeti()

begin

      # Decide which pixel size to use 

      if (clgeti("dispaxis") == 1) {
         pixsize = xpixel
         binsize = xb
      } else {
         pixsize = ypixel
         binsize = yb
      }

      # Now calculate the bit you want.

      disp_app = dispmm*pixsize*binsize*0.001d0
      return(disp_app)
end

double procedure aatrgowave(mode,gratangle,iord,gratrule,camera,pixsize)

# Parameters

int mode,iord,gratrule,camera
double gratangle,pixsize,result

# Local variables

double deltac,ang1,ang2

begin

      # Camera constant and useful angles...

      switch (camera) {
         case 25:
            deltac = -44.95d0
         case 82:
            deltac = 15.19d0
      }
      ang1 = DEGTORAD(gratangle - 49.75d0)
      ang2 = ang1 - DEGTORAD(deltac)

      # Section for finding the central wavelength

      if (mode == 1) {
         result = abs((sin(ang1) + sin(ang2))/(double(iord*gratrule)))*1.0d7
  
      # Section for finding dispersion...

      } else {
         result = 1.0d3*abs(cos(ang2)*pixsize)/double(iord*gratrule*camera)
      }

      # Return the value

      return (result)
end

int procedure aatrgodflip(camera,grblzdir)

# Parameters

int camera,iflip
char grblzdir[ARB]

# Function 

bool streq()

begin

      # Calculate dispersion direction

      iflip = 1
      if (streq(grblzdir,"coll")) 
         iflip = -1
      if (camera == 82) 
         iflip = -iflip

      return(iflip)
end

