include <gio.h>

procedure t_ids()
begin
       call reduce("ids")
end

procedure t_isis()
begin
       call reduce("isis")
end

procedure t_rbs()
begin
       call reduce("rbs")
end

procedure reduce(instr)

# Parameters

char instr[ARB]

# Local variables

pointer sp,gp,strval,cmd,rp1,rp2,guifile,tmp,pp1,pp2,logfile,fullname
pointer telname,instname
int wcs,key
real x,y

# Functions

int clgcur(),strlen()
pointer gopenui(),gopen(),clopset(),ps_open()
bool streq()
extern gflush()

begin

      # Get some workspace

      call smark(sp)
      call salloc(telname,SZ_LINE,TY_CHAR)
      call salloc(instname,SZ_LINE,TY_CHAR)
      call salloc(cmd,SZ_LINE,TY_CHAR)
      call salloc(strval,SZ_LINE,TY_CHAR)
      call salloc(guifile,SZ_FNAME,TY_CHAR)
      call salloc(tmp,SZ_FNAME,TY_CHAR)
      call salloc(logfile,SZ_FNAME,TY_CHAR)
      call salloc(fullname,SZ_PATHNAME,TY_CHAR)
    
      # Get the telescope name and instrument name.  If they are blank, then
      # get out of here and issue some advice.

      call clgstr("telescope",Memc[telname],SZ_LINE)
      call clgstr("instrument",Memc[instname],SZ_LINE)
      if (strlen(Memc[telname]) == 0 || strlen(Memc[instname]) == 0) {
         call error(0,
            "Telescope and/or instrument undefined -- run setup first")
      }

      # If the instrument name and the instrument which is specified in 
      # the call to this subroutine don't match, the get out of here.

      if (!streq(Memc[instname],instr)) {
         call eprintf("Setup instrument: %s,  requested instrument: %s\n")
            call pargstr(Memc[instname])
            call pargstr(instr)
         call error(0,"Run setup first and try again")
      }

      # Block for when you are using a gui...

      call clgstr("guifile",Memc[guifile],SZ_FNAME)
      if (!streq(Memc[guifile],"")) {

         # Get descriptor for pset

         pp1 = clopset(instr)
         pp2 = clopset("rpars")

         # Do work on gui

         gp = gopen("stdgraph",NEW_FILE,STDGRAPH)
         call gclose(gp)
         gp = gopenui("stdgraph",NEW_FILE,Memc[guifile],STDGRAPH)
         call gclear(gp)

         # Get the logfile name.  Append the full directory name and send it
         # down to the gui.
 
         call clgstr("logfile",Memc[logfile],SZ_FNAME)
         call fmapfn(Memc[logfile],Memc[tmp],SZ_FNAME)
         call strupk(Memc[tmp],Memc[fullname],SZ_PATHNAME)
         if (streq(Memc[fullname],Memc[logfile])) {
            call sprintf(Memc[fullname],SZ_PATHNAME,"./%s")
               call pargstr(Memc[logfile])
            call fmapfn(Memc[fullname],Memc[tmp],SZ_FNAME)
            call strupk(Memc[tmp],Memc[fullname],SZ_PATHNAME)
         }
         call gmsg(gp,"logfile",Memc[fullname])

         # Open two parameter structures, one for the standard parameters of
         # the task and the other for the rpars pset.

         call sprintf(Memc[tmp],SZ_LINE,"spec2$%s.par")
            call pargstr(instr)
         rp1 = ps_open(Memc[tmp])
         rp2 = ps_open("spec2$rpars.par")
         call ps_fill(rp1)
         call ps_fill(rp2)

         # Setup standard and rpars pset values

         call setpars(gp,1,rp1)
         call setpars(gp,2,rp2)

         # Set the gui directory so that your gui doesn't get blown away by
         # a call to splot.

         call clcmdw("set guidir = spec2$lib/")

         # Cursor loop

         while (clgcur("cursor",x,y,wcs,key,Memc[strval],SZ_LINE) != EOF) {
            switch (key) {
               case 'e':
                  break
               case 'r':
                  call fillpars(rp1)
                  call gmsgi(gp,"working",1)
                  call gdeactivate(gp,0)
                  call clcmdw("rspecred")
                  call printf("Processing Done\n")
                  call greactivate(gp,0)
                  call gmsgi(gp,"working",0)
               case 'h':
                  call gmsgi(gp,"working",1)
                  call gdeactivate(gp,0)
                  call sprintf(Memc[tmp],SZ_LINE,"spec2$doc/%shelp.key")
                     call pargstr(instr)
                  call gpagefile(gp,Memc[tmp],"SPEC2 help")
                  call greactivate(gp,0)
                  call gmsgi(gp,"working",0)
               case ':':
                  call getparam(gp,pp1,pp2,rp1,rp2,Memc[strval])
            }
         }

         # Release pset structures, SPEC2 parameter system structures and close
         # the graphics...

         call clcpset(pp1)
         call clcpset(pp2)
         call ps_close(rp1)
         call ps_close(rp2)
         call gclose(gp)

      # Block for not using a gui...

      } else {

         # Get descriptor for pset descriptor for main parameters only.
         # Open a SPEC2 parameter structure for them and fill them in.

         pp1 = clopset(instr)
         call sprintf(Memc[tmp],SZ_LINE,"spec2$%s.par")
            call pargstr(instr)
         rp1 = ps_open(Memc[tmp])
         call ps_fill(rp1)

         # Now fill the parameters for 'rspecred' with these values and
         # execute the script.

         call fillpars(rp1)
         call clcmdw("rspecred")
 
         # Release workspace for structures...

         call clcpset(pp1)
         call ps_close(rp1)
      }

      # Ditch the last bit of workspace

      call sfree(sp)
end
