procedure rspecred()

string     imlist = ""    {prompt="List of objects to be reduced"}
string     arclist = ""   {prompt="List of arc files"}
string     biases = ""    {prompt="List of bias frames"}
string     flats = ""     {prompt="List of tungsten flat field frames"}
string     illums = ""    {prompt="List of sky flat field frames"}
file       mbias = ""     {prompt="Name for mean bias frame"}
file       mflat = ""     {prompt="Name for mean flat frame"}
file       millum = ""    {prompt="Name for mean illumination frame"}
bool       dispcor = yes  {prompt="Dispersion correct spectra?"}
bool       tippex = yes   {prompt="Tippex out sky line residuals?"}
file       tfile = ""     {prompt="Name of tippex file"}
int        autoid = 1     {prompt="What level of automation in arcline id?"}
bool       redo = no      {prompt="Redo operations if previously done?"}
bool       dcorarcs = yes {prompt="Dispersion correct arcs?"}
bool       fluxcal = yes  {prompt="Do a flux calibration?"}
bool       backgrnd = no  {prompt="Do a background subtraction?"}
bool       displ = yes    {prompt="Display resulting frames?"}
file       sensfunc = ""  {prompt="Sensitivity file for flux calibration"}
file       coorlist       {prompt="Name of arcline file"}

pset       *rpars = ""
struct     *fd

begin

      string objs,ars,bis,fls,log1,log2,extn,ra,dec,imtype,idname,fmtname,junk
      string fitname,ils
      struct title,data
      file objfile,arcfile,biasfile,flatfile,temp1,ofile,outfile
      file arc1,arc2,aoname1,aoname2,arefs,rname,tf,sf,illfile
      int isign,i,nsc,nx,dc_flag,jst,jfn,isign2,junki,isect,xorder,yorder
      real srange,cw_est,disp_est,cw_est2,dw_est2,aplow,aphigh
      bool reextract,splotr,dcor,tipp,reid,test1,test2,do_ext,do_dcor,do_tipp
      bool zapcrs,dcar,flxcal,do_flx,bgrnd,do_bgrnd,xlog,ylog,disp,err

      # Get the lists of files to be processed

      objs = imlist
      ars = arclist
      bis = biases
      fls = flats
      ils = illums

      # Make sure that there are some objects listed...

      if (objs == "")
         error(0,"No object files have been specified")

      # See if you want to redo the work on the object files if previously 
      # done and see if you want to scrunch arcs.

      reextract = redo
      dcar = dcorarcs

      # Set up logging...

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose)
         log2 = "STDOUT"
      else
         log2 = "dev$null"

      # Set up some temporary files which will contain lists of files
      # to be processed

      objfile = mktemp("tmp$iraf")
      arcfile = mktemp("tmp$iraf")
      biasfile = mktemp("tmp$iraf")
      flatfile = mktemp("tmp$iraf")
      illfile = mktemp("tmp$iraf")
      temp1 = mktemp("tmp$iraf")

      # Now fill the file lists

      filllist(objs,objfile)
      filllist(ars,arcfile)
      filllist(bis,biasfile)
      filllist(fls,flatfile)
      filllist(ils,illfile)

      # Define multispec extension

      imtype = "."//envget("imtype")
      extn = ".tds"//imtype

      # Sort out a few switches.  If there are no arc files, then you can't
      # possibly dispersion correct.  Similarly, if there is no tippex file,
      # then you can't do tippexing...

      dcor = dispcor
      if (ars == "" && dcor) {
         dcor = no
         print("No arc files specified => no dispersion correction possible") |
            tee(log1, > log2)
      }
      tipp = tippex
      tf = tfile
      if (tipp && !access(tf)) {
         tipp = no
         print("Cannot find a tippex file => no tippexing will be done") |
            tee(log1, > log2)
      }
      flxcal = fluxcal
      sf = ""
      strpextn(sensfunc) | scan(sf)
      if (flxcal && !access(sf//imtype)) {
         flxcal = no
         print("Cannot find sensitivity file => no flux calib. will be done") |
            tee(log1, > log2)
      }
      bgrnd = backgrnd
      if (bgrnd && rpars.b_sample == "") {
         bgrnd = no
         print("No sample area specified -- no background subtraction done") |
            tee(log1, > log2)
      }

      # If you want to display then check that there is a display server 
      # on the screen.  This is a bit of a kludge, but I couldn't think of
      # another way to do it...

      disp = displ
      if (disp) {
         print(":allframes\n q") | imexamine(>&temp1,imagecur="STDIN")
         if (access(temp1)) {
            fd = temp1
            err = no
            while (fscan(fd,data) != EOF) {
               if (substr(data,1,21) == "cannot access display") {
                  err = yes
                  break
               }
            }
            if (err) 
               print("!ximtool -g -0-0&") | cl(>&"dev$null")
            delete(temp1,verify=no)
            fd = ""
         }
      }

      # Sort out the data and bias sections for each of the non-bias frames.

      evalexpr("BIASSEC","@"//objfile)
      evalexpr("TRIMSEC","@"//objfile)
      if (dcor) {
         evalexpr("BIASSEC","@"//arcfile)
         evalexpr("TRIMSEC","@"//arcfile)
      }
      if (fls != "") {
         evalexpr("BIASSEC","@"//flatfile)
         evalexpr("TRIMSEC","@"//flatfile)
      }
      if (ils != "") {
         evalexpr("BIASSEC","@"//illfile)
         evalexpr("TRIMSEC","@"//illfile)
      }
      
      # Prepare the bias and flat frames and set up the ccd instrumental
      # parameters for ccdred.

      preccd("@"//biasfile,"@"//flatfile,"@"//illfile,mbias,mflat,millum,
         rpars.interactive)

      # Now do the ccd processing on the object and arc frames

      ccdproc("@"//objfile)
      if (dcor)
         ccdproc("@"//arcfile)

      # Set the Julian dates for all of the arc files (if you have any)

      if (dcor) {
         junk = "@"//arcfile//",@"//objfile
         evalexpr("RA",junk)
         evalexpr("DEC",junk)
         evalexpr("EPOCH",junk)
         evalexpr("UT",junk)
         evalexpr("ST",junk)
         evalexpr("DATE_OBS",junk)
         evalexpr("EXPOSURE",junk)
         setjd("@"//arcfile,observatory=observatory,date="date_obs",
            time="ut",exposure="exposure",ra="ra",dec="dec",epoch="epoch",
            jd="jd",hjd="hjd",ljd="ljd",utdate=yes,uttime=yes,
            listonly=no) | tee (log1, > log2)
      }

      # Edit the header item "REFSPEC1" into the header of the of each
      # arc file so that you can fool 'refspec' into thinking that they have
      # already been used in 'identify' and are hence proper reference images

      hedit("@"//arcfile,"REFSPEC1","($I)",add=yes,delete=no,verify=no,show=no,
         update=yes) | tee(log1, >log2)

      # Now loop for the individual object files

      fd = objfile
      while (fscan(fd,ofile) != EOF) {

         # Get output file name and see if it exists.  If it does exist and
         # you want to redo this file, then delete the output file.  Check
         # what else can be done to this file.  The variables do_??? specify
         # what can be done _in principle_.  Remember, we can only do tippexing
         # on spectra which have either been wavelength calibrated or which
         # will be wavelength calibrated on this run. (Same with fluxing).

         print("Beginning work on ",ofile)
         do_ext = yes; do_dcor = yes; do_tipp = yes; do_flx = yes; do_bgrnd = yes
         outfile = ofile//extn
         if (access(outfile) && reextract) {
            imdelete(outfile,verify=no)
            delete(database//"/ap"//ofile,verify=no)
         } else if (access(outfile) && !reextract) {
            do_ext = no
            dc_flag = -1
            hselect(outfile,"DCLOG1",yes) | scan(junk)
            if (nscan() == 1) {
               dc_flag = 0
               do_dcor = no
            }
            hselect(outfile,"tp-flag",yes) | scan(junk)
            if (nscan() == 0) {
               if (dc_flag < 0 && !(do_dcor && dcor))
                  do_tipp = no
            } else {
               do_tipp = no
            }
            hselect(outfile,"ca-flag",yes) | scan(junk)
            if (nscan() == 0) {
               if (dc_flag < 0 && !(do_dcor && dcor))
                  do_flx = no
            } else {
               do_flx = no
            }
            hselect(outfile,"bk-flag",yes) | scan(junk)
            if (nscan() == 0) {
               if (dc_flag < 0 && !(do_dcor && dcor))
                  do_bgrnd = no
            } else {
               do_bgrnd = no
            }
         }
         if (!do_ext && !(do_dcor && dcor) && !(do_tipp && tipp) && !(do_flx && flxcal)) {
            printf("Output file %s already exists\n",outfile) | tee(log1,>log2)
            next
         }

         # "Extraction" in this case is simply copying over the CCD processed
         # frame to the output file.

         if (do_ext) 
            imcopy(ofile,outfile,verbose=no)

         # Do this next block only if you are doing a dispersion correction.
         # Start by setting Julian date for output program frame

         if (do_dcor && dcor) {
            setjd(outfile,observatory=observatory,date="date_obs",
               time="ut",exposure="exposure",ra="ra",dec="dec",epoch="epoch",
               jd="jd",hjd="hjd",ljd="ljd",utdate=yes,uttime=yes,
               listonly=no) | tee (log1, > log2)
            setairmass(outfile,observatory=observatory,intype="beginning",
               outtype="effective",date="date_obs",exposure="exposure",
               airmass="airmass",utmiddle="utmiddle",show=no,update=yes,
               override=yes, >& "dev$null")

            # Decide which arc files are appropriate for the object file

            refspectra(outfile,references="@"//arcfile,apertures="",
               refaps="",ignoreaps=yes,select=rpars.select,sort=rpars.sort,
               time=rpars.time,timewrap=rpars.timewrap,override=yes,
               confirm=no,assign=yes,logfiles="STDOUT",verbose=yes) | tee(log1, > log2)
            arc1 = "" ; arc2 = ""
            hselect(outfile,"refspec1",yes) | scan(data)
            nsc = nscan()
            print(data) | scan(arc1)
            if (nsc == 0 || arc1 == "") {
               printf("No reference arcs for file: ",outfile) | tee(log1, >log2)
               next
            }
            hselect(outfile,"refspec2",yes) | scan(data)
            if (nscan() != 0)
               print(data) | scan(arc2)

            # Get an estimate of the central wavelength, the dispersion
            # and the dispersion direction for the first arc

            evalexpr("CENWAVE",arc1,ordno=1)
            evalexpr("DISP_APP",arc1,ordno=1)
            evalexpr("DFLIP",arc1,ordno=1)
            hselect(arc1,"cenwave,disp_app,dflip",yes) | scan(cw_est,disp_est,isign)

            # Create the first reference arc. 

            aoname1 = arc1//".tds"
            if (access(aoname1//imtype) && reextract) 
               imdelete(aoname1,verify=no)
            if (!access(aoname1//imtype))
               imcopy(arc1,aoname1,verbose=no)

            # If you want one, create the second arc...

            aoname2 = ""
            if (arc2 != "") {
               aoname2 = arc2//".tds"
               if (access(aoname2//imtype) && reextract) 
                  imdelete(aoname2,verify=no)
               if (!access(aoname2//imtype))
                  imcopy(arc2,aoname2,verbose=no)
            }

            # Reset the reference names in the object file header to be the
            # _extracted_ arcs...

            arefs = aoname1
            if (arc2 != "")
               arefs = arefs//","//aoname2
            hedit(outfile,"refspec?",value="",add=no,delete=yes,verify=no,
               show=no,update=yes)
            hedit(arefs,"REFSPEC1","($I)",add=yes,delete=no,verify=no,show=no,
               update=yes) | tee(log1, >log2)
            refspectra(outfile,references=arefs,apertures="",
               refaps="",ignoreaps=yes,select=rpars.select,sort=rpars.sort,
               time=rpars.time,timewrap=rpars.timewrap,override=yes,confirm=no,
               assign=yes,logfiles="STDOUT",verbose=yes) | tee(log1, > log2)

            # Check and see if either arc1 or arc2 have been identified 

            rname = ""
            idname = database//"/id"//aoname1
            test1 = no; test2 = no
            if (access(idname)) {
               if (reextract) 
                  delete(idname,verify=no)
               else {
                  rname = aoname1
                  test1 = yes
               }
            }
            if (arc2 != "") {
               idname = database//"/id"//aoname2
               if (access(idname)) {
                  if (reextract)
                     delete(idname,verify=no)
                  else {
                     if (rname == "") 
                        rname = aoname2
                     test2 = yes
                  }
               }
            }

            # Now check the arc log and see if another arc with a
            # similar central wavelength and dispersion has been identified.

            if (rname == "") {
               hselect(aoname1,"i_naxis"//dispaxis,yes) | scan(nx)
               srange = nx*disp_est
               sarclog(arc1,srange,cw_est,disp_est,0.05,0.05,"jd") | 
                  scan(rname)
            }

            # If you are using the first or second autoid levels, then use
            # any existing arcs which have already been identified.

            if (autoid <= 2) 
               reid = (rname != "")

            # If you've opted to identify all spectra by hand, then
            # don't allow reidentifications
            
            else
               reid = no

            # If you can reidentify the whole spectrum from a previous
            # reference spectrum, then do that now.  Otherwise, identify
            # the first arc using which ever method you have chosen.

            if (!test1) {
               if (reid) {
                  junki = 3 - dispaxis
                  hselect(rname,"i_naxis"//junki,yes) | scan(isect)
                  if (nscan() == 0) 
                     isect = 1
                  else
                     isect = isect/2
                  i_reid(rname,aoname1,isect,coordlist=coorlist) | tee(log1, >log2)
               } else {
                  junki = 3 - dispaxis
                  hselect(aoname1,"i_naxis"//junki,yes) | scan(isect)
                  isect = isect/2
                  i_initid(aoname1,isect,autoid,coordlist=coorlist,cenwave=cw_est,
                     disper=disp_est,dflip=isign)
               }
               mkarclog(aoname1,cw_est,disp_est)
               arcstat(aoname1,outfile,1) | tee(log1,>log2)
            }

            # Ok...Now if there there is a second arc then reidentify
            # it with the first arc as a reference (if you have chosen the
            # correct level of automation).  If you want to do it manually,
            # then do so...

            if (arc2 != "") {
               if (!test2)  {
                  if (autoid <= 2) {
                     junki = 3 - dispaxis
                     hselect(aoname1,"i_naxis"//junki,yes) | scan(isect)
                     isect = isect/2
                     i_reid(aoname1,aoname2,isect,coordlist=coorlist) | tee(log1, >log2)
                  } else {
                     junki = 3 - dispaxis
                     hselect(aoname2,"i_naxis"//junki,yes) | scan(isect)
                     isect = isect/2
                     i_initid(aoname2,isect,autoid,coordlist=coorlist,cenwave=cw_est,
                        disper=disp_est,dflip=isign)
                  }
                  mkarclog(aoname2,cw_est,disp_est)
                  arcstat(aoname2,outfile,2) | tee(log1,>log2)
               }
            }

            # Fit the distortion coordinates
 
            arefs = aoname1
            fitname = aoname1
            if (arc2 != "") {
               arefs = arefs//","//aoname2
               fitname = fitname//aoname2
            }
            if (access(database//"/"//fitname) && reextract)
               delete(database//"/"//fitname,verify=no)
            if (!access(database//"/"//fitname)) {
               if (dispaxis == 1) {
                  xorder = rpars.i_order
                  yorder = rpars.sp_order
               } else {
                  xorder = rpars.sp_order
                  yorder = rpars.i_order
               }
               fitcoords(arefs,fitname=fitname,interactive=no,combine=yes,
                  database=database,deletions="",function=rpars.i_function,
                  xorder=xorder,yorder=yorder,logfiles="STDOUT",plotfile="",
                  graphics="stdgraph",cursor="") | tee(log1, > log2)
            }

            # Dispersion correct the arcs if that's what you want...

            if (dispaxis == 1) {
               xlog = rpars.log
               ylog = no
            } else {
               xlog = no
               ylog = rpars.log
            }
            if (dcar) {
               hselect(aoname1,"dc-flag",yes) | scan(dc_flag)
               if (nscan() == 0) {
                  hedit(aoname1,"dispaxis",dispaxis,add=yes,delete=no,
                     verify=no,show=no,update=yes)
                  transform(aoname1,aoname1,fitname,database=database,
                     interptype=interp,x1=INDEF,x2=INDEF,dx=INDEF,nx=INDEF,
                     xlog=no,y1=INDEF,y2=INDEF,dy=INDEF,ny=INDEF,ylog=no,
                     flux=rpars.flux,logfiles="STDOUT") | tee(log1,>log2)
               }
               if (arc2 != "") {
                  hselect(aoname2,"dc-flag",yes) | scan(dc_flag)
                  if (nscan() == 0) {
                     hedit(aoname2,"dispaxis",dispaxis,add=yes,delete=no,
                        verify=no,show=no,update=yes)
                     transform(aoname2,aoname2,fitname,database=database,
                        interptype=interp,x1=INDEF,x2=INDEF,dx=INDEF,nx=INDEF,
                        xlog=no,y1=INDEF,y2=INDEF,dy=INDEF,ny=INDEF,ylog=no,
                        flux=rpars.flux,logfiles="STDOUT") | tee(log1,>log2)
                  }
               }
            }

            # Finally dispersion correct the spectra

            hedit(outfile,"dispaxis",dispaxis,add=yes,delete=no,
               verify=no,show=no,update=yes)
            transform(outfile,outfile,fitname,database=database,
               interptype=interp,x1=INDEF,x2=INDEF,dx=INDEF,nx=INDEF,
               xlog=no,y1=INDEF,y2=INDEF,dy=INDEF,ny=INDEF,ylog=no,
               flux=rpars.flux,logfiles="STDOUT") | tee(log1,>log2)
         }

         # If doing background subtraction, then do that now...

         if (do_bgrnd && bgrnd) {
            junki = 3 - dispaxis
            background(outfile,outfile,axis=junki,interactive=no,
               sample=rpars.b_sample,naverage=rpars.b_naverage,
               function=rpars.b_function,order=rpars.b_order,
               low_reject=rpars.b_low_reject,high_reject=rpars.b_high_reject,
               niterate=rpars.b_niterate,grow=rpars.b_grow,
               graphics="stdgraph",cursor="") | tee(log1,>log2)
            hedit(outfile,"bk-flag",0,add=yes,show=no,verify=no,update=yes)
            hedit(outfile,"bsamp",rpars.b_sample,add=yes,show=no,verify=no,
               update=yes)
         }

         # If tippexing, then do that now.

         if (do_tipp && tipp) {
            tippex(tf,outfile,temp1) | tee (log1, > log2)
            imdelete(outfile,verify=no)
            imrename(temp1,outfile,verbose=no)
         }

         # If doing flux calibration, do that now.

         if (do_flx && flxcal) {
            calibrate(outfile,outfile,extinct=yes,flux=yes,
               extinction=extfile,observatory=observatory,
               ignoreaps=yes,sensitivity=sf,fnu=no) | tee(log1,>log2)
         }

         # Add this file into the running log

         runlog(ofile,ired=2,instfile=ccdred.instrument)

         # If displaying the final frame...

         if (disp)
            display(outfile,1,bpmask="",bpdisplay="none",bpcolors="red",
               overlay=no,ocolors="green",erase=yes,border_erase=no,
               select_frame=yes,repeat=no,fill=yes,zscale=yes,contrast=0.25,
               zrange=yes,zmask="",nsample=1000,xcenter=0.5,ycenter=0.5,
               xsize=1.0,ysize=1.0,xmag=1.0,ymag=1.0,order=0,ztrans="linear",
               lutfile="",>>"dev$null")
      }
end
