include <error.h>
include <imhdr.h>

procedure t_arcstat()

# Local variables

pointer sp,iname,rootname,dbname,line,dt,iptr,outfile,optr
int record,nfeat,ntot,i,iwt,junkint,dispaxis,spaxis,nl,isp,nsp,iref
real sum,sumsq,junkr,w1,w2,resid,mean,sig

# Functions

int dtlocate(),dtgeti(),dtscan(),clgeti()
pointer immap()
                     
begin

      # Get some workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(rootname,SZ_FNAME,TY_CHAR)
      call salloc(dbname,SZ_FNAME,TY_CHAR)
      call salloc(line,SZ_LINE,TY_CHAR)
      call salloc(outfile,SZ_FNAME,TY_CHAR)

      # Get input file name and database name. Strip any extra rubbish
      # off the input file name. 

      call clgstr("input",Memc[iname],SZ_FNAME)
      iferr (iptr = immap(Memc[iname],READ_ONLY,0)) {
         call eprintf("Can't find file: %s\n")
            call pargstr(Memc[iname])
         call erract(EA_ERROR)
      }
      call xt_imroot(Memc[iname],Memc[rootname],SZ_FNAME)
      call sprintf(Memc[iname],SZ_LINE,"id%s")
         call pargstr(Memc[rootname])
      call clgstr("database",Memc[dbname],SZ_FNAME)

      # Get the dispersion axis and spatial axis.  Find the maximum number
      # of lines in the arc file which could have been identified.

      dispaxis = clgeti("dispaxis")
      spaxis = 3 - dispaxis
      nsp = IM_LEN(iptr,spaxis)
      call imunmap(iptr)

      # Open the output file.  This is the header of the object file.  We want
      # to write the rms results here.  If we can't open them then don't bother
      # to write the results to a header.

      call clgstr("outfile",Memc[outfile],SZ_FNAME)
      iferr (optr = immap(Memc[outfile],READ_WRITE,0))
         optr = NULL

      # Is this reference spectrum 1 or 2?

      iref = clgeti("iref")

      # Open the database file...

      dt = NULL
      iferr (call dtremap(dt,Memc[dbname],Memc[iname],READ_ONLY)) {
         call eprintf("Can't map file %s in %s/\n")
            call pargstr(Memc[iname])
            call pargstr(Memc[dbname])
         call erract(EA_ERROR)
      }

      # Initialise a few things...

      sum = 0.0
      sumsq = 0.0
      ntot = 0
      nl = 0

      # Now loop for each spectrum

      for (isp = 1; isp <= nsp; isp = isp + 1) {

         # Get database record for aperture and find out how many
         # features were identified

         if (dispaxis == 1)
            call sprintf(Memc[line],SZ_LINE,"identify %s[*,%d]")
         else
            call sprintf(Memc[line],SZ_LINE,"identify %s[%d,*]")
         call pargstr(Memc[rootname])
         call pargi(isp)
         iferr (record = dtlocate(dt,Memc[line])) 
            next
         nfeat = dtgeti(dt,record,"features")
         nl = nl + 1

         # Now read the feature information and do the required summations
         
         do i = 1,nfeat {
            junkint = dtscan(dt)
            call gargr(junkr)
            call gargr(w1)
            call gargr(w2)
            call gargr(junkr)
            call gargi(junkint)
            call gargi(iwt)
            if (IS_INDEF(w1) || IS_INDEF(w2)) iwt = 0
            if (iwt > 0) {
               resid = w1 - w2
               sum = sum + resid
               sumsq = sumsq + resid**2
               ntot = ntot + 1
            }
         }
      }

      # Calculate rms.  Deal with error situations...

      if (ntot <= 2)
         sig = INDEF
      else {
         mean = sum/real(ntot)
         sig = sumsq/real(ntot) - mean*mean
         if (sig < 0.0) 
            sig = INDEF
         else
            sig = sqrt(sig)
      }
       
      # Now report the situation...

      call printf(
         "Image: %s,  No. Spectra: %d,  No. lines: %d,  RMS error: %0.6g\n")
         call pargstr(Memc[iname])
         call pargi(nl)
         call pargi(ntot)
         call pargr(sig)

      # If the header to an output file exists, then write the result here.

      if (optr != NULL) {
         call sprintf(Memc[line],SZ_LINE,"DCE%03d_%d")
            call pargi(1)
            call pargi(iref)
         call imaddr(optr,Memc[line],sig)
      }

      # Free workspace and close things up

      call imunmap(optr)
      call dtunmap(dt)
      call sfree(sp)
end
