include <gio.h>
include <chars.h>
include <fset.h>

define  XGTERMC  "xgterm -sb -fn 7x14 -fb black -bg white -T monitor -e "
define  LENCMD   1023
define  SZTMPPTH 2048

procedure t_ids()
begin
      call reduce("ids")
end

procedure t_isis()
begin
      call reduce("isis")
end

procedure t_aatrgo()
begin
      call reduce("aatrgo")
end

procedure t_aatfors()
begin
      call reduce("aatfors")
end



procedure reduce(instr)

# Parameters

char instr[ARB]

# Local variables

pointer sp,gp,strval,cmd,rp1,rp2,rp3,guifile,tmp,pp1,pp2,logfile,fullname
pointer tmp1,tmp2,tmp3,tmp4,telname,instname,dirnm,qnm,rnm,hnm
int wcs,key,junk
real x,y

# Functions

int clgcur(),strlen(),access(),oscmd()
pointer gopenui(),clopset(),ps_open()
bool streq()
extern gflush()

begin

      # Get some workspace

      call smark(sp)
      call salloc(telname,SZ_LINE,TY_CHAR)
      call salloc(instname,SZ_LINE,TY_CHAR)
      call salloc(cmd,LENCMD,TY_CHAR)
      call salloc(strval,SZ_LINE,TY_CHAR)
      call salloc(guifile,SZ_FNAME,TY_CHAR)
      call salloc(tmp,SZ_PATHNAME,TY_CHAR)
      call salloc(tmp1,SZ_FNAME,TY_CHAR)
      call salloc(tmp2,SZ_FNAME,TY_CHAR)
      call salloc(tmp3,SZ_FNAME,TY_CHAR)
      call salloc(tmp4,SZ_FNAME,TY_CHAR)
      call salloc(logfile,SZ_FNAME,TY_CHAR)
      call salloc(fullname,SZ_PATHNAME,TY_CHAR)
      call salloc(dirnm,SZTMPPTH,TY_CHAR)
      call salloc(qnm,SZTMPPTH,TY_CHAR)
      call salloc(rnm,SZTMPPTH,TY_CHAR)
      call salloc(hnm,SZTMPPTH,TY_CHAR)
    
      # Get the telescope name and instrument name.  If they are blank, then
      # get out of here and issue some advice.

      call clgstr("telescope",Memc[telname],SZ_LINE)
      call clgstr("instrument",Memc[instname],SZ_LINE)
      if (strlen(Memc[telname]) == 0 || strlen(Memc[instname]) == 0) {
         call error(0,
            "Telescope and/or instrument undefined -- run setup first")
      }

      # If the instrument name and the instrument which is specified in 
      # the call to this subroutine don't match, the get out of here.

      if (!streq(Memc[instname],instr)) {
         call eprintf("Setup instrument: %s,  requested instrument: %s\n")
            call pargstr(Memc[instname])
            call pargstr(instr)
         call error(0,"Run setup first and try again")
      }

      # Block for when you are using a gui...

      call clgstr("guifile",Memc[guifile],SZ_FNAME)
      if (!streq(Memc[guifile],"")) {

         # Get descriptor for pset

         pp1 = clopset(instr)
         pp2 = clopset("rpars")

         # Do work on gui

         gp = gopenui("stdgraph",NEW_FILE,Memc[guifile],STDGRAPH)

         # Make some temporary file names

         call mktemp("tmp$iraf",Memc[tmp1],SZ_FNAME)
         call mktemp("tmp$iraf",Memc[tmp2],SZ_FNAME)
         call mktemp("tmp$iraf",Memc[tmp3],SZ_FNAME)
         call mktemp("tmp$iraf",Memc[tmp4],SZ_FNAME)

         # Get the logfile name.  Append the full directory name and send it
         # down to the gui.
 
         call clgstr("logfile",Memc[logfile],SZ_FNAME)
         call fmapfn(Memc[logfile],Memc[tmp],SZ_PATHNAME)
         call strupk(Memc[tmp],Memc[fullname],SZ_PATHNAME)
         if (streq(Memc[fullname],Memc[logfile])) {
            call sprintf(Memc[fullname],SZ_PATHNAME,"./%s")
               call pargstr(Memc[logfile])
            call fmapfn(Memc[fullname],Memc[tmp],SZ_PATHNAME)
            call strupk(Memc[tmp],Memc[fullname],SZ_PATHNAME)
         }
         call gmsg(gp,"logfile",Memc[fullname])

         # Open two parameter structures, one for the standard parameters of
         # the task and the other for the rpars pset. Open the one for
         # rspecred to be used later when setting the parameters for that
         # task.  This is used to weed out any instrument specific parameters
         # from the main parameter file.

         call sprintf(Memc[tmp],SZ_PATHNAME,"spec2$%s.par")
            call pargstr(instr)
         rp1 = ps_open(Memc[tmp])
         rp2 = ps_open("spec2$rpars.par")
         rp3 = ps_open("spec2$rspecred.par")
         call ps_fill(rp1)
         call ps_fill(rp2)
         call ps_fill(rp3)

         # Setup standard and rpars pset values

         call setpars(gp,pp1,rp1,1)
         call setpars(gp,pp2,rp2,2)

         # Create login.cl and .hushiraf files in a special directory

         call mklogincl("spec2",instr,Memc[tmp1],Memc[tmp2],Memc[tmp3],
            Memc[tmp4],Memc[dirnm],SZTMPPTH)

         # Get the full name of the window cshell script

         call fmapfn("spec2$mwindow",Memc[tmp],SZ_PATHNAME)
         call strupk(Memc[tmp],Memc[fullname],SZ_PATHNAME)

         # Start up xgterm...

         call sprintf(Memc[cmd],LENCMD,"unset time; cd %s; %s %s &")
            call pargstr(Memc[dirnm])
            call pargstr(XGTERMC)
            call pargstr(Memc[fullname])
         junk = oscmd(Memc[cmd],"","dev$null","dev$null")

         # Create semaphore file names...

         call sprintf(Memc[qnm],SZ_FNAME,"%s/quit.sp2")
            call pargstr(Memc[dirnm])
         call sprintf(Memc[rnm],SZ_FNAME,"%s/reduce.sp2")
            call pargstr(Memc[dirnm])
         call sprintf(Memc[hnm],SZ_FNAME,"%s/help.sp2")
            call pargstr(Memc[dirnm])

         # Cursor loop

         while (clgcur("cursor",x,y,wcs,key,Memc[strval],SZ_LINE) != EOF) {
            switch (key) {
               case 'e':
                  iferr (call fcopy("spec2$dummy.sp2",Memc[qnm]))
                     next
                  break
               case 'r':
                  call fillpars(rp1,rp3)
                  call prepfiles(Memc[tmp1],"rspecred",NO)
                  call prepfiles(Memc[tmp2],"rpars",NO)
                  call prepfiles(Memc[tmp3],"instrpars",NO)
                  call prepfiles(Memc[tmp4],instr,YES)
                  iferr (call fcopy("spec2$dummy.sp2",Memc[rnm])) {
                     call gmsg(gp,"errmsg",
                        "Wait a moment -- I'm in the middle of another reduction")
                     next
                  }
               case 'h':
                  iferr (call fcopy("spec2$dummy.sp2",Memc[hnm]))
                     next
               case ':':
                  iferr (call getparam(gp,pp1,pp2,rp1,rp2,Memc[strval]))
                     next
            }
         }

         # Delete these files before the user notices...

#         call dellogincl(Memc[dirnm],SZTMPPTH)

         # Delete temporary files...
  
         if (access(Memc[tmp1],0,0) == YES) 
            call delete(Memc[tmp1])
         if (access(Memc[tmp2],0,0) == YES) 
            call delete(Memc[tmp2])
         if (access(Memc[tmp3],0,0) == YES) 
            call delete(Memc[tmp3])
         if (access(Memc[tmp4],0,0) == YES) 
            call delete(Memc[tmp4])

         # Release pset structures, SPEC2 parameter system structures and close
         # the graphics...

         call clcpset(pp1)
         call clcpset(pp2)
         call ps_close(rp1)
         call ps_close(rp2)
         call ps_close(rp3)
         call gclose(gp)

      # Block for not using a gui...

      } else {

         # Get descriptor for pset descriptor for main parameters only.
         # Open a SPEC2 parameter structure for them and fill them in.

         pp1 = clopset(instr)
         call sprintf(Memc[tmp],SZ_PATHNAME,"spec2$%s.par")
            call pargstr(instr)
         rp1 = ps_open(Memc[tmp])
         rp3 = ps_open("spec2$rspecred.par")
         call ps_fill(rp1)
         call ps_fill(rp3)

         # Now fill the parameters for 'rspecred' with these values and
         # execute the script.

         call fillpars(rp1,rp3)
         call clcmdw("rspecred")
 
         # Release workspace for structures...

         call clcpset(pp1)
         call ps_close(rp1)
         call ps_close(rp3)
      }

      # Ditch the last bit of workspace

      call sfree(sp)
end

procedure prepfiles(filename,psetname,edit)

# Parameters

char filename[ARB],psetname[ARB]
int edit

# Local variables

int fd1,fd2
pointer tmp,tmpfil,sp

# Functions

int access(),open(),getline(),strsearch()

begin

      # Get some workspace

      call smark(sp)
      call salloc(tmp,SZ_LINE,TY_CHAR)
      call salloc(tmpfil,SZ_FNAME,TY_CHAR)

      # If you can find the file, then delete it

      if (access(filename,0,0) == YES) 
         call delete(filename)
      
      # Do the requisite dpar

      call sprintf(Memc[tmp],SZ_PATHNAME,"dpar %s > %s")
         call pargstr(psetname)
         call pargstr(filename)
      call clcmdw(Memc[tmp])

      # If you want to edit $nargs out, then do that here...

      if (edit == YES) {
         call mktemp("tmp$iraf",Memc[tmpfil],SZ_FNAME)
         call rename(filename,Memc[tmpfil])
         fd1 = open(Memc[tmpfil],READ_ONLY,TEXT_FILE)
         fd2 = open(filename,NEW_FILE,TEXT_FILE)
         while (getline(fd1,Memc[tmp]) != EOF) {
            if (strsearch(Memc[tmp],"$nargs") != 0) 
               next
            else
               call putline(fd2,Memc[tmp])
         }
         call close(fd1)
         call close(fd2)
         call delete(Memc[tmpfil])
      }
end

