procedure tippex(tfile,input,output)

file   tfile     {prompt="Name of tippex file"}
file   input     {prompt="Name of file to be cleaned"}
file   output    {prompt="Name of output file"}

struct *fd

begin

      string tf,in,out,temp1,temp2,temp3
      real w1,w2,cen,width
      int i,nspec,sp_axis

      # Temporary file names...

      temp1 = mktemp("tmp$iraf")
      temp2 = mktemp("tmp$iraf")
      temp3 = mktemp("tmp$iraf")

      # Get tippex file, open it and create a cursor file with a single entry

      tf = tfile
      if (strlen(tf) == 0) {
         print("No tippex file was specified -- skipping")
         return
      } else if (!access(tf)) {
         print("Can\'t access tippex file: ",tf," -- skipping")
         return
      }
      fd = tf
      while (fscan(fd,cen,width) != EOF) {
         w1 = cen - width
         w2 = cen + width
         print(w1," 1.0 1 x", >> temp1)
         print(w2," 1.0 1 x", >> temp1)
      }
      print("0.0 0.0 1 i", >> temp1)
      print("0.0 0.0 1 )", >> temp1)
      fd = ""

      # Get input and output names

      in = input
      out = output

      # Now find out how many spectra there are to clean and create a cursor
      # file with entries for all of the spectra...

      sp_axis = 3 - dispaxis
      hselect(in,"i_naxis"//sp_axis,yes) | scan(nspec)
      for (i = 1; i <= nspec; i+=1) 
         type(temp1,>>temp2)
      print("0.0 0.0 1 q", >>temp2)
      delete(temp1,verify=no)

      # If the dispersion axis is Y, then transpose the image

      if (dispaxis == 1) 
         temp1 = in
      else
         imtranspose(in,temp1,len_blk=512)

      # Set the mode of the new_image and overwrite parameters in "splot" so
      # that you never get prompted for them

      splot.new_image.p_mode="h"
      splot.overwrite.p_mode="h"
      splot.new_image = temp3
      splot.overwrite=yes

      # Do the tippexing...

      splot(temp1,1,units="",options="auto wreset",xmin=INDEF,xmax=INDEF,
         ymin=INDEF,ymax=INDEF,save_file="",graphics="stdgraph",
         cursor=temp2,sigma0=INDEF,invgain=INDEF,function="legnedre",
         order=1,low_reject=4,high_reject=4,niterate=0,grow=1.0,
         markrej=yes, >G "dev$null", > "dev$null")

      # Reset the splot parameter modes

      splot.new_image.p_mode="q"
      splot.overwrite.p_mode="q"

      # Ok, now copy/transpose the image back to the input file if need be

      if (dispaxis == 2) {
         imtranspose(temp3,out,len_blk=512)
         imdelete(temp1,verify=no)
         imdelete(temp3,verify=no)
      } else {
         imrename(temp3,out,verbose=no)
      }

      # Zap the temporary text file

      delete(temp2,verify=no)

      # Put something into the header to tell the user that this file has
      # been tippexed

      hedit(out,"tp-flag",0,add=yes,show=no,verify=no,update=yes)
end
