procedure ffos2(image)

# Script to locate and extract FOS2 data from raw frames.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'pfos2.cl'.  Both scripts are called from the task 'fos2'
# in the package 'rgo.quickie' and it is recommended that you use the
# fos2 task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 
# 
# Updates:
#      May 1995 -- Added bias subtraction, flat fielding and a check
#                  for header information.  Also fixed a number of
#                  bugs.  JRL and BJB
#      20 Feb 1996 -- Major overhaul.  JRL

string  image   {prompt="Input image(s)"}
bool    sec     {yes,prompt="Extract 2nd order as well?"}
bool    fconv   {no,prompt="Convert from FITS?"}
bool    crproc  {yes,prompt="Remove cosmic rays for extraction?"}
bool    lcal    {yes,prompt="Wavelength calibrate?"}
bool    comb    {no,prompt="Combine first and second order spectra?"}
bool    head    {yes,prompt="Does header information exist?"}
bool    bsub    {yes,prompt="Perform bias subraction"}
bool    ffield  {no,prompt="Perform flat field"}
string  fframe  {"",prompt="Flat field name"}
#string  sfunc   {"",prompt="Name of sensitivity function?"}


struct  *inimglist
struct  *tlist

begin

        string  dsect           # data section
        string  imglist         # equals image
        string  imtype          # image type
        string  mstype          # multispec image type
        string  infile          # temporary list for files
        string  img             # image name from fscan with extension stripped
        string  ct              # type of chip
        string  ca              # isis arm
        string  refim           # reference image for spectral tracing
        string  outfil          # dummy string
        string  tempname        # dummy string
        string  dumstr          # dummy string
        string  log1            # file for logging
        string  ff              # input flat field frame
        string  sf              # input sensitivity function
        string  bsect           # bias strip data section
        string  ysamp           # region to sample for polynomial fit to bias
        string  ccdwmode        # ccd windowing mode
        real    cw              # central wavelegnth
        real    defsizx,defsizy # default sizes in case header items are missing
        real    endp            # end pixel for wavelength calibration
        real    app             # dispersion A/pix
        real    mpp             # mm/pix
        real    start           # starting pixel for wavelength calibration
        real    stw             # start wavelength
        real    rd              # CCD readnoise
        real    gn              # CCD gain
        real    xs,xz,xsw,xzw   # X dimensions of chip/window
        real    xsize           # X size of chip
        real    ys,yz,ysw,yzw   # Y dimensions of chip/window
        real    ysize           # Y size of chip
        real    xb,yb           # binning factor
        real    dumreal         # dummy real
        real    pos             # spectrum position
        int     bxstart,bxend   # X position of bias strip
        int     bystart,byend   # Y position of bias strip
        int     mm              # dummy
        int     nspec           # Number of spectra to extract
        int     i               # Size of a character string
        int     nfound          # Number of spectra actually extracted
        bool    second          # TRUE if extracting second order
        bool    combine         # TRUE if combining first and second order
        bool    fcon            # TRUE if converting from FITs
        bool    crp             # TRUE if processing cosmic rays
        bool    fproc           # TRUE if dividing by a flat field
        bool    bproc           # TRUE if subtracting a bias estimate
        bool    procccd         # TRUE if CCD processing is done

# Get query parameter
        imglist = image

# Decide whether to do the second order or not...
        second = sec 
        if (second)
           nspec = 2
        else
           nspec = 1
        combine = no
        if (second) combine = comb

# Get a few more parameters
        fcon = fconv
        crp = crproc

# Set up logging
        tee.append = yes
        log1 = "fos2.log"

# Expand file lists into temporary files.
        infile =  mktemp("temp")
        sections (imglist,option="fullname",>infile)
        inimglist = infile

# Get file name extension
        imtype = "."//envget("imtype")
        mstype = ".ms"//imtype

# Strip extension from flat field image and sensitivity function
# (if either exist...)
        ff = fframe
        strpextn(ff) | scan(ff)
#        sf = sfunc
#        strpextn(sf) | scan(sf)

# Loop through images
        while (fscan(inimglist,img) != EOF) {

# Strip off any extensions.  This is to avoid conflicts with names which
# might arise with FITS files or if the user has used meta-characters in
# the input file list
           strpextn(img) | scan(img)

# Clobber old output files
           imdelete(img//mstype,ver-, >& "dev$null")
           if (combine)   
              imdelete(img//"C"//mstype,ver-,>& "dev$null")

# If the input file is a fits image, then convert it to an iraf file...
# This will become unnecessary when v2.11 arrives with its FITS image kernel
           if (fcon) {
              print(" Reading FITS image") | tee(log1)
              rfits(img//".fits", "", img, make+, lo-, sh-,
                 datatype="", blank=0.0, scale+, oldir-, offset=0)
           }

# Get the chip name in the isis arm.
           ct = ""; ca = ""
           q_gets(img,"CCDCHIP","") | ucase | scan(ct)
           q_gets(img,"ISIARM","") | ucase | scan(ca)
           if (ct == "" || ca == "")
              print("Warning: Info on chip name or ISIS arm don't exist in header") | tee (log1)
           if (ca != "FOS" && ca != "")
              error(0,"This isn't a FOS frame!")
           
# Get information about windowing of the chip
           q_getr(img,"CCDXBIN",1.0) | scan(xb)
           q_getr(img,"CCDYBIN",1.0) | scan(yb)

           q_getr(img,"i_naxis1",1.0) | scan(defsizx)
           q_getr(img,"i_naxis2",1.0) | scan(defsizy)
           q_getr(img,"CCDXIMST",1.0) | scan(xs)
           q_getr(img,"CCDXIMSI",defsizx) | scan(xz)
           q_getr(img,"CCDXSIZE",defsizx) | scan(xsize)

           q_getr(img,"CCDYIMST",1.0) | scan(ys)
           q_getr(img,"CCDYIMSI",defsizy) | scan(yz)
           q_getr(img,"CCDYSIZE",defsizy) | scan(ysize)

           q_gets(img,"CCDWMODE","F") | ucase | scan(ccdwmode)
           if (ccdwmode == "T") {
              q_getr(img,"CCDWXO1",0.0) | scan(xsw)
              q_getr(img,"CCDWXS1",defsizx) | scan(xzw)
              q_getr(img,"CCDWYO1",0.0) | scan(ysw)
              q_getr(img,"CCDWYS1",defsizy) | scan(yzw)
           } else {
              xsw = 0.0
              xzw = defsizx
              ysw = 0.0
              yzw = defsizy
           }

           start = max(1.0,real(nint((ys-ysw-1.0)/yb+1.0)))
           endp = min(start+yzw/yb-1.0,start+yz/yb-1.0)

# Get the chip readout noise and gain.  If it doesn't exist, then
# substitute plausible values.
           q_getr(img,"READNOIS",5.0) | scan(rd)
           q_getr(img,"GAIN",1.0) | scan(gn)
           print(" ") | tee(log1)
           print(" CCD frame: ",img,"   FOS: ",ca) | tee(log1)
           print(" ") | tee(log1)
           print(" Chip: ",ct,"   Readnoise = ",rd," Gain = ",gn,) | tee(log1)
           print(" ") | tee(log1)

# Do you want to do CCD processing?
           bproc = bsub
           if ((xs - 2) <= xsw) bproc = no
           fproc = ffield
           if (fproc && !access(ff//imtype)) {
              fproc = no
              print("Warning: can't access flat-field image") | tee(log1)
           }
           procccd = no
           if(fproc || bproc){
              if (!access("backup/"))
                 mkdir("backup/")
              ccdred.verbose = no
              ccdred.backup = "backup/"
              ccdred.logfile = "fos2.log"

# Create a bias section
              bxstart = (2 - xsw)/xb
              if (bxstart <= 0) bxstart = 1
              bxend = (xs - 2 - xsw)/xb
              if (bxend < bxstart) bxend = bxstart
              bystart = (10 - ysw)/yb
              if (bystart <= 0) bystart = 1
              byend = (yzw)/yb
              if(byend > (578.0/yb) || byend <= 0.0) byend = 578.0/yb
              bsect="["//int(bxstart)//":"//int(bxend)//","//int(bystart)//":"//int(byend)//"]"
              ysamp = int(bystart)//":"//int(byend)
            
# Perform bias subtraction and flat-field if required
              if(bproc)print (" Performing overscan-correction") | tee(log1)
              if(fproc)print (" Performing flat-field division") | tee(log1)
              ccdproc (img,ccdtype=" ", max_cache=0, noproc=no, fixpix=no, 
                 overscan=bproc, trim=no, zerocor=no, darkcor=no, flatcor=fproc, 
                 illumcor=no, fringecor=no, readcor=no, scancor=no, 
                 readaxis="line", fixfile="", biassec=bsect, trimsec="",
                 zero="", dark="", flat=ff, illum="", fringe="", minreplace=1.,
                 scantype="shortscan", nscan=1, interactive=no, function="legendre",
                 order=1, sample=ysamp, naverage=1, niterate=1, low_reject=3., 
                 high_reject=3., grow=0.)
              procccd = yes
           }

# Set airmass and Julian date  - you never know when they might come in handy.....
# However, only do it if header information exists
           if(head){
              setjd (img,observatory="lapalma", date="date-obs", time="utobs", 
                 exposure="exptime",ra="ra", dec="dec", epoch="epoch", 
                 jd="jd", hjd="hjd", ljd="ljd",utdate=yes, uttime=yes,
                 listonly=no, >& "dev$null")
              setairmass (img,observatory="lapalma", intype="beginning", 
                 outtype="effective",date="date-obs", exposure="exptime", 
                 airmass="airmass", utmiddle="utmiddle", show=yes, update=yes, 
                 override=yes, >& "dev$null")
           }

# Remove cosmic rays from the frame to facilitate locating the spectra. Note
# that this is not the frame from which the spectra will be extracted.
           refim=img
           if (crp) {
              refim=mktemp(img)
              print(" Removing Cosmic Rays") | tee(log1)
              imdelete("_fmed*"//imtype, ver-, >& "dev$null")
              imdelete("_cr*"//imtype, ver-, >& "dev$null")
              imdelete("_mask*"//imtype, ver-, >& "dev$null")
              xzap(img,outlist=refim, nsigma=2., xboxsz=1, yboxsz=15, 
                 nobjsigma=0., nrings=0, donegative-,nnegsigma=5.,
                 imgsec="[*,*]", deletemasks+, cleanup+, verbose-,
                 inimglist="", outimglist="", statlist="", >& "dev$null")
              imdelete("_fmed"//img, ver-)
           }

# Trace the spectrum...
           print(" Tracing spectrum") | tee(log1)

           apextract.dispaxis = 2
           apall (refim, output=refim, apertures= "",format="multispec", 
              references="", nfind=nspec, profiles="", interactive-, find+,
              recenter+, resize-, edit-, trace+, fittrace-,
              extract-, extras-, review-, line=150, nsum=20, 
              lower=-4., upper=4., apidtable="", b_function="chebyshev", 
              b_order=1, b_sample="-12:-7,7:12", b_naverage=-10, 
              b_niterate=1, b_low_reject=3., b_high_rejec=3., b_grow=0., 
              width=10., radius=10., threshold=0., minsep=30., maxsep=200., 
              order="increasing", aprecenter="", npeaks=nspec, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+,
              r_grow=0., avglimits-, t_nsum=10, t_step=10, t_nlost=1, 
              t_function="legendre", t_order=8, t_sample="*", t_naverage=1, 
              t_niterate=3, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance", pfit="fit1d", 
              clean+, saturation=INDEF, readnoise=rd, gain=gn, lsigma=4., 
              usigma=4., nsubaps=1, > "dev$null")

# Extract the spectrum...
           print(" Extracting spectrum ") | tee(log1)

           apall (img,output=img, apertures="", format="multispec", 
              references=refim, nfind=nspec, profiles="", interactive-, find-,
              recenter-, resize-, edit-, trace-, fittrace-, extract+, 
              extras-, review-, line=150, nsum=20, lower=-4., upper=4., 
              apidtable="", b_function="chebyshev", b_order=1, 
              b_sample="-12:-7,7:12", b_naverage=-10, b_niterate=1, 
              b_low_reject=3., b_high_rejec=3., b_grow=0., width=10., 
              radius=10., threshold=0., minsep=30.,maxsep=200., 
              order="increasing", aprecenter="", npeaks=nspec, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+, 
              r_grow=0., avglimits-, t_nsum=10, t_step=10, t_nlost=1, 
              t_function="legendre", t_order=8, t_sample="*", t_naverage=1,
              t_niterate=3, t_low_reject=3, t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance",pfit="fit1d", 
              clean+, saturation=65000, readnoise=rd, gain=gn, lsigma=3., 
              usigma=3., nsubaps=1, > "dev$null")
 
# Send out a friendly message and clean up the database directory.  Remove
# the cosmic ray processed frame if it exists...
           print(" ") | tee(log1)
           tempname = mktemp("temp")
           type("database/aplast",map+,dev="terminal") | 
              match("begin","",stop-,print+,meta+,>tempname) 
           tlist = tempname
           nfound = 0
           while (fscan(tlist,dumstr,dumstr,dumstr,mm,pos,dumreal) != EOF) {
              print(" Spectrum # ",mm, "found in column ",pos) | tee(log1)
              nfound = nfound + 1
           }
           print(" ") | tee(log1)
           tlist = ""
           delete(tempname, ver-, >& "dev$null")
           delete("database/ap"//refim, ver-, >& "dev$null")
           if (crp) imdel(refim, ver-)

# Create a data section string for use later
           if (nfound > 1) 
              dsect = "["//int(start)//":"//int(endp)//",*]"
           else
              dsect = "["//int(start)//":"//int(endp)//"]"

# If you did CCD processing on this frame, then get rid of the processed
# frame and bring back the original...
           if (procccd) {
              imdelete(img, ver-)
              imrename("backup/"//img//imtype,img,verbose-)
           }

# Chop off the bits you don't want...
           tempname = mktemp("temp")
           outfil = img//mstype
           imrename(outfil,tempname,verbose-, >& "dev$null")
           imcopy(tempname//dsect,outfil,verbose-,>&"dev$null")
           imdelete(tempname,verify-)

# If you want to do a wavelength calibration then decide where on the chip
# the spectrum lies...
           if(lcal){
              start = max(ys,ysw)
              endp = min(ys+yz-1,ysw+yzw-1)
   
# Get the central wavelength and dispersion.  Also sort out which arm/chip
# combinations reverse the sense of the dispersion (e.g. red to the left).
              print(" Applying mean wavelength solution") | tee(log1)
              print(" ") | tee(log1)
              stw = 7212.0 + (ysize*0.5 - start)*8.7
              app = -8.7

# Edit the WCS to include the wavelength information for first order.
              wcsreset(outfil,wcs="physical",verbose-)
              sapertures(outfil,apertures="1",apidtable="",wcsreset=no,
                 verbose-,beam=INDEF,dtype=0,w1=stw,dw=app,z=INDEF,
                 aplow=INDEF,aphigh=INDEF,title="INDEF")
              print(" First order:   Start wavelength ",stw," Angstroms/pix ",
                 app," Binning factor ",xb) | tee(log1)

# Do the same for second order...
              if (nspec == 2) {
                 stw = 3646.0 + (ysize*0.5 - start)*4.3
                 app = -4.3
                 sapertures(outfil,apertures="2",apidtable="",wcsreset=no,
                    verbose-,beam=INDEF,dtype=0,w1=stw,dw=app,z=INDEF,
                    aplow=INDEF,aphigh=INDEF,title="INDEF")
                 print(" Second order:  Start wavelength ",stw,
                    " Angstroms/pix ",app," Binning factor ",xb) | tee(log1)
              }
              dispcor(outfil,outfil,linearize+,database="",table="",
                 w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,log-,flux+,samedisp-,
                 global-,ignoreaps-,confirm-,listonly-,verbose-,logfile=log1)
              print(" ")  | tee(log1)

# If not wavelength calibrating, then set up the logical and physical
# coordinates to be the same
           } else {
              wcsreset(outfil,wcs="physical",verbose-)
           }

# Perform flux calibration if required....
#           if (fcal) 
#              calibrate (outfil,outfil, extinct=yes, flux=yes, 
#                 extinction="rgo$flux/lpoext.tab",observatory="lapalma", 
#                 ignoreaps=yes, sensitivity=sf, fnu=no)
           
# If both orders have been extracted, then combine them if that's what you want
           if (second && combine) {
              scombine(outfile,img//"C"//mstype,noutput="",
                 logfile="STDOUT",apertures="1-2",group="all",
                 combine="average",reject="none",first-,w1=INDEF,
                 w2=INDEF,dw=INDEF,nw=INDEF,log-,scale="none",zero="none",
                 weight="none",sample="",lthreshold=INDEF,hthreshold=INDEF,
                 nlow=1,nhigh=1,nkeep=1,mclip+,lsigma=3.0,hsigma=3.0,
                 rdnoise="0.0",gain="1.0",snoise="0.0",sigscale=0.1,
                 pclip=-0.5,grow=0,blank=0.0, >& "dev$null")
           }
# If this started out as a FITS file, then delete the IRAF file...
           if (fcon) imdelete(img, ver-, >& "dev$null")
        }

# Tidy and exit
        delete (infile, ver-)
        inimglist = ""
end
                      
