procedure fisis(image)

# Script to locate and extract ISIS data from raw frames.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'pisis.cl'.  Both scripts are called from the task 'isis'
# in the package 'rgo.quickie' and it is recommended that you use the
# isis task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 
# 
# Updates:
#      May 1995 -- Added bias subtraction, flat fielding and a check
#                  for header information.  Also fixed a number of
#                  bugs.  JRL and BJB
#      20 Feb 1996 -- Major overhaul.  JRL

string  image   {prompt="Input image(s)"}
int     nspec   {1,prompt="Number of spectra in image?"}
int     blkfact {1,min=1,prompt="Blocking factor in spectral domain"}
bool    fconv   {no,prompt="Convert from FITS?"}
bool    crproc  {yes,prompt="Remove cosmic rays for extraction?"}
bool    lcal    {yes,prompt="Wavelength calibrate?"}
bool    fcal    {no,prompt="Perform flux calibration?"}
bool    comb    {no,prompt="Combine all spectra into one?"}
bool    head    {yes,prompt="Does header information exist?"}
bool    bsub    {yes,prompt="Perform bias subraction"}
bool    ffield  {no,prompt="Perform flat field"}
string  fframe  {"",prompt="Flat field name"}
#string  sfunc   {"",prompt="Name of sensitivity function?"}


struct  *inimglist
struct  *tlist

begin

        string  chipname        # name of chip
        string  dsect           # data section
        string  bsect           # bias region
        string  ysamp           # Y-range of overscan
        string  imglist         # equals image
        string  imtype          # image type
        string  mstype          # multispec image type
        string  infile          # temporary list for files
        string  img             # image name from fscan with extension stripped
        string  ct              # type of chip
        string  refim           # reference image for spectral tracing
        string  outfil          # dummy string
        string  tempname        # dummy string
        string  dumstr          # dummy string
        string  instring        # dummy string
        string  ff              # Flat field frame
        string  sf              # Sensitivity function
        string  log1            # Name of log file
        string  ccdwmode        # ccd windowing mode
        real    cw              # central wavelegnth
        real    defsizx,defsizy # default size in case header items are missing
        real    endp            # end pixel for wavelength calibration
        real    app             # dispersion A/pix
        real    mpp             # mm/pix
        real    start           # starting pixel for wavelength calibration
        real    stw             # start wavelength
        real    rd              # CCD readnoise
        real    gn              # CCD gain
        real    xs,xz,xsw,xzw   # X dimensions of chip/window
        real    ys,yz,ysw,yzw   # Y dimensions of chip/window
        real    xsize           # X size of chip
        real    ysize           # Y size of chip
        real    xb,yb           # binning factor
        real    bxstart,bxend   # X position of bias strip
        real    bystart,byend   # Y position of bias strip
        real    dumreal         # dummy real
        real    pos             # spectrum position
        int     i               # File name length
        int     mm              # dummy
        int     iblk            # blocking factor in spectral domain
        int     nsp             # number of spectra
        int     junk            # dummy
        int     nfound          # Number of spectra actually extracted
        int     dispaxis        # dispersion axis
        bool    bproc           # TRUE if performing bias subraction
        bool    fproc           # TRUE if performing flat field correction 
        bool    crp             # TRUE if removing cosmic rays
        bool    fcon            # TRUE if input file is a FITS file
        bool    procccd         # TRUE if the file has been CCD processed
        bool    lcal2           # TRUE if doing wavelength calibration
        struct  ca              # Name of ISIS arm
        

# Get query parameter
        imglist = image

# Expand file lists into temporary files.
        infile =  mktemp("temp")
        sections (imglist,option="fullname",>infile)
        inimglist = infile

# Get a few parameters
        iblk = blkfact
        nsp = nspec
        crp = crproc
        fcon = fconv

# Set up logging
        tee.append = yes
        log1 = "isis.log"

# Get file name extension
        imtype = "."//envget("imtype")
        mstype = ".ms"//imtype

# Strip extension from flat field image and sensitivity function
# (if either exist...)
        ff = fframe
        strpextn(ff) | scan(ff)
#        sf = sfunc
#        strpextn(sf) | scan(sf)

# Loop through images
        while (fscan(inimglist,img) != EOF) {

# Strip off any extensions.  This is to avoid conflicts with names which
# might arise with FITS files or if the user has used meta-characters in
# the input file list
           strpextn(img) | scan(img)

# Clobber old output files
           imdelete(img//mstype,ver-, >& "dev$null")
           if (comb) 
              imdelete(img//"C"//mstype,ver-,>& "dev$null")

# If the input file is a fits image, then convert it to an iraf file...
# This will become unnecessary when v2.11 arrives with its FITS image kernel
           if (fcon) {
              print(" Reading FITS image") | tee(log1)
              rfits(img//".fits", "", img, make+, lo-, sh-,
                 datatype="", blank=0.0, scale+, oldir-, offset=0)
           }

# Get the chip name in the isis arm.
           ct = ""; ca = ""
           q_gets(img,"CCDCHIP","") | ucase | scan(ct)
           q_gets(img,"ISIARM","") | ucase | scan(ca)
           if (ct == "" || ca == "") 
              print("Warning: Info on chip name or ISIS arm don't exist in header") | tee (log1)

# Get information about windowing of the chip
           q_getr(img,"CCDXBIN",1.0) | scan(xb)
           q_getr(img,"CCDYBIN",1.0) | scan(yb)

           q_getr(img,"i_naxis1",1.0) | scan(defsizx)           
           q_getr(img,"i_naxis2",1.0) | scan(defsizy)
           q_getr(img,"CCDXIMST",1.0) | scan(xs)
           q_getr(img,"CCDXIMSI",defsizx) | scan(xz)
           q_getr(img,"CCDXSIZE",defsizx) | scan(xsize)

           q_getr(img,"CCDYIMST",1.0) | scan(ys)
           q_getr(img,"CCDYIMSI",defsizy) | scan(yz)
           q_getr(img,"CCDYSIZE",defsizy) | scan(ysize)

           q_gets(img,"CCDWMODE","F") | ucase | scan(ccdwmode)
           if (ccdwmode == "T") { 
              q_getr(img,"CCDWXO1",0.0) | scan(xsw)
              q_getr(img,"CCDWXS1",defsizx) | scan(xzw)
              q_getr(img,"CCDWYO1",0.0) | scan(ysw)
              q_getr(img,"CCDWYS1",defsizy) | scan(yzw)
           } else {
              xsw = 0.0
              xzw = defsizx
              ysw = 0.0
              yzw = defsizy
           }

# Get chip name so that you can trap for vertical dispaxis...
           refim=img
           hselect(refim,"DETECTOR",yes) | scan(chipname)
           if (nscan() == 0) {
               dispaxis = 1
           } else {
               if (chipname >= "EEV11" || chipname <= "EEV16") {
                   dispaxis = 2
               } else {
                   dispaxis = 1
               }
           }

           if (dispaxis == 1) {
               start = max(1.0,real(nint((xs-xsw-1.0)/xb+1.0)))
               endp = min(start+xzw/xb-1.0,start+xz/xb-1.0)
           } else {
               start = max(1.0,real(nint((ys-ysw-1.0)/yb+1.0)))
               endp = min(start+yzw/yb-1.0,start+yz/yb-1.0)
           }

# Get the chip readout noise and gain.  If it doesn't exist, then
# substitute plausible values.
           q_getr(img,"READNOIS",5.0) | scan(rd)
           q_getr(img,"GAIN",1.0) | scan(gn)
           if (rd <= 0.0) {
               rd = 5.0
           } 
           if (gn <= 0.0) {
               gn = 1.0
           }
           print(" ") | tee (log1)
           print(" CCD frame: ",img,"   ISIS: ",ca) | tee (log1)
           print(" ") | tee (log1)
           print(" Chip: ",ct,"   Readnoise = ",rd," Gain = ",gn,) | tee (log1)
           print(" ") | tee (log1)

# Do you want to do CCD processing?
           bproc = bsub
           if ((xs - 6) <= xsw) bproc = no
           fproc = ffield
           if (fproc && !access(ff//imtype)) {
              fproc = no
              print("Warning: can't access flat-field image") | tee(log1)
           }
           procccd = no
           if(fproc || bproc){
              if (!access("backup/"))
                 mkdir("backup/")
              ccdred.verbose = no
              ccdred.backup = "backup/"
              ccdred.logfile = "isis.log"

# Create a bias section
              bxstart = (6 - xsw)/xb
              if (bxstart <= 0) bxstart = 1
              bxend = (xs - 6 - xsw)/xb
              bystart = (101 - ysw)/yb
              if (bystart <= 0) bystart = 1
              byend = (yzw)/yb
              if (byend > (1000/yb)) byend = (1000)/yb
              bsect="["//int(bxstart)//":"//int(bxend)//","//int(bystart)//":"//int(byend)//"]"
              ysamp = int(bystart)//":"//int(byend)
            
# Perform bias subtraction and flat-field if required
              if(bproc)print (" Performing overscan-correction") | tee(log1)
              if(fproc)print (" Performing flat-field division") | tee(log1)
              ccdproc (img,ccdtype=" ", max_cache=0, noproc=no, fixpix=no, 
                 overscan=bproc, trim=no, zerocor=no, darkcor=no, flatcor=fproc, 
                 illumcor=no, fringecor=no, readcor=no, scancor=no, 
                 readaxis="line", fixfile="", biassec=bsect, trimsec="",
                 zero="", dark="", flat=ff, illum="", fringe="", minreplace=1.,
                 scantype="shortscan", nscan=1, interactive=no, function="legendre",
                 order=1, sample=ysamp, naverage=1, niterate=1, low_reject=3., 
                 high_reject=3., grow=0.)
              procccd = yes
           }

# Set airmass and Julian date  - you never know when they might come in handy.....
# However, only do it if header information exists
           if(head){
              setjd (img,observatory="lapalma", date="date-obs", time="utobs", 
                 exposure="exptime",ra="ra", dec="dec", epoch="epoch", 
                 jd="jd", hjd="hjd", ljd="ljd",utdate=yes, uttime=yes,
                 listonly=no, >& "dev$null")
              setairmass (img,observatory="lapalma", intype="beginning", 
                 outtype="effective",date="date-obs", exposure="exptime", 
                 airmass="airmass", utmiddle="utmiddle", show=yes, update=yes, 
                 override=yes, >& "dev$null")
           }

# Remove cosmic rays from the frame to facilitate locating the spectra. Note
# that this is not the frame from which the spectra will be extracted.
           if (crp) {
              refim=mktemp(img)
              print(" Removing Cosmic Rays") | tee (log1)
              imdelete("_fmed*"//imtype, ver-, >& "dev$null")
              imdelete("_cr*"//imtype, ver-, >& "dev$null")
              imdelete("_mask*"//imtype, ver-, >& "dev$null")
              xzap(img,outlist=refim, nsigma=2., xboxsz=15, yboxsz=1, 
                 nobjsigma=0., nrings=0, donegative-,nnegsigma=5.,
                 imgsec="[*,*]", deletemasks+, cleanup+, verbose-,
                 inimglist="", outimglist="", statlist="", >& "dev$null")
              imdelete("_fmed"//img, ver-)
           }


# Trace the spectrum...
           print(" Tracing spectrum") | tee (log1)

           apextract.dispaxis = dispaxis
           apall (refim, output=refim, apertures="", format="multispec", references="",
              nfind=nsp, profiles="", interactive-, find+,
              recenter+, resize+, edit-, trace+, fittrace-,
              extract-, extras-, review-, line=INDEF, nsum=-100, 
              lower=-5., upper=5., apidtable="", b_function="chebyshev", 
              b_order=1, b_sample="-50:-21,21:50", b_naverage=-10, 
              b_niterate=0, b_low_reject=3., b_high_rejec=3., b_grow=0., 
              width=10., radius=10., threshold=0., minsep=5., maxsep=1000., 
              order="increasing", aprecenter="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+,
              r_grow=0., avglimits-, t_nsum=50, t_step=50, t_nlost=3, 
              t_function="legendre", t_order=4, t_sample="*", t_naverage=1, 
              t_niterate=2, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance", pfit="fit1d", 
              clean-, saturation=INDEF, readnoise=rd, gain=gn, lsigma=4., 
              usigma=4., nsubaps=1, > "dev$null")

# Extract the spectrum...
           print(" Extracting spectrum ") | tee (log1)

           apall (img,output=img, apertures="", format="multispec", references=refim,
              nfind=nsp, profiles="", interactive-, find-, recenter-, 
              resize-, edit-, trace-, fittrace-, extract+, extras-, review-, 
              line=INDEF, nsum=10, lower=-5., upper=5., 
              apidtable="", b_function="chebyshev", b_order=1, 
              b_sample="-50:-21,21:50", b_naverage=-10, b_niterate=0, 
              b_low_reject=3., b_high_rejec=3., b_grow=0., width=10., 
              radius=10., threshold=0., minsep=5.,maxsep=1000., 
              order="increasing", aprecenter="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+, 
              r_grow=0., avglimits-, t_nsum=50, t_step=50, t_nlost=3, 
              t_function="legendre", t_order=4, t_sample="*", t_naverage=1, 
              t_niterate=2, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance",pfit="fit1d", 
              clean+, saturation=65000, readnoise=rd, gain=gn, lsigma=3., 
              usigma=3., nsubaps=1, > "dev$null")
 
# Send out a friendly message and clean up the database directory.  Remove
# the cosmic ray processed frame if it exists...
           print(" ") | tee (log1)
           tempname = mktemp("temp")
           type("database/aplast",map+,dev="terminal") | 
              match("begin","",stop-,print+,meta+,>tempname) 
           nfound = 0
           tlist = tempname
           while(fscan(tlist,dumstr,dumstr,dumstr,mm,dumreal,pos) != EOF) {
              if (dispaxis == 1) {
                  print(" Spectrum # ",mm, "found in row ",pos) | tee (log1)
              } else {
                  print(" Spectrum # ",mm, "found in row ",dumreal) | tee (log1)
              }        
              nfound = nfound + 1
           }
           print(" ") | tee (log1)
           tlist = ""
           delete(tempname, ver-, >& "dev$null")
           delete("database/ap"//refim, ver-, >& "dev$null")
           if (crp) imdelete(refim, ver-)

# Create a data section string for use later
           if (nfound > 1) 
              dsect = "["//int(start)//":"//int(endp)//",*]"
           else
              dsect = "["//int(start)//":"//int(endp)//"]"

# If you did CCD processing on this frame, then get rid of the processed
# frame and bring back the original...
           if (procccd) {
              imdelete(img, ver-)
              imrename("backup/"//img//imtype,img,verbose-)
           }

# Chop off the bits you don't want...
           tempname = mktemp("temp")
           outfil = img//mstype
           imrename(outfil,tempname,verbose-, >& "dev$null")
           imcopy(tempname//dsect,outfil,verbose-,>&"dev$null")
           imdelete(tempname,verify-)

# See if some basic wavelength info is in the header. If it isn't then
# you can't wavelength calibrate...
           q_getr(img,"CENWAVE",-1.0) | scan(cw)
           q_getr(img,"DISPERSI",0.0) | scan(app)
           lcal2 = lcal
           if (cw == -1.0 || app == 0.0) {
              print("Dispersion information absent from header") | tee(log1)
              lcal2 = no
           }

# If you want to do a wavelength calibration then decide where on the chip
# the spectrum lies...
           if(lcal2){
              if (dispaxis == 1) {
                  start = max(xs,xsw)
                  endp = min(xs+xz-1,xsw+xzw-1)
              } else {
                  start = max(ys,ysw)
                  endp = min(ys+yz-1,ysw+yzw-1)
              }
   
# Get the central wavelength and dispersion.  Also sort out which arm/chip
# combinations reverse the sense of the dispersion (e.g. red to the left).
              print(" Applying wavelength scale from header") | tee (log1)
              print(" ") | tee (log1)
              q_getr(img,"CCDXPIXE",0.0) | scan(mpp)
              mpp = mpp*0.001
              if (mpp == 0.0) {
                 if (substr(ct,1,3) == "TEK") 
                    mpp=0.024
                 else if (substr(ct,1,5) == "EEV42") 
                    mpp=0.0135
                 else if(substr(ct,1,3) == "EEV")
                    mpp=0.0225
                 else
                    mpp=0.024
              }
              app=app*mpp
              if (ca == "BLUE ARM") 
                 if (ct == "TEK2" || ct == "EEV6" || ct == "TEK5") app = -app
              else if (ca == "RED ARM")
                 if (ct == "TEK1" || ct == "EEV3") app = -app
              if (dispaxis == 1) {
                  stw = cw - (xsize*0.5 - start)*app
              } else {
                  stw = cw - (ysize*0.5 - start)*app
              }
              app=xb*app
              print(" Start wavelength ",stw," Angstroms/pix ",
                 app," Binning factor ",xb) | tee (log1)
              print(" ")  | tee (log1)

# Edit the WCS to include the wavelength information
              wcsreset(outfil,wcs="physical",verbose-)
              sapertures(outfil,apertures="",apidtable="",wcsreset=no,
                 verbose-,beam=INDEF,dtype=0,w1=stw,dw=app,z=INDEF,
                 aplow=INDEF,aphigh=INDEF,title="INDEF")
              dispcor(outfil,outfil,linearize+,database="",table="",
                 w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,log-,flux+,samedisp+,
                 global-,ignoreaps-,confirm-,listonly-,verbose-,logfile=log1)

# If not doing a wavelength calibration, then reset the WCS so that logical
# and physical coordinates match...
           } else {
              wcsreset(outfil,wcs="physical",verbose-)
           }


# Perform flux calibration if required....
#           if (fcal) 
#              calibrate (outfil,outfil, extinct=yes, flux=yes, 
#                 extinction="rgo$flux/lpoext.tab",observatory="lapalma", 
#                 ignoreaps=yes, sensitivity=sf, fnu=no)
           
# Block average the result if you want...
           if (iblk > 1)
              blkavg(outfil,outfil,iblk,1,1,1,1,1,1,option="average")

# If this came originally from a FITS file, then delete the IRAF file...
           if (fcon) imdelete(img, ver-, >& "dev$null")
        }

# If combining spectra, then do that now...
        if (comb) {
           inimglist = infile
           instring = ""
           while (fscan(inimglist,img) != EOF) {
              strpextn(img) | scan(img)
              instring = instring//img//".ms,"
           }
           tempname = substr(instring,1,strlen(instring)-1)
           inimglist = infile
           junk = fscan(inimglist,outfil)
           dumstr = outfil//"C"
           scombine(input=tempname,output=outfil,noutput="",apertures="",
              logfile="STDOUT",group="apertures",combine="average",
              reject="none", first=no, w1=INDEF, w2=INDEF, dw=INDEF,
              nw=INDEF, scale="none", zero="none", weight="none",
              sample="",lthreshold=INDEF, hthreshold=INDEF, nlow=1, nhigh=1,
              nkeep=1, mclip-,>& "dev$null")
        }

# Tidy and exit
        delete (infile, ver-, >& "dev$null")
        inimglist = ""
end
