procedure fues(image)

# Script to locate and extract UES data from raw frames.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'pues.cl'.  Both scripts are called from the task 'ues'
# in the package 'rgo.quickie' and it is recommended that you use the
# ues task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 
# 
# Updates:
#      May 1995 -- Added bias subtraction, flat fielding and a check
#                  for header information.  Also fixed a number of
#                  bugs.  JRL and BJB

string  image   {prompt="Input image(s)"}
int     nspec   {1,prompt="Number of orders on image?"}
int     blkfact {1,min=1,prompt="Blocking factor in spectral domain"}
bool    fconv   {no,prompt="Convert from FITS?"}
bool    crproc  {yes,prompt="Remove cosmic rays for extraction?"}
bool    lcal    {yes,prompt="Wavelength calibrate?"}
bool    comb    {no,prompt="Combine all spectra into one?"}
bool    head    {yes,prompt="Does header information exist?"}
bool    bsub    {yes,prompt="Perform bias subraction"}
bool    ffield  {no,prompt="Perform flat field"}
string  fframe  {"",prompt="Flat field name"}
#string  sfunc   {"",prompt="Name of sensitivity function?"}
string  apref   {"",prompt="Name of aperture reference file"}

struct  *inimglist
struct  *tlist

begin

        string  dsect           # data section
        string  det             # detector name
        string  ech             # echelle name
        string  imglist         # equals image
        string  imtype          # image type
        string  infile          # temporary list for files
        string  img             # image name from fscan with extension stripped
        string  refim           # reference image for spectral tracing
        string  outfil          # dummy string
        string  tempname        # dummy string
        string  dumstr          # dummy string
        string  tempname2       # dummy string
        string  tabfil          # table file with echelle info
        string  swap            # echelle order in frame
        string  ccdwmode        # windowing mode for ccd
        string  ysamp           # sampling window for fit to bias strip
        string  ccdpfile        # list of files for ccd processing
        string  bsect           # bias strip image section
        string  msg             # message string
        file    oldref          # old name of aperture reference
        file    log1            # name of log file
        file    ff              # name of input flat field image
        file    apr             # name of input aperture reference image
        real    cent            # centre of Y window
        real    cw              # central wavelegnth in header
        real    ccw             # central wavelegnth of central order
        real    cwaveues        # central wavelength for ues in central order
        real    endp            # end pixel for wavelength calibration
        real    endw            # end wavelength
        real    app             # dispersion A/pix
        real    start           # starting pixel for wavelength calibration
        real    stw             # start wavelength
        real    rd              # CCD readnoise
        real    gn              # CCD gain
        real    xs,xz,xsw,xzw   # X dimensions of chip/window
        real    ys,yz,ysw,yzw   # Y dimensions of chip/window
        real    xsize           # X size of chip
        real    dsize,ssize     # Dispersion and spatial sizes of detector
        real    xb,yb,db        # binning factor (x,y,dispersion)
        real    ysize           # Y size of chip
        real    dumreal         # dummy real
        real    poff            # pixel offset
        real    pos,pos1,pos2   # spectrum positions
        real    minsep          # minimum distance between order and y centre
        real    pixsiz          # X length of pixel in microns
        real    y1,y2,diff      # dummy real variables
        real    dy              # Y length of pixel in microns
        real    bxstart,bxend   # X dims of bias strip
        real    bystart,byend   # Y dims of bias strip
        real    isign           # sign for dispersion
        int     defsizx,defsizy # a default size
        int     npred           # number of spectra predicted
        int     mm              # dummy
        int     mrem            # spectrum number closest to y centre
        int     npl             # number of plots on a page
        int     nfound          # number of spectra actually found
        int     cord            # central order
        int     stord           # starting order 
        int     len             # string length
        int     ord,ord1        # orders
        int     row,row1        # rows in standard file
        int     nsp             # number of spectra
        int     nsum            # number of rows to sum up for finding spectra
        int     iblk            # blocking factor in spectral domain
        int     tord            # trace order
        int     nimtype         # number of charcters in image type extension
        int     i               # number of characters in a string
        bool    crp             # TRUE if taking out cosmic rays
        bool    combine         # TRUE if combining all output spectra into one
        bool    bproc           # TRUE if doing bias subtraction
        bool    fproc           # TRUE if doing flat fielding
        bool    procccd         # TRUE if ccd processing has been done
        bool    already         # TRUE if apref is already traced
        bool    fcon            # TRUE if converting from FITS

# Get query parameter
        imglist = image

# Expand file lists into temporary files.
        infile =  mktemp("temp")
        sections (imglist,option="fullname",>infile)
        inimglist = infile
        nsp = nspec
        fcon = fconv

# Set up logging
        tee.append = yes
        log1 = "ues.log"

# Get file name extension
        imtype = "."//envget("imtype")
        nimtype = strlen(imtype)

# Strip extension from flat field image and sensitivity function
# (if either exist...)
        ff = fframe
        i = strlen(ff)
        if (i > nimtype)
           if (substr(ff,i-nimtype,i) == imtype)
              ff = substr(ff,1,i-nimtype)
        apr = apref
        i = strlen(apr)
        if (i > nimtype)
           if (substr(apr,i-nimtype,i) == imtype)
              apr = substr(apr,1,i-nimtype)
#        sf = sfunc
#        i = strlen(sf)
#        if (i > nimtype)
#           if (substr(sf,i-nimtype,i) == imtype)
#              sf = substr(sf,1,i-nimtype)

# Check to see if aperture reference is already traced.
        if (access("database/ap"//apr))
           already = yes
        else
           already = no

# Cache the command 'keypar' as protection in case you are running this
# script in the background...
        cache("keypar")

# Loop through images
        while (fscan(inimglist,img) != EOF) {

# Strip off any extensions.  This is to avoid conflicts with names which
# might arise with FITS files or if the user has used meta-characters in
# the input file list
           i = strlen(img)
           if (i > nimtype)
              if (substr(img,i-nimtype,i) == imtype)
                 img = substr(img,1,i-nimtype)

# Clobber old output files
           imdel(img//".0*"//imtype, go+, ver-, def+, >& "dev$null")
           imdel(img//"C"//imtype, go+, ver-, def+, >& "dev$null")

# If the input file is a fits image, then convert it to an iraf file...
# This will become unnecessary when v2.11 arrives with its FITS image kernel
           if (fcon) {
              print(" Reading FITS image") | tee(log1)
              rfits(img//".fits", "", img, make+, lo-, sh-,
                 datatype="", blank=0.0, scale+, oldir-, offset=0)
              if (!access(apr//imtype))
                 rfits(apr//".fits", "", img, make+, lo-, sh-,
                    datatype="", blank=0.0, scale+, oldir-, offset=0)
           }

# Get the name of the detector and the echelle
           hselect(img,"DETECTOR",yes) | scan(det)
           if (nscan() == 0) det = ""
           hselect(img,"UESECHEL",yes) | scan(ech)
           if (nscan() == 0) ech = ""
           if (ech == "" || det == "")
              error(0,"No header information on detector or echelle")

# If this is the IPCS, then set up the sizes of the windows...
           if (det == "IPCS") {
              xs = 1.0
              hselect(img,"i_naxis1,i_naxis2",yes) | scan(xz,yz)
              xsize = xz
              xsw = 0.0
              xzw = xz
              xb = 1.0
              ysize = yz
              yb = 1.0
              dsize = xsize
              ssize = ysize
              db = xb
           } else {
              hselect(img,"CCDXBIN",yes) | scan(xb)
              if (nscan() == 0) xb = 1.0
              hselect(img,"CCDYBIN",yes) | scan(yb)
              if (nscan() == 0) yb = 1.0         

              hselect(img,"i_naxis1,i_naxis2",yes) | scan(defsizx,defsizy)
              hselect(img,"CCDXIMST",yes) | scan(xs)
              if (nscan() == 0) xs = 1.0
              hselect(img,"CCDXIMSI",yes) | scan(xz)
              if (nscan() == 0) xz = defsizx
              hselect(img,"CCDXSIZE",yes) | scan(xsize)
              if (nscan() == 0) xsize = defsizx

              hselect(img,"CCDYIMST",yes) | scan(ys)
              if (nscan() == 0) ys = 1.0
              hselect(img,"CCDYIMSI",yes) | scan(yz)
              if (nscan() == 0) yz = defsizy
              hselect(img,"CCDYSIZE",yes) | scan(ysize)
              if (nscan() == 0) ysize = defsizy

              hselect(img,"CCDWMODE",yes) | scan(ccdwmode)
              if (nscan() == 0) ccdwmode = "F"
              if (ccdwmode == "T") { 
                 hselect(img,"CCDWXO1",yes) | scan(xsw)
                 if (nscan() == 0) xsw = 0.0
                 hselect(img,"CCDWXS1",yes) | scan(xzw)
                 if (nscan() == 0) xzw = defsizx
                 hselect(img,"CCDWYO1",yes) | scan(ysw)
                 if (nscan() == 0) ysw = 0.0
                 hselect(img,"CCDWYS1",yes) | scan(yzw)
                 if (nscan() == 0) yzw = defsizy
              } else {
                 xsw = 0.0
                 xzw = defsizx
                 ysw = 0.0
                 yzw = defsizy
              }
              dsize = ysize/yb
              ssize = xsize/xb
              db = yb
           }

# Create a data section string for use later
           if (det == "IPCS") {
              start = 1.0
              endp = xz
              dsect = "[*]"
           } else {
              start = max(1.0,real(nint((ys-ysw-1.0)/yb+1.0)))
              endp = min(start+yzw/yb-1.0,start+yz/yb-1.0)
              if (start == 1.0 && endp == yzw)
                 dsect = "[*]"
              else 
                 dsect = "["//int(start)//":"//int(endp)//"]"
           }

# Get the chip readout noise and gain.  If it doesn't exist, then
# substitute plausible values.  Substitute the right values for IPCS.
           if (det == "IPCS") {
              rd = 0.0
              gn = 1.0
           } else {
              hselect(img,"READNOIS",yes) | scan(rd)
              if (nscan() == 0) rd = 5.0
              hselect(img,"GAIN",yes) | scan(gn)
              if (nscan() == 0) gn = 1.0
           }
           print(" ") | tee(log1)
           print(" UES frame: ",img, "   Echelle: ",ech) | tee(log1)
           print(" ") | tee(log1)
           print(" Detector: ",det,"   Readnoise = ",rd," Gain = ",gn,) | tee(log1)
           print(" ") | tee(log1)

           if (det == "IPCS") 
              swap="increasing"
           else if (det == "TEK1") 
              swap = "decreasing"
           else if (det == "TEK2") 
              swap = "increasing"
           else if (det == "TEK5") 
              swap = "increasing"
           else if (det == "EEV3")
              swap = "decreasing"
           else
              swap = "increasing"
         
# Do you want to do CCD processing?
           if (det != "IPCS") {
              bproc = bsub
              if ((xs - 6) <= xsw) bproc = no
              fproc = ffield
              if (fproc && !access(ff//imtype)) {
                 fproc = no
                 print("Warning: can't access flat-field image") | tee(log1)
              }
              procccd = no
              if(fproc || bproc){
                 if (!access("backup/"))
                    mkdir("backup/")
                 ccdred.verbose = no
                 ccdred.backup = "backup/"
                 ccdred.logfile = log1

# Create a bias section
                 bxstart = (6 - xsw)/xb
                 if (bxstart <= 0) bxstart = 1
                 bxend = (xs - 6 - xsw)/xb
                 bystart = (101 - ysw)/yb
                 if (bystart <= 0) bystart = 1
                 byend = (yzw)/yb
                 if (byend > (1000/yb)) byend = (1000)/yb
                 bsect="["//int(bxstart)//":"//int(bxend)//","//int(bystart)//":"//int(byend)//"]"
                 ysamp = int(bystart)//":"//int(byend)
            
# Perform bias subtraction and flat-field if required
                 if(bproc)print (" Performing overscan-correction") | tee(log1)
                 if(fproc)print (" Performing flat-field division") | tee(log1)
                 ccdpfile = img
                 if (access(apr//imtype) && img != apr)
                    ccdpfile = ccdpfile//","//apr
                 ccdproc (ccdpfile,ccdtype=" ", max_cache=0, noproc=no, fixpix=no, 
                    overscan=bproc, trim=no, zerocor=no, darkcor=no, flatcor=fproc, 
                    illumcor=no, fringecor=no, readcor=no, scancor=no, 
                    readaxis="line", fixfile="", biassec=bsect, trimsec="",
                    zero="", dark="", flat=ff, illum="", fringe="", minreplace=1.,
                    scantype="shortscan", nscan=1, interactive=no, function="legendre",
                    order=1, sample=ysamp, naverage=1, niterate=1, low_reject=3., 
                    high_reject=3., grow=0.)
                 procccd = yes
              }
           } else 
              procccd = no

# Set airmass and Julian date  - you never know when they might come in handy.....
# However, only do it if header information exists
           if(head){
              setjd (img,observatory="lapalma", date="date-obs", time="utobs", 
                 exposure="exptime",ra="ra", dec="dec", epoch="epoch", 
                 jd="jd", hjd="hjd", ljd="ljd",utdate=yes, uttime=yes,
                 listonly=no, >& "dev$null")
              setairmass (img,observatory="lapalma", intype="beginning", 
                 outtype="effective",date="date-obs", exposure="exptime", 
                 airmass="airmass", utmiddle="utmiddle", show=yes, update=yes, 
                 override=yes, >& "dev$null")
           }

# Remove cosmic rays from the frame to facilitate locating the spectra. Note
# that this is not the frame from which the spectra will be extracted.
           if (access(apr//imtype))
              refim = apr
           else
              refim=img
           if (det == "IPCS" || already)
              crp = no
           else 
              crp = crproc
           if (crp) {
              oldref=mktemp(refim)
              imrename(refim,oldref,verbose-)
              print(" Removing Cosmic Rays") | tee(log1)
              imdelete("_fmed*"//imtype, ver-, >& "dev$null")
              imdelete("_cr*"//imtype, ver-, >& "dev$null")
              imdelete("_mask*"//imtype, ver-, >& "dev$null")
              xzap(oldref,outlist=refim, nsigma=2., xboxsz=1, yboxsz=15, 
                 nobjsigma=0., nrings=0, donegative-,nnegsigma=5.,
                 imgsec="[*,*]", deletemasks+, cleanup+, verbose-,
                 inimglist="", outimglist="", statlist="", >& "dev$null")
              imdel("_fmed"//oldref, go+, ver-, def+)
           }

# Find central order and decide a minimum distance between orders...
           hselect(img,"UESCORDE",yes) | scan(cord)
           if (nscan() == 0) {
              print("Warning: No info on central order") | tee(log1)
              cord = 0
           }
           len = strlen(ech)
           tabfil = "quickie$stdfiles/ues"//substr(ech,2,len)
           keypar(tabfil,"ordst")
           ord1 = int(keypar.value)
           row = cord - ord1 + 1
           tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
              columns="CDCEN",rows=row,pwidth=80) | scan(y1)
           tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
              columns="CDCEN",rows=row+1,pwidth=80) | scan(y2)
           if (det == "IPCS") 
              hselect(img,"UESPIXL",yes) | scan(dy)
           else 
              hselect(img,"UESPIXW",yes) | scan(dy)
           if (nscan() == 0) {
              print("Warning: No info on order size in header") | tee(log1)
              dy = 1.0
           }
           npred = int(ssize*dy/(abs(y1-y2)*1000.0))
           row1 = min(1,row-npred/2)
           minsep = ssize
           for (row=row1; row<=row1+npred-2; row+=1) {
              tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                 columns="CDCEN",rows=row,pwidth=80) | scan(y1)
              tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                 columns="CDCEN",rows=row+1,pwidth=80) | scan(y2)
              diff = abs(y1 - y2)*1000.0/dy
              if (diff < minsep) minsep = diff
           }
           minsep = 0.9*minsep/xb
           nsum = int(0.25*dsize)
              
# Trace the spectrum...
           print(" Tracing spectra") | tee(log1)

           if (det == "IPCS") {
              apextract.dispaxis = 1
              tord = 6
           } else {
              apextract.dispaxis = 2
              tord = 3
           }
           if (!already) {
              apall (refim, output="", apertures="", format="onedspec", references="",
                 nfind=nsp, profiles="", interactive-, find+,
                 recenter+, resize+, edit-, trace+, fittrace-,
                 extract-, extras-, review-, line=INDEF, nsum=nsum, 
                 lower=-5., upper=5., apidtable="", b_function="chebyshev", 
                 b_order=1, b_sample="-10:-6,6:10", b_naverage=-10, 
                 b_niterate=1, b_low_reject=3., b_high_rejec=3., b_grow=0., 
                 width=10., radius=10., threshold=0., minsep=minsep, 
                 maxsep=1000.0, order=swap, aprecenter="", npeaks=nsp, shift+,
                 llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+,
                 r_grow=0., avglimits-, t_nsum=20, t_step=20, t_nlost=3, 
                 t_function="legendre", t_order=tord,t_sample="*",t_naverage=1,
                 t_niterate=3, t_low_reject=2., t_high_rejec=2., t_grow=0., 
                 background="median", skybox=1, weights="none", pfit="fit1d", 
                 clean-, saturation=INDEF, readnoise=rd, gain=gn, lsigma=4., 
                 usigma=4., nsubaps=1, > "dev$null")
           }

# Extract the spectrum...
           print(" Extracting spectra ") | tee(log1)

           apall (img,output="", apertures="", format="onedspec", references=refim,
              nfind=nsp, profiles="", interactive-, find-, recenter-, 
              resize-, edit-, trace-, fittrace-, extract+, extras-, review-, 
              line=INDEF, nsum=nsum, lower=-5., upper=5., 
              apidtable="", b_function="chebyshev", b_order=1, 
              b_sample="-10:-6,6:10", b_naverage=-10, b_niterate=0, 
              b_low_reject=3., b_high_rejec=3., b_grow=0., width=10., 
              radius=10., threshold=0., minsep=minsep, maxsep=1000., 
              order=swap, aprecenter="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+, 
              r_grow=0., avglimits-, t_nsum=20, t_step=20, t_nlost=3, 
              t_function="legendre", t_order=tord, t_sample="*", t_naverage=1,
              t_niterate=3, t_low_reject=2., t_high_rejec=2., t_grow=0., 
              background="median", skybox=1, weights="variance",pfit="fit1d", 
              clean+, saturation=INDEF, readnoise=rd, gain=gn, lsigma=2.5, 
              usigma=2.5, nsubaps=1, > "dev$null")
 
# Send out a friendly message and clean up the database directory.  Remove
# the cosmic ray processed frame if it exists...Also find out which spectrum
# is the central order...
           print(" ") | tee(log1)
           tempname = mktemp("temp")
           type("database/aplast",map+,dev="terminal") | 
              match("begin","",stop-,print+,meta+,>tempname) 
           minsep = ssize
           cent = ssize*0.5
           tlist = tempname
           nfound = 0
           if (det == "IPCS") 
              msg = "row"
           else
              msg = "column"
           while(fscan(tlist,dumstr,dumstr,dumstr,mm,pos1,pos2) != EOF) {
              nfound = nfound + 1
              if (apextract.dispaxis == 1)
                 pos = pos2
              else
                 pos = pos1
              printf(" Spectrum #%d found in %s %7.2f\n",mm,msg,pos) | tee(log1)
              if (abs(pos-cent) < minsep) {
                 minsep = abs(pos-cent)
                 mrem = mm
              }
           }
           if (nfound != nsp) 
              print("Warning: Only ",nfound," spectra found") | tee(log1)
           print(" ") | tee(log1)
           tlist = ""
           delete(tempname, go+, ver-, def+, all+, sub+, >& "dev$null")
           if (crp) {
              imdel(refim, go+, ver-, def+)
              imrename(oldref,refim,verbose-)
           }

# If you did CCD processing on thess frames, then get rid of the processed
# frames and bring back the originals...
           if (procccd) {
              imdelete(ccdpfile, ver-)
              imrename("backup/*"//imtype,".",verbose-)
           }

# Chop off the bits you don't want...
          tempname = mktemp("temp")
          if (dsect != "[*]") {
             for (mm=1; mm<= nfound; mm+=1) {
                names(img,str(mm),append="",check-) | scan(outfil)
                imrename(outfil,tempname,ver-, >& "dev$null")
                imcopy(tempname//dsect,outfil,ver-, >& "dev$null")
                imdelete(tempname, go+, ver-, def+)
             }
           }

# Find the starting order number
           stord = cord + mrem - 1

# If you want to do a wavelength calibration then decide where on the chip
# the spectrum lies...
           if(lcal){
              if (det == "IPCS") {
                 start = max(xs,xsw)
                 endp = min(xs+xz-1,xsw+xzw-1)
                 isign = 1.0
              } else {
                 start = max(ys,ysw)
                 endp = min(ys+yz-1,ysw+yzw-1)
                 if (substr(det,1,3) == "TEK")
                    isign = -1.0
                 else
                    isign = 1.0
              }

# Find wavelength centre
              row1 = stord - ord1 + 1
              print(" Applying wavelength scale ") | tee(log1)
              print(" ") | tee(log1)
              hselect(img,"UESPIXW,UESCWAVE,UESDISPX",yes) | scan(pixsiz,
                 cwaveues,app)
              if (nscan() != 3) {
                 print("Warning: Wavelength information not in header") | tee(log1)
                 next
              }
              app = isign*app*db 
              tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                 columns="WAVCENTR",rows=row1-mrem+1,pwidth=80) | scan(ccw)
              poff = (cwaveues - ccw)/app

# Now loop for all the spectra...
              for (mm=1; mm<=nfound; mm+=1) {
                 row = row1 - mm + 1
                 ord = stord - mm + 1

# Get the wavelength info from the standard table...
                 tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                    columns="WAVCENTR",rows=row,pwidth=80) | scan(cw)
                 tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                    columns="LD",rows=row,pwidth=80) | scan(app)
                 app = isign*app*pixsiz*0.001
                 stw = cw - (0.5*(endp-start) - poff)*app
                 printf(" Spectrum: %d  Order: %d ",mm,ord) | tee(log1)
                 printf("  Start wavelength %9.3f  Dispersion %6.4f\n",
                    stw,app) | tee(log1)
                 names(img,str(mm),append="",check-) | scan(outfil)

# Edit the WCS to include the wavelength information and order number
                 wcsedit(outfil,"cd",app,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crval",stw,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crpix",1,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"units","angstroms",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"label","wavelength",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 hedit(outfil,"ordnum",ord,add+,del-,ver-,show-,upd+,
                    >& "dev$null")
              }

# If not doing a wavelength calibration, then reset the WCS so that logical
# and physical coordinates match...
           } else {
              for (mm=1; mm<=nfound; mm+=1) {
                 names(img,str(mm),append="",check-) | scan(outfil)
                 wcsedit(outfil,"cd",1.0,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crval",1.0,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crpix",1,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"label","X",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"units","pixels",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
              }
           }

# Do you want the spectra block averaged?  If so, then do that here...
           iblk = blkfact
           if (iblk > 1) {
              for (mm=1; mm<=nfound; mm+=1) {
                 names(img,str(mm),append="",check-) | scan(outfil)
                 blkavg(outfil,outfil,iblk,1,1,1,1,1,1,option="average")
              }

# If this started out as a FITS file, then delete the IRAF file (but
# only for the image and not the aperture reference.
           if (fcon) imdelete(img, ver-, >& "dev$null")
           }

# Do you want to combine the spectra?
           combine = comb
           if (combine) {
              scomb(img//".0*"//imtype,output=img//"C"//imtype,noutput="",
                 logfile="STDOUT",apertures="",group="all",reject="none",
                 first-,w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,log-,scale="none",
                 zero="none",weight="none",sample="",lthreshold=INDEF,
                 hthreshold=INDEF,nlow=1,nhigh=1,nkeep=1,mclip-,lsigma=3.0,
                 hsigma=3.0,rdnoise="0.0",gain="1.0",snoise="0.0",
                 sigscale=0.1,pclip=-0.5,grow=0,blank=0.0, >& "dev$null")
           }
        }

# Tidy and exit
        delete (infile, go+, ver-, def+, all+, sub+)
        inimglist = ""
end
