procedure findfos2(image)

# Script to locate and extract FOS2 data from raw frames.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'plotfos2.cl'.  Both scripts are called from the task 'fos2'
# in the package 'rgo.quickie' and it is recommended that you use the
# fos2 task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 

string  image   {prompt="Input image(s)"}
bool    sec     {no,prompt="Extract 2nd order as well?"}
bool    fconv   {no,prompt="Convert from FITS?"}
bool    crproc  {yes,prompt="Remove cosmic rays for extraction?"}
bool    lcal    {yes,prompt="Wavelength calibrate?"}
bool    comb    {yes,prompt="Combine first and second order spectra?"}


struct  *inimglist
struct  *tlist

begin

        string  dsect           # data section
        string  imglist         # equals image
        string  imtype          # image type
        string  infile          # temporary list for files
        string  img             # image name from fscan with extension stripped
        string  imgi            # image name from fscan
        string  ct              # type of chip
        string  ca              # isis arm
        string  refim           # reference image for spectral tracing
        string  outfil          # dummy string
        string  tempname        # dummy string
        string  dumstr          # dummy string
        string  plfil           # file for plotting
        real    cw              # central wavelegnth
        real    defsiz          # default size in case header items are missing
        real    endp            # end pixel for wavelength calibration
        real    app             # dispersion A/pix
        real    mpp             # mm/pix
        real    start           # starting pixel for wavelength calibration
        real    stw             # start wavelength
        real    rd              # CCD readnoise
        real    gn              # CCD gain
        real    ys,yz,ysw,yzw   # X dimensions of chip/window
        real    ysize           # X size of chip
        real    xb,yb           # binning factor
        real    dumreal         # dummy real
        real    pos             # spectrum position
        int     mm              # dummy
        int     nspec           # Number of spectra to extract
        bool    second          # TRUE if extracting second order
        bool    combine         # TRUE if combining first and second order
        bool    fcon            # TRUE if converting from FITs
        bool    crp             # TRUE if processing cosmic rays

# Get query parameter
        imglist = image

# Decide whether to do the second order or not...
        second = sec 
        if (second)
           nspec = 2
        else
           nspec = 1
        combine = no
        if (second) combine = comb

# Get a few more parameters
        fcon = fconv
        crp = crproc

# Expand file lists into temporary files.
        infile =  mktemp("temp")
        sections (imglist,option="fullname",>infile)
        inimglist = infile
        imtype = envget("imtype")

# Loop through images
        while (fscan(inimglist,imgi) != EOF) {

# Strip off any extensions.  This is to avoid conflicts with names which
# might arise with FITS files or if the user has used meta-characters in
# the input file list
           fileroot(imgi,val-)
           img = fileroot.root

# Clobber old output files
           imdel(img//".0*."//imtype, go+, ver-, def+, >& "dev$null")
           if (combine) imdel(img//"C."//imtype, go+, ver-, def+,
               >& "dev$null")

# If the input file is a fits image, then convert it to an iraf file...
# This will become unnecessary when v2.11 arrives with its FITS image kernel
           if (fcon) {
              print(" Reading FITS image")
              print(" Reading FITS image",>>"fos2.log")
              rfits(img//".fits", "", img, make+, lo-, sh-,
              datatype="", blank=0.0, scale+, oldir-, offset=0)
           }

# Get the chip name in the isis arm.
           imgets (img,"CCDCHIP")
           ct=(imgets.value)
           imgets (img,"ISIARM")
           ca=(imgets.value)
           if (ct == "0" || ca == "0") {
              print("Warning: Info on chip name or ISIS arm don't exist in header")
              print("Warning: Info on chip name or ISIS arm don't exist in header",>>"fos2.log")
           }
           if (ca != "FOS" && ca != "0")
              error(0,"This isn't a FOS frame!")
           
# Get information about windowing of the chip
         
           imgets(img, "i_naxis2", >& "dev$null")
           defsiz = real(imgets.value)
           imgets (img, "CCDYIMST", >& "dev$null")
           ys=real(imgets.value)
           if (ys == 0.0) ys = 1.0
           imgets (img, "CCDYIMSI", >& "dev$null")
           yz=real(imgets.value) 
           if (yz == 0.0) yz = defsiz
           imgets (img, "CCDYSIZE", >& "dev$null")
           ysize=real(imgets.value)
           if (ysize == 0.0) ysize = defsiz

           imgets (img, "CCDWYO1", >& "dev$null")
           ysw=real(imgets.value)
           imgets (img, "CCDWYS1", >& "dev$null")
           yzw=real(imgets.value)
           if (yzw == 0.0) yzw = defsiz
           imgets (img, "CCDXBIN", >& "dev$null")
           xb=real(imgets.value)
           if (xb == 0.0) xb = 1.0
           imgets (img, "CCDYBIN", >& "dev$null")
           yb=real(imgets.value)
           if (yb == 0.0) xb = 1.0
           start = max(1.0,ys-ysw)
           endp = min(start+yzw-1.0,start+yz-1.0)

# Create a data section string for use later
           dsect = "["//int(start)//":"//int(endp)//"]"

# Get the chip readout noise and gain.  If it doesn't exist, then
# substitute plausible values.
           imgets (img, "READNOISE", >& "dev$null")
           rd=real(imgets.value)
           imgets (img, "GAIN", >& "dev$null")
           gn=real(imgets.value) 
           if(rd ==  0)rd=5
           if(gn ==  0)gn=1
           print(" ")
           print(" CCD frame: ",img,"   FOS: ",ca)
           print(" ")
           print(" Chip: ",ct,"   Readnoise = ",rd," Gain = ",gn,)
           print(" ")
           print(" ",>>"fos2.log")
           print(" CCD frame: ",img,"   FOS: ",ca,>>"fos2.log")
           print(" ",>>"fos2.log")
           print(" Chip: ",ct,"   Readnoise = ",rd," Gain = ",gn,,>>"fos2.log")
           print(" ",>>"fos2.log")

# Remove cosmic rays from the frame to facilitate locating the spectra. Note
# that this is not the frame from which the spectra will be extracted.
           refim=img
           if (crp) {
              refim=mktemp(img)
              print(" Removing Cosmic Rays")
              print(" Removing Cosmic Rays",>>"fos2.log")
              xzap(img,outlist=refim, nsigma=2., xboxsz=1, yboxsz=15, 
                 nobjsigma=0., nrings=0, donegative-,nnegsigma=5.,
                 imgsec="[*,*]", deletemasks+, cleanup+, verbose-,
                 inimglist="", outimglist="", statlist="", >& "dev$null")
              imdel("_fmed"//img, go+, ver-, def+)
           }

# Trace the spectrum...
           print(" Tracing spectrum")
           print(" Tracing spectrum",>>"fos2.log")

           apextract.dispaxis = 2
           apall (refim, output=refim, format="onedspec", references="",
              nfind=nspec, profiles="", interactive-, find+,
              recenter+, resize-, edit-, trace+, fittrace-,
              extract-, extras-, review-, line=150, nsum=20, 
              lower=-5., upper=5., apidtable="", b_function="chebyshev", 
              b_order=1, b_sample="-12:-7,7:12", b_naverage=-10, 
              b_niterate=0, b_low_reject=3., b_high_rejec=3., b_grow=0., 
              width=10., radius=10., threshold=0., minsep=30., maxsep=200., 
              order="increasing", apertures="", npeaks=nspec, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+,
              r_grow=0., avglimits-, t_nsum=10, t_step=10, t_nlost=1, 
              t_function="legendre", t_order=8, t_sample="*", t_naverage=1, 
              t_niterate=3, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance", pfit="fit2d", 
              clean-, saturation=INDEF, readnoise=rd, gain=gn, lsigma=4., 
              usigma=4., nsubaps=1, > "dev$null")

# Extract the spectrum...
           print(" Extracting spectrum ")
           print(" Extracting spectrum ",>>"fos2.log")

           apall (img,output=img, format="onedspec", references=refim,
              nfind=nspec, profiles="", interactive-, find-, recenter-, 
              resize-, edit-, trace-, fittrace-, extract+, extras-, review-, 
              line=150, nsum=20, lower=-5., upper=5., 
              apidtable="", b_function="chebyshev", b_order=1, 
              b_sample="-12:-7,7:12", b_naverage=-10, b_niterate=0, 
              b_low_reject=3., b_high_rejec=3., b_grow=0., width=10., 
              radius=10., threshold=0., minsep=30.,maxsep=200., 
              order="increasing", apertures="", npeaks=nspec, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+, 
              r_grow=0., avglimits-, t_nsum=10, t_step=10, t_nlost=1, 
              t_function="legendre", t_order=8, t_sample="*", t_naverage=1,
              t_niterate=3, t_low_reject=3, t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance",pfit="fit2d", 
              clean+, saturation=65000, readnoise=rd, gain=gn, lsigma=3., 
              usigma=3., nsubaps=1, > "dev$null")
 
# Send out a friendly message and clean up the database directory.  Remove
# the cosmic ray processed frame if it exists...
           print(" ")
           print(" ",>>"fos2.log")
           tempname = mktemp("temp")
           tlist = tempname
           type("database/aplast",map+,dev="terminal") | 
              match("begin","",stop-,print+,meta+,>tempname) 
           while (fscan(tlist,dumstr,dumstr,dumstr,mm,pos,dumreal) != EOF) {
              print(" Spectrum # ",mm, "found in column ",pos)
              print(" Spectrum # ",mm, "found in column ",pos,>>"fos2.log")
           }
           print(" ")
           print(" ",>>"fos2.log")
           tlist = ""
           delete(tempname, go+, ver-, def+, all+, sub+, >& "dev$null")
           delete("database/ap"//refim, go+, ver-, def+, all+, sub+,
              >& "dev$null")
           if (crp) imdel(refim, go+, ver-, def+)

# Chop off the bits you don't want...
           for (mm=1; mm<= nspec; mm+=1) {
              names(img,str(mm),append="",check-) | scan(outfil)
              tempname = mktemp("temp")
              imrename(outfil,tempname,ver-, >& "dev$null")
              imcopy(tempname//dsect,outfil,ver-, >& "dev$null")
              imdelete(tempname, go+, ver-, def+)
           }

# If you want to do a wavelength calibration then decide where on the chip
# the spectrum lies...
           if(lcal){
              start = max(ys,ysw)
              endp = min(ys+yz-1,ysw+yzw-1)
   
# Get the central wavelength and dispersion.  Also sort out which arm/chip
# combinations reverse the sense of the dispersion (e.g. red to the left).
              print(" Applying mean wavelength solution")
              print(" Applying mean wavelength solution",>>"fos2.log")
              print(" ")
              print(" ",>>"fos2.log")
              stw = 7212.0 + (ysize*0.5 - start)*8.7
              app = -8.7

# Edit the WCS to include the wavelength information for first order.
              wcsedit(img//".0001."//imtype,"cd",app,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0001."//imtype,"crval",stw,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0001."//imtype,"crpix",1,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0001."//imtype,"units","angstroms",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0001."//imtype,"label","wavelength",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
              print(" First order:   Start wavelength ",stw," Angstroms/pix ",
                 app," Binning factor ",xb)
              print(" First order:   Start wavelength ",stw," Angstroms/pix ",
                 app," Binning factor ",xb,>>"fos2.log")

# Do the same for second order...
              if (nspec == 2) {
                 stw = 3646.0 + (ysize*0.5 - start)*4.3
                 app = -4.3
                 wcsedit(img//".0002."//imtype,"cd",app,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(img//".0002."//imtype,"crval",stw,1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 wcsedit(img//".0002."//imtype,"crpix",1,1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 wcsedit(img//".0002."//imtype,"units","angstroms",1,1,
                    wcs="world",inter-,commands="",verbose-,update+,
                    >& "dev$null")
                 wcsedit(img//".0002."//imtype,"label","wavelength",1,1,
                    wcs="world",inter-,commands="",verbose-,update+,
                    >& "dev$null")
                 print(" Second order:  Start wavelength ",stw,
                    " Angstroms/pix ",app," Binning factor ",xb)
                 print(" Second order:  Start wavelength ",stw,
                    " Angstroms/pix ",app," Binning factor ",xb,>>"fos2.log")
              }
              print(" ")  
              print(" ",>>"fos2.log")

# If not wavelength calibrating, then set up the logical and physical
# coordinates to be the same
           } else {
              wcsedit(img//".0*."//imtype,"cd",1.0,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"crval",1.0,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"crpix",1,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"label","X",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"units","pixels",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
           }

# If both orders have been extracted, then combine them if that's what you want
           if (second && combine) {
              scomb(img//".0*."//imtype,img//"C."//imtype,noutput="",
                 logfile="STDOUT",apertures="",group="all",
                 combine="average",reject="none",first-,w1=INDEF,
                 w2=INDEF,dw=INDEF,nw=INDEF,log-,scale="none",zero="none",
                 weight="none",sample="",lthreshold=INDEF,hthreshold=INDEF,
                 nlow=1,nhigh=1,nkeep=1,mclip+,lsigma=3.0,hsigma=3.0,
                 rdnoise="0.0",gain="1.0",snoise="0.0",sigscale=0.1,
                 pclip=-0.5,grow=0,blank=0.0, >& "dev$null")
              plfil = img//"C."//imtype
           } else {
              plfil = img//".0*."//imtype
           }
# If this started out as a FITS file, then delete the IRAF file...
           if (fcon) imdelete(img, go+, ver-, def+, >& "dev$null")
        }

# Tidy and exit
        delete (infile, go+, ver-, def+, all+, sub+)
        inimglist = ""
end
                                                           
