procedure findids(image)

# Script to locate and extract IDS data from raw frames.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'plotids.cl'.  Both scripts are called from the task 'ids'
# in the package 'rgo.quickie' and it is recommended that you use the
# ids task rather than to call these scripts individually. 
# Most of the beutiful programing style and GUI interface was stolen
# from Jim Lewis ISIS script, which in turn stole ideas from
# scripts written by Brian Boyle (aka Ryan O'Bible).
# Ryan O'Bible,  RGO, February 1995 

string  image   {prompt="Input image(s)"}
int     nspec   {1,prompt="Number of spectra in image?"}
real    stw     {8500,prompt="Start wavelength"}
real    app     {-2.,prompt="Angstroms/pix"}
int     blkfact {1,min=1,prompt="Blocking factor in spectral domain"}
bool    fconv   {no,prompt="Convert from FITS?"}
bool    crproc  {yes,prompt="Remove cosmic rays for extraction?"}
bool    lcal    {yes,prompt="Wavelength calibrate?"}


struct  *inimglist
struct  *tlist

begin

        string  dsect           # data section
        string  imglist         # equals image
        string  imtype          # image type
        string  infile          # temporary list for files
        string  img             # image name from fscan with extension stripped
        string  imgi            # image name from fscan
        string  ct              # type of chip
        string  ca              # ids arm
        string  refim           # reference image for spectral tracing
        string  outfil          # dummy string
        string  tempname        # dummy string
        string  dumstr          # dummy string
        string  instring        # dummy string
        real    cw              # central wavelegnth
        real    defsiz          # default size in case header items are missing
        real    endp            # end pixel for wavelength calibration
        real    mpp             # mm/pix
        real    start           # starting pixel for wavelength calibration
        real    rd              # CCD readnoise
        real    gn              # CCD gain
        real    xs,xz,xsw,xzw   # X dimensions of chip/window
        real    xsize           # X size of chip
        real    xb,yb           # binning factor
        real    dumreal         # dummy real
        real    pos             # spectrum position
        int     mm              # dummy
        int     iblk            # blocking factor in spectral domain
        int     nsp             # number of spectra
        int     junk            # dummy
        bool    crp             # TRUE if removing cosmic rays
        bool    fcon            # TRUE if input file is a FITS file

# Get query parameter
        imglist = image

# Expand file lists into temporary files.
        infile =  mktemp("temp")
        sections (imglist,option="fullname",>infile)
        inimglist = infile
        imtype = envget("imtype")

# Get a few parameters
        iblk = blkfact
        nsp = nspec
        crp = crproc
        fcon = fconv

# Loop through images
        while (fscan(inimglist,imgi) != EOF) {

# Strip off any extensions.  This is to avoid conflicts with names which
# might arise with FITS files or if the user has used meta-characters in
# the input file list
           fileroot(imgi,val-)
           img = fileroot.root

# Clobber old output files
           imdel(img//".0*."//imtype, go+, ver-, def+, > "dev$null")

# If the input file is a fits image, then convert it to an iraf file...
# This will become unnecessary when v2.11 arrives with its FITS image kernel
           if (fcon) {
              print(" Reading FITS image")
              print(" Reading FITS image",>>"ids.log")
              rfits(img//".fits", "", img, make+, lo-, sh-,
              datatype="", blank=0.0, scale+, oldir-, offset=0)
           }

# Get the chip name in the ids arm.
           imgets (img,"DETHEAD")
           ct = (imgets.value)
           if (ct == "0") ct="unknown"

# Get information about windowing of the chip
         
           imgets(img, "i_naxis1", >& "dev$null")
           defsiz = real(imgets.value)
           imgets (img, "AXSTART", >& "dev$null")
           xs=real(imgets.value)
           if (xs == 0.0) xs = 1.0
           imgets (img, "AXEND", >& "dev$null")
           xz=real(imgets.value) 
           if (xz == 0.0) xz = defsiz
           xsize=xz-xs
           if (xsize == 0.0) xsize = defsiz

           imgets (img, "DXSTART", >& "dev$null")
           xsw=real(imgets.value)
           imgets (img, "DXEND", >& "dev$null")
           xzw=real(imgets.value)
           if (xzw == 0.0) xzw = defsiz
           imgets (img, "XBIN", >& "dev$null")
           xb=real(imgets.value)
           if (xb == 0.0) xb = 1.0
           imgets (img, "YBIN", >& "dev$null")
           yb=real(imgets.value)
           if (yb == 0.0) xb = 1.0
           start = max(1.0,xs-xsw)
           endp = min(start+xzw-1.0,start+xz-1.0)

# Create a data section string for use later
           dsect = "["//int(start)//":"//int(endp)//"]"

# Get the chip readout noise and gain.  If it doesn't exist, then
# substitute plausible values.
           imgets (img, "READNOISE", >& "dev$null")
           rd=real(imgets.value)
           imgets (img, "GAIN", >& "dev$null")
           gn=real(imgets.value) 
           if(rd ==  0)rd=5
           if(gn ==  0)gn=1
           print(" ")
           print(" CCD frame: ",img," IDS ")
           print(" ")
           print(" Chip: ",ct,"   Readnoise = ",rd," Gain = ",gn,)
           print(" ")
           print(" ",>>"ids.log")
           print(" CCD frame: ",img," IDS ",>>"ids.log")
           print(" ",>>"ids.log")
           print(" Chip: ",ct,"   Readnoise = ",rd," Gain = ",gn,>>"ids.log")
           print(" ",>>"ids.log")

# Remove cosmic rays from the frame to facilitate locating the spectra. Note
# that this is not the frame from which the spectra will be extracted.
           refim=img
           if (crp) {
              refim=mktemp(img)
              print(" Removing Cosmic Rays")
              print(" Removing Cosmic Rays",>>"ids.log")
              xzap(img,outlist=refim, nsigma=2., xboxsz=15, yboxsz=1, 
                 nobjsigma=0., nrings=0, donegative-,nnegsigma=5.,
                 imgsec="[*,*]", deletemasks+, cleanup+, verbose-,
                 inimglist="", outimglist="", statlist="", >& "dev$null")
              imdel("_fmed"//img, go+, ver-, def+)
           }

# Trace the spectrum...
           print(" Tracing spectrum")
           print(" Tracing spectrum",>>"ids.log")

           apextract.dispaxis = 1
           apall1.polysep = 0.6
           apall (refim, output=refim, format="onedspec", references="",
              nfind=nsp, profiles="", interactive-, find+,
              recenter+, resize-, edit-, trace+, fittrace-,
              extract-, extras-, review-, line=INDEF, nsum=100, 
              lower=-5., upper=5., apidtable="", b_function="chebyshev", 
              b_order=1, b_sample="-50:-21,21:50", b_naverage=-10, 
              b_niterate=0, b_low_reject=3., b_high_rejec=3., b_grow=0., 
              width=10., radius=10., threshold=0., minsep=5., maxsep=1000., 
              order="increasing", apertures="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+,
              r_grow=0., avglimits-, t_nsum=50, t_step=50, t_nlost=3, 
              t_function="legendre", t_order=4, t_sample="*", t_naverage=1, 
              t_niterate=2, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance", pfit="fit2d", 
              clean-, saturation=INDEF, readnoise=rd, gain=gn, lsigma=4., 
              usigma=4., nsubaps=1, > "dev$null")

# Extract the spectrum...
           print(" Extracting spectrum ")
           print(" Extracting spectrum ",>>"ids.log")

           apall (img,output=img, format="onedspec", references=refim,
              nfind=nsp, profiles="", interactive-, find-, recenter-, 
              resize-, edit-, trace-, fittrace-, extract+, extras-, review-, 
              line=INDEF, nsum=10, lower=-5., upper=5., 
              apidtable="", b_function="chebyshev", b_order=1, 
              b_sample="-50:-21,21:50", b_naverage=-10, b_niterate=0, 
              b_low_reject=3., b_high_rejec=3., b_grow=0., width=10., 
              radius=10., threshold=0., minsep=5.,maxsep=1000., 
              order="increasing", apertures="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+, 
              r_grow=0., avglimits-, t_nsum=50, t_step=50, t_nlost=3, 
              t_function="legendre", t_order=4, t_sample="*", t_naverage=1, 
              t_niterate=2, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="fit", skybox=1, weights="variance",pfit="fit2d", 
              clean+, saturation=65000, readnoise=rd, gain=gn, lsigma=3., 
              usigma=3., nsubaps=1, > "dev$null")
 
# Send out a friendly message and clean up the database directory.  Remove
# the cosmic ray processed frame if it exists...
           print(" ")
           print(" ",>>"ids.log")
           tempname = mktemp("temp")
           type("database/aplast",map+,dev="terminal") | 
              match("begin","",stop-,print+,meta+,>tempname) 
           tlist = tempname
           while(fscan(tlist,dumstr,dumstr,dumstr,mm,dumreal,pos) != EOF) {
              print(" Spectrum # ",mm, "found in row ",pos)
              print(" Spectrum # ",mm, "found in row ",pos,>>"ids.log")
           }
           print(" ")
           print(" ",>>"ids.log")
           tlist = ""
           delete(tempname, go+, ver-, def+, all+, sub+, >& "dev$null")
           delete("database/ap"//refim, go+, ver-, def+, all+, sub+,
              >& "dev$null")
           if (crp) imdel(refim, go+, ver-, def+)

# Chop off the bits you don't want...
           for (mm=1; mm<= nsp; mm+=1) {
              names(img,str(mm),append="",check-) | scan(outfil)
              tempname = mktemp("temp")
              imrename(outfil,tempname,ver-, >& "dev$null")
              imcopy(tempname//dsect,outfil,ver-, >& "dev$null")
              imdelete(tempname, go+, ver-, def+)
           }

# If you want to do a wavelength calibration then decide where on the chip
# the spectrum lies...
           if(lcal){
              start = max(xs,xsw)
              endp = min(xs+xz-1,xsw+xzw-1)
   
              print(" Start wavelength ",stw," Angstroms/pix ",
                 app," Binning factor ",xb)
              print(" Start wavelength ",stw," Angstroms/pix ",
                 app," Binning factor ",xb,>>"ids.log")
              print(" ")  
              print(" ",>>"ids.log")

# Edit the WCS to include the wavelength information
              wcsedit(img//".0*."//imtype,"cd",app,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"crval",stw,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"crpix",1,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"units","angstroms",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"label","wavelength",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")


# If not doing a wavelength calibration, then reset the WCS so that logical
# and physical coordinates match...
           } else {
              wcsedit(img//".0*."//imtype,"cd",1.0,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"crval",1.0,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"crpix",1,1,1,wcs="world",inter-,
                 commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"label","X",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
              wcsedit(img//".0*."//imtype,"units","pixels",1,1,wcs="world",
                 inter-,commands="",verbose-,update+,>& "dev$null")
           }

# Block average the result if you want...
           for (mm=1; mm<= nsp; mm+=1) {
              names(img,str(mm),append="",check-) | scan(outfil)
              blkavg(outfil,outfil,iblk,1,1,1,1,1,1,option="average")
           }

# If this came originally from a FITS file, then delete the IRAF file...
           if (fcon) imdelete(img, go+, ver-, def+, >& "dev$null")
        }

# Tidy and exit
        delete (infile, go+, ver-, def+, all+, sub+, >& "dev$null")
        inimglist = ""
end
