procedure findues(image)

# Script to locate and extract UES data from raw frames.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'plotues.cl'.  Both scripts are called from the task 'ues'
# in the package 'rgo.quickie' and it is recommended that you use the
# ues task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 

string  image   {prompt="Input image(s)"}
int     nspec   {1,prompt="Number of spectra in image?"}
int     blkfact {1,min=1,prompt="Blocking factor in spectral domain"}
bool    fconv   {no,prompt="Convert from FITS?"}
bool    crproc  {yes,prompt="Remove cosmic rays for extraction?"}
bool    lcal    {yes,prompt="Wavelength calibrate?"}
bool    comb    {no,prompt="Combine all spectra into one?"}


struct  *inimglist
struct  *tlist

begin

        string  dsect           # data section
        string  det             # detector name
        string  ech             # echelle name
        string  imglist         # equals image
        string  imtype          # image type
        string  infile          # temporary list for files
        string  img             # image name from fscan with extension stripped
        string  imgi            # image name from fscan
        string  refim           # reference image for spectral tracing
        string  outfil          # dummy string
        string  tempname        # dummy string
        string  dumstr          # dummy string
        string  tabfil          # table file with echelle info
        real    cent            # centre of Y window
        real    cw              # central wavelegnth in header
        real    ccw             # central wavelegnth of central order
        real    cwaveues        # central wavelength for ues in central order
        real    endp            # end pixel for wavelength calibration
        real    endw            # end wavelength
        real    app             # dispersion A/pix
        real    start           # starting pixel for wavelength calibration
        real    stw             # start wavelength
        real    rd              # CCD readnoise
        real    gn              # CCD gain
        real    xs,xz,xsw,xzw   # X dimensions of chip/window
        real    ys,yz,ysw,yzw   # Y dimensions of chip/window
        real    xsize           # X size of chip
        real    dsize,ssize     # Dispersion and spatial sizes of detector
        real    xb,yb           # binning factor
        real    ysize           # Y size of chip
        real    dumreal         # dummy real
        real    poff            # pixel offset
        real    pos,pos1,pos2   # spectrum positions
        real    minsep          # minimum distance between order and y centre
        real    pixsiz          # X length of pixel in microns
        real    y1,y2,diff      # dummy real variables
        real    dy              # Y length of pixel in microns
        int     defsiz          # a default size
        int     npred           # number of spectra predicted
        int     mm              # dummy
        int     mrem            # spectrum number closest to y centre
        int     npl             # number of plots on a page
        int     nfound          # number of spectra actually found
        int     cord            # central order
        int     stord           # starting order 
        int     len             # string length
        int     ord,ord1        # orders
        int     row,row1        # rows in standard file
        int     nsp             # number of spectra
        int     nsum            # number of rows to sum up for finding spectra
        int     iblk            # blocking factor in spectral domain
        int     tord            # trace order
        bool    crp             # TRUE if taking out cosmic rays
        bool    combine         # TRUE if combining all output spectra into one

# Get query parameter
        imglist = image

# Expand file lists into temporary files.
        infile =  mktemp("temp")
        sections (imglist,option="fullname",>infile)
        inimglist = infile
        imtype = envget("imtype")
        nsp = nspec

# Loop through images
        while (fscan(inimglist,imgi) != EOF) {

# Strip off any extensions.  This is to avoid conflicts with names which
# might arise with FITS files or if the user has used meta-characters in
# the input file list
           fileroot(imgi,val-)
           img = fileroot.root

# Clobber old output files
           imdel(img//".0*."//imtype, go+, ver-, def+, >& "dev$null")
           imdel(img//"C."//imtype, go+, ver-, def+, >& "dev$null")

# If the input file is a fits image, then convert it to an iraf file...
# This will become unnecessary when v2.11 arrives with its FITS image kernel
           if (fconv) {
              print(" Reading FITS image")
              print(" Reading FITS image",>>"ues.log")
              rfits(img//".fits", "", img, make+, lo-, sh-,
              datatype="", blank=0.0, scale+, oldir-, offset=0)
           }

# Get the name of the detector and the echelle
           imgets(img,"DETECTOR")
           det = (imgets.value)
           imgets(img,"UESECHEL")
           ech = (imgets.value)

# If this is the IPCS, then set up the sizes of the windows...
           if (det == "IPCS") {
              xs = 1.0
              imgets(img, "i_naxis1", >& "dev$null")
              xz = real(imgets.value)
              xsize = xz
              xsw = 0.0
              xzw = xz
              xb = 1.0
              imgets(img, "i_naxis2", >& "dev$null")
              ysize = real(imgets.value)
              yb = 1.0
              dsize = xsize
              ssize = ysize
           } else {
              imgets(img, "i_naxis2", >& "dev$null")
              defsiz = real(imgets.value)
              imgets (img, "CCDYIMST", >& "dev$null")
              ys=real(imgets.value)
              if (ys == 0.0) ys = 1.0
              imgets (img, "CCDYIMSI", >& "dev$null")
              yz=real(imgets.value) 
              if (yz == 0.0) yz = defsiz
              imgets (img, "CCDYSIZE", >& "dev$null")
              ysize=real(imgets.value)
              if (ysize == 0.0) ysize = defsiz

              imgets (img, "CCDWYO1", >& "dev$null")
              ysw=real(imgets.value)
              imgets (img, "CCDWYS1", >& "dev$null")
              yzw=real(imgets.value)
              if (yzw == 0.0) yzw = defsiz
              imgets (img, "CCDXBIN", >& "dev$null")
              xb=real(imgets.value)
              if (xb == 0.0) xb = 1.0
              imgets (img, "CCDYBIN", >& "dev$null")
              yb=real(imgets.value)
              if (yb == 0.0) xb = 1.0
              imgets(img, "i_naxis1", >& "dev$null")
              defsiz = real(imgets.value)
              imgets (img, "CCDXSIZE", >& "dev$null")
              xsize=real(imgets.value)
              if (xsize == 0.0) xsize = defsiz
              dsize = ysize
              ssize = xsize
           }

# Create a data section string for use later
           if (det == "IPCS") {
              start = 1.0
              endp = xz
              dsect = "[*]"
           } else {
              start = max(1.0,ys-ysw)
              endp = min(start+yzw-1.0,start+yz-1.0)
              if (start == 1.0 && endp == yzw)
                 dsect = "[*]"
              else 
                 dsect = "["//int(start)//":"//int(endp)//"]"
           }


# Get the chip readout noise and gain.  If it doesn't exist, then
# substitute plausible values.  Substitute the right values for IPCS.
           if (det == "IPCS") {
              rd = 0.0
              gn = 1.0
           } else {
              imgets (img, "READNOISE", >& "dev$null")
              rd=real(imgets.value)
              imgets (img, "GAIN", >& "dev$null")
              gn=real(imgets.value) 
              if(rd ==  0)rd=5
              if(gn ==  0)gn=1
           }
           print(" ")
           print(" UES frame: ",img, "   Echelle: ",ech)
           print(" ")
           print(" Detector: ",det,"   Readnoise = ",rd," Gain = ",gn,)
           print(" ")
           print(" ",>>"ues.log")
           print(" UES frame: ",img, "   Echelle: ",ech,>>"ues.log")
           print(" ",>>"ues.log")
           print(" Detector: ",det,"   Readnoise = ",rd," Gain = ",gn,,>>"ues.log")
           print(" ",>>"ues.log")

# Remove cosmic rays from the frame to facilitate locating the spectra. Note
# that this is not the frame from which the spectra will be extracted.
           refim=img
           if (det == "IPCS")
              crp = no
           else 
              crp = crproc
           if (crp) {
              refim=mktemp(img)
              print(" Removing Cosmic Rays")
              print(" Removing Cosmic Rays",>>"ues.log")
              xzap(img,outlist=refim, nsigma=2., xboxsz=1, yboxsz=15, 
                 nobjsigma=0., nrings=0, donegative-,nnegsigma=5.,
                 imgsec="[*,*]", deletemasks+, cleanup+, verbose-,
                 inimglist="", outimglist="", statlist="", >& "dev$null")
              imdel("_fmed"//img, go+, ver-, def+)
           }

# Find central order and decide a minimum distance between orders...
           imgets(img, "UESCORDE")
           cord = int(imgets.value)
           len = strlen(ech)
           tabfil = "quickie$stdfiles/ues"//substr(ech,2,len)
           keypar(tabfil,"ordst")
           ord1 = int(keypar.value)
           row = cord - ord1 + 1
           tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
              columns="CDCEN",rows=row,pwidth=80) | scan(y1)
           tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
              columns="CDCEN",rows=row+1,pwidth=80) | scan(y2)
           if (det == "IPCS") 
              imgets(img,"UESPIXL")
           else 
              imgets(img,"UESPIXW")
           dy = real(imgets.value)
           npred = int(ssize*dy/(abs(y1-y2)*1000.0))
           row1 = min(1,row-npred/2)
           minsep = ssize
           for (row=row1; row<=row1+npred-2; row+=1) {
              tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                 columns="CDCEN",rows=row,pwidth=80) | scan(y1)
              tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                 columns="CDCEN",rows=row+1,pwidth=80) | scan(y2)
              diff = abs(y1 - y2)*1000.0/dy
              if (diff < minsep) minsep = diff
           }
           minsep = 0.9*minsep
           nsum = int(0.25*dsize)
              
# Trace the spectrum...
           print(" Tracing spectra")
           print(" Tracing spectra",>>"ues.log")

           if (det == "IPCS") {
              apextract.dispaxis = 1
              tord = 6
           } else {
              apextract.dispaxis = 2
              tord = 2
           }
           apall (refim, output="", format="onedspec", references="",
              nfind=nsp, profiles="", interactive-, find+,
              recenter+, resize-, edit-, trace+, fittrace-,
              extract-, extras-, review-, line=INDEF, nsum=nsum, 
              lower=-5., upper=5., apidtable="", b_function="chebyshev", 
              b_order=1, b_sample="-10:-6,6:10", b_naverage=-10, 
              b_niterate=0, b_low_reject=3., b_high_rejec=3., b_grow=0., 
              width=10., radius=10., threshold=0., minsep=minsep, maxsep=1000.,
              order="increasing", apertures="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+,
              r_grow=0., avglimits-, t_nsum=150, t_step=150, t_nlost=3, 
              t_function="legendre", t_order=tord, t_sample="*", t_naverage=1, 
              t_niterate=1, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="median", skybox=1, weights="none", pfit="fit2d", 
              clean-, saturation=INDEF, readnoise=rd, gain=gn, lsigma=4., 
              usigma=4., nsubaps=1, > "dev$null")

# Extract the spectrum...
           print(" Extracting spectra ")
           print(" Extracting spectra ",>>"ues.log")

           apall (img,output="", format="onedspec", references=refim,
              nfind=nsp, profiles="", interactive-, find-, recenter-, 
              resize-, edit-, trace-, fittrace-, extract+, extras-, review-, 
              line=INDEF, nsum=nsum, lower=-5., upper=5., 
              apidtable="", b_function="chebyshev", b_order=1, 
              b_sample="-10:-6,6:10", b_naverage=-10, b_niterate=0, 
              b_low_reject=3., b_high_rejec=3., b_grow=0., width=10., 
              radius=10., threshold=0., minsep=minsep, maxsep=1000., 
              order="increasing", apertures="", npeaks=nsp, shift+,
              llimit=INDEF, ulimit=INDEF, ylevel=0.1, peak+, bkg+, 
              r_grow=0., avglimits-, t_nsum=150, t_step=150, t_nlost=3, 
              t_function="legendre", t_order=tord, t_sample="*", t_naverage=1,
              t_niterate=1, t_low_reject=3., t_high_rejec=3., t_grow=0., 
              background="median", skybox=1, weights="none",pfit="fit2d", 
              clean-, saturation=INDEF, readnoise=rd, gain=gn, lsigma=3., 
              usigma=3., nsubaps=1, > "dev$null")
 
# Send out a friendly message and clean up the database directory.  Remove
# the cosmic ray processed frame if it exists...Also find out which spectrum
# is the central order...
           print(" ")
           print(" ",>>"ues.log")
           tempname = mktemp("temp")
           type("database/aplast",map+,dev="terminal") | 
              match("begin","",stop-,print+,meta+,>tempname) 
           minsep = ssize
           cent = ssize*0.5
           tlist = tempname
           nfound = 0
           while(fscan(tlist,dumstr,dumstr,dumstr,mm,pos1,pos2) != EOF) {
              nfound = nfound + 1
              if (apextract.dispaxis == 1)
                 pos = pos2
              else
                 pos = pos1
              printf(" Spectrum #%d found in row %7.2f\n",mm,pos)
              printf(" Spectrum #%d found in row %7.2f\n",mm,pos,>>"ues.log")
              if (abs(pos-cent) < minsep) {
                 minsep = abs(pos-cent)
                 mrem = mm
              }
           }
           if (nfound != nsp) {
              print("Warning: Only ",nfound," spectra found")
              print("Warning: Only ",nfound," spectra found",>>"ues.log")
           }
           print(" ")
           print(" ",>>"ues.log")
           tlist = ""
           delete(tempname, go+, ver-, def+, all+, sub+, >& "dev$null")
           delete("database/ap"//refim, go+, ver-, def+, all+, sub+,
              >& "dev$null")
           if (crp) imdel(refim, go+, ver-, def+)

# Chop off the bits you don't want...
           if (dsect != "[*]") {
              for (mm=1; mm<= nfound; mm+=1) {
                 names(img,str(mm),append="",check-) | scan(outfil)
                 tempname = mktemp("temp")
                 imrename(outfil,tempname,ver-, >& "dev$null")
                 imcopy(tempname//dsect,outfil,ver-, >& "dev$null")
                 imdelete(tempname, go+, ver-, def+)
              }
           }

# Find the starting order number
           stord = cord + mrem - 1

# If you want to do a wavelength calibration then decide where on the chip
# the spectrum lies...
           if(lcal){
              if (det == "IPCS") {
                 start = max(xs,xsw)
                 endp = min(xs+xz-1,xsw+xzw-1)
              } else {
                 start = max(ys,ysw)
                 endp = min(ys+yz-1,ysw+yzw-1)
              }

# Find wavelength centre
              row1 = stord - ord1 + 1
              print(" Applying wavelength scale ")
              print(" ")
              print(" Applying wavelength scale ",>>"ues.log")
              print(" ",>>"ues.log")
              imgets(img,"UESPIXW")
              pixsiz = real(imgets.value)
              imgets(img,"UESCWAVE")
              cwaveues = real(imgets.value)
              imgets(img,"UESDISPX")
              app = real(imgets.value)
              tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                 columns="WAVCENTR",rows=row1-mrem+1,pwidth=80) | scan(ccw)
              poff = (cwaveues - ccw)/app

# Now loop for all the spectra...
              for (mm=1; mm<=nfound; mm+=1) {
                 row = row1 - mm + 1
                 ord = stord - mm + 1

# Get the wavelength info from the standard table...
                 tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                    columns="WAVCENTR",rows=row,pwidth=80) | scan(cw)
                 tdump(tabfil,cdfile="",pfile="",datafile="STDOUT",
                    columns="LD",rows=row,pwidth=80) | scan(app)
                 app = app*pixsiz*0.001
                 stw = cw - (0.5*(endp-start) - poff)*app
                 printf(" Spectrum: %d  Order: %d ",mm,ord)
                 printf("  Start wavelength %9.3f  Dispersion %6.4f\n",
                    stw,app)
                 printf(" Spectrum: %d  Order: %d ",mm,ord,>>"ues.log")
                 printf("  Start wavelength %9.3f  Dispersion %6.4f\n",
                    stw,app,>>"ues.log")
                 names(img,str(mm),ap="",check-) | scan(outfil)

# Edit the WCS to include the wavelength information and order number
                 wcsedit(outfil,"cd",app,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crval",stw,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crpix",1,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"units","angstroms",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"label","wavelength",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 hedit(outfil,"ordnum",ord,add+,del-,ver-,show-,upd+,
                    >& "dev$null")
              }

# If not doing a wavelength calibration, then reset the WCS so that logical
# and physical coordinates match...
           } else {
              for (mm=1; mm<=nfound; mm+=1) {
                 names(img,str(mm),ap="",check-) | scan(outfil)
                 wcsedit(outfil,"cd",1.0,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crval",1.0,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"crpix",1,1,1,wcs="world",inter-,
                    commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"label","X",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
                 wcsedit(outfil,"units","pixels",1,1,wcs="world",
                    inter-,commands="",verbose-,update+,>& "dev$null")
              }
           }

# Do you want the spectra block averaged?  If so, then do that here...
           iblk = blkfact
           if (iblk > 1) {
              for (mm=1; mm<=nfound; mm+=1) {
                 names(img,str(mm),ap="",check-) | scan(outfil)
                 blkavg(outfil,outfil,iblk,1,1,1,1,1,1,option="average")
              }
           }

# Do you want to combine the spectra?
           combine = comb
           if (combine) {
              scomb(img//".0*."//imtype,output=img//"C."//imtype,noutput="",
                 logfile="STDOUT",apertures="",group="all",reject="none",
                 first-,w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,log-,scale="none",
                 zero="none",weight="none",sample="",lthreshold=INDEF,
                 hthreshold=INDEF,nlow=1,nhigh=1,nkeep=1,mclip-,lsigma=3.0,
                 hsigma=3.0,rdnoise="0.0",gain="1.0",snoise="0.0",
                 sigscale=0.1,pclip=-0.5,grow=0,blank=0.0, >& "dev$null")
           }
        }

# Tidy and exit
        delete (infile, go+, ver-, def+, all+, sub+)
        inimglist = ""
end
