procedure plotisis()

# Script to plot ISIS data which have been extracted with findisis. These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'findisis.cl'.  Both scripts are called from the task 'isis'
# in the package 'rgo.quickie' and it is recommended that you use the
# isis task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 

string  image    {prompt="Input image"}
int     nplot    {prompt="Number of plots on a page"}
bool    comb     {prompt="Combine spectra?"}
bool    hcopy    {prompt="Hardcopy of final plot?"}

struct  *inimglist

begin

      int    npl         # Number of plots
      int    nf          # Number of spectra found
      int    mm          # Dummy
      int    mm1         # Dummy
      int    mm2         # Dummy
      bool   quit        # Dummy
      bool   combine     # Combine spectra?
      string img         # Image name
      string input       # Image name
      string outfil      # Output file name
      string tempfile    # Dummy
      string infile      # Input file list
      string title       # Title string
      string sttitle     # Starting bit of title

      # Get some parameters

      img = image
      npl = nplot
      combine = comb
      tempfile = mktemp("temp")

      # Expand the input list into a temporary file...

      infile = mktemp("tmp$temp")
      sections(img,option="fullname",>infile)
      inimglist = infile

      # Now loop for this list

      while (fscan(inimglist,input) != EOF) {
         fileroot(input,val-)
         input = fileroot.root
         if (combine) input = input//"C"

         # How many files are there?

         quit = no
         for (mm=1; ; mm+=1) {
            outfil = ""
            names(input,str(mm),append="",check+) | scan(outfil)
            if (strlen(outfil) == 0) break
         }
         nf = mm - 1
         if (nf < npl) npl = nf

         # If there are no files, then get out of here...

         if (nf == 0) {
            print(" \n")
            print("There are no files to be plotted\n")
            print(" \n")
            print(" \n",>>"isis.log")
            print("There are no files to be plotted\n",>>"isis.log")
            print(" \n",>>"isis.log")
            return
         }

         # Do the plotting...

         if (nf < npl) npl = nf
         if (!combine) 
            sttitle = input//" Spectrum # "
         else
            sttitle = img//" Combined spectra # "
         for (mm=1; mm<=nf; mm+=1) {
            title = sttitle//mm
            names(input,str(mm),ap="",check-) | scan(outfil)
            imgets(outfil,"CD1_1")
            if (real(imgets.value) < 0.0) outfil = outfil//"[-*]"
            prow(outfil,1,wcs="world",wx1=0.0,wx2=0.0,wy1=0.0,wy2=0.0,
               point-,mark="",szmark=0.005,logx-,logy-,xlabel="",
               ylabel="counts",title=title,xformat="wcsformat",
               vx1=0.0,vx2=0.0,vy1=0.0,vy2=0.0,majrx=5,minrx=5,majry=5,
               minry=5,round-,fill+,app-,device="stdgraph", >>G tempfile)
         }
         for (mm=1; mm<=nf; mm+=npl) {
            mm1 = mm
            mm2 = min(nf,mm1+npl-1)
            outfil = mktemp("temp")
            gkiextract(tempfile,mm1//"-"//mm2,go+,ver-,def+,>outfil)
            if (mm2 != nf)
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter+,cursor="",>& "dev$null")
            else
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter-,cursor="",>& "dev$null")
            delete(outfil,go+,ver-,def+,all+,sub+, >& "dev$null")
         }
         if (hcopy) {
            outfil = mktemp("temp")
            gkimosaic(tempfile,device="",output=outfil,nx=1,ny=npl,
               rotate-,fill+,inter-,cursor="",>& "dev$null")
            stdplot(outfil,dev="stdplot",gen-,deb-,ver-,gkiun-)
            delete(outfil,go+,ver-,def+,all+,sub+, >& "dev$null")
         }
         delete(tempfile, go+, ver-, def+, all+, sub+, >& "dev$null")
         if (combine) break
      }
      delete(infile, go+, ver-, def+, all+, sub+, >& "dev$null")
      inimglist = ""
end      
