procedure pfos2()

# Script to plot FOS2 data which have been extracted with ffos2.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'ffos2.cl'.  Both scripts are called from the task 'fos2'
# in the package 'rgo.quickie' and it is recommended that you use the
# fos2 task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 

string  image    {prompt="Input image"}
bool    sec      {yes,prompt="Extract 2nd order?"}
bool    comb     {no,prompt="Combine spectra?"}
bool    hcopy    {no,prompt="Hardcopy of final plot?"}

struct  *inimglist

begin

      int    npl         # Number of plots
      int    nf          # Number of spectra found
      int    mm          # Dummy
      int    mm1         # Dummy
      int    mm2         # Dummy
      int    nimtype     # Size of image type extension
      int    i           # Size of a character string
      int    npix        # Number of pixels in a spectrum
      bool   quit        # Dummy
      bool   second      # Extract 2nd order?
      bool   combine     # Combine spectra?
      string img         # Image name
      string input       # Image name
      string outfil      # Output file name
      string tempfile    # Dummy
      string infile      # Input file list
      string title       # Title string
      string tit         # Title string
      string imtype      # Image type
      string log1        # Name of log file
      string mstype      # Multispec image type
      real   cd1         # Plot dispersion

      # Get some parameters

      img = image
      combine = comb
      second = sec
      tempfile = mktemp("temp")

      # Set up logging
      
      tee.append = yes
      log1 = "fos2.log"

      # Get file name extension

      mstype = ".ms."//envget("imtype")

      # Expand the input list into a temporary file...

      infile = mktemp("tmp$temp")
      sections(img,option="fullname",>infile)
      inimglist = infile

      # Now loop for this list

      while (fscan(inimglist,input) != EOF) {

         strpextn(input) | scan(input)
         if (combine) input = input//"C"
         outfil = input//mstype

         # How many spectra are there?

         if (!combine) {
            hselect(outfil,"i_naxis2",yes) | scan(nf)
            if (nscan() == 0) {
               hselect(outfil,"i_naxis1",yes) | scan(npix)
               if (nscan() == 0) {
                  print(" \n") | tee(log1)
                  printf("There are no spectra to be plotted from %s\n",input) | tee(log1)
                  print(" \n") | tee(log1)
                  next
               } else
                  nf = 1
            }
         } else {
            nf = 1
            if (!access(outfil)) {
               printf("Can't access file %s\n",outfil) | tee(log1)
               next
            }
         }

         # How many to plot on a panel?

         if (combine) 
            npl = 1
         else if (!combine && second)
            npl = 2
         else if (!combine && !second)
            npl = 1

         # Do the plotting... 

         if (!combine) 
            tit = input//" Order # "
         else
            tit = img//" Combined orders "
         for (mm=1; mm<=nf; mm+=1) {
            if (!combine) 
               title = tit//mm
            else 
               title = tit
            prow(outfil,mm,wcs="world",wx1=0.0,wx2=0.0,wy1=0.0,wy2=0.0,
               point-,mark="",szmark=0.005,logx-,logy-,xlabel="",
               ylabel="counts",title=title,xformat="wcsformat",
               vx1=0.0,vx2=0.0,vy1=0.0,vy2=0.0,majrx=5,minrx=5,majry=5,
               minry=5,round-,fill+,app-,device="stdgraph", >>G tempfile)
         }
         for (mm=1; mm<=nf; mm+=npl) {
            mm1 = mm
            mm2 = min(nf,mm1+npl-1)
            outfil = mktemp("temp")
            gkiextract(tempfile,mm1//"-"//mm2,go+,ver-,def+,>outfil)
            if (mm2 != nf)
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter+,cursor="",>& "dev$null")
            else
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter-,cursor="",>& "dev$null")
            delete(outfil,ver-,>& "dev$null")
         }
         if (hcopy) {
            outfil = mktemp("temp")
            gkimosaic(tempfile,device="",output=outfil,nx=1,ny=npl,
               rotate-,fill+,inter-,cursor="",>& "dev$null")
            stdplot(outfil,dev="stdplot",gen-,deb-,ver-,gkiun-)
            delete(outfil,ver-,>& "dev$null")
         }
         delete(tempfile, ver-, >& "dev$null")
      }
      delete(infile, ver-, >& "dev$null")
      inimglist = ""
end      
