procedure pids()

# Script to plot IDS data which have been extracted with fids. These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'fids.cl'.  Both scripts are called from the task 'ids'
# in the package 'rgo.quickie' and it is recommended that you use the
# ids task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 

string  image    {prompt="Input image"}
int     nplot    {2,prompt="Number of plots on a page"}
bool    hcopy    {no,prompt="Hardcopy of final plot?"}

struct  *inimglist

begin

      int    npl         # Number of plots
      int    nf          # Number of spectra found
      int    mm          # Dummy
      int    mm1         # Dummy
      int    mm2         # Dummy
      int    npix        # Number of pixels in a spectrum
      string img         # Image name
      string input       # Image name
      string outfil      # Output file name
      string tempfile    # Dummy
      string infile      # Input file list
      string title       # Title string
      string sttitle     # Starting bit of title string
      string log1        # Name of log file
      string mstype      # Multispec image type extension

      # Get some parameters

      img = image
      npl = nplot
      tempfile = mktemp("temp")

      # Expand the input list into a temporary file...

      infile = mktemp("tmp$temp")
      sections(img,option="fullname",>infile)
      inimglist = infile

      # Set up logging

      tee.append = yes
      log1 = "ids.log"

      # Get file name extension

      mstype = ".ms."//envget("imtype")

      # Now loop for this list

      while (fscan(inimglist,input) != EOF) {
         strpextn(input) | scan(input)
         outfil = input//mstype

         # How many spectra are there?

         hselect(outfil,"i_naxis2",yes) | scan(nf)
         if (nscan() == 0) {
            hselect(outfil,"i_naxis1",yes) | scan(npix)
            if (nscan() == 0) {
               print(" \n") | tee(log1)
               printf("There are no spectra to be plotted from %s\n",input) | tee(log1)
               print(" \n") | tee(log1)
               next
            } else
               nf = 1
         }

         # Do the plotting...

         if (nf < npl) npl = nf
         sttitle = input//" Spectrum # "
         for (mm=1; mm<=nf; mm+=1) {
            title = sttitle//mm
            prow(outfil,mm,wcs="world",wx1=0.0,wx2=0.0,wy1=0.0,wy2=0.0,
               point-,mark="",szmark=0.005,logx-,logy-,xlabel="",
               ylabel="counts",title=title,xformat="wcsformat",
               vx1=0.0,vx2=0.0,vy1=0.0,vy2=0.0,majrx=5,minrx=5,majry=5,
               minry=5,round-,fill+,app-,device="stdgraph", >>G tempfile)
         }
         for (mm=1; mm<=nf; mm+=npl) {
            mm1 = mm
            mm2 = min(nf,mm1+npl-1)
            outfil = mktemp("temp")
            gkiextract(tempfile,mm1//"-"//mm2,go+,ver-,def+,>outfil)
            if (mm2 != nf)
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter+,cursor="",>& "dev$null")
            else
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter-,cursor="",>& "dev$null")
            delete(outfil,ver-,>& "dev$null")
         }
         if (hcopy) {
            outfil = mktemp("temp")
            gkimosaic(tempfile,device="",output=outfil,nx=1,ny=npl,
               rotate-,fill+,inter-,cursor="",>& "dev$null")
            stdplot(outfil,dev="stdplot",gen-,deb-,ver-,gkiun-)
            delete(outfil,ver-,>& "dev$null")
         }
         delete(tempfile,ver-,>& "dev$null")
      }
      delete(infile, ver-, >& "dev$null")
      inimglist = ""
end      
