procedure pues()

# Script to plot UES data which have been extracted with fues.  These
# extractions are only good for quicklook data reduction and the
# users should not attempt to do any real analysis with their resulting
# spectra.  This script is meant to be used in conjunction with the
# script 'fues.cl'.  Both scripts are called from the task 'ues'
# in the package 'rgo.quickie' and it is recommended that you use the
# ues task rather than to call these scripts individually. Many of the
# ideas were stolen from scripts written by Brian Boyle (aka Ryan O'Bible).
# Jim Lewis,  RGO, January 1995 

string  image    {prompt="Input image"}
int     nplot    {2,prompt="Number of plots on a page"}
bool    combine  {no,prompt="Is this one combined spectrum?"}
bool    hcopy    {no,prompt="Hardcopy of final plot?"}

begin

      int    npl         # Number of plots
      int    nf          # Number of spectra found
      int    mm          # Dummy
      int    mm1         # Dummy
      int    mm2         # Dummy
      int    ord         # Order number
      bool   quit        # Dummy
      bool   comb        # Combine spectra?
      string img         # Image name
      string outfil      # Output file name
      string imtype      # Image type
      string tempfile    # Dummy
      file   log1        # Name of log file
      real   cd1         # Dispersion

      # Get some parameters
      img = image
      npl = nplot
      comb = combine
      tempfile = mktemp("temp")

      # Get image type

      imtype = "."//envget("imtype")

      # Set up logging...

      tee.append = yes
      log1 = "ues.log"

      # How many files are there?

      if (! comb) {
         quit = no
         for (mm=1; ; mm+=1) {
            outfil = ""
            names(img,str(mm),append="",check+) | scan(outfil)
            if (strlen(outfil) == 0) break
         }
         nf = mm - 1
         if (nf < npl) npl = nf
      } else {
         nf = 1
         if (!access(img//"C"//imtype)) nf = 0
      }

      # If there are no files, then get out of here...

      if (nf == 0) {
         print(" ") | tee(log1)
         print("There are no files to be plotted") | tee(log1)
         print(" ") | tee(log1)
         return
      }

      # Do the plotting...

      if (! combine) {
         if (nf < npl) npl = nf
         for (mm=1; mm<=nf; mm+=1) {
            names(img,str(mm),ap="",check-) | scan(outfil)
            hselect(outfil,"ORDNUM",yes) | scan(ord)
            if (nscan() == 0) {
               print("Warning: No order number for spectrum",mm) | tee(log1)
               next
            }
            hselect(outfil,"CD1_1",yes) | scan(cd1)
            if (nscan() == 0) {
               print("Warning: No dispersion for spectrum",mm) | tee(log1)
               next
            }
            if (cd1 < 0.0) outfil = outfil//"[-*]"
            prow(outfil,1,wcs="world",wx1=0.0,wx2=0.0,wy1=0.0,wy2=0.0,
               point-,mark="",szmark=0.005,logx-,logy-,xlabel="",
               ylabel="counts",title="Order #"//ord,xformat="wcsformat",
               vx1=0.0,vx2=0.0,vy1=0.0,vy2=0.0,majrx=5,minrx=5,majry=5,
               minry=5,round-,fill+,app-,device="stdgraph", >>G tempfile)
         }
         for (mm=1; mm<=nf; mm+=npl) {
            mm1 = mm
            mm2 = min(nf,mm1+npl-1)
            outfil = mktemp("temp")
            gkiextract(tempfile,mm1//"-"//mm2,go+,ver-,def+,>outfil)
            if (mm2 != nf)
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter+,cursor="",>& "dev$null")
            else
               gkimosaic(outfil,device="stdgraph",output="",nx=1,ny=npl,
                  rotate-,fill+,inter-,cursor="",>& "dev$null")
            delete(outfil,go+,ver-,def+,all+,sub+, >& "dev$null")
         }
      } else {
         npl = 1
         prow(img//"C"//imtype,1,wcs="world",wx1=0.0,wx2=0.0,wy1=0.0,
            wy2=0.0,point-,mark="",szmark=0.005,logx-,logy-,xlabel="",
            ylabel="counts",title="Combined orders",xformat="wcsformat",
            vx1=0.0,vx2=0.0,vy1=0.0,vy2=0.0,majrx=5,minrx=5,majry=5,
            minry=5,round-,fill+,app-,device="stdgraph", >>G tempfile)
         gkimosaic(tempfile,device="stdgraph",output="",nx=1,ny=npl,
            rotate-,fill+,inter-,cursor="",>& "dev$null")
      }
      if (hcopy) {
         outfil = mktemp("temp")
         gkimosaic(tempfile,device="",output=outfil,nx=1,ny=npl,
            rotate-,fill+,inter-,cursor="",>& "dev$null")
         stdplot(outfil,dev="stdplot",gen-,deb-,ver-,gkiun-)
         delete(outfil,go+,ver-,def+,all+,sub+, >& "dev$null")
      }
      delete(tempfile, go+, ver-, def+, all+, sub+, >& "dev$null")
end      
