procedure xzap(inlist,outlist)

string	inlist	    {prompt="Image(s) for cosmic ray cleaning"}
string	outlist	    {prompt="Output image(s)"}
real	nsigma	    {5,prompt="Number of sky sigma for zapping threshold"}
int	xboxsz	    {15,prompt="X Box size for zapping"}
int	yboxsz	    {1,prompt="X Box size for zapping"}
real	nobjsigma   {2,prompt="Number of sky sigma for object identification"}
int	nrings	    {0,prompt="Number of pixels to flag as buffer around CRs"}
bool	donegative  {no,prompt="Clean deviant negative pixels?"}
real	nnegsigma   {5,prompt="Number of sky sigma for negative zapping"}
string	imgsec	    {prompt="Image section to use for computing sky sigma"}
bool	deletemasks {yes,prompt="Delete deviant pixel mask after execution?"}
bool	cleanup	    {yes,prompt="Delete temporary working images afterwards?"}
bool	verbose     {yes,prompt="Verbose output?"}

struct	*inimglist
struct	*outimglist
struct	*statlist

begin

	real	skysig,skymode
	real 	crthresh,objthresh,negthresh
	real	dmin,dmax
	int	krap,nbox,nbox2,ilen
	string 	infile, outfile, img, outimg, statfile
	bool	maskobj

# Check input parameters.
	if (nsigma <= 0.) error (1,"ERROR:  nsigma must be positive.")
	if (donegative && nnegsigma <= 0.) 
		error (1,"ERROR:  nnegsigma must be positive.")
	if (nobjsigma < 0.) error (1,"ERROR:  nobjsigma must be non-negative.")
	if (nobjsigma == 0.) {
#		print ("No object masking will be used during zapping.")
		maskobj = no
	   } else {
		maskobj = yes
	}
	if (nrings < 0) error (1,"ERROR:  nrings must be non-negative.")

# Expand file lists into temporary files.
	infile =  mktemp("tmp$zap")
	outfile = mktemp("tmp$zap")
	sections (inlist,option="fullname",>infile)
	sections (outlist,option="fullname",>outfile)
	inimglist = infile
	outimglist = outfile

# Loop through input files:
	while (fscan(inimglist,img) != EOF) {
		ilen = strlen(img)
		if (substr(img,ilen-3,ilen) == ".imh") 
			img = substr(img,1,ilen-4)

# Read names of output file.
		krap = fscan(outimglist,outimg)
		ilen = strlen(outimg)
		if (substr(outimg,ilen-3,ilen) == ".imh") 
			outimg = substr(outimg,1,ilen-4)
		if (verbose) print ("Working on ",img," ---ZAP!--> ",outimg)

# Calulate sky mean and RMS noise using iterative sigma rejection.
		if (verbose) print ("     Computing statistics.")
		iterstat (img//imgsec,maxiter=3,verbose=no)
		skymode = iterstat.mean
		skysig  = iterstat.sigma

# Median filter image to produce file _fmed//img
		if (verbose) print ("     Median filtering.")
		median (img,"_fmed"//img,xw=xboxsz,yw=yboxsz,
		   zloreject=INDEF,zhireject=INDEF,boundary="reflect")
# Take difference to produce "unsharp masked" image _cr//img
		imarith (img,"-","_fmed"//img,"_cr"//img)
		imcopy ("_cr"//img,"_mask1"//img//".imh",ve-)


# Threshold _cr//img at nsigma*skysig to make CR mask _mask1//img.  
#    Potential CRs --> 1
#    "Blank sky"   --> 0
# Note that crthresh will be positive by definition.
		if (verbose) print ("     Masking potential CR events.")
		crthresh = nsigma*skysig   
		imreplace ("_mask1"//img//".imh",0.,upper=crthresh,lower=INDEF)
		imreplace ("_mask1"//img//".imh",1.,lower=crthresh,upper=INDEF)
# Make PL format version of _mask1//img and delete IMH format version.
		imcopy   ("_mask1"//img//".imh","_mask1"//img//".pl",ve-)
		imdelete ("_mask1"//img//".imh",ve-)

# Object masking:  create mask identifying where objects might be.

		if (maskobj) {
			if (verbose) print ("     Creating object mask.")

#    The first step is just a nicety.  If the user wants to "clean up" afterward,
#    then we can save time here by just renameing _fmed//img to _mask2//img 
#    before thresholding.  If the user doesn't clean up, they may want to SEE 
#    the original _fmed//img frame, so we copy it to _mask2//img instead of 
#    renaming.   
			if (cleanup) {
				imrename ("_fmed"//img,"_mask2"//img//".imh",ve-)
			} else {
				imcopy ("_fmed"//img,"_mask2"//img//".imh",ve-)
			}

# Set object threshold for _fmed//img at sky+objthresh to make 
# object mask _mask2//img
#    Objects     --> 0
#    "Blank sky" --> 1
		    objthresh = skymode+nobjsigma*skysig  
# objthresh may in principle be negative or (harder still) between 0 and 1.
# This necessitates handling various cases:
		    if (objthresh > 1.) {
			imreplace ("_mask2"//img//".imh",1.,upper=objthresh,lower=INDEF)
			imreplace ("_mask2"//img//".imh",0.,lower=objthresh,upper=INDEF)
		    } else if (objthresh < 0.) {
			imreplace ("_mask2"//img//".imh",0.,lower=objthresh,upper=INDEF)
			imreplace ("_mask2"//img//".imh",1.,upper=objthresh,lower=INDEF)
		    } else {
			imreplace ("_mask2"//img//".imh",1.,lower=objthresh,upper=INDEF)
			imreplace ("_mask2"//img//".imh",2.,upper=objthresh,lower=INDEF)
			iarith    ("_mask2"//img//".imh","-",1.,"_mask2"//img//".imh")
		    }
# Make PL format version of _mask2//img and delete IMH format version.
		    imcopy   ("_mask2"//img//".imh","_mask2"//img//".pl",ve-)
		    imdelete ("_mask2"//img//".imh",ve-)
		}

# If not masking objects, final CR mask is just _mask1//img.  If we are
# masking objects, take product of object and CR masks to make _crmask//img

		if (maskobj) {
			imarith ("_mask1"//img//".pl","*","_mask2"//img//".pl",
				 "_crmask"//img//".pl")
		   } else {
			imcopy  ("_mask1"//img//".pl","_crmask"//img//".pl",ve-)
		}

# Grow additional buffer region around identified CRs.
		if (nrings > 0) {
			if (verbose) print ("     Growing mask rings around CR hits.")
			nbox = 2 * nrings + 1
			nbox2 = nbox * nbox
			imarith ("_crmask"//img//".pl","*",nbox2,"_crmask"//img//".pl")
			boxcar ("_crmask"//img//".pl","_crmask"//img//".pl",
				xw=nbox,yw=nbox)
			imreplace ("_crmask"//img//".pl",1,lower=1,upper=INDEF)
		}

# Identify negative pixels if desired.  No "rings" are grown around negative 
		if (donegative) {
			if (verbose) print ("     Masking deviant negative pixels.")
			imcopy ("_cr"//img,"_maskneg"//img,ve-)
			negthresh = -1.*nnegsigma*skysig
			imreplace ("_maskneg"//img,0.,
				lower=negthresh,upper=INDEF)
			imreplace ("_maskneg"//img,1.,
				upper=negthresh,lower=INDEF)
			imcopy ("_maskneg"//img//".imh","_maskneg"//img//".pl",ve-)
			imdelete ("_maskneg"//img//".imh",ve-)
			imarith ("_crmask"//img//".pl","+","_maskneg"//img//".pl",
				 "_crmask"//img//".pl")
			imreplace ("_crmask"//img//".pl",1,lower=1,upper=INDEF)
		}

		if (verbose) print ("     Replacing CR hits with local median.")
# Multiply CR mask by _crmask//img to make "comic rays only" image _cronly//img
		imarith ("_cr"//img//".imh","*","_crmask"//img//".pl",
			 "_cronly"//img//".imh")

# Subtract _cronly//img from data to produce clean image "outimg". 
#      Note that this effectively replaces the masked regions with the local
#      median, since _cronly//img = img - _fmed//img.
		imarith (img,"-","_cronly"//img,outimg)

# Clean up.
		if (deletemasks) {
			delete ("_mask1"//img//".pl",ve-)
			if (maskobj) delete ("_mask2"//img//".pl",ve-)
			if (donegative) delete ("_maskneg"//img//".pl",ve-)
			delete ("_crmask"//img//".pl",ve-)
		}
		if (cleanup) {
			imdelete ("_cr"//img,ve-)
			imdelete ("_cronly"//img,ve-)
		}
              } 
	if (verbose) print ("     Done.")

end
