include <imhdr.h>

pointer procedure rt_crfile(name,mode,template,ndim,dims,type)
#------------------------------------------------------------------------------
#      RT_CRFILE -- Create an output file from scratch
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) name:       (char) Name for the file
#      (>) mode:       (int) Mapping mode
#      (>) template:   (pointer) Template on which file is based
#      (>) ndim:       (int) Number of dimensions in data array
#      (>) dims:       (int[ndim]) Dimensions of data array
#      (>) type:       (int) Data type of data array
#
#      Returns:
#          fptr:       (pointer) Pointer to the new output file
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        30-MAR-1993
#      UPDATE:      09-JUN-1994 -- Fixed bug which restricted scope to 2d.(JRL)
#
#------------------------------------------------------------------------------

# Parameters

char name[ARB]
int mode,ndim,dims[ndim],type
pointer template

# Local variables

pointer fptr,ptr1
int i
long vin[IM_MAXDIM],vout[IM_MAXDIM],one_l

# Functions

pointer immap(),impgsr()

errchk immap,impgsr

begin

      # Begin by mapping the new file

      fptr = immap(name,mode,template)

      # Now set the necessary header items...

      IM_NDIM(fptr) = ndim
      do i = 1,ndim
         IM_LEN(fptr,i) = dims[i]
      IM_PIXTYPE(fptr) = type

      # Set up some data array indicies

      one_l = 1
      call amovkl(one_l,vin,IM_MAXDIM)
      call amovkl(one_l,vout,IM_MAXDIM)

      # Now map some data and close the file.  This creates the data array.
      # Reopen it READ_WRITE access...

      ptr1 = impgsr(fptr,vin,vout,ndim)
      call imunmap(fptr)
      fptr = immap(name,READ_WRITE,0)

      # Return the pointer

      return (fptr)
end
