procedure rt_fstat(filename,mode,mustexist,exist,clobber)
#-------------------------------------------------------------------------------
#      RT_FSTAT -- Checks whether a file exists and/or can be clobbered
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) filename:   (char) File name
#      (>) mode:       (int) File name mode of access
#      (>) mustexist:  (bool) TRUE if the file must exist
#      (<) exist:      (bool) TRUE if the file exists
#      (<) clobber:    (bool) TRUE if files can be clobbered
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        25-OCT-1993
#      UPDATE:
#
#-------------------------------------------------------------------------------

# Parameters 

char filename[ARB]
int mode
bool mustexist,exist,clobber

# Local variables

pointer sp,cp1

# Functions

bool envgetb(),rt_filex()

begin

      # Get stack workspace

      call smark(sp)
      call salloc(cp1,SZ_LINE,TY_CHAR)

      # Check to see if the file exists

      exist = rt_filex(filename)

      # Get the environment variable "clobber"

      clobber = envgetb("clobber")

      # Section for new files

      if (mode == NEW_FILE || mode == NEW_COPY) {

         # If the file already exists and you can't clobber it, then
         # signal an error.

         if (exist && !clobber) {
            call sprintf(Memc[cp1],SZ_LINE,
               "File %s already exists and can't be clobbered")
            call pargstr(filename)
            call error(0,Memc[cp1]) 
         }

         # If the file already exists and you are allowed to clobber it,
         # then delete the old file.

         else if (exist && clobber) {
            call imdelete(filename)
         }
      }

      # Section for old files

      else {

         # If the file doesn't exist and it must exist, then signal an error

         if (!exist && mustexist) {
            call sprintf(Memc[cp1],SZ_LINE,"File %s does not exist")
            call pargstr(filename)
            call error(0,Memc[cp1])
         }
      }

      # Ditch workspace

      call sfree(sp)
end
