procedure prekeep(final)
#------------------------------------------------------------------------------
#
#   -------------
#   P R E K E E P
#   -------------
#
#   Description
#   -----------
#   Save the spectra which were extracted on the last run of preview.
#
#   
#   Scope of program
#   ----------------
#   - Dimensionality irrelevant
#   - Data type irrlevant
#   - Error files ignored
#   - Batch execution supported
#
#
#   Environment
#   -----------
#   IRAF
#
#
#   Language
#   --------
#   CL script
#
#
#   Parameters
#   ----------
#   final      This is the root name for the final output file name.
#              (character)(prompted for)
#
#   gumby      This parameter, if switched on, gives the new user advice on
#              how to use the program.  If gumby is used, then each prompt for
#              information is preceded by a small explanation of what is 
#              required of the user.  To use gumby mode, one should type: 
#              `prekeep gumby+' on the command line and not use epar.
#
#
#   Propogation of data structure
#   -----------------------------
#   - Files are simply renamed.
#
#   
#   Method
#   ------
#   - The user specifies a root output name.
#   - If only one spectrum was extracted, the the output name is the
#     root with the original frame's run number appended.
#   - If more than one spectrum were extracted, the output names are the
#     root with the original frame's run number and a spectrum number
#     attached, e.g. alpha4444_001 ==> root = 'alpha', run number = 4444,
#     this is the first spectrum ==> _001
#
#
#   External applications called
#   ----------------------------
#   Package images:
#      imrename
#
#
#   Possible future upgrades
#   ------------------------
#
#
#   Author/s
#   --------
#   Jim Lewis      RGO
#
#
#   History
#   -------
#   08-DEC-93     - Original program 
#
#
#------------------------------------------------------------------------------

# Query Parameters

string       final     {prompt="Root name for output file"}

# Hidden Parameters

bool         gumby     {no,prompt="Do you need lots of extra help to run this?"}
struct       *temp

begin

      # Local variables

      string ofile,ifile,rootname,wildcard,run,telescope
      int len,ipos,ipos2
      bool gum

      # See if the user is a gumby or not...(i.e. whether lots of extra
      # help is required)

      gum = gumby

      # Open up the text file with the names to be used

      temp = "sam_ofiles"
      if (!access("sam_ofiles")) 
         error(0,"Cannot access file sam_ofiles")

      # Get final name of output files and strip off extension

      if (gum) {
         print("############################################################")
         print("#                        PREKEEP                           #")
         print("#  This program will save your most recently extracted     #")
         print("#  spectra.  It does so simply by renaming the file from   #")
         print("#  the temporary name assigned by preview to something     #")
         print("#  which is more meaningful to you.  It is important to    #")
         print("#  note that this will only save the most recent spectra   #")
         print("#  as preview deletes old spectra before it creates new    #")
         print("#  ones.  What I need from you now is a root name to use.  #")
         print("#  If the last extraction was of a single spectrum, then   #")
         print("#  the output will be your root name with the file's run   #")
         print("#  number attatched.  If multiple spectra were extracted   #")
         print("#  a spectrum number will be attatched as well.            #")
         print("#                                                          #")
         print("#  Examples:                                               #")
         print("#    1 spectrum, run number = 1234, root name = `basil'    #")
         print("#    output file = `basil1234'                             #")
         print("#                                                          #")
         print("#    2 spectra, run number = 3333, root name = `sybil      #")
         print("#    output files = `sybil333_001' and `sybil333_002'      #")
         print("############################################################")
      }         
      ofile = final
      len = strlen(ofile)
      if (substr(ofile,len-3,len) == "."//envget("imtype")) 
         ofile = substr(ofile,1,len-4)

      # Now rename the files. Tack the run number on

      while (fscan(temp,ifile) != EOF) {
         ipos = stridx("_???",ifile)
         if (ipos == 0) {
            ipos2 = stridx("."//envget("imtype"),ifile) - 1
            rootname = substr(ifile,1,ipos2)
            wildcard = ""
            imgets(rootname,"TELESCOP")
            telescope = imgets.value
            if (telescope == "WHT") {
               imgets(rootname,"RUN")
            } else {
               imgets(rootname,"NUMRUN")
            }
         } else {
            rootname = substr(ifile,1,ipos-1)
            wildcard = "_???"
            imgets(rootname//"_001","TELESCOP")
            telescope = imgets.value
            if (telescope == "WHT") {
               imgets(rootname//"_001","RUN")
            } else {
               imgets(rootname//"_001","NUMRUN")
            }
         }
         run = imgets.value
         if (run == "") run = "00000"
         imrename(ifile,
         "%"//rootname//"%"//ofile//run//"%"//wildcard//"."//envget("imtype"),
         ver-)
      }
end
