procedure sam_check(nord,trace,spec_rng)
#--------------------------------------------------------------------------------------
#      SAM_CHECK -- Check that the requested spectral range isn't greater than
#                   the spectral range used to define the trace
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) nord:       (int) Order of trace polynomial
#      (>) trace:      (real[nord+4]) Trace coefficients
#      (>) spec_rng:   (real[2]) Requested spectral range
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int nord
real trace[nord+4],spec_rng[2]

# Local variables

int ty1,ty2,sy1,sy2
bool panic

begin

      # Convert ranges to integers for comparison

      ty1 = trace[nord+2]
      ty2 = trace[nord+3]
      sy1 = nint(spec_rng[1])
      sy2 = nint(spec_rng[2])

      # See if there is cause to panic

      if (sy1 == 0 && sy2 ==  0) {
         panic = false }
      else if (sy1 < ty1 || sy2 > ty2) {
         panic = true }
      else {
         panic = false
      }

      # If there is cause to panic, then print out some info...

      if (panic) {
         call eprintf("Trace is good in spectral pixels %d %d \n")
            call pargi(ty1)
            call pargi(ty2)
         call eprintf("Spectrum is to cover pixels %d %d \n")
            call pargi(sy1)
            call pargi(sy2)
         call error(0,"***You will be extrapolating the trace !!***")
      }
end
