real procedure sam_determ(array,nterms)
#--------------------------------------------------------------------------------------
#      SAM_DETERM -- Calculate the determinant of a square matrix
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (!) array:      (double[nterms,nterms]) Input matrix
#      (>) nterms:     (int) Dimension of the input matrix
#
#      Returns:
#         determ:      (real) Value of the determinant
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993 -- Based on p. 294 of Bevington
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int nterms
double array[nterms,nterms]

# Local variables

int k,i,j,k1
real determ
double save
bool ok

begin

      # For Zeroth order case...

      if (nterms == 1) {
         determ = array[1,1] }

      # All other cases...Interchange columns if diagonal element is zero

      else {
         determ = 1.0
         do k = 1,nterms {
            if (array[k,k] == 0.0d0) {
               ok = false
               for (j = k; j <= nterms && !ok; j = j + 1) {
                  if (array[k,j] != 0.0d0) ok = true
               }
               if (!ok) {
                  determ = 0.0
                  call error(0,"SAM_DETERM: No non-zero off diagonal elements")
               }
               do i = k,nterms {
                  save = array[i,j]
                  array[i,j] = array[i,k]
                  array[i,k] = save
               }
            determ = -determ
            }

            # Subtract row K from lower rows to get diagonal matrix

            determ = determ*array[k,k]
            if (k - nterms < 0) {
               k1 = k + 1
               do i = k1,nterms {
                  do j = k1,nterms {
                     array[i,j] = array[i,j] - array[i,k]*array[k,j]/array[k,k]
                  }
               }
            }
         }
      }
      return (determ)
end
