int procedure sam_filex(listin)
#-------------------------------------------------------------------------------
#      SAM_FILEX -- Check an output file list to see if any of the files
#                   already exist.  If the files can be clobbered and they
#                   exist, the routine deletes them. If any of the files
#                   exist and they can't be clobbered, then the routine prints
#                   out the name of each of the offending files and tallies
#                   how many there are.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) listin:     (int) Output file list descriptor
#
#      Returns:
#          nexist:     (int) Number of files in the list which already exist.
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        04-MAR-1993
#      UPDATE:      26-OCT-1993 -- Uses rtools routine for determining
#                                  existence. (JRL)
#
#-------------------------------------------------------------------------------

# Parameters 

int listin

# Local variables

pointer sp,filename
int nexist
bool clobber,exist

# Functions

int imtgetim()
bool envgetb(),rt_filex()

begin

      # Get stack workspace

      call smark(sp)
      call salloc(filename,SZ_FNAME,TY_CHAR)

      # Initialise the number of files which already exist

      nexist = 0

      # See if it's alright to clobber files 

      clobber = envgetb("clobber")

      # Now loop for each file in the list. See if it exists.  If
      # it does and you can clobber it, then do so. Otherwise, signal
      # an error.

      while (imtgetim(listin,Memc[filename],SZ_FNAME) != EOF) {
         exist = rt_filex(Memc[filename])
         if (exist && !clobber) {
            nexist = nexist + 1
            call eprintf("File %s already exists\n")
              call pargstr(Memc[filename])
         }
         else if (exist && clobber) {
            call imdelete(Memc[filename])
         }
      }

      # Rewind the template list

      call imtrew(listin)

      # Get rid of stack space

      call sfree(sp)

      # Return the number of files which exist and couldn't be clobbered

      return (nexist)
end
