include <imhdr.h>

pointer procedure sam_gtrace(track,ny,tptr,sametrace,iobj)
#------------------------------------------------------------------------------
#      SAM_GTRACE -- Get the trace for the current object
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) track:      (bool) TRUE if a trace file exists
#      (>) ny:         (int) Maximum distance is spectral direction for trace
#      (>) tptr:       (pointer) Pointer to trace file header
#      (>) sametrace:  (bool) TRUE if the same trace is used for each object
#      (>) iobj:       (int) Number of the object
#
#      Returns:
#          ptr_tr:     (pointer) Pointer to trace for current object
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        12-NOV-1993
#      UPDATE:
#
#------------------------------------------------------------------------------

# Parameters

bool track,sametrace
pointer tptr
int ny,iobj

# Local variables

int nord
pointer ptr_tr

# Functions

pointer imgl2r()

errchk imgl2r

begin

      # If you have a trace, then read the appropriate row.

      if (track) {
         if (sametrace) {
            ptr_tr = imgl2r(tptr,1) }
         else {
            ptr_tr = imgl2r(tptr,iobj)
         }
         nord = IM_LEN(tptr,1) - 4 
      }

      # If not using a trace file, create a fake one, but only the first
      # time, since all objects will use the same trace.

      else {
         if (iobj == 1) {
            call malloc(ptr_tr,4,TY_REAL)
            Memr[ptr_tr] = 0.0
            Memr[ptr_tr+1] = 1.0
            Memr[ptr_tr+2] = real(ny)
            Memr[ptr_tr+3] = 1.0/real(ny)
         }
      }

      # Return the pointer now

      return (ptr_tr)
end
