include <imhdr.h>
include <error.h>

procedure sam_ifile(iname,iptr,nx,ny,ename,eptr,er_ex)
#--------------------------------------------------------------------------------------
#      SAM_IFILE -- Prompt for an input file and open it. Check for error
#                   file and open it if it exists.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iname:      (char[ARB]) Name of input file
#      (<) iptr:       (pointer) Pointer to input file header
#      (<) nx:         (int) X dimension of input file 
#      (<) ny:         (int) Y dimension of input file
#      (<) ename:      (char[ARB]) Name of error file associated with input file
#      (<) eptr:       (pointer) Pointer to error file header
#      (<) er_ex:      (bool) TRUE if an error file exists for the input file
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

char iname[ARB],ename[ARB]
int nx,ny
bool er_ex
pointer iptr,eptr

# Functions

pointer immap(),err_open()
bool err_exist()

errchk immap,err_exist,err_open

begin
      
      # Get the input file.  Open it and check its dimensions...

      iptr = immap(iname,READ_WRITE,0)
      if (IM_NDIM(iptr) != 2) 
         call error(0,"Only 2-d arrays are considered!")
      nx = IM_LEN(iptr,1)
      ny = IM_LEN(iptr,2)

      # Look for variance array file

      call err_fname(iptr,ename)
      er_ex = err_exist(iptr,READ_WRITE,ename,false)
      if (er_ex) eptr = err_open(iptr,READ_WRITE,false)
end

