include <imhdr.h>
include <error.h>

procedure sam_otrace(track,tname,nobj,dptr,nord,tptr,sametrace)
#------------------------------------------------------------------------------
#      SAM_OTRACE -- Open the trace file if it exists.  Do all sorts
#                    of checks for consistency.
#
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) track:      (bool) TRUE if a trace file exists
#      (>) tname:      (pointer) Pointer to trace file name
#      (>) nobj:       (int) Number of objects
#      (>) dptr:       (pointer) Pointer to dekker header
#      (<) nord:       (int) Order of trace polynomial
#      (<) tptr:       (pointer) Pointer to trace file header
#      (<) sametrace:  (bool) TRUE if all spectra to use the same trace.
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        11-NOV-1993
#      UPDATE:      
#
#------------------------------------------------------------------------------

# Parameters

bool track,sametrace
pointer tname,tptr,dptr
int nord,nobj

# Local variables

int nobjt,ndim,i
pointer ptr_tr,ptr_de
real spec_rng[2]

# Functions

pointer immap(),imgl2r()

errchk immap,imgl2r

begin

      # If you have a trace, then open it...

      if (track) {
         tptr = immap(Memc[tname],READ_ONLY,0)
         nord = IM_LEN(tptr,1) - 4 
         ndim = IM_NDIM(tptr)

         # Check on the dimensionality of the trace file.

         if (ndim != 2)  call error(0,"Trace file must be 2d only")

         # Check that there are at least 4 entries per line

         if (nord < 0) 
            call error(0,"Trace must have dimension of at least 4 in X")

         # Get the number of objects in the trace file...

         nobjt = IM_LEN(tptr,2)

         # Check on the number of lines in the trace file.  If there
         # are fewer lines than there are objects, then signal an error
         # unless there is only one.  If there is only one,  then
         # all objects use the same trace.  If there are too many,
         # signal an error.  If the number of objects initially given
         # is less than zero, then it is set to the number of objects
         # in the trace file.

         sametrace = false
         if (nobj < 0) {
            nobj = nobjt 
         }
         else if (nobjt < nobj) {
            if (nobjt == 1) {
               call eprintf("All objects will use the same trace\n")
               sametrace = true }
            else {
               call error(0,"Not enough rows in trace file")
            }
         }
         else if (nobjt > nobj) {
            call error(0,"Too many entries in trace file")
         }

         # Now check on the spectral limits implied by the trace and
         # the dekker files (so long as the dekker file is open).

         if (dptr != NULL) {
            do i = 1,nobj {
               ptr_de = imgl2r(dptr,i)
               call amovr(Memr[ptr_de+2],spec_rng,2)
               if (sametrace) 
                  ptr_tr = imgl2r(tptr,1)
               else 
                  ptr_tr = imgl2r(tptr,i)
               iferr(call sam_check(nord,Memr[ptr_tr],spec_rng)) {
                  call eprintf("Spectrum # %d\n")
                  call pargi(i)
                  call erract(EA_WARN)
               }
            }
         }
      }

      # If there is no trace, then set up a few variables...

      else {
         nord = 0
      }
end
