define   ORDER   4    # Fit to cubic spline

procedure sam_peaks(x,y,npts,x0,sigma)
#------------------------------------------------------------------------------
#      SAM_PEAKS -- Locate peaks in a 1d distribution.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) x:          (real[npts]) X pos of pixels. 
#      (>) y:          (real[npts]) Height of pixels. 
#      (>) npts:       (int) Number of pixels in the profile
#      (<) x0:         (real) Centre of the profile
#      (<) sigma:      (real) Dispersion of the profile
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        15-NOV-1993
#      UPDATE:
#
#------------------------------------------------------------------------------

# Parameters

real x[npts],y[npts],x0,sigma
int npts

# Local variables

pointer sp,wsp1,wsp2
int ier,irem,i
real ymax,xeval,deriv,lderiv,rderiv,xleft,xright,slope,aint,x1,x2

# Functions

real seval()

begin

      # Get some workspace for spline fitting

      call smark(sp)
      call salloc(wsp1,2*npts+30,TY_REAL)
      call salloc(wsp2,(2*ORDER-1)*npts,TY_REAL)

      # Calculate spline coefs
      
      call spline(x,y,npts,Memr[wsp1],Memr[wsp2],ORDER,ier)
      if (ier != 0) {
         call sfree(sp)
         call error(0,"Error in peak finding routine")
      }

      # Find the maximum point

      irem = 1
      ymax = y[1]
      do i = 2,npts {
         if (y[i] > ymax) {
            ymax = y[i]
            irem = i
         }
      }
      if (irem == 1 || irem == npts) {
         call sfree(sp)
         call error(0,"Peak is too far off to one side")
      }

      # Now find where the first derivative is zero. This gives 
      # the true maximum

      xeval = x[irem]
      deriv = seval(xeval,1,Memr[wsp1])
      if (deriv >= 0.0) {
         lderiv = deriv
         xleft = xeval
         xeval = x[irem+1]
         rderiv = seval(xeval,1,Memr[wsp1])
         xright = xeval
      } else {
         rderiv = deriv
         xright = xeval
         xeval = x[irem-1]
         lderiv = seval(xeval,1,Memr[wsp1])
         xleft = xeval
      }
      slope = (rderiv - lderiv)/(xright - xleft)
      aint = rderiv - slope*xright
      x0 = -aint/slope 

      # Now find where the second derivative is zero on the lhs of
      # the peak

      for (i = irem; i >= 1; i = i - 1) {
         xeval = x[i]
         lderiv = seval(xeval,2,Memr[wsp1])
         if (lderiv >= 0.0) break
      }
      if (i == 0) {
         call sfree(sp)
         call error(1,"Profile is too broad")
      }
      xleft = xeval
      xright = x[i+1]
      rderiv = seval(xright,2,Memr[wsp1])
      slope = (rderiv - lderiv)/(xright - xleft)
      aint = rderiv - slope*xright
      x1 = -aint/slope 
      
      # Now find where the second derivative is zero on the rhs of
      # the peak

      for (i = irem; i <= npts; i = i + 1) {
         xeval = x[i]
         rderiv = seval(xeval,2,Memr[wsp1])
         if (rderiv >= 0.0) break
      }
      if (i == npts+1) {
         call sfree(sp)
         call error(1,"Profile is too broad")
      }
      xright= xeval
      xleft = x[i-1]
      lderiv = seval(xleft,2,Memr[wsp1])
      slope = (rderiv - lderiv)/(xright - xleft)
      aint = rderiv - slope*xright
      x2 = -aint/slope 
      
      # Now average these distances to get a mean value for sigma

      sigma = 0.5*((x0 - x1) + (x2 - x0))

      # Ditch workspace

      call sfree(sp)
end
                     
