include <math.h>

real procedure sam_qint(i,x,pol_dist)
#--------------------------------------------------------------------------------------
#      SAM_QINT -- Do interpolation to find contribution of a polynomial to
#                  the emission in a particular pixel
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) i:          (int) Pixel postion in question
#      (>) x:          (real) Mean position of polynomial in question
#      (>) pol_dist:   (real) Space between two polynomials
#
#      Returns:
#          val:        (real) Value of interpolation coefficient
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int i
real x,pol_dist

# Local variables

real c1,c2,c3,c4,c5,delta,adelta,val

begin 

      # Set up a few useful variables

      c1 = 0.5 - pol_dist
      c2 = abs(c1)
      c3 = 1.0 - (c1/pol_dist)**2
      c4 = 0.5 + pol_dist
      c5 = SQRTOF2*pol_dist

      # Now calculate the result

      delta = x - real(i)
      adelta = abs(delta)

      # The first two options are for a pixel which has a polynomial overlapping it

      if (adelta >= c2) {
   
         # Pixel which has a polynomial overlapping from the outside...

         if (adelta >= 0.5) {
            val = ((adelta - c4)/c5)**2 }

         # Pixel which has a polynomial within it, but that polynomial overlaps
         # another pixel on one side...

         else {
            val = 1.0 - ((adelta - c1)/c5)**2
         }
      }

      # Pixel which contains a whole polynomial...

      else if (adelta <= c1) {
         val = 1.0 }

      # Pixel which contains a polynomial which overlaps both sides

      else {
         val = c3 - (delta/pol_dist)**2
      }

      # Return value

      return (val)
end
