include <imhdr.h>

pointer procedure sam_rsect(iname,ispace1,ispace2,ispec1,ispec2,track,trace,
   nord,axis,nt,toff)
#--------------------------------------------------------------------------------------
#      SAM_RSECT -- Read a section from a file following the trace if necessary
#                   and taking into account the dispersion axis.  Copy the subset
#                   into a temporary file.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iname:      (char[ARB]) Name of input file
#      (>) ispace1:    (int) Lower limit in spacial domain of section to be read
#      (>) ispace2:    (int) Upper limit in spacial domain of section to be read
#      (>) ispec1:     (int) Lower limit in spectral domaine of section to be read
#      (>) ispec2:     (int) Upper limit in spectral domaine of section to be read
#      (>) track:      (bool) TRUE if a trace file exists
#      (>) trace:      (real[nord+4]) Trace coefficients
#      (>) nord:       (int) Order of trace polynomial
#      (>) axis:       (int) Spectral axis
#      (>) nt:         (int) Number of elements in offset array
#      (>) toff:       (short[nt]) Array of offsets
#
#      Returns:
#          optr:       (pointer) Pointer to temporary section file
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

char iname[ARB]
int ispace1,ispace2,ispec1,ispec2,nord,axis,nt,j
short toff[nt]
bool track
real trace[nord+4]
pointer optr

# Local variables

pointer sp,section,fsect,tptr1,tptr2,iptr,co,ptr1,ptr2,tempfile
int nnx,nny,maxbuf,m,ixoff,iy1,iy2,npix,ix1,ix2,jst,jfn,i
short to1
long one_l,v1[IM_MAXDIM],v2[IM_MAXDIM]
real spec_pos,xoff,zero

# Functions

pointer immap(),sam_maket(),cogetr(),comap(),imgl2r(),impl2r()
real apolr()
int impnlr(),imgnlr()

errchk immap,sam_maket,imgnlr,impnlr,imunmap,impl2r,imgl2r,sam_swap

begin

      # Get some stack workspace

      call smark(sp)
      call salloc(section,SZ_FNAME,TY_CHAR)
      call salloc(fsect,SZ_FNAME,TY_CHAR)
      call salloc(tempfile,SZ_FNAME,TY_CHAR)

      # Block for spectra with no TRACE

      if (!track) {
         toff[1] = ispace1 - 1
         call strcpy(iname,Memc[fsect],SZ_FNAME)
         call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")

         # If dispersion axis is X axis then open a subset of the frame and
         # swap this into a temporary file...

         if (axis == 1) {         
            call pargi(ispec1)
            call pargi(ispec2)
            call pargi(ispace1)
            call pargi(ispace2)
            call strcat(Memc[section],Memc[fsect],SZ_FNAME)
            tptr1 = immap(Memc[fsect],READ_ONLY,0)
            nnx = IM_LEN(tptr1,1)
            nny = IM_LEN(tptr1,2)
            tptr2 = sam_maket(nny,nnx,TY_REAL,tptr1)
            call strcpy(IM_HDRFILE(tptr2),Memc[tempfile],SZ_FNAME)
            call sam_swap(tptr1,tptr2)
            call imunmap(tptr1)
            call imunmap(tptr2)
            tptr2 = immap(Memc[tempfile],READ_WRITE,0)
            optr = tptr2 }

         # If the dispersion axis is the Y axis, then copy the subset, but don't
         # bother swapping the axes

         else {
            call pargi(ispace1)
            call pargi(ispace2)
            call pargi(ispec1)
            call pargi(ispec2)
            call strcat(Memc[section],Memc[fsect],SZ_FNAME)
            tptr1 = immap(Memc[fsect],READ_ONLY,0)
            nnx = IM_LEN(tptr1,1)
            nny = IM_LEN(tptr1,2)
            tptr2 = sam_maket(nnx,nny,TY_REAL,tptr1)
            call strcpy(IM_HDRFILE(tptr2),Memc[tempfile],SZ_FNAME)
            one_l = 1
            call amovkl(one_l,v1,IM_MAXDIM)
            call amovkl(one_l,v2,IM_MAXDIM)
            while (imgnlr(tptr1,ptr1,v1) != EOF) {
               npix = impnlr(tptr2,ptr2,v2)
               call amovr(Memr[ptr1],Memr[ptr2],nnx)
            }
            call imunmap(tptr1)
            call imunmap(tptr2)
            tptr2 = immap(Memc[tempfile],READ_WRITE,0)
            optr = tptr2
         } }

      # Block for spectra with a TRACE

      else {

         # If the dispersion axis is the X axis, then do column access and 
         # swap this into a temporary file

         if (axis == 1) {

            # Set up for column access

            maxbuf = (ispace2 - ispace1)*1000
            iptr = immap(iname,READ_ONLY,0)
            co = comap(iptr,maxbuf)
           
            # Create temporary file

            nnx = ispace2 - ispace1 + 1
            nny = ispec2 - ispec1 + 1
            tptr1 = sam_maket(nnx,nny,TY_REAL,iptr)
            call strcpy(IM_HDRFILE(tptr1),Memc[tempfile],SZ_FNAME)
            one_l = 1
            call amovkl(one_l,v1,IM_MAXDIM)
            m = 0

            # Now access columns whilst simulaneously following the trace.
            # Write the rows to the temporary file...

            zero = apolr(trace[nord+2]*trace[nord+4],trace,nord+1)
            do j = ispec1,ispec2 {
               m = m + 1
               spec_pos = real(j)*trace[nord+4]
               xoff = apolr(spec_pos,trace,nord+1) - zero
               ixoff = nint(xoff)
               iy1 = ispace1 + ixoff
               iy2 = ispace2 + ixoff
               toff[m] = iy1 - 1
               ptr1 = cogetr(co,j,iy1,iy2)
               npix = impnlr(tptr1,ptr2,v1)
               call amovr(Memr[ptr1],Memr[ptr2],npix)
            }
            call counmap(co)
            call imunmap(iptr)
            call imunmap(tptr1)
            tptr1 = immap(Memc[tempfile],READ_WRITE,0)
            optr = tptr1 }
            
         # If the dispersion axis is the Y axis then follow the trace and write
         # the resulting subset to the temporary file...

         else {
            nnx = ispace2 - ispace1 + 1
            nny = ispec2 - ispec1 + 1
            tptr1 = sam_maket(nnx,nny,TY_REAL,NULL)
            call strcpy(IM_HDRFILE(tptr1),Memc[tempfile],SZ_FNAME)
            call imunmap(tptr1)

            # Evaluate trace for future reference...

            zero = apolr(trace[nord+2]*trace[nord+4],trace,nord+1)
            m = 0
            do j = ispec1,ispec2 {
               m = m + 1
               spec_pos = real(j)*trace[nord+4]
               xoff = apolr(spec_pos,trace,nord+1) - zero
               toff[m] = ispace1 + nint(xoff) - 1
            }

            # Find a block where the trace implies the same integral pixel offset
            # and write to that block

            to1 = toff[1]
            jst = 1
            jfn = 1
            while (jfn <= nny) {
               while (toff[jfn] - to1 == 0 && jfn <= nny) {
                  jfn = jfn + 1
               }
               jfn = jfn - 1
               iy1 = jst
               iy2 = jfn
               ix1 = 1
               ix2 = nnx

               # Open appropriate sections of files

               call strcpy(Memc[tempfile],Memc[fsect],SZ_FNAME)
               call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")
                  call pargi(ix1)
                  call pargi(ix2)
                  call pargi(iy1)
                  call pargi(iy2)
               call strcat(Memc[section],Memc[fsect],SZ_FNAME)
               tptr1 = immap(Memc[fsect],READ_WRITE,0)
               ix1 = toff[jst] + 1
               ix2 = ix1 + nnx - 1
               iy1 = ispec1 + iy1 - 1
               iy2 = ispec1 + iy2 - 1
               call strcpy(iname,Memc[fsect],SZ_FNAME)
               call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")
                  call pargi(ix1)
                  call pargi(ix2)
                  call pargi(iy1)
                  call pargi(iy2)
               call strcat(Memc[section],Memc[fsect],SZ_FNAME)
               tptr2 = immap(Memc[fsect],READ_ONLY,0)

               # Move the data over

               do i = 1,iy2-iy1+1 {
                  ptr1 = impl2r(tptr1,i)
                  ptr2 = imgl2r(tptr2,i)
                  call amovr(Memr[ptr2],Memr[ptr1],nnx)
               }               
               jst = jfn + 1
               jfn = jst
               if (jst <= nny) to1 = toff[jst]
               call imunmap(tptr1)
               call imunmap(tptr2) 
            }
         optr = immap(Memc[tempfile],READ_WRITE,0)
         }
      }

      # Ditch workspace

      call sfree(sp)

      # Return variable

      return (optr)
end
                           
