include <imhdr.h>

procedure sam_swap(iptr,optr)
#--------------------------------------------------------------------------------------
#      SAM_SWAP -- Swap the axes of an input file.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iptr:       (pointer) Input file pointer
#      (>) optr:       (pointer) Output file pointer
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

pointer iptr,optr

# Local variables

int len_blk

# Functions

int clgeti()

errchk imtranspose

begin

      # Get block size for swap...

      len_blk = clgeti("len_blk")

      # Use the IMAGES routine to swap the indicies...

      call imtranspose(iptr,optr,len_blk)
end

###  From here down has been taken from the core IRAF routine imtrans.
# Copyright(c) 1986 Association of Universities for Research in Astronomy Inc.

# IMTRANSPOSE -- Transpose an image.
#
# Divide the image into square blocks of size len_blk by len_blk.
# Transpose each block with a generic array transpose operator.

procedure imtranspose (im_in, im_out, len_blk)

pointer im_in                           # Input image descriptor
pointer im_out                          # Output image descriptor
int     len_blk                         # 1D length of transpose block

int     x1, x2, nx
int     y1, y2, ny
pointer buf_in, buf_out

pointer imgs2s(), imps2s(), imgs2i(), imps2i(), imgs2l(), imps2l()
pointer imgs2r(), imps2r(), imgs2d(), imps2d(), imgs2x(), imps2x()

begin
        # Output image is a copy of input image with dims transposed.

        IM_LEN (im_out, 1) = IM_LEN (im_in, 2)
        IM_LEN (im_out, 2) = IM_LEN (im_in, 1)

        # Break the input image into blocks of at most len_blk by len_blk.

        do x1 = 1, IM_LEN (im_in, 1), len_blk {
            x2 = x1 + len_blk - 1
            if (x2 > IM_LEN(im_in, 1))
               x2 = IM_LEN(im_in, 1)
            nx = x2 - x1 + 1

            do y1 = 1, IM_LEN (im_in, 2), len_blk {
                y2 = y1 + len_blk - 1
                if (y2 > IM_LEN(im_in, 2))
                   y2 = IM_LEN(im_in, 2)
                ny = y2 - y1 + 1

                # Switch on the pixel type to optimize IMIO.

                switch (IM_PIXTYPE (im_in)) {
                case TY_SHORT:
                    buf_in = imgs2s (im_in, x1, x2, y1, y2)
                    buf_out = imps2s (im_out, y1, y2, x1, x2)
                    call imtr2s (Mems[buf_in], Mems[buf_out], nx, ny)
                case TY_INT:
                    buf_in = imgs2i (im_in, x1, x2, y1, y2)
                    buf_out = imps2i (im_out, y1, y2, x1, x2)
                    call imtr2i (Memi[buf_in], Memi[buf_out], nx, ny)
                case TY_USHORT, TY_LONG:
                    buf_in = imgs2l (im_in, x1, x2, y1, y2)
                    buf_out = imps2l (im_out, y1, y2, x1, x2)
                    call imtr2l (Meml[buf_in], Meml[buf_out], nx, ny)
                case TY_REAL:
                    buf_in = imgs2r (im_in, x1, x2, y1, y2)
                    buf_out = imps2r (im_out, y1, y2, x1, x2)
                    call imtr2r (Memr[buf_in], Memr[buf_out], nx, ny)
                case TY_DOUBLE:
                    buf_in = imgs2d (im_in, x1, x2, y1, y2)
                    buf_out = imps2d (im_out, y1, y2, x1, x2)
                    call imtr2d (Memd[buf_in], Memd[buf_out], nx, ny)
                case TY_COMPLEX:
                    buf_in = imgs2x (im_in, x1, x2, y1, y2)
                    buf_out = imps2x (im_out, y1, y2, x1, x2)
                    call imtr2x (Memx[buf_in], Memx[buf_out], nx, ny)
                default:
                    call error (0, "unknown pixel type")
                }
            }
        }
end


# IMTR2 -- Generic transpose.  The arrays need not be identical.

procedure imtr2s (a, b, nx, ny)

short	a[nx, ny], b[ny, nx]
int	nx, ny, x, y

begin
	do x = 1, nx
	   do y = 1, ny
	       b[y, x] = a[x, y]
end



# IMTR2 -- Generic transpose.  The arrays need not be identical.

procedure imtr2i (a, b, nx, ny)

int	a[nx, ny], b[ny, nx]
int	nx, ny, x, y

begin
	do x = 1, nx
	   do y = 1, ny
	       b[y, x] = a[x, y]
end



# IMTR2 -- Generic transpose.  The arrays need not be identical.

procedure imtr2l (a, b, nx, ny)

long	a[nx, ny], b[ny, nx]
int	nx, ny, x, y

begin
	do x = 1, nx
	   do y = 1, ny
	       b[y, x] = a[x, y]
end



# IMTR2 -- Generic transpose.  The arrays need not be identical.

procedure imtr2r (a, b, nx, ny)

real	a[nx, ny], b[ny, nx]
int	nx, ny, x, y

begin
	do x = 1, nx
	   do y = 1, ny
	       b[y, x] = a[x, y]
end



# IMTR2 -- Generic transpose.  The arrays need not be identical.

procedure imtr2d (a, b, nx, ny)

double	a[nx, ny], b[ny, nx]
int	nx, ny, x, y

begin
	do x = 1, nx
	   do y = 1, ny
	       b[y, x] = a[x, y]
end



# IMTR2 -- Generic transpose.  The arrays need not be identical.

procedure imtr2x (a, b, nx, ny)

complex	a[nx, ny], b[ny, nx]
int	nx, ny, x, y

begin
	do x = 1, nx
	   do y = 1, ny
	       b[y, x] = a[x, y]
end



