procedure sam_trspec(iptr,spec_rng,nord,trace,nprof,profile)
#--------------------------------------------------------------------------------------
#      SAM_TRSPEC -- Trace a spectrum along the spectral axis and sum it into a
#                    1d array.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iptr:       (pointer) Pointer to input data header
#      (>) spec_rng:   (real[2]) Requested spectral range
#      (>) nord:       (int) Order of trace polynomial
#      (>) trace:      (real[nord+4]) Trace coefficients
#      (>) nprof:      (int) Size of 1d array 
#      (<) profile:    (real[nprof]) 1d array with summed spectrum
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:      09-NOV-1993 -- Fixed bug in interpolation for 
#                                  section files (JRL)
#
#--------------------------------------------------------------------------------------

# Parameters

pointer iptr
int nord,nprof
real trace[nord+4],profile[nprof],spec_rng[2]

# Local variables

pointer ptr1
real yscale,zero,x,y,w0,w1,w2,v1,v2
int j,i,irow

# Functions

pointer imgl2r()
real apolr()

errchk imgl2r

begin

      # Set up some constants

      yscale = trace[nord+4]
      zero = apolr(trace[nord+2]*yscale,trace,nord+1)

      # Loop over the spectral range to be considered. Get the starting
      # points and the weights organised

      irow = 0
      do j = nint(spec_rng[1]),nint(spec_rng[2]) {
         irow = irow + 1
         y = real(j)*yscale
         x = apolr(y,trace,nord+1) - zero
         v2 = x - int(x)
         v1 = 1.0 - v2
         if (nint(x) - int(x) > 0) {
            w0 = v1
            w1 = v2
            w2 = 0.0 }
         else {
            w0 = 0.0
            w1 = v1
            w2 = v2
         }

         # Get the relevant row of data

         ptr1 = imgl2r(iptr,irow)

         # Now loop over the spacial pixels.  Do a linear interpolation as you
         # go along.

         do i = 2,nprof-1 {
            profile[i] = profile[i] + Memr[ptr1+i-2]*w0 + Memr[ptr1+i-1]*w1 + 
            Memr[ptr1+i]*w2
         }
         profile[1] = profile[1] + Memr[ptr1]
         profile[nprof] = profile[nprof] + Memr[ptr1+nprof-1]
      }
end

