include <imhdr.h>

procedure sam_wsect(iptr,oname,ispace1,ispace2,ispec1,ispec2,track,axis,
   nt,toff)
#--------------------------------------------------------------------------------------
#      SAM_WSECT -- Write data from a temporary section file back into an output file
#                   following the trace if necessary and taking the spectral axis
#                   into account.
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iptr:       (pointer) Pointer to input section file
#      (>) oname:      (char[ARB]) Name of output file
#      (>) ispace1:    (int) Lower limit in spacial domain of section to be read
#      (>) ispace2:    (int) Upper limit in spacial domain of section to be read
#      (>) ispec1:     (int) Lower limit in spectral domaine of section to be read
#      (>) ispec2:     (int) Upper limit in spectral domaine of section to be read
#      (>) track:      (bool) TRUE if a trace file exists
#      (>) axis:       (int) Spectral axis
#      (>) nt:         (int) Number of elements in offset array
#      (>) toff:       (short[nt]) Array of offsets
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

pointer iptr
char oname[ARB]
int ispace1,ispace2,ispec1,ispec2,axis,nt
short toff[nt]
bool track

# Local variables

pointer sp,section,fsect,tptr1,tptr2,ptr1,ptr2,tempname
long one_l,v1[IM_MAXDIM],v2[IM_MAXDIM]
int npix,ix1,ix2,iy1,iy2,nnx,jst,jfn,nny
short to1

# Functions

pointer immap()
int impnlr(),imgnlr()

errchk imunmap,immap,sam_swap,impnlr,imgnlr

begin

      # Get some stack workspace

      call smark(sp)
      call salloc(section,SZ_FNAME,TY_CHAR)
      call salloc(fsect,SZ_FNAME,TY_CHAR)
      call salloc(tempname,SZ_FNAME,TY_CHAR)

      # Close the original file and save the file name

      call strcpy(IM_HDRFILE(iptr),Memc[tempname],SZ_FNAME)
      call imunmap(iptr)

      # Block for spectra with no TRACE

      if (!track) {
         call strcpy(oname,Memc[fsect],SZ_FNAME)
         call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")

         # If the dispersion axis is the X axis then open a subset of
         # the output file and swap the input file into it.

         if (axis == 1) {
            call pargi(ispec1)
            call pargi(ispec2)
            call pargi(ispace1)
            call pargi(ispace2)
            call strcat(Memc[section],Memc[fsect],SZ_FNAME)
            tptr1 = immap(Memc[fsect],READ_WRITE,0)
            tptr2 = immap(Memc[tempname],READ_ONLY,0)
            call sam_swap(tptr2,tptr1)
            call imunmap(tptr1)
            call imunmap(tptr2) }
         
         # If the dispersion axis is the Y axis, then just copy the file straight
         # into the output file section.

         else {
            call pargi(ispace1)
            call pargi(ispace2)
            call pargi(ispec1)
            call pargi(ispec2)
            call strcat(Memc[section],Memc[fsect],SZ_FNAME)
            tptr1 = immap(Memc[fsect],READ_WRITE,0)
            tptr2 = immap(Memc[tempname],READ_ONLY,0)
            one_l = 1
            call amovkl(one_l,v1,IM_MAXDIM)
            call amovkl(one_l,v2,IM_MAXDIM)
            while (impnlr(tptr1,ptr1,v1) != EOF) {
               npix = imgnlr(tptr2,ptr2,v2) 
               call amovr(Memr[ptr2],Memr[ptr1],npix)
            }
            call imunmap(tptr1)
            call imunmap(tptr2)
         } }

      # Block for spectra with a TRACE

      else {

         # If the dispersion axis is the X axis, then write results to 
         # the correct column, while following the trace.

         if (axis == 1) {
            nnx = ispec2 - ispec1 + 1
            nny = ispace2 - ispace1 + 1

            # Find a block where the trace implies the same integral pixel offset
            # and write to that block

            to1 = toff[1]
            jst = 1
            jfn = 1
            while (jfn <= nnx) {
               while (toff[jfn] - to1 == 0 && jfn <= nnx) {
                  jfn = jfn + 1
               }
               jfn = jfn - 1
               ix1 = ispec1 + jst - 1
               ix2 = ispec1 + jfn - 1
               iy1 = toff[jst] + 1
               iy2 = iy1 + nny - 1

               # Open appropriate sections of files

               call strcpy(oname,Memc[fsect],SZ_FNAME)
               call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")
                  call pargi(ix1)
                  call pargi(ix2)
                  call pargi(iy1)
                  call pargi(iy2)
               call strcat(Memc[section],Memc[fsect],SZ_FNAME)
               tptr1 = immap(Memc[fsect],READ_WRITE,0)
               iy1 = 1
               iy2 = nny
               call strcpy(Memc[tempname],Memc[fsect],SZ_FNAME)
               call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")
                  call pargi(iy1)
                  call pargi(iy2)
                  call pargi(jst)
                  call pargi(jfn)
               call strcat(Memc[section],Memc[fsect],SZ_FNAME)
               tptr2 = immap(Memc[fsect],READ_ONLY,0)

               # Swap the data over

               call sam_swap(tptr2,tptr1)
               jst = jfn + 1
               jfn = jst
               if (jst <= nnx) to1 = toff[jst]
               call imunmap(tptr1)
               call imunmap(tptr2) 
            }
         }

         # If the dispersion axis is the Y axis, then write the results to 
         # the correct row, while following the trace.

         else {
            nnx = ispace2 - ispace1 + 1
            nny = ispec2 - ispec1 + 1

            # Find a block where the trace implies the same integral pixel offset
            # and write to that block

            to1 = toff[1]
            jst = 1
            jfn = 1
            while (jfn <= nny) {
               while (toff[jfn] - to1 == 0 && jfn <= nny) {
                  jfn = jfn + 1
               }
               jfn = jfn - 1
               iy1 = ispec1 + jst - 1
               iy2 = ispec1 + jfn - 1
               ix1 = toff[jst] + 1
               ix2 = ix1 + nnx - 1

               # Open appropriate sections of files

               call strcpy(oname,Memc[fsect],SZ_FNAME)
               call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")
                  call pargi(ix1)
                  call pargi(ix2)
                  call pargi(iy1)
                  call pargi(iy2)
               call strcat(Memc[section],Memc[fsect],SZ_FNAME)
               tptr1 = immap(Memc[fsect],READ_WRITE,0)
               ix1 = 1
               ix2 = nnx
               call strcpy(Memc[tempname],Memc[fsect],SZ_FNAME)
               call sprintf(Memc[section],SZ_FNAME,"[%d:%d,%d:%d]")
                  call pargi(ix1)
                  call pargi(ix2)
                  call pargi(jst)
                  call pargi(jfn)
               call strcat(Memc[section],Memc[fsect],SZ_FNAME)
               tptr2 = immap(Memc[fsect],READ_ONLY,0)

               # Move the data over

               one_l = 1
               call amovkl(one_l,v1,IM_MAXDIM)
               call amovkl(one_l,v2,IM_MAXDIM)
               while (imgnlr(tptr2,ptr2,v2) != EOF) {
                  npix = impnlr(tptr1,ptr1,v1)
                  call amovr(Memr[ptr2],Memr[ptr1],nnx)
               }               
               jst = jfn + 1
               jfn = jst
               if (jst <= nny) to1 = toff[jst]
               call imunmap(tptr1)
               call imunmap(tptr2) 
            }

         }
      }

      # Reopen input file...

      iptr = immap(Memc[tempname],READ_WRITE,0)

      # Ditch the workspace

      call sfree(sp)
end

              
