include <gset.h>

define  HELPFIL   "rgo$sam/curfile/findhlp.key"
define  PROMPT    "find: keystrokes for defining spatial limits"

procedure sub_findi(iptr,dekker,trace,nord,spec_rng,nx,work,
   profile,pos,width,sky1,sky2)
#------------------------------------------------------------------------------
#      SUB_FINDI -- Main subroutine for finding a spectrum interactively
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iptr:       (pointer) Pointer to input file
#      (>) dekker:     (real[2]) Limits of dekker
#      (>) trace:      (real[nord+4]) Trace coefficients
#      (>) nord:       (int) Order of trace polynomial
#      (>) spec_rng:   (real[2]) Requested spectral range
#      (>) nx:         (int) Number of pixels between dekker limits
#      (W) work:       (real[nx]) Workspace for plotting routine
#      (W) profile:    (real[nx]) Workspace
#      (<) pos:        (real) Position of the object
#      (<) width:      (real[2]) Limits of seeing disc for the object
#      (<) sky1:       (real[2]) First sky band for the object
#      (<) sky2:       (real[2]) Second sky band for the object
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:      08-NOV-1993 -- Major upgrade to remove object loop (JRL)
#                   16-AUG-1994 -- Added help in cursor mode. (JRL)
#
#------------------------------------------------------------------------------

# Parameters

pointer iptr
int nord,nx,width[2],sky1[2],sky2[2]
real dekker[2],trace[nord+4],spec_rng[2],profile[nx],pos,work[nx]

# Local variables

pointer sp,cmd,gp
int i,nst,wcs,key,idum
real xmin,xmax,ymin,ymax,dummy,val[2],dum2

# Functions

real alovr(),ahivr()
pointer gopen()
int clgcur()

errchk sam_trspec,gopen

begin

      # Get some stack workspace

      call smark(sp)
      call salloc(cmd,SZ_LINE,TY_CHAR)

      # Begin by summing the spectrum along the spectral axis

      call sam_trspec(iptr,spec_rng,nord,trace,nx,profile)

      # Get min and max of profile and set up axis array

      do i = 1,nx {
         work[i] = dekker[1] + real(i) - 1.0
      }
      xmin = work[1]
      xmax = work[nx]
      ymin = alovr(profile,nx)
      ymax = ahivr(profile,nx)

      # Set up the plot device

      gp = gopen("stdgraph",NEW_FILE,STDGRAPH)

      # Set up the window and plot the profile

      call gswind(gp,xmin,xmax,ymin,ymax)
      call glabax(gp,"Summed Dekker Profile","X","Dekker Profile")
      call gpline(gp,work,profile,nx)

      # Get position of the peak

      call printf("Indicate the centre of spectrum\n")
      i = 0
      while (i < 1) {
         nst = clgcur("graphcur",pos,dummy,wcs,key,Memc[cmd],SZ_LINE)
         switch (key) {
            case 'q':
               call error(0,"Point selection ended prematurely")
            case '?':
               call gpagefile(gp,HELPFIL,PROMPT)
            default:
               i = i + 1
         }
      }
      call gmark(gp,pos,dummy,GM_CROSS,1.0,1.0)

      # Get the position of the limits of the spectrum

      call printf("Indicate the limits of the spectrum with 2 points\n")
      i = 0
      while (i < 2) {
         nst = clgcur("graphcur",dum2,dummy,wcs,key,Memc[cmd],SZ_LINE)
         switch (key) {
            case 'q':
               call error(0,"Point selection ended prematurely")
            case '?':
               call gpagefile(gp,HELPFIL,PROMPT)
            default:
               i = i + 1
               val[i] = dum2
               call gmark(gp,val[i],dummy,GM_CROSS,1.0,1.0)
         }
      }
      call sam_band(val[1],val[2])
      width[1] = nint(val[1])
      width[2] = nint(val[2])

      # Get position of first sky band

      call printf("Indicate the limits of the first sky band\n")
      i = 0
      while (i < 2) {
         nst = clgcur("graphcur",dum2,dummy,wcs,key,Memc[cmd],SZ_LINE)
         switch (key) {
            case 'q':
               call error(0,"Point selection ended prematurely")
            case '?':
               call gpagefile(gp,HELPFIL,PROMPT)
            default:
               i = i + 1
               val[i] = dum2
               call gmark(gp,val[i],dummy,GM_CROSS,1.0,1.0)
         }
      }
      call sam_band(val[1],val[2])
      sky1[1] = nint(val[1])
      sky1[2] = nint(val[2])

      # Get position of second sky band

      call printf("Indicate the limits of the second sky band\n")
      i = 0
      while (i < 2) {
         nst = clgcur("graphcur",dum2,dummy,wcs,key,Memc[cmd],SZ_LINE)
         switch (key) {
            case 'q':
               call error(0,"Point selection ended prematurely")
            case '?':
               call gpagefile(gp,HELPFIL,PROMPT)
            default:
               i = i + 1
               val[i] = dum2
               call gmark(gp,val[i],dummy,GM_CROSS,1.0,1.0)
         }
      }
      call sam_band(val[1],val[2])
      sky2[1] = nint(val[1])
      sky2[2] = nint(val[2])
        
      # Make sure the "first" sky band is the left-hand sky band

      if (sky1[1] > sky2[1]) {
         do i = 1,2 {
            idum = sky2[i]
            sky2[i] = sky1[i]
            sky1[i] = idum
         }
      }

      # Make sure the sky bands don't overlap

      if (sky2[1] < sky1[2])
         call error(0,"Your defined sky bands overlap")    

      # Make sure the sky bands don't overlap onto the object

      if (sky1[2] > width[1] || sky2[1] < width[2])
         call error(0,"A sky band overlaps the object")

      # Close up the graphics stream

      call gclose(gp)

      # Free workspace

      call sfree(sp)
end
