procedure sub_optx(iptr,eptr,nx,ny,axis,nt,toff,ext_thr,pptr,qual,output,outvar)
#--------------------------------------------------------------------------------------
#      SUB_OPTX -- Main subroutine for extracting spectra given optimal extraction
#                  profile weights
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) iptr:       (pointer) Pointer to input file
#      (>) eptr:       (pointer) Pointer to input error file
#      (>) nx:         (int) X dimension of input data
#      (>) ny:         (int) Y dimension of input data
#      (>) axis:       (int) Dispersion axis
#      (>) nt:         (int) Size of offset array
#      (>) toff:       (short[nt]) Offset array
#      (>) ext_thr:    (real) Threshold for rejection of cosmic rays
#      (>) pptr:       (pointer) Pointer to input profile file
#      (W) qual:       (short[nx,ny]) Array used for marking bad pixels
#      (<) output:     (real[ny]) Output spectrum
#      (<) outvar:     (real[ny]) Output variance array
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        22-FEB-1993
#      UPDATE:      03-MAR-1993 -- Now uses offset array to give correct location
#                                  of zapped pixels
#
#--------------------------------------------------------------------------------------

# Parameters

pointer iptr,eptr,pptr
int nx,ny,axis,nt
short qual[nx,ny],toff[nt]
real ext_thr,output[ny],outvar[ny]

# Local variables

pointer ptr1,ptr2,ptr3
int ndeltot,iter,i,j,irem,jrem,ioff
bool cut
real rmax,sum1,sum2,dat,var,prof,resid

# Functions 

pointer imgl2r()

errchk imgl2r

begin

      # Set up a few things...

      ndeltot = 0
      iter = 0
      cut = true

      # Loop until you haven't deleted any more points...

      while (cut) {
         iter = iter + 1
         rmax = 0.0
         cut = false

         # Loop through the spectral axis...

         do j = 1,ny {
            sum1 = 0.0
            sum2 = 0.0
            ptr1 = imgl2r(iptr,j)
            ptr2 = imgl2r(eptr,j)
            ptr3 = imgl2r(pptr,j)

            # Loop through the seeing profile

            do i = 1,nx {
               if (qual[i,j] != 1) {
                  dat = Memr[ptr1+i-1]
                  var = Memr[ptr2+i-1]
                  prof = Memr[ptr3+i-1]
                  sum1 = sum1 + prof*dat/var
                  sum2 = sum2 + prof*prof/var
               }
            }

            # If there is a positive contribution, then calculate the spectrum
            # and its variance. Otherwise flag it by making it and its variance zero

            if (sum2 > 0.0) {
               output[j] = sum1/sum2
               outvar[j] = 1.0/sum2 }
            else {
               output[j] = 0.0
               outvar[j] = 0.0
            }

            # Now see if there is anything worth clipping...

            do i = 1,nx {
               dat = Memr[ptr1+i-1]
               var = Memr[ptr2+i-1]
               prof = Memr[ptr3+i-1]
               if (qual[i,j] == 0) {
                  resid = abs(output[j]*prof - dat)/sqrt(var)
                  if (resid > ext_thr) {
                     cut = true
                     if (resid > rmax) {
                        rmax = resid
                        irem = i
                        jrem = j
                     }
                  }
               }
            }
         } # End of row loop

         # Clip out the worst point if it was worth doing so...

         if (cut) {
            ndeltot = ndeltot + 1
            qual(irem,jrem) = 1
            call printf("Iteration %d:  Pixel clipped at: %d %d \n")
               call pargi(iter)
            if (nt == 1) {
               ioff = toff[1] }
            else {
               ioff = toff[jrem]
            }
            if (axis == 2) {
               call pargi(irem+ioff)
               call pargi(jrem) }
            else {
               call pargi(jrem)
               call pargi(irem+ioff)
            }
            call printf("%16t with absolute residual %8.2f sigmas\n")
               call pargr(rmax)
         }
      } # End of iteration loop  

      # A final message...

      call printf("A total of %d pixels have been deleted\n")
         call pargi(ndeltot)
end
