include <error.h>

procedure t_whole()
#------------------------------------------------------------------------------
#
#   ---------
#   W H O L E
#   ---------
#
#   Description
#   -----------
#   Set up a frame so that all the information between the two dekker 
#   limits will be extracted
#
#   
#   Scope of program
#   ----------------
#   - Dimesionsality irrelevant
#   - Data types irrelevant
#   - Variance arrays irrelevant
#   - Batch execution supported
#
#
#   Environment
#   -----------
#   IRAF
#
#
#   Language
#   --------
#   SPP
#
#
#   Parameters
#   ----------
#   input      Image(s) with spectra to be extracted (string)(prompted for)
#
#   axis       Axis to which spectra are parallel -- i.e. dispersion axis.
#              1 = X, 2 = Y. (int)(hidden)
#
#   track      TRUE if a trace file is being used. (bool)(hidden)
#
#   trace      Name of the trace file if required. (string)(hidden)
#
#   dekker     Name of the dekker file. (string)(hidden)
#
#
#   Propogation of data structure
#   -----------------------------
#   Not relevant
#
#   
#   Method
#   ------
#   - The requested information and the dekker limits are written to the
#     input file header.  This can then be used by 'extract' to extract 
#     the data
#
#
#   External functions & subroutines called
#   ---------------------------------------
#   Library CLIO:
#      clgetb
#      clgeti
#      clgstr
#
#   Library ETC:
#      erract
#
#   Library FMTIO:
#      pargi
#      sprintf
#   
#   Library IMIO:
#      imaddb
#      imaddi
#      imaddr
#      imastr
#      imgl1r
#      immap
#      imtclose
#      imtgetim
#      imtopen
#      imunmap
#
#   Library MEMIO:
#      salloc
#      sfree
#      smark
#
#   Library SAMSUB:
#      sam_gdekker
#      sam_odekker
#
#   Library VOPS
#      amovr
#
#
#   Internal subroutines called
#   ---------------------------
#   None
#
#
#   INCLUDE statements
#   ------------------
#   include <error.h>
#
#
#   Possible future upgrades
#   ------------------------
#
#
#   Author/s
#   --------
#   Jim Lewis      RGO
#
#
#   History
#   -------
#   26-FEB-93     - Original program based on VAX program SAM_WHOLE
#   12-NOV-93     - Modified to do multiple objects (JRL)
#
#
#------------------------------------------------------------------------------

# Local variables

int        axis               # Dispersion axis
pointer    cp1                # Pointer for dummy character
real       dek[2]             # Location of current dekker limits
pointer    dname              # Pointer for dekker file name
pointer    dptr               # Pointer for dekker file header
real       dummy              # Dummy real variable
int        idum               # Dummy integer variable
pointer    imtlist            # Pointer to image file list
pointer    iname              # Pointer for input file name
int        iobj               # Counter for object number                   
pointer    iptr               # Pointer for input file header
int        j                  # Integer counter
int        list               # List descriptor
int        nobj               # Number of objects
pointer    ptr_de             # Pointer for dekker file
pointer    sp                 # Stack marker
pointer    tname              # Pointer for trace file name
bool       track              # TRUE if a trace file exists

# Functions

pointer immap(),sam_gdekker()
int clgeti(),imtopen(),imtgetim()
bool clgetb()

begin

      # Mark stack and get some workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(dname,SZ_FNAME,TY_CHAR)
      call salloc(tname,SZ_FNAME,TY_CHAR)
      call salloc(imtlist,SZ_LINE,TY_CHAR)
      call salloc(cp1,SZ_LINE,TY_CHAR)

      # Get name of input file list

      call clgstr("input",Memc[imtlist],SZ_LINE)

      # Get axis, dekker file name and trace file name

      axis = clgeti("axis")
      track = clgetb("track")
      if (track) call clgstr("trace",Memc[tname],SZ_FNAME)
      call clgstr("dekker",Memc[dname],SZ_FNAME)
      iferr (call sam_odekker(dname,nobj,dptr)) call erract(EA_ERROR)

      # Set up the input file list

      list = imtopen(Memc[imtlist])

      # Loop for all of the files in the list

      while (imtgetim(list,Memc[iname],SZ_FNAME) != EOF) {

         # Open the file...

         iferr (iptr = immap(Memc[iname],READ_WRITE,0)) {
            call erract(EA_WARN)
            next
         }

         # Write some of these parameters into the header of the file

         call imaddi(iptr,"axis",axis)
         call imaddb(iptr,"track",track)
         if (track) call imastr(iptr,"trace",Memc[tname])
         call imastr(iptr,"dekker",Memc[dname])
         call imaddi(iptr,"nobj",nobj)

         # Loop for objects...

         do iobj = 1,nobj {

            # Set the dekker limits for this object
        
            ptr_de = sam_gdekker(dptr,iobj)
            call amovr(Memr[ptr_de],dek,2)

            # Set a dummy position

            call sprintf(Memc[cp1],SZ_LINE,"pos%d")
               call pargi(iobj)
            dummy = 0.5*(dek[1] + dek[2])
            call imaddr(iptr,Memc[cp1],dummy)

            # Set the spectral width as the limits of the dekker

            do j = 1,2 {
               call sprintf(Memc[cp1],SZ_LINE,"swid%d_%d")
                  call pargi(iobj)
                  call pargi(j)
               idum = nint(dek[j])
               call imaddi(iptr,Memc[cp1],idum)
            }

            # The first sky band

            do j = 1,2 {
               call sprintf(Memc[cp1],SZ_LINE,"skya%d_%d")
                  call pargi(iobj)
                  call pargi(j)
               call imaddi(iptr,Memc[cp1],0)
            }

            # The second sky band

            do j = 1,2 {
               call sprintf(Memc[cp1],SZ_LINE,"skyb%d_%d")
                  call pargi(iobj)
                  call pargi(j)
               call imaddi(iptr,Memc[cp1],0)
            }
         } # End object loop

         # Unmap the file

         call imunmap(iptr)
      } # End image template loop

      # Close the image list

      call imtclose(list)

      # Close dekker file

      call imunmap(dptr)

      # Ditch the workspace

      call sfree(sp)
end

                                 
