include <mach.h>

procedure stats(nelm,array,ave,sig,clip)

#--------------------------------------------------------------------------------------
#      STATS -- Do average and sigma stats 
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) nelm:       (int) Number of elements in array
#      (>) array:      (real[nelm] Input array
#      (<) ave:        (real) Average of array
#      (<) sig:        (real) Sigma of array
#      (>) clip:       (real) Number of sigma for clipping
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int      nelm
real     array[nelm],ave,sig,clip

# Local variables

int      i,k,ntot
real     val,cut
double   sum,sumsq,dval,dave,sig2,dntot

begin

      # Do statistics with a clip

      cut = MAX_REAL
      ave = 0.0

      # Do two iterations -- for the first one the clip threshold is so high
      # that clipping can't be done

      do k = 1,2 {

         # Initialise summation counters

         sum = 0.0d0; sumsq = 0.0d0
         ntot = 0

         # Now do the summations

         do i = 1,nelm {
            val = array[i]
            if (abs(val-ave) <= cut) {
               dval = double(val)
               ntot = ntot + 1
               sum = sum + dval
               sumsq = sumsq + dval*dval
            }
         }

         # If all the data in the array was indefinite, then signal an error here...

         if (ntot == 0) break

         # Now find mean and sigma. Check for sigma**2 < 0.0

         dntot = double(ntot)
         dave = sum/dntot
         ave = real(dave)
         sig2 = sumsq/dntot - dave*dave
         if (sig2 >= 0.0d0) {
            sig = real(sqrt(sig2))
            cut = clip*sig }
         else {
            break 
         }
      }

      # Check for funny results...

      if (ntot == 0) {
         call error(1,"All points clipped out of stats calculation") }
      else if (sig2 < 0.0) {
         call error(2,"Bad value of sigma")
      }

end
