procedure w_sum_frames(nelm,array,var,isuma,isumw,osuma,osumw)
#--------------------------------------------------------------------------------------
#      W_SUM_FRAMES -- Do running sum of frames with weights
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) nelm:       (int) Number of elements in array
#      (>) array:      (real[nelm]) Input array
#      (>) var:        (real[nelm]) Variance of input array
#      (>) isuma:      (real[nelm]) Current running weighted sum
#      (>) isumw:      (real[nelm]) Current running sum of weights
#      (<) osuma:      (real[nelm]) Output running weighted sum
#      (<) osumw:      (real[nelm]) Output running sum of weights
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:      11-FEB-1993 -- Now checks for zero divides (JRL)
#
#--------------------------------------------------------------------------------------

# Parameters

int nelm
real array[nelm],var[nelm],isuma[nelm],isumw[nelm],osuma[nelm],osumw[nelm]

# Local variables

int i
real weight,val

# Functions

real check_div()

begin

      # Do the weighted sum

      do i = 1,nelm {
         val = array[i]
         weight = 1.0/check_div(var[i])
         osuma[i] = isuma[i] + weight*val
         osumw[i] = isumw[i] + weight
      }
end
