procedure cos_rays(nx,ny,iptr,threshold,cell_size,ncos)
#--------------------------------------------------------------------------------------
#      COS_RAYS -- Rough and ready cosmic ray zapper
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) nx:         (int) X dimension of data subset 
#      (>) ny:         (int) Y dimension of data subset
#      (>) iptr:       (pointer) Pointer to input data subset
#      (>) threshold:  (real) Threshold in counts over the second highest pixel in
#                             a cell for which a pixel is considered to be z CR.
#      (>) cell_size:  (int) Size of the cell (square) for analysis
#      (<) ncos:       (int) Number of cosmic rays fixed in this subset
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        11-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

pointer iptr
int nx,ny,cell_size,ncos
real threshold

# Local variables

pointer wpt1,wpt2,wpt3,ptr1
int nw,npts,ncent,idc,jst,ist,jfn,ifn,i,j,irem,ii,jj,ninc
real hival,second,flux,replace,val,sum

# Functions

pointer imgs2r()
real ahivr(),asumr()

begin

      # Get some workspace for storage of results

      call malloc(wpt1,100,TY_REAL)
      call malloc(wpt2,100,TY_INT)
      call malloc(wpt3,100,TY_INT)
      nw = 100

      # Initialise a few things...

      npts = cell_size**2
      ncent = npts/2 + 1
      ncos = 0

      # Loop for all the pixels except those on the borders of a cell

      idc = cell_size/2
      jst = idc + 1
      ist = idc + 1
      jfn = ny - idc
      ifn = nx - idc
      do j = jst,jfn {
         do i = ist,ifn {

            # Is this pixel a local maxium?

            ptr1 = imgs2r(iptr,i-idc,i+idc,j-idc,j+idc)
            hival = ahivr(Memr[ptr1],npts)
            if (Memr[ptr1+ncent-1] < hival) next

            # Find next highest value...

            second = Memr[ptr1]
            do ii = 2,npts {
               if (ii == ncent) next
               val = Memr[ptr1+ii-1]
               if (val > second) {
                  second = val
                  irem = ii
               }
            }

            # Find the total flux in the cell excluding these two points and see if the
            # flux of the central point exceeds this mean by the threshold.

            flux = (asumr(Memr[ptr1],npts) - hival - second)/real(npts-2)
            if (hival < flux + threshold) next

            # Find a value to use instead of the real one...

            sum = 0.0
            jj = ncent - cell_size
            ninc = 0
            if (jj != irem) {
               sum = sum + Memr[ptr1+jj-1]
               ninc = ninc + 1
            }
            jj = ncent + cell_size
            if (jj != irem) {
               sum = sum + Memr[ptr1+jj-1]
               ninc = ninc + 1
            }
            jj = ncent + 1
            if (jj != irem) {
               sum = sum + Memr[ptr1+jj-1]
               ninc = ninc + 1
            }
            jj = ncent - 1
            if (jj != irem) {
               sum = sum + Memr[ptr1+jj-1]
               ninc = ninc + 1
            }
            replace = sum/real(ninc)
            ncos = ncos + 1

            # Store the results away...

            if (ncos > nw) {
               nw = nw + 100
               call realloc(wpt1,nw,TY_REAL)
               call realloc(wpt2,nw,TY_INT)
               call realloc(wpt3,nw,TY_INT)
            }
            Memr[wpt1+ncos-1] = replace
            Memi[wpt2+ncos-1] = i
            Memi[wpt3+ncos-1] = j
         }
      }

      # Now replace all the naughty values...

      call rep_cr(iptr,nw,ncos,Memr[wpt1],Memi[wpt2],Memi[wpt3])
end
