procedure dud_mask_set(fd,nx,ny,mask,ndud)

#--------------------------------------------------------------------------------------
#      DUD_MASK_SET -- Set up the mask for dud pixels
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) fd:         (int) Descriptor for chip file
#      (>) nx:         (int) X dimension of data
#      (>) ny:         (int) Y dimension of data
#      (<) mask:       (short[nx.ny]) Output mask
#      (<) ndud:       (int) Number of dud pixels in mask
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int fd,nx,ny,ndud
short mask[nx,ny]

# Local variables

pointer sp,lp1
int iline,status,ip,lims[4],ix1,ix2,iy1,iy2,i,j

# Functions

int ctoi()
real asums()

define errexit_ 99

begin

     # Get a bit of workspace

     call smark(sp)
     call salloc(lp1,SZ_LINE,TY_CHAR)

     # Read the dud pixel info and fix the mask

      ndud = 0
      iline = 0
      status = 0
      iferr (call chip_data(iline,fd,Memc[lp1])) goto errexit_
      iferr (call chip_data(iline,fd,Memc[lp1])) goto errexit_
      while (status == 0) {

         # Get the info off the line now

         ip = 1
         for (i = 1; i <= 4 && ctoi(Memc[lp1],ip,lims[i]) != 0; i = i + 1) ;

         # If all went well, then find the regions depending on zero point of section

         if (i == 5) {
            ix1 = lims[1]
            ix2 = lims[2] 
            iy1 = lims[3] 
            iy2 = lims[4] 

            # If these make sense, then set the mask

            if ((ix1 >= 1) && (iy1 >= 1) && (ix2 <= nx) && (iy2 <= ny)) {
               do j = iy1,iy2 {
                  do i = ix1,ix2 {
                     mask[i,j] = 1
                  }
               }
            }
         }

         # Read the next line

         iferr (call chip_data(iline,fd,Memc[lp1])) break
      }

      # Find out how many dud pixels there really are...

      ndud = nint(asums(mask,nx*ny))

      # Free the workspace

errexit_
      call sfree(sp)
end
