procedure dud_pixels(nx,ny,array,mask,drow,iys,nfixed,nunable,optr)
#--------------------------------------------------------------------------------------
#      DUD_PIXELS -- Remove dud pixels by interpolating to nearest neighbours
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) nx:         (int) X dimension of input data subset
#      (>) ny:         (int) Y dimension of input data subset
#      (>) array:      (real[nx,ny]) Input data subset
#      (>) mask:       (short[nx,ny]) Input mask subset
#      (>) drow:       (int) Row of subset to be examined
#      (>) iys:        (int) Row of original data frame where subset starts
#      (<) nfixed:     (int) Number of dud pixels fixed
#      (<) nunable:    (int) Number of dud pixels which couldn't be fixed
#      (>) optr:       (pointer) Pointer to output data array
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        11-FEB-1993
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int nx,ny,nfixed,nunable,drow,iys
short mask[nx,ny]
real array[nx,ny]
pointer optr

# Local variables

int ndud,i,ix,iy,ix1,ix2,iy1,iy2,ngadj,w,val,idiff
real sum,sumw
pointer ptr1

# Functions

real asums()
pointer imps2r()

define exitpt_ 99

begin

      # Begin by checking whether there are any dud pixels.  Sum the mask along
      # the middle row (if there are only 2 rows, say at top of a frame, then
      # this isn't the middle row...fussy fussy).

      ndud = nint(asums(mask[1,drow],nx))
      if (ndud == 0) goto exitpt_

      # Initialise a few things...

      nfixed = 0
      nunable = 0

      # If there are any naughty pixels then fix them...

      do i = 1,nx {
         if (mask[i,drow] == 1) {
            ix1 = max(1,i-1)
            ix2 = min(nx,i+1)
            iy1 = max(1,drow-1)
            iy2 = min(ny,drow+1)

            # Initialise summations...

            sum = 0.0; sumw = 0.0; ngadj = 0

            # Do an average-of-neighbours interpolation so long as there
            # is at lease one good pixel touching the bad one.  If the
            # pixel in question is surrounded by bad pixels, that's tough...

            do iy = iy1,iy2 {
               do ix = ix1,ix2 {
                  idiff = abs(ix-i) + abs(iy-drow)
                  if (mask[ix,iy] == 0) {
                     val = array[ix,iy]
                     if (idiff == 1) {
                        ngadj = ngadj + 1
                        w = 1.0 }
                     else {
                        w = 0.71
                     }
                     sum = sum + val*w
                     sumw = sumw + w 
                  }
               }
            }

            # If there is at least one good adjacent (e.g. not a corner)
            # pixel, then do the average.  If all the adjacent pixels
            # are bad, then leave the pixel alone.

            if (ngadj >= 1) {
               nfixed = nfixed + 1
               ptr1 = imps2r(optr,i,i,drow+iys-1,drow+iys-1)
               Memr[ptr1] = sum/sumw}
            else {
               nunable = nunable + 1
            }
         }
      } # End of pixel do loop

      # Exit point

exitpt_
       return
end
