include <imhdr.h>

procedure open_mean(fname,nx,ny,sect,type,want_err,should_map,ex_err,
   ff,use_sect,read2,phot,iptr,eptr)

#--------------------------------------------------------------------------------------
#      OPEN_MEAN -- Open a mean bias/dark/flat frame
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) fname:      (char) Name of mean file
#      (>) nx:         (int) X dimension of data
#      (>) ny:         (int) Y dimension of data
#      (>) sect:       (char) Data section to be used
#      (>) type:       (char) Type of file you are currently reducing
#      (>) want_err:   (bool) True if you want output error arrays.
#      (>) should_map: (bool) True if you want to map error arrays of mean files
#      (<) ex_err:     (bool) True if an error array exists in the mean file
#      (>) ff:         (bool) True if the mean file currently is a mean flat file
#      (>) use_sect:   (bool) True if using a data section
#      (>) read2:      (real) Readout noise **2
#      (>) phot:       (real) Inverse gain in adu/e-
#      (>) iptr:       (pointer) Pointer to input mean data file
#      (>) eptr:       (pointer) Pointer to associated error file
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:      19-JAN-1994 -- Added parameter 'use_sect' to cure mapping
#                                  error (JRL)
#
#--------------------------------------------------------------------------------------

# Parameters

real read2,phot
int nx,ny
char fname[ARB],sect[ARB],type[ARB]
bool want_err,should_map,ex_err,ff,use_sect
pointer iptr,eptr

# Local variables

pointer cp1,sp,ptr1,ptr2
int npix
long one_l,v1[IM_MAXDIM],v2[IM_MAXDIM]

# Functions

pointer immap(),err_open()
bool err_exist()
int imgnlr(),impnlr()

# Check errors

errchk check_size,err_exist,err_open,immap

begin

      # Get some workspace

      call smark(sp)
      call salloc(cp1,SZ_FNAME,TY_CHAR)

      # Open mean frame and check it out

      iptr = immap(fname,READ_ONLY,0)
      call check_size(iptr,nx,ny)

      # If using a data section, then close the file and reopen it...

      if (use_sect) {
         call imunmap(iptr)
         call strcpy(fname,Memc[cp1],SZ_FNAME)
         call strcat(sect,Memc[cp1],SZ_FNAME)
         iptr = immap(Memc[cp1],READ_ONLY,0)
      }

      # Do this next section only if you really want to consider error
      # arrays. (Generally not so for mean bias frames...)

      if (should_map) {

         # Deal with existence of error array in the file...

         if (want_err) {
            call err_fname(iptr,Memc[cp1])
            ex_err = err_exist(iptr,READ_ONLY,Memc[cp1],false)

            # If the error array exists then map it...

            if (ex_err) {
               eptr = err_open(iptr,READ_ONLY,false) }

            # If it doesn't exist then only create one.  Fill it with the usual
            # noise model, unless it's a flat field.  This will be filled with
            # zeros.

            else {
               one_l = 1.0
               npix = IM_LEN(iptr,1)
               eptr = err_open(iptr,NEW_FILE,false)
               call amovkl(one_l,v1,IM_MAXDIM)
               if (!ff) {
                  call amovkl(one_l,v2,IM_MAXDIM)
                  while (imgnlr(iptr,ptr1,v1) != EOF && 
                     impnlr(eptr,ptr2,v2) != EOF) {
                     call cre_var(npix,Memr[ptr1],read2,phot,Memr[ptr2])
                  }
               }
               else {
                  while (impnlr(eptr,ptr1,v1) != EOF) {
                     call amovkr(0.0,Memr[ptr1],npix)
                  }
               }
               call imflush(eptr)
            }
         }
      }

      # Free workspace

      call sfree(sp)
end
