include <error.h>
include <imhdr.h>

procedure prep_os(nx,ny,bias,overscan,bias_frame,bsect,log_descr,fbias)

#--------------------------------------------------------------------------------------
#      PREP_OS -- Prepare mean bias frame if it wasn't created in this run
#
#      Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
#
#      (>) nx:         (int) X dimension of chip 
#      (>) ny:         (int) Y dimension of chip
#      (>) bias:       (bool) True if bias frames have been processed
#      (>) overscan:   (bool) True if overscan correction is being done
#      (>) bias_frame: (char) Name of output mean bias frame
#      (>) bsect:      (char) Bias section
#      (>) log_descr:  (int) Descriptor for log file
#      (<) fbias:      (real) Value of average bias in overscan region of output frame
#
#
#      AUTHOR:      Jim Lewis -- RGO
#      DATE:        18-NOV-1992
#      UPDATE:
#
#--------------------------------------------------------------------------------------

# Parameters

int nx,ny,log_descr
bool bias,overscan
char bias_frame[ARB],bsect[ARB]
real fbias

# Local variables

pointer ibias,sp,lp,iptr
int npix,nrow
real sig

# Functions

pointer immap(),imgs2r()
int errcode()

begin

      # Get a bit of workspace

      call smark(sp)
      call salloc(lp,SZ_FNAME,TY_CHAR)
      
      # If you have processed bias frames in this run already,
      # then use the overscan bias you have alread found.  Otherwise
      # find it from the mean bias map. First open this map and check
      # its dimensions.

      if (! bias && overscan) {
         ibias = immap(bias_frame,READ_ONLY,0)
         iferr(call check_size(ibias,nx,ny)) call erract(EA_ERROR)
         call imunmap(ibias)

         # Now reopen the bias strip only

         call strcpy(bias_frame,Memc[lp],SZ_FNAME)
         call strcat(bsect,Memc[lp],SZ_FNAME)
         ibias = immap(Memc[lp],READ_ONLY,0)
         npix = IM_LEN(ibias,1)
         nrow = IM_LEN(ibias,2)
         iptr = imgs2r(ibias,1,npix,1,nrow)

         # Do the stats on the bias strip

         iferr (call stats(npix*nrow,Memr[iptr],fbias,sig,3.0)) {
            if (errcode() == 1) {
               call fprintf(log_descr,
                  "Bias strip in mean bias frame has all bad values\n")}
            else if (errcode() == 2) {
                  call fprintf(log_descr,
                     "Bias strip sig**2 < 0 in mean bias frame\n")
            }
            call erract(EA_ERROR) }
         else {
            call fprintf(log_descr,
               "    Bias strip in mean bias frame average was %0.7g +/- %0.7g \n")
               call pargr(fbias)
               call pargr(sig)
         }
         call imunmap(ibias)
      }

      # Free workspace

      call sfree(sp)
end
