procedure sub_lamda(wave_cal,fsr_cal,gap,cenph_cal,mpu0_cal,dmpu_cal,
   nz,mode,wave_want,mpu0_obs,dmpu_obs,verbose,logd,cenph_obs,lam0_obs,
   dlam_obs,fsr_obs)

# Parameters

real wave_cal,fsr_cal,gap,cenph_cal,mpu0_cal,dmpu_cal,wave_want,mpu0_obs,
   dmpu_obs,cenph_obs,lam0_obs,dlam_obs,fsr_obs
int nz,mode,logd
bool verbose

# Local variables

real cp_c_pix,fsr_c_pix,fsr_c_microns,dlam_cal,wave0_cal,arg,gap_new,
   wave0_obs,fsr_o_pix,fsr_o_microns,cp_o_pix
int n_cal,n_obs,nz2

begin

      # Convert central phase and FSR of cal cube to pixels

      cp_c_pix = (cenph_cal - mpu0_cal)/dmpu_cal + 1.0
      fsr_c_pix = fsr_cal/dmpu_cal

      # Define dispersion and wavelength zero point in cal cube...
  
      fsr_c_microns = wave_cal**2/(2.0*gap)
      dlam_cal = real(mode)*fsr_c_microns/fsr_c_pix
      wave0_cal = wave_cal - dlam_cal*cp_c_pix

      # Now define a new gap estimate based on the order number
      # of the cal cube...

      arg = 2.0*gap/wave_cal
      n_cal = nint(arg)
      gap_new = real(n_cal)*wave_cal*0.5

      # Now find the order of the obs cube and calculate its wavelength
      # zeropoint based on the idea that n*lamda = const

      n_obs = int(2.0*gap_new/wave_want)
      wave0_obs = real(n_cal)*wave0_cal/real(n_obs)

      # Find the dispersion in the obs cube from estimates of the FSR

      fsr_o_pix = wave_want*fsr_c_pix/wave_cal
      fsr_obs = fsr_o_pix*dmpu_obs
      fsr_o_microns = wave_want**2/(2.0*gap_new)
      dlam_obs = real(mode)*fsr_o_microns/fsr_o_pix

      # Now find out what the wavelength zeropoint will be in the corrected
      # obs cube.

      nz2 = nz/2
      lam0_obs = wave_want - real(nz2)*dlam_obs

      # Finally find the central phase of the obs cube and convert to MPUs

      cp_o_pix = (wave_want - wave0_obs)/dlam_obs
      cenph_obs = (cp_o_pix - 1.0)*dmpu_obs + mpu0_obs

      # If requested, send info...

      if (verbose) {
         call printf(" \n")
         call printf("The wavelength of the CAL cube is %9.6f microns\n")
            call pargr(wave_cal)
         call printf(
            "The FSR of the CAL cube is %8.3f pixels and %0.6g microns\n")
            call pargr(fsr_c_pix)
            call pargr(fsr_c_microns)
         call printf("The central phase of the CAL cube is %8.3f pixels\n")
            call pargr(cp_c_pix)
         call printf("The mode variable is %d \n")
            call pargi(mode)
         call printf("Therefore, in the CAL cube:\n")
         call printf("   wave0 = %9.6f microns\n")
            call pargr(wave0_cal)
         call printf("   dlam = %0.6g microns/pixel \n")
            call pargr(dlam_cal)
         call printf("The CAL cube is in order %d \n")
            call pargi(n_cal)
         call printf("The original gap estimate was %11.6f microns\n")
            call pargr(gap)
         call printf("The new gap estimate was %11.6f microns\n")
            call pargr(gap_new)
         call printf(" \n")
         call printf("The wavelength of the OBS cube is %9.6f microns\n")
            call pargr(wave_want)
         call printf("The OBS cube is in order %d \n")
            call pargi(n_obs)
         call printf(
            "The FSR of the OBS cube is %8.3f pixels and %0.6g microns\n")
            call pargr(fsr_o_pix)
            call pargr(fsr_o_microns)
         call printf("Therefore from n*lamda = const in the OBS cube:\n")
         call printf("   wave0 = %9.6f microns\n")
            call pargr(wave0_obs)
         call printf("Thus in the corrected cube\n")
         call printf("   lam0 = %0.6g microns\n")
            call pargr(lam0_obs)
         call printf("   dlam = %0.6g microns/pixel\n")
            call pargr(dlam_obs)
         call printf("The central phase of the OBS cube is %8.3f pixels\n")
            call pargr(cp_o_pix)
         call printf(" \n")
         call printf(" \n")
      }

      # Send similar info to the log file...

      call fprintf(logd,"#CAL cube wave0: %30t %0.6g microns\n")
         call pargr(wave0_cal)
      call fprintf(logd,"#CAL cube dlam: %30t %0.6g microns/pixel \n")
         call pargr(dlam_cal)
      call fprintf(logd,"#CAL cube FP order: %30t %d \n")
         call pargi(n_cal)
      call fprintf(logd,"#Original gap estimate: %30t %0.6g microns\n")
         call pargr(gap)
      call fprintf(logd,"#New gap estimate: %30t %0.6g microns\n")
         call pargr(gap_new)
      call fprintf(logd,"#OBS cube FP order: %30t %d \n")
         call pargi(n_obs)
      call fprintf(logd,"#OBS cube FSR: %30t %0.6g pixels\n")
         call pargr(fsr_o_pix)
      call fprintf(logd,"#OBS cube FSR: %30t %0.6g microns\n")
         call pargr(fsr_o_microns)
      call fprintf(logd,"#OBS cube wave0: %30t %0.6g microns\n")
         call pargr(wave0_obs)
      call fprintf(logd,"#Corrected OBS cube wave0: %30t %0.6g microns\n")
         call pargr(lam0_obs)
      call fprintf(logd,"#OBS cube dlam: %30t %0.6g microns/pixel\n")
         call pargr(dlam_obs)
      call fprintf(logd,"#OBS cube cen. phase: %30t %0.6g pixels\n")
         call pargr(cp_o_pix)

end
