include <imhdr.h>
include <error.h>
include <mach.h>

define HELPFIL      "rgo$taucal/curfils/findpks.key"
define PROMPT       "findpks:  Options for peak definition"
define DOTSIZE      1

procedure t_findpks()

# Local variables

pointer sp,iname,dp,iw,iptr
int frame,dims[2],nrad,wcs_status
short colour
real thresh

# Functions

int clgeti()
pointer imd_mapframe(),iw_open(),immap()
real clgetr()
short clgets()

begin

      # Get some workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)

      # Get the input file name.  This should be the file which has been
      # displayed on the image display.

      frame = clgeti("frame")
      colour = clgets("colour")
      iferr {
         dp = imd_mapframe(frame,READ_WRITE,YES)
         iw = iw_open(dp,frame,Memc[iname],SZ_FNAME,wcs_status)
      } then {
         call eprintf("Error opening display window\n")
         call erract(EA_ERROR)
      }

      # Open the input file and get its dimensions...

      iferr (iptr = immap(Memc[iname],READ_ONLY,0)) call erract(EA_ERROR)
      if (IM_NDIM(iptr) != 2) 
         call error(0,"Input file can only be 2d!")
      dims[1] = IM_LEN(iptr,1)
      dims[2] = IM_LEN(iptr,2)

      # Now get a few more parameters

      nrad = clgeti("nrad")
      thresh = clgetr("thresh")

      # Go to the main processing routine

      call sub_find(iptr,dims,dp,iw,frame,colour,nrad,thresh)

      # Free workspace

      call imunmap(iptr)
      call imunmap(dp)
      call iw_close(iw)
      call sfree(sp)
end

procedure sub_find(iptr,dims,dp,iw,frame,colour,nrad,thresh)

# Parameters

pointer iptr,dp,iw
int dims[2],frame,nrad
real thresh
short colour

# Local variables

pointer sp,cmd,template,name,ptr,wpt1,wpt2,wpt3,wpt4,wpt5,wpt6,xc,yc,
   outstr,col,offscr,wptr
int nxy,npos,ngpix,nabove,i,npks,nsize,ii,jj,junk,wcs,i1,i2,j1,j2,npix,
   irem,kk,fd,key,npos0,memsiz
bool auto,off
real cut,cx,cy,rmax,rad,fx,fy,clipmean,sig

# Functions

pointer imgs2r(),imgs2s(),imps2s()
int aravr(),clgcur(),tau_log()
real clgetr()

begin

      # Get a bit of workspace

      call smark(sp)
      call salloc(cmd,SZ_LINE,TY_CHAR)
      call salloc(template,SZ_FNAME,TY_CHAR)
      call salloc(name,SZ_FNAME,TY_CHAR)
      call salloc(outstr,SZ_LINE,TY_CHAR)
      memsiz = (2*DOTSIZE + 1)**2

      # Map the whole array...

      ptr = imgs2r(iptr,1,dims[1],1,dims[2])
      nxy = dims[1]*dims[2]

      # Enter cursor loop
      
      auto = true
      npos = 0
      repeat {

         if (auto) {

            # First do a statistical analysis to find a mean background

            ngpix = aravr(Memr[ptr],nxy,clipmean,sig,thresh)
            cut = clipmean + thresh*sig
            nabove = 0
            do i = 1,nxy 
               if (Memr[ptr+i-1] > cut) nabove = nabove + 1
     
            # Now find the pixels above this threshold
 
            call malloc(wpt1,nabove,TY_INT)
            call malloc(wpt2,nabove,TY_INT)
            call malloc(wpt3,nabove,TY_REAL)
            call pkfind(Memr[ptr],dims[1],dims[2],cut,nabove,npks,Memi[wpt1],
               Memi[wpt2],Memr[wpt3])

            # If no peaks were found then quit...

            if (npks == 0) call error(0,"No peaks found above background...")

            # Sort these into decreasing order of brightness...

            call malloc(wpt4,npks,TY_INT)
            call malloc(wpt5,npks,TY_INT)
            call malloc(wpt6,npks,TY_REAL)
            call pksort(npks,Memi[wpt1],Memi[wpt2],Memr[wpt3],Memi[wpt4],
               Memi[wpt5],Memr[wpt6])

            # Free the workspace from the sort and reallocate it for the
            # centroiding routine...

            call mfree(wpt4,TY_INT)
            call mfree(wpt5,TY_INT)
            call mfree(wpt6,TY_REAL)
            nsize = 2*nrad + 1
            call malloc(wpt4,nsize,TY_REAL)
            call malloc(wpt5,nsize,TY_REAL)
            call malloc(wpt6,npks,TY_INT)

            # Do centroid fit to each peak

            call pkcent(Memr[ptr],dims[1],dims[2],npks,Memi[wpt1],Memi[wpt2],
               clipmean,nsize,Memr[wpt4],Memr[wpt5],Memi[wpt6],npos,xc,yc)
   
            # Now ditch the workspace from the finding and centroiding routines

            call mfree(wpt1,TY_INT)
            call mfree(wpt2,TY_INT)
            call mfree(wpt3,TY_REAL)
            call mfree(wpt4,TY_REAL)
            call mfree(wpt5,TY_REAL)
            call mfree(wpt6,TY_INT)

            # Get initial workspace for dots

            call malloc(col,npos,TY_INT)
            call malloc(offscr,npos,TY_BOOL)
 
            # Display these peaks on the image screen...

            do i = 1,npos {
               call iw_im2fb(iw,Memr[xc+i-1],Memr[yc+i-1],fx,fy)
               call pkdlim(dp,fx,fy,i1,i2,j1,j2,npix,off)
               call malloc(wptr,memsiz,TY_SHORT)
               Memi[col+i-1] = wptr
               if (!off) {
                  call amovs(Mems[imgs2s(dp,i1,i2,j1,j2)],Mems[wptr],npix)
                  call amovks(colour,Mems[imps2s(dp,i1,i2,j1,j2)],npix)
                  call imflush(dp)
               } else {
                  call amovks(INDEFS,Mems[wptr],memsiz)
               }
               Memb[offscr+i-1] = off
            }
            call imflush(dp)
         auto = false
         }

         # Get a cursor stroke...

         junk = clgcur("imagecur",cx,cy,wcs,key,Memc[cmd],SZ_LINE)

         # Now switch for the keystroke...

         switch (key) {
            case 'q':    # Get out of the loop (quit)
               break
            case 'd':    # delete the point nearest the cursor
               rmax = MAX_REAL
               do i = 1,npos {
                  rad = sqrt((Memr[xc+i-1]-cx)**2 + (Memr[yc+i-1]-cy)**2)
                  if (rad < rmax) {
                     irem = i
                     rmax = rad
                  }
               }
               if (!Memb[offscr+irem-1]) {
                  call iw_im2fb(iw,Memr[xc+irem-1],Memr[yc+irem-1],fx,fy)
                  call pkdlim(dp,fx,fy,i1,i2,j1,j2,npix,off)
                  call amovs(Mems[Memi[col+irem-1]],
                     Mems[imps2s(dp,i1,i2,j1,j2)],npix)
                  call imflush(dp)
               }
               call mfree(Memi[col+irem-1],TY_SHORT)
               do i = irem,npos-1 {
                  Memr[xc+i-1] = Memr[xc+i]
                  Memr[yc+i-1] = Memr[yc+i]
                  Memi[col+i-1] = Memi[col+i]
                  Memb[offscr+i-1] = Memb[offscr+i]
               }
               npos = npos - 1
               if (npos == 0) {
                  call mfree(xc,TY_REAL)
                  call mfree(yc,TY_REAL)
                  call mfree(col,TY_INT)
                  call mfree(offscr,TY_BOOL)
               } else {
                  call realloc(xc,npos,TY_REAL)
                  call realloc(yc,npos,TY_REAL)
                  call realloc(col,npos,TY_INT)
                  call realloc(offscr,npos,TY_BOOL)
               }
            case 'a':    # add the point nearest the cursor
               ii = nint(cx)
               jj = nint(cy)
               call malloc(wpt4,nsize,TY_REAL)
               call malloc(wpt5,nsize,TY_REAL)
               npos0 = npos
               call pkcent(Memr[ptr],dims[1],dims[2],1,ii,jj,clipmean,nsize,
                  Memr[wpt4],Memr[wpt5],kk,npos,xc,yc)
               call mfree(wpt4,TY_REAL)
               call mfree(wpt5,TY_REAL)
               if (npos > npos0) {
                  call iw_im2fb(iw,Memr[xc+npos-1],Memr[yc+npos-1],fx,fy)
                  call pkdlim(dp,fx,fy,i1,i2,j1,j2,npix,off)
                  call malloc(wptr,memsiz,TY_SHORT)
                  Memi[col+npos-1] = wptr
                  if (!off) {
                     call amovs(Mems[imgs2s(dp,i1,i2,j1,j2)],Mems[wptr],npix)
                     call amovks(colour,Mems[imps2s(dp,i1,i2,j1,j2)],npix)
                     call imflush(dp)
                  } else {
                     call amovks(INDEFS,Mems[wptr],memsiz)
                  }
                  call realloc(col,npos,TY_INT)
                  call realloc(offscr,npos,TY_BOOL)
                  Memi[col+npos-1] = wptr
                  Memb[offscr+npos-1] = off
               }
            case 't':  # Change the theshold for the search
               thresh = clgetr("thresh")
               auto = true
               do i = 1,npos {
                  if (!Memb[offscr+i-1]) {
                     call iw_im2fb(iw,Memr[xc+i-1],Memr[yc+i-1],fx,fy)
                     call pkdlim(dp,fx,fy,i1,i2,j1,j2,npix,off)
                     call amovs(Mems[Memi[col+i-1]],
                        Mems[imps2s(dp,i1,i2,j1,j2)],npix)
                  }
                  call mfree(Memi[col+i-1],TY_SHORT)
               }
               call imflush(dp)
               call mfree(xc,TY_REAL)
               call mfree(yc,TY_REAL)
               call mfree(offscr,TY_BOOL)
               call mfree(col,TY_INT)
               npos = 0
            case 'I':    # Interrupt immediately
               call fatal(0,"Interrupted!!")
            case '?':    # HELP!!!
               call pagefile(HELPFIL,PROMPT)
         }  
      }  # End of repeat block

      # Get rid of some workspace

      do i = 1,npos 
         call mfree(Memi[col+i-1],TY_SHORT)
      call mfree(col,TY_INT)
      call mfree(offscr,TY_BOOL)

      # Open a log file...

      fd = tau_log("findpks.log",NEW_FILE)

      # Write some initial information to the file...

      call sysid(Memc[outstr],SZ_LINE)
      call fprintf(fd,"#%s\n")
         call pargstr(Memc[outstr])
      call fprintf(fd,"#Program: %30t FINDPKS\n")
      call fprintf(fd,"#Input file name: %30t %s\n")
         call pargstr(IM_HDRFILE(iptr))
      call fprintf(fd,"#Display frame used: %30t %d\n")
         call pargi(frame)
      call fprintf(fd,"#Marker colour used: %30t %d\n")
         call pargs(colour)
      call fprintf(fd,"#Threshold for peaks: %30t %0.6g\n")
         call pargr(thresh)
      call fprintf(fd,"#Radius for centroid: %30t %d\n")
         call pargi(nrad)
      call fprintf(fd,"#Number of peaks found: %30t %d\n")
         call pargi(npos)
      do i = 1,npos {
         call fprintf(fd,"%0.6g %0.6g %d\n")
            call pargr(Memr[xc+i-1])
            call pargr(Memr[yc+i-1])
            call pargi(i)
      }
      call close(fd)

      # Friendly message to the user

      call printf("%d peaks found\n")
         call pargi(npos)

      # Right, now get rid of the remaining workspace

      call mfree(xc,TY_REAL)
      call mfree(yc,TY_REAL)
      call sfree(sp)
end


procedure pkdlim(dp,fx,fy,i1,i2,j1,j2,npix,off)

# Parameters

pointer dp
int i1,i2,j1,j2,npix
real fx,fy
bool off

begin

      # Find limits

      i1 = nint(fx) - DOTSIZE
      i2 = i1 + 2*DOTSIZE
      j1 = nint(fy) - DOTSIZE
      j2 = j1 + 2*DOTSIZE

      # Are any of these such that the dot would be completely off
      # the screen?

      if (i1 > IM_LEN(dp,1) || i2 < 1 || j1 > IM_LEN(dp,2) || j2 < 1) {
         npix = 1
         off = true
      
      # If it's on the screen in any way, make sure that it doesn't spill
      # over anywhere...

      } else {
         i1 = max(i1,1)
         i2 = min(i2,IM_LEN(dp,1))
         j1 = max(j1,1)
         j2 = min(j2,IM_LEN(dp,2))
         npix = (i2 - i1 + 1)*(j2 - j1 + 1)
         off = false
      }
end
 

procedure pkfind(input,nx,ny,cut,nmax,npks,xpeaks,ypeaks,vpeaks)

# Parameters

int nx,ny,nmax,npks,xpeaks[nmax],ypeaks[nmax]
real input[nx,ny],cut,vpeaks[nmax]

# Local variables

real hann[3,3],sumhann,val
int jx,jy,ii,ix,iy
pointer wptr

data hann/1.0,2.0,1.0,2.0,4.0,2.0,1.0,2.0,1.0/,sumhann/16.0/

# Functions

real asumr()

begin

      # Initialise a few things

      npks = 0
      call malloc(wptr,9,TY_REAL)

      # Loop for all pixels except the ends

      do jy = 2,ny-1 {
         do jx = 2,nx-1 {

            # Is this value above the threshold? If it is then smooth the
            # 3x3 matrix around it and test again.

            if (input[jx,jy] > cut) {
               ii = 0
               do iy = jy-1,jy+1 {
                  do ix = jx-1,jx+1 {
                     ii = ii + 1
                     Memr[wptr+ii-1] = input[ix,iy]
                  }
               }
               call amulr(Memr[wptr],hann,Memr[wptr],9)
               val = asumr(Memr[wptr],9)/sumhann

               # If the smoothed value passes the cut, then keep it

               if (val > cut) {
                  npks = npks + 1
                  xpeaks(npks) = jx
                  ypeaks(npks) = jy
                  vpeaks(npks) = input[jx,jy]
               }
            }
         }
      }

      # Ditch workspace

      call mfree(wptr,TY_REAL)
end
      
procedure pksort(np,ix,iy,iv,ox,oy,ov)

# Parameters

int np,ix[np],iy[np],ox[np],oy[np]
real iv[np],ov[np]

# Local variables

int i,j,jrem
real vmax

begin

      # Loop for each peak and reset the maximum value

      do i = 1,np {
         vmax = -MAX_REAL

         # Loop for each peak and find the brightest

         do j = 1,np {
            if (iv[j] > vmax) {
               jrem = j
               vmax = iv[j]
            }
         }

         # Store the brightest and reset it to something very low to keep
         # it from being considered again

         ox[i] = ix[jrem]
         oy[i] = iy[jrem]
         ov[i] = iv[jrem]
         iv[jrem] = -MAX_REAL
      }

      # Now copy over the result into the original arrays

      call amovi(ox,ix,np)
      call amovi(oy,iy,np)
      call amovr(ov,iv,np)
end


procedure pkcent(input,nx,ny,npks,ix,iy,mean,nsize,work1,work2,ipunt,
   npos,xc,yc)

# Parameters

int nx,ny,npks,ix[npks],iy[npks],nsize,ipunt[npks],npos
real input[nx,ny],work1[nsize],work2[nsize],mean
pointer xc,yc

# Local variables

int i,nrad,j
real xcn,ycn,dx,dy,rad

errchk pkctroid

begin

      # Start by initialising the flag array and some counters

      call amovki(1,ipunt,npks)
      i = 0
      nrad = (nsize - 1)/2

      # Loop until you have run out of peaks...

      while (i < npks) {
         i = i + 1

         # If the current local maximum hasn't been examined already, then
         # try and do a centroid on it

         if (ipunt[i] != 0) {
            ifnoerr(call pkctroid(input,nx,ny,ix[i],iy[i],mean,nsize,
               work1,work2,xcn,ycn)) {
   
               # If the centroid worked, then store the value away and
               # flag any peaks which might be associated with the
               # current one.

               npos = npos + 1
               if (npos == 1) {
                  call malloc(xc,npos,TY_REAL)
                  call malloc(yc,npos,TY_REAL)
               } else {
                  call realloc(xc,npos,TY_REAL)
                  call realloc(yc,npos,TY_REAL)
               }
               Memr[xc+npos-1] = xcn
               Memr[yc+npos-1] = ycn
               do j = 1,npks {
                  dx = xcn - real(ix[j])
                  dy = ycn - real(iy[j])
                  rad = sqrt(dx*dx + dy*dy)
                  if (rad <= real(nrad)) ipunt[j] = 0
               }
            } else {
               call eprintf("Couldn't do centroid on peak at %d %d \n")
               call pargi(ix[i])
               call pargi(iy[i])
               call erract(EA_WARN)
            }
         }
      }
end

procedure pkctroid(input,nx,ny,ix,iy,mean,nsize,sumx,sumy,xcn,ycn)

# Parameters

int nx,ny,ix,iy,nsize
real input[nx,ny],sumx[nsize],sumy[nsize],xcn,ycn,mean

# Local variables

real x,y,xorig,yorig,oldx,oldy,value,zx,zy
int nrad,niter,i,ox,oy,oxst,oyst,j,k

begin

      # Initialise a few things...

      x = real(ix)
      y = real(iy)
      xorig = x
      yorig = y
      nrad = (nsize - 1)/2
      niter = 5

      # Now do a maximum number of iterations...

      do k = 1,niter {
         oldx = x
         oldy = y
         ox = int(oldx + 0.5)
         oy = int(oldy + 0.5)
         oxst = ox - nrad - 1
         oyst = oy - nrad - 1

         # Is this too near the edge?

         if (ox-nrad < 1 || ox+nrad > nx || oy-nrad < 1 || oy+nrad > ny)
            call error(3,"centroid: pixel too near edge of frame")

         # Do the necessary summations

         call aclrr(sumx,nsize)
         call aclrr(sumy,nsize)
         do j = 1,nsize {
            do i = 1,nsize {
               value = input[oxst+i,oyst+j] - mean
               sumx[i] = sumx[i] + value
               sumy[j] = sumy[j] + value 
            }
         }

         # Initialise some counters

         zx = 0.0
         zy = 0.0
         x = 0.0
         y = 0.0

         # Centroids

         do i = 1,nsize {
            x = x + sumx[i]*real(oxst+i)
            zx = zx + sumx[i]
            y = y + sumy[i]*real(oyst+i)
            zy = zy + sumy[i]
         }

         # Check for negative total intensity...

         if (zx <= 0.0 || zy <= 0.0) 
            call error(1,"centroid: negative total intensity")

         # Centroid solutions

         x = x/zx
         y = y/zy

         # Check for a runaway solution

         if (sqrt((x-xorig)**2+(y-yorig)**2) > real(nrad)) 
            call error(4,"centroid: runaway solution")

         # If it has converged then get out of here

         if (sqrt((oldx-x)**2 + (oldy-y)**2) < 1.0) break
      }

      # If it never converged, then send out an error message

      if (k > niter) call error(2,"centroid: never converged")

      # Send back the centres

      xcn = x
      ycn = y
end
                                         
