include "rings.h"
include <imhdr.h>
include <fset.h>
include <mwset.h>
include <error.h>
include <math/iminterp.h>
include <mach.h>

define    ESTIMS      "|before|after|average|"

procedure t_fpcor()

# Local variables

pointer sp,ilist,olist,iname,oname,rname,gname,rvtname,rvfname,interp,params,
   rp,rvfptr,rvtptr,rptr,tgp,slopes,inters,iptr,optr,runs,nord,asip,estname,
   outstr,mw,rr,ww,cd
int listi,listo,nrefx,nrefy,infunc,logd,nser,mode,i,junk,dims[3],obsrun,est,
   nruns,pdim
real wave_cal,pixx,fsr_cal,gap,xcen,ycen,cenph_cal,mpu0_cal,dmpu_cal,cam_flen,
   fscale,wave_want,mpu0_obs,dmpu_obs,dx,dy,dz,dr,cenph_obs,lam0_obs,dlam_obs,
   fsr_obs
bool use_grid,verbose

# Functions

int fstati(),imtopen(),imtlen(),tbhgti(),tau_log(),imtgetim(),imgeti(),
   clgwrd(),rvnfrm(),mw_stati()
pointer ring_init(),rvopen(),immap(),tgio_init(),rt_crfile(),mw_openim()
bool tbhgtb(),clgetb()
real tbhgtr(),clgetr(),imgetr()

begin

      # Get workspace for file names...

      call smark(sp)
      call salloc(ilist,SZ_LINE,TY_CHAR)
      call salloc(olist,SZ_LINE,TY_CHAR)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(oname,SZ_FNAME,TY_CHAR)
      call salloc(rname,SZ_FNAME,TY_CHAR)
      call salloc(gname,SZ_FNAME,TY_CHAR)
      call salloc(rvtname,SZ_FNAME,TY_CHAR)
      call salloc(rvfname,SZ_FNAME,TY_CHAR)
      call salloc(interp,10,TY_CHAR)
      call salloc(estname,10,TY_CHAR)
      call salloc(params,8,TY_CHAR)
      call salloc(outstr,SZ_LINE,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Start by getting input file template.  Also get list of output
      # processed cubes...

      call clgstr("inlist",Memc[ilist],SZ_LINE)
      listi = imtopen(Memc[ilist])
      call clgstr("olist",Memc[olist],SZ_LINE)
      listo = imtopen(Memc[olist])
      if (imtlen(listi) != imtlen(listo)) 
         call error(0,"Input and output file lists not the same length")

      # Get the RVF file name and the RVT if one is being used.
      # Open them if you can...

      rp = ring_init()
      call clgstr("rvfname",Memc[rvfname],SZ_FNAME)
      iferr (rvfptr = rvopen(rvfname,READ_ONLY)) {
         call eprintf("Can't open input RVF table %s\n")
            call pargstr(Memc[rvfname])
         call erract(EA_ERROR)
      }
      call clgstr("rvtname",Memc[rvtname],SZ_FNAME)
      if (Memc[rvtname] != EOS) {
         iferr (rvtptr = rvopen(rvtname,READ_ONLY)) {
            call eprintf("Can't open input RVT table %s\n")
               call pargstr(Memc[rvtname])
            call erract(EA_ERROR)
         }
         est = clgwrd("est",Memc[estname],10,ESTIMS)
      } else {
         rvtptr = NULL
      }
         
      # Get optional residual phase map 

      call clgstr("resname",Memc[rname],SZ_FNAME)
      if (Memc[rname] != EOS) {
         iferr (rptr = immap(Memc[rname],READ_ONLY,NULL)) {
            call eprintf("Can't open input residual phase map %s\n")
               call pargstr(Memc[rname])
            call erract(EA_ERROR)
         }
      } else {
         rptr = NULL
      }

      # Get optional grid.  First get a bit of info from the header of the RVF

      nrefx = tbhgti(rvfptr,"reflenx")
      nrefy = tbhgti(rvfptr,"refleny")
      use_grid = tbhgtb(rvfptr,"use_grid")
      call tbhgtt(rvfptr,"gridname",Memc[gname],SZ_FNAME)
      iferr (tgp = tgio_init(use_grid,1,gname,nrefx,nrefy))
         call erract(EA_ERROR)

      # Get interpolation method

      infunc = clgwrd("interp",Memc[interp],10,II_FUNCTIONS)
      call asiinit(asip,infunc)      

      # Verbose output?  Open output logfile as well.

      verbose = clgetb("verbose")
      logd = tau_log("fpcor.log",NEW_FILE)

      # Read calibration cube info from RVF

      nser = tbhgti(rvfptr,"nser")
      wave_cal = tbhgtr(rvfptr,"wave")
      pixx = tbhgtr(rvfptr,"pixx")
      fsr_cal = tbhgtr(rvfptr,"fsrcen")
      gap = tbhgtr(rvfptr,"gap")
      xcen = tbhgtr(rvfptr,"xc")
      ycen = tbhgtr(rvfptr,"yc")
      mode = tbhgti(rvfptr,"mode")
      cenph_cal = tbhgtr(rvfptr,"cenph")
      mpu0_cal = tbhgtr(rvfptr,"mpu0")
      dmpu_cal = tbhgtr(rvfptr,"dmpu")

      # Get some workspace for the slopes and intercepts. Then read them...
  
      call malloc(slopes,nser,TY_REAL)
      call malloc(inters,nser,TY_REAL)
      do i = 1,nser {
         call sprintf(Memc[params],8,"slopes%d")
            call pargi(i)
         Memr[slopes+i-1] = tbhgtr(rvfptr,Memc[params])
         call sprintf(Memc[params],8,"inters%d")
            call pargi(i)
         Memr[inters+i-1] = tbhgtr(rvfptr,Memc[params])
      }

      # Get camera focal length

      cam_flen = clgetr("cam_flen")*1000.0
      fscale = pixx/cam_flen

      # Write some stuff to the log file...

      call sysid(Memc[outstr],SZ_LINE)
      call fprintf(logd,"#%s\n")
         call pargstr(Memc[outstr])
      call fprintf(logd,"#Program: %30t FPCOR\n")
      call fprintf(logd,"#RVF table: %30t %s\n")
         call pargstr(Memc[rvfname])
      call fprintf(logd,"#Number of series fit: %30t %d\n")
         call pargi(nser)
      do i = 1,nser {
         call fprintf(logd,"#Intercept for series %d: %30t %0.6g MPUs\n")
            call pargi(i)
            call pargr(Memr[inters+i-1])
         call fprintf(logd,"#Slope for series %d: %30t %0.6g pix**2/MPU\n")
            call pargi(i)
            call pargr(Memr[slopes+i-1])
      }
      call fprintf(logd,"#Calibration wavelength: %30t %0.6g microns\n")
         call pargr(wave_cal)
      call fprintf(logd,"#FSR for CAL cube: %30t %0.6g MPUs\n")
         call pargr(fsr_cal)
      call fprintf(logd,"#Gap estimate: %30t %0.6g microns\n")
         call pargr(gap)
      call fprintf(logd,"#Mean Xc for CAL cube: %30t %0.6g\n")
         call pargr(xcen)
      call fprintf(logd,"#Mean Yc for CAL cube: %30t %0.6g\n")
         call pargr(ycen)
      call fprintf(logd,"#Mode variable in RVF: %30t %d \n")
         call pargi(mode)
      call fprintf(logd,"#CAL cube central phase: %30t %0.6g MPUs\n")
         call pargr(cenph_cal)
      call fprintf(logd,"#CAL cube MPU0: %30t %0.6g MPUs\n")
         call pargr(mpu0_cal)
      call fprintf(logd,"#CAL cube dmpu: %30t %0.6g MPUs/frame\n")
         call pargr(dmpu_cal)
      call fprintf(logd,"#RVT table: %30t %s\n")
      if (rvtptr != NULL) {
         call pargstr(Memc[rvtname])
         call fprintf(logd,"#RVT estimation method: %30t %s\n")
            call pargstr(Memc[estname])
      } else {
         call pargstr("none used")
      }
      call fprintf(logd,"#Residual phase map: %30t %s\n")
      if (rptr != NULL)
         call pargstr(Memc[rname])
      else
         call pargstr("none used")
      call fprintf(logd,"#Grid map: %30t %s \n")
      if (use_grid) 
         call pargstr(Memc[gname])
      else
         call pargstr("none used")
      call fprintf(logd,"#Interpolation method: %30t %s \n")
         call pargstr(Memc[interp])
      call fprintf(logd,"#Camera focal length: %30t %0.6g mm\n")
         call pargr(cam_flen*0.001)
      call fprintf(logd,"#Pixel size: %30t %0.6g microns\n")
         call pargr(pixx)
      call fprintf(logd,"#\n")
      call fprintf(logd,"#\n")

      # Now loop for each cube...

      while (imtgetim(listi,Memc[iname],SZ_FNAME) != EOF) {
         junk = imtgetim(listo,Memc[oname],SZ_FNAME)

         # Open the input and output files...

         if (verbose) {
            call printf(" \n")
            call printf("Beginning work on file %s \n")
               call pargstr(Memc[iname])
         }
         iferr (iptr = immap(Memc[iname],READ_ONLY,NULL)) {
            call eprintf("Can't open input file %s \n")
               call pargstr(Memc[iname])
            next
         }
         do i = 1,3
            dims[i] = IM_LEN(iptr,i)
         optr = rt_crfile(Memc[oname],NEW_COPY,iptr,IM_NDIM(iptr),dims,
            IM_PIXTYPE(iptr))

         # Read header info from input file

         wave_want = imgetr(iptr,"wave")
         obsrun = imgeti(iptr,"run")
         mpu0_obs = imgetr(iptr,"mpu0")
         dmpu_obs = imgetr(iptr,"dmpu")

         # Write some stuff to the log file...

         call fprintf(logd,"#Input file: %30t %s\n")
            call pargstr(Memc[iname])
         call fprintf(logd,"#Output file: %30t %s\n")
            call pargstr(Memc[oname])
         call fprintf(logd,"#Wavelegth of observation %30t %0.6g microns\n")
            call pargr(wave_want)
         call fprintf(logd,"#Run number of OBS cube: %30t %d\n")
            call pargi(obsrun)
         call fprintf(logd,"#OBS cube MPU0 %30t %0.6g MPUs\n")
            call pargr(mpu0_obs)
         call fprintf(logd,"#OBS cube dmpu %30t %0.6g MPUs/frame\n")
            call pargr(dmpu_obs)

         # If you want to do rvt correction, do it now...

         if (rvtptr != NULL) {
            call fprintf(logd,"# \n")
            call fprintf(logd,"#Results from RVT analysis\n")
            call fprintf(logd,"# \n")
            nruns = rvnfrm(rvtptr)
            call malloc(runs,nruns,TY_INT)
            call malloc(nord,nruns,TY_INT)
            iferr(call sub_xrvt(rvtptr,rvfptr,rp,nruns,Memi[runs],Memi[nord],
               obsrun,xcen,ycen,mpu0_cal,dmpu_cal,est,nser,Memr[slopes],
               Memr[inters],verbose,logd,dx,dy,dr,dz)) call erract(EA_ERROR)
            call mfree(runs,TY_INT)
            call mfree(nord,TY_INT)
         } else {
            dx = 0.0
            dy = 0.0
            dr = 0.0
            dz = 0.0
         }

         # Sort out wavelength calibration...

         call fprintf(logd,"# \n")
         call fprintf(logd,"#Results from wavelength analysis\n")
         call fprintf(logd,"# \n")
         call sub_lamda(wave_cal,fsr_cal,gap,cenph_cal,mpu0_cal,dmpu_cal,
            IM_LEN(iptr,1),mode,wave_want,mpu0_obs,dmpu_obs,verbose,logd,
            cenph_obs,lam0_obs,dlam_obs,fsr_obs)

         # Finally, do the correction...

         call sub_correct(iptr,gap,fsr_obs,mode,mpu0_obs,dmpu_obs,
            cenph_obs,fscale,wave_want,tgp,xcen,ycen,dx,dy,dz,rptr,
            asip,verbose,optr)

         # Set up the wcs of the spectral axis...

call eprintf("hi1\n")
         mw = mw_openim(iptr)
         pdim = mw_stati(mw,MW_NPHYSDIM)
         call malloc(ww,pdim,TY_DOUBLE)
         call malloc(rr,pdim,TY_DOUBLE)
         call malloc(cd,pdim,TY_DOUBLE)
call eprintf("hi1\n")
         call fpmw(mw,lam0_obs,dlam_obs,Memd[rr],Memd[ww],Memd[cd],pdim)
call eprintf("hi1\n")
         call mw_saveim(mw,optr)
call eprintf("hi1\n")
         call mw_close(mw)
         call mfree(ww,TY_DOUBLE)
         call mfree(rr,TY_DOUBLE)
         call mfree(cd,TY_DOUBLE)
call eprintf("hi1\n")

         # Close up the files...

         call imunmap(iptr)
         call imunmap(optr)
      }

      # Now close up the rest of the files and free workspace...

call eprintf("hi1\n")
      call close(logd)
      call asifree(asip)
      call tgio_close(tgp)
      if (rptr != NULL) call imunmap(rptr)
      call rvclose(rvtptr)
      call rvclose(rvfptr)
call eprintf("hi1\n")
      call imtclose(listi)
      call imtclose(listo)
      call mfree(slopes,TY_REAL)
      call mfree(inters,TY_REAL)
      call sfree(sp)
end
         
procedure sub_correct(iptr,gap,fsr_obs,mode,mpu0,dmpu,cenph,fscale,
   wave_want,tgp,xcen,ycen,dx,dy,dz,rptr,asip,verbose,optr)

# Parameters

int mode
real wave_want,gap,fsr_obs,mpu0,dmpu,cenph,fscale,xcen,ycen,dx,dy,dz
pointer iptr,tgp,rptr,asip,optr
bool verbose

# Local variables

real free_shift,beta,fscale2,censh,fsr_pix,xnew,ynew,r2,sec_theta,
   fsr,result,respix
pointer ptrr,ptrg,ptri,ptro
int i,j,index,nx,ny,nz

# Functions

pointer imgs3r(),imps3r(),imgl2r(),tggetl()

begin

      # First set up a few constants...

      nx = IM_LEN(iptr,2)
      ny = IM_LEN(iptr,3)
      nz = IM_LEN(iptr,1)
      free_shift = 0.5*real(nz)*dmpu
      beta = real(mode)*2.0*fsr_obs*gap/wave_want
      fscale2 = fscale**2
      censh = cenph - mpu0 + dmpu
      fsr_pix = fsr_obs/dmpu

      # Get a bit of workspace for dummy residuals...

      if (rptr == NULL)
         call calloc(ptrr,nx,TY_REAL)

      # Loop for Y first...

      do j = 1,ny {
  
         # If there is a residual phase map, then get a row of it

         if (rptr != NULL) 
            ptrr = imgl2r(rptr,j)

         # Get a line of the grid...

         ptrg = tggetl(tgp,j)

         # Get a slab of spectra, and a slab of the output file

         ptri = imgs3r(iptr,1,nz,1,nx,j,j)
         ptro = imps3r(optr,1,nz,1,nx,j,j)        
 
         # Now loop for X and calculate the phase for each pixel...

         do i = 1,nx {
            xnew = Memr[ptrg+2*(i-1)] - (xcen - dx)
            ynew = Memr[ptrg+2*(i-1)+1] - (ycen - dy)
            r2 = xnew**2 + ynew**2
            sec_theta = sqrt(r2*fscale2 + 1.0)
            fsr = fsr_obs*sec_theta
            result = censh + beta*(sec_theta - 1.0)
            result = result + Memr[ptrr+i-1] + dz - free_shift
            if (abs(result) >= abs(fsr)) {
               index = int(result/fsr)
               result = result - fsr*real(index)
            }
            respix = result/dmpu
            
            # Correct the data now...

            call cslide(Memr[ptri+(i-1)*nz],nz,respix,fsr_pix,asip,
               Memr[ptro+(i-1)*nz])
         }

         # Message to the user if it is wanted...
 
         if (verbose && mod(j,10) == 0) {
            call printf("Line %d phase corrected \n")
               call pargi(j)
         }
      }

      # Release the last bit of workspace, if it was allocated in
      # the first place...

      if (rptr == NULL) call mfree(ptrr,TY_REAL)
end

procedure cslide(input,nz,respix,fsr,asip,output)

# Parameters

real input[nz],output[nz],respix,fsr
int nz
pointer asip

# Local variables

pointer wptr
int i,index
real x,y,diff

# Functions

real asieval()

begin

      # Get some workspace and copy the input spectrum to it

      call malloc(wptr,nz,TY_REAL)
      call amovr(input,Memr[wptr],nz)

      # Fit the data now and ditch the workspace

      call asifit(asip,Memr[wptr],nz)
      call mfree(wptr,TY_REAL)

      # Now loop for the input pixels...

      do i = 1,nz {
         x = real(i) + respix
         if (x > real(nz)) {
            diff = x - real(nz)
            index = int(diff/fsr) + 1
         } else if (x < 1.0) {
            diff = x - 1.0
            index = int(diff/fsr) - 1
         } else {
            index = 0
         }
         x = x - real(index)*fsr
         y = asieval(asip,x)
         output[i] = y
      }
end

procedure sub_lamda(wave_cal,fsr_cal,gap,cenph_cal,mpu0_cal,dmpu_cal,
   nz,mode,wave_want,mpu0_obs,dmpu_obs,verbose,logd,cenph_obs,lam0_obs,
   dlam_obs,fsr_obs)

# Parameters

real wave_cal,fsr_cal,gap,cenph_cal,mpu0_cal,dmpu_cal,wave_want,mpu0_obs,
   dmpu_obs,cenph_obs,lam0_obs,dlam_obs,fsr_obs
int nz,mode,logd
bool verbose

# Local variables

real cp_c_pix,fsr_c_pix,fsr_c_microns,dlam_cal,wave0_cal,arg,gap_new,
   wave0_obs,fsr_o_pix,fsr_o_microns,cp_o_pix
int n_cal,n_obs,nz2

begin

      # Convert central phase and FSR of cal cube to pixels

      cp_c_pix = (cenph_cal - mpu0_cal)/dmpu_cal + 1.0
      fsr_c_pix = fsr_cal/dmpu_cal

      # Define the interference order of the cal cube and a new 
      # estimate of the gap based on the idea that the order must
      # be an integer

      arg = 2.0*gap/wave_cal
      n_cal = int(arg)
      gap_new = real(n_cal)*wave_cal*0.5

      # Define dispersion and wavelength zero point in cal cube...
  
      fsr_c_microns = wave_cal**2/(2.0*gap_new)
      dlam_cal = real(mode)*fsr_c_microns/fsr_c_pix
      wave0_cal = wave_cal - dlam_cal*cp_c_pix

      # Now find the order of the obs cube and calculate the wavelength
      # at the position of the cal cube central phase, based on the
      # idea that n*lamda = const

      n_obs = int(2.0*gap_new/wave_want)
      wave0_obs = real(n_cal)*wave_cal/real(n_obs)

      # Find the dispersion in the obs cube from estimates of the FSR

      fsr_obs = fsr_cal*wave0_obs/wave_cal
      fsr_o_pix = fsr_obs/dmpu_obs
      fsr_o_microns = wave0_obs**2/(2.0*gap_new)
      dlam_obs = real(mode)*fsr_o_microns/fsr_o_pix

      # Now find out what the wavelength zeropoint will be in the corrected
      # obs cube.

      nz2 = nz/2
      lam0_obs = wave_want - real(nz2)*dlam_obs

      # Finally find the central phase of the obs cube and convert to MPUs

      cenph_obs = cenph_cal + ((wave_want - wave0_obs)/dlam_obs)*dmpu_obs
      cp_o_pix = (cenph_obs - mpu0_obs)/dmpu_obs + 1.0

      # If requested, send info...

      if (verbose) {
         call printf(" \n")
         call printf("The wavelength of the CAL cube is %9.6f microns\n")
            call pargr(wave_cal)
         call printf(
            "The FSR of the CAL cube is %8.3f pixels and %0.6g microns\n")
            call pargr(fsr_c_pix)
            call pargr(fsr_c_microns)
         call printf("The central phase of the CAL cube is %8.3f pixels\n")
            call pargr(cp_c_pix)
         call printf("The mode variable is %d \n")
            call pargi(mode)
         call printf("Therefore, in the CAL cube:\n")
         call printf("   wave0 = %9.6f microns\n")
            call pargr(wave0_cal)
         call printf("   dlam = %0.6g microns/pixel \n")
            call pargr(dlam_cal)
         call printf("The CAL cube is in order %d \n")
            call pargi(n_cal)
         call printf("The original gap estimate was %11.6f microns\n")
            call pargr(gap)
         call printf("The new gap estimate was %11.6f microns\n")
            call pargr(gap_new)
         call printf(" \n")
         call printf("The wavelength of the OBS cube is %9.6f microns\n")
            call pargr(wave_want)
         call printf("The OBS cube is in order %d \n")
            call pargi(n_obs)
         call printf(
            "The FSR of the OBS cube is %8.3f pixels and %0.6g microns\n")
            call pargr(fsr_o_pix)
            call pargr(fsr_o_microns)
         call printf("Therefore from n*lamda = const in the OBS cube:\n")
         call printf("   wave_cpcal = %9.6f microns\n")
            call pargr(wave0_obs)
         call printf("Thus in the corrected cube\n")
         call printf("   lam0 = %0.6g microns\n")
            call pargr(lam0_obs)
         call printf("   dlam = %0.6g microns/pixel\n")
            call pargr(dlam_obs)
         call printf("The central phase of the OBS cube is %8.3f pixels\n")
            call pargr(cp_o_pix)
         call printf(" \n")
         call printf(" \n")
      }

      # Send similar info to the log file...

      call fprintf(logd,"#CAL cube wave0: %30t %0.6g microns\n")
         call pargr(wave0_cal)
      call fprintf(logd,"#CAL cube dlam: %30t %0.6g microns/pixel \n")
         call pargr(dlam_cal)
      call fprintf(logd,"#CAL cube FP order: %30t %d \n")
         call pargi(n_cal)
      call fprintf(logd,"#Original gap estimate: %30t %0.6g microns\n")
         call pargr(gap)
      call fprintf(logd,"#New gap estimate: %30t %0.6g microns\n")
         call pargr(gap_new)
      call fprintf(logd,"#OBS cube FP order: %30t %d \n")
         call pargi(n_obs)
      call fprintf(logd,"#OBS cube FSR: %30t %0.6g pixels\n")
         call pargr(fsr_o_pix)
      call fprintf(logd,"#OBS cube FSR: %30t %0.6g microns\n")
         call pargr(fsr_o_microns)
      call fprintf(logd,"#OBS cube wave at CAL cp: %30t %0.6g microns\n")
         call pargr(wave0_obs)
      call fprintf(logd,"#Corrected OBS cube wave0: %30t %0.6g microns\n")
         call pargr(lam0_obs)
      call fprintf(logd,"#OBS cube dlam: %30t %0.6g microns/pixel\n")
         call pargr(dlam_obs)
      call fprintf(logd,"#OBS cube cen. phase: %30t %0.6g pixels\n")
         call pargr(cp_o_pix)
end

procedure sub_xrvt(rvtptr,rvfptr,rp,nruns,runs,nord,obsrun,xcen,ycen,
   mpu0,dmpu,est,nser,slopes,inters,verbose,logd,dx,dy,dr,dz)

# Parameters

pointer rvtptr,rvfptr,rp
int nruns,runs[nruns],nord[nruns],obsrun,est,nser,logd
real xcen,ycen,dx,dy,dr,dz,mpu0,dmpu,slopes[nser],inters[nser]
bool verbose

# Local variables

int i,flag,iuse1,iuse2,dumrun,iser,status,nr
real mpu,xc,yc,rad,sig,peak,back,ell,meanerr,mpurvt,radrvt,radrvf,
   radrt1,radrt2,radrf1,radrf2,dr1,dr2,dx1,dx2,dy1,dy2

begin

      # Start by getting the run numbers and sorting them 

      nr = 0
      do i = 1,nruns {
         call rvgetl(rvtptr,rp,i,false,dumrun,mpu,xc,yc,rad,sig,peak,back,
            ell,meanerr,flag)
         if (flag == RING_OK || flag == RING_DODGY) {
            nr = nr + 1
            runs[nr] = dumrun
         }
      }
      if (nr == 0) call error(0,"No good rings in RVT")
      call csort(nr,runs,nord)

      # Find which rings to use for this particular obs cube...

      for (i=1; i <= nr && runs[nord[i]] < obsrun; i = i + 1)
         ;
      if (est == 1) {
         if (i == 1)
            iuse1 = nord[1]
         else if (i > nr)
            iuse1 = nord[nr]
         else
            iuse1 = nord[i-1]
      } else if (est == 2) {
         if (i > nr) 
            iuse1 = nord[nr]
         else
            iuse1 = nord[i]
      } else if (est == 3) {
         if (i == 1) {
            iuse1 = nord[1]
            iuse2 = nord[1]
         } else if (i > nr) {
            iuse1 = nord[nr]
            iuse2 = nord[nr]
         } else {
            iuse1 = nord[i-1]
            iuse2 = nord[i]
         }
      }

      # Now for each estimation method get a value of Xc, Yc and radius

      switch (est) {
      
         # 1 -- Use the results from the run just previous to the obs cube
         # 2 -- Ditto, but just after...

         case 1, 2:
            call rvgetl(rvtptr,rp,iuse1,false,dumrun,mpurvt,xc,yc,radrvt,
               sig,peak,back,ell,meanerr,flag)
            call cfindr(mpurvt,nser,slopes,inters,radrvf,iser,status)
            if (status .ne. 0) 
               call error(0,"Z of ring isn't in RVF range of MPUs")
            dr = radrvf**2 - radrvt**2 
            dz = dr/slopes[iser]
            dx = xcen - xc
            dy = ycen - yc

         # 3 -- Average the results of the two rings on either side...

         case 3:
            call rvgetl(rvtptr,rp,iuse1,false,dumrun,mpurvt,xc,yc,radrt1,
               sig,peak,back,ell,meanerr,flag)
            call cfindr(mpurvt,nser,slopes,inters,radrf1,iser,status)
            if (status .ne. 0) 
               call error(0,"Z of ring isn't in RVF range of MPUs")
            dr1 = (radrf1**2 - radrt1**2)
            dx1 = xcen - xc
            dy1 = ycen - yc
            call rvgetl(rvtptr,rp,iuse2,false,dumrun,mpurvt,xc,yc,radrt2,
               sig,peak,back,ell,meanerr,flag)
            call cfindr(mpurvt,nser,slopes,inters,radrf2,iser,status)
            if (status .ne. 0) 
               call error(0,"Z of ring isn't in RVF Z range")
            dr2 = (radrf2**2 - radrt2**2)
            dx2 = xcen - xc
            dy2 = ycen - yc
            dr = 0.5*(dr1 + dr2)
            dz = dr/slopes[iser]
            dx = 0.5*(dx1 + dx2)
            dy = 0.5*(dy1 + dy2)
      }

      # Now send some messages...

      if (verbose) {
         call printf(" \n")
         call printf("Phase shift info for obs cube run %d \n")
            call pargi(obsrun)
         call printf(" \n")
         call printf("Estimate of phase shift calculated from ring%s")
         if (est == 1) {
            call pargstr(" just before obs cube\n")
            call printf("Run number of the ring was %d \n")
               call pargi(runs[iuse1])
         } else if (est == 2) {
            call pargstr(" just after obs cube\n")
            call printf("Run number of the ring was %d \n")
               call pargi(runs[iuse1])
         } else if (est == 3) {
            call pargstr("s either side of obs cube\n")
            call printf("Run number of the rings were %d %d\n")
               call pargi(runs[iuse1])
               call pargi(runs[iuse2])
         }
         call printf("d(r**2) = %0.6g\n")
            call pargr(dr)
         call printf("dz = %0.6g\n")
            call pargr(dz)
         call printf("dx = %0.6g\n")
            call pargr(dx)
         call printf("dy = %0.6g\n")
            call pargr(dy)
      }

      # Write some stuff to log file...

      if (est != 3) {
         call fprintf(logd,"#RVT estimate from run: %30t %d\n")
            call pargi(runs[iuse1])
      } else {
         call fprintf(logd,"#RVT estimate from runs: %30t %d %d\n")
            call pargi(runs[iuse1])
            call pargi(runs[iuse2])
      }
      call fprintf(logd,"#d(r**2): %30t %0.6g\n")
         call pargr(dr)
      call fprintf(logd,"#dz: %30t %0.6g\n")
         call pargr(dz)
      call fprintf(logd,"#dx: %30t %0.6g\n")
         call pargr(dx)
      call fprintf(logd,"#dy: %30t %0.6g\n")
         call pargr(dy)
end


procedure cfindr(mpurvt,nser,slopes,inters,radrvf,iser,status)

# Parameters

real mpurvt,radrvf,slopes[nser],inters[nser]
int nser,status,iser

# Local variables

real rsmall,rad2
int i
bool found

begin

      # Find out which series gives the smallest positive value of r**2.

      rsmall = MAX_REAL
      do i = 1,nser {
         rad2 = slopes[i]*mpurvt + inters[i]
         if (rad2 < rsmall && rad2 > 0.0) {
            rsmall = rad2
            iser = i
            found = true
         }
      }

      # If a candidate was found, then take the square root and that's your
      # answer

      if (found) {
         radrvf = sqrt(rsmall)
         status = 0
      } else {
         radrvf = 0.0
         status = 1
      }
end



procedure csort(nruns,runs,nord)

# Parameters

int nruns,runs[nruns],nord[nruns]

# Local variables

int i,minrun,j,jrem
pointer wptr

begin

      # Get some workspace and copy the run numbers to it...

      call malloc(wptr,nruns,TY_INT)
      call amovi(runs,Memi[wptr],nruns)

      # Now do the sort

      do i = 1,nruns {
         minrun = MAX_INT
         do j = 1,nruns {
            if (Memi[wptr+j-1] < minrun) {
               jrem = j
               minrun = Memi[wptr+j-1]
            }
         }
         Memi[wptr+jrem-1] = MAX_INT
         nord[i] = jrem
      }

      # Ditch the workspace

      call mfree(wptr,TY_INT)
end
           

procedure fpmw(mw,wave0,disp,r,w,cd,pdim)

# Parameters

pointer mw
real wave0,disp
int pdim
double r[pdim],w[pdim],cd[pdim,pdim]

# Local variables

int ival,axbits,axes[IM_MAXDIM],naxes,axmap,ax

# Functions

int mw_stati()

begin
      # Set the wcs

      iferr (call mw_newsystem(mw,"world",pdim)) 
         call mw_ssystem(mw,"world")

      # Get an axis list

      ival = 2**pdim - 1
      call bitpak(ival,axbits,1,pdim)
      call mw_gaxlist(mw,axbits,axes,naxes)
      if (naxes < 2) return

      # Switch off axis mapping

      axmap = mw_stati(mw,MW_USEAXMAP)
      call mw_seti(mw,MW_USEAXMAP,NO)

      # Get the Wterm of the wcs

      call mw_gwtermd(mw,r,w,cd,pdim)      

      # Which axis is this?

      ax = axes[1]

      # Set the dispersion and wavelength zeropoint in the Wterm 

      cd[ax,ax] = double(disp)
      r[ax] = 1.0d0
      w[ax] = double(wave0)
      call mw_swtermd(mw,r,w,cd,pdim)

      # Define the wavelength attributes for the spectral axis

      call mw_swtype(mw,ax,1,"linear","")
      call mw_swattrs(mw,ax,"label","Wavelength")
      call mw_swattrs(mw,ax,"units","Angstroms")

      # Reset axis mapping

      call mw_seti(mw,MW_USEAXMAP,axmap)
end
