include <mach.h>
include <math.h>
include <fset.h>
include <gset.h>
include <pkg/gtools.h>
include <math/nlfit.h>
include <error.h>
include <imhdr.h>
include <tbset.h>
include "rings.h"

define    MAXCPTS      20
define    HELPFIL      "rgo$taucal/curfils/prering.key"
define    PROMPT       "prering:  Options for point selection"
define    DISPFRAME    1
define    SIGGAU       3.0
define    DOTSIZE      1

procedure t_prering()

# Local variables

pointer sp,inlist,otab,slptr,spptr,optr,rp
bool rvf,dispall
int list,nfiles,nser,nread
real dtheta,restrict
short colour

# Functions

bool clgetb()
int imtopen(),imtlen(),clgeti(),fstati()
real clgetr()
short clgets()
pointer ring_init(),rvcreate()

begin

      # Get some workspace first

      call smark(sp)
      call salloc(inlist,SZ_LINE,TY_CHAR)
      call salloc(otab,SZ_FNAME,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Find out if this is an RVF or an RVT analysis

      rvf = clgetb("rvf")

      # Get the file names

      call clgstr("input",Memc[inlist],SZ_LINE)

      # Open the list, get the number of files, and check if this is ok...

      list = imtopen(Memc[inlist])
      nfiles = imtlen(list)
      if (rvf && nfiles != 1)
         call error(0,"Only one file allowed for an RVF")

      # If this is an RVF, then get the number of series, the limits
      # of the series and the frames to be analysed for each series

      if (rvf) {
         nser = clgeti("nser")
         call malloc(slptr,2*nser,TY_INT)
         call malloc(spptr,2*nser,TY_INT)
         call rt_par_arri("serlims",2*nser,nread,Memi[slptr])
         call rt_par_arri("serproc",2*nser,nread,Memi[spptr])

      # If this is an RVT then decide whether to do a display of each
      # ring frame in turn...

      } else {
         dispall = clgetb("dispall")
      }

      # Get a few other parameters

      call clgstr("output",Memc[otab],SZ_FNAME)
      dtheta = clgetr("dtheta")
      restrict = clgetr("restrict")
      colour = clgets("colour")

      # Open the rings data structure...

      rp = ring_init()

      # Open output rvf/rvt file and define the columns...

      optr = rvcreate(rp,otab)

      # Do the processing

      call sub_prering(list,dtheta,restrict,rvf,nser,slptr,spptr,colour,
         dispall,rp,optr)

      # Do some cleaning up...

      if (rvf) {
         call mfree(slptr,TY_INT)
         call mfree(spptr,TY_INT)
      }
      call ring_close(rp)
      call rvclose(optr)
      call sfree(sp)
end

procedure sub_prering(list,dtheta,restrict,rvf,nser,slptr,spptr,
  colour,dispall,rp,optr)

# Parameters

pointer list,slptr,spptr,rp,optr
int nser
real dtheta,restrict
bool rvf,dispall
short colour

# Local variables

pointer sp,iname,iptr,gp,gt,parname
int nfiles,nring,ifile,junk,run,nr,nring1,nlim1,nring2,nlim2,j,i,flag
bool disp
real xc,yc,rad,peak,back,sig,mpu0,dmpu,slope,aint,xc1,yc1,rad1,peak1,back1,
   sig1,xc2,yc2,rad2,peak2,back2,sig2,rsig,mpu,wave

# Functions

int imtlen(),imtgetim(),imgeti()
real imgetr()
pointer immap(),gt_init(),gopen()

begin

      # Get some workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(parname,SZ_LINE,TY_CHAR)

      # Open up graphics

      gp = gopen("stdgraph",NEW_FILE,STDGRAPH)
      gt = gt_init()
      call gt_sets(gt,GTTITLE,"Intensity vs Distance from centre")
      call gt_sets(gt,GTXLABEL,"Distance from Centre of circle(pixels)")
      call gt_sets(gt,GTYLABEL,"Mean Intensity (counts)")
      call gdeactivate(gp,0)

      # Section for RVT analysis...

      if (!rvf) {
     
         # Get the number of files on offer

         nfiles = imtlen(list)
         nring = 1

         # Now loop for this number.  Start off by opening each frame

         do ifile = 1,nfiles {
            junk = imtgetim(list,Memc[iname],SZ_FNAME)
            iferr(iptr = immap(Memc[iname],READ_ONLY,0)) {
               call eprintf("Can't open file %s\n")
               call pargstr(Memc[iname])
               call erract(EA_WARN)
               next
            }
            
            # Do the processing of the ring...

            if (ifile == 1)
               disp = true
            else
               disp = dispall
            call procring(iptr,nring,colour,dtheta,restrict,disp,gp,gt,
               xc,yc,rad,peak,back,sig) 

            # Get some info from the file header

            iferr {
               run = imgeti(iptr,"run")
               mpu0 = imgetr(iptr,"mpu0")
            } then {
               call eprintf("Header information missing in %s\n")
                  call pargstr(Memc[iname])
               run = INDEFI
               mpu0 = INDEFR
            }

            # Now fill in the appropriate row of the output table

            call rvputl(optr,rp,ifile,false,run,mpu0,xc,yc,rad,sig,peak,back,
               1.0,0.0,RING_PRE)
            call rvputl(optr,rp,ifile,true,run,mpu0,0.0,0.0,0.0,0.0,0.0,0.0,
               0.0,0.0,RING_PRE)

            # Put file names in rvt header

            call sprintf(Memc[parname],SZ_LINE,"fi%04d")
               call pargi(ifile)
            call tbhadt(optr,Memc[parname],Memc[iname])
            if (ifile == 1) {
               call tbhadi(optr,"reflenx",IM_LEN(iptr,1))
               call tbhadi(optr,"refleny",IM_LEN(iptr,2))
               call tbhadb(optr,"rvf",rvf)
            }

            # Close the input file

            call imunmap(iptr)
         }

      # Section for RVF analysis

      } else {
      
         # Get the name of the cube and open it

         junk = imtgetim(list,Memc[iname],SZ_FNAME)
         iferr(iptr = immap(Memc[iname],READ_ONLY,0)) {
            call eprintf("Can't open file %s\n")
            call pargstr(Memc[iname])
            call erract(EA_ERROR)
         }
         iferr {
            run = imgeti(iptr,"run")
            mpu0 = imgetr(iptr,"mpu0")
            dmpu = imgetr(iptr,"dmpu")
         } then {
            call eprintf("Header information missing in %s\n")
               call pargstr(Memc[iname])
            run = INDEFI
            mpu0 = INDEFR
            dmpu = 0.0
         }

         # Initialise the output table

         call rvfinit(optr,rp,IM_LEN(iptr,3),mpu0,dmpu,run)

         # Always do a display for RVF

         disp = true

         # Loop for the series

         nr = 0
         do i = 1,nser {

            # Process the two rings for this series

            nr = nr + 1
            nring1 = Memi[spptr+nr-1]
            nlim1 = Memi[slptr+nr-1]
            call procring(iptr,nring1,colour,dtheta,restrict,disp,gp,gt,xc1,
               yc1,rad1,peak1,back1,sig1)
            nr = nr + 1
            nring2 = Memi[spptr+nr-1]
            nlim2 = Memi[slptr+nr-1]
            call procring(iptr,nring2,colour,dtheta,restrict,disp,gp,gt,xc2,
               yc2,rad2,peak2,back2,sig2)

            # Now do some averages

            xc = 0.5*(xc1 + xc2)
            yc = 0.5*(yc1 + yc2)
            back = 0.5*(back1 + back2)
            rsig = 0.5*(rad1*sig1 + rad2*sig2)
            peak = 0.5*(peak1 + peak2)

            # Now set up the slope and intercept for RVF

            slope = (rad2*rad2 - rad1*rad1)/real(nring2 - nring1)
            aint = rad2*rad2 - slope*real(nring2)

            # Finally write out info for each frame in the series

            do j = nlim1,nlim2 {
               rad = slope*real(j) + aint
               if (rad < 0.0) {
                  rad = 0.0
                  sig = 0.0
                  flag = RING_SKIP
               } else {
                  rad = sqrt(rad)
                  sig = rsig/rad
                  flag = RING_PRE
               }
               mpu = mpu0 + real(j-1)*dmpu
               call rvputl(optr,rp,j,false,run,mpu,xc,yc,rad,sig,peak,back,
                  1.0,0.0,flag)
               call rvputl(optr,rp,j,true,run,mpu,0.0,0.0,0.0,0.0,0.0,0.0,
                  0.0,0.0,flag)
            }

            # Put the series limits into the header

            call sprintf(Memc[parname],SZ_LINE,"ser%d_1")
               call pargi(i)
            call tbhadi(optr,Memc[parname],nlim1)
            call sprintf(Memc[parname],SZ_LINE,"ser%d_2")
               call pargi(i)
            call tbhadi(optr,Memc[parname],nlim2)
         }

         # Put name of file in header of rvf.  Add other useful bits of info

         call sprintf(Memc[parname],SZ_LINE,"fi0001")
         call tbhadt(optr,Memc[parname],Memc[iname])
         call tbhadi(optr,"reflenx",IM_LEN(iptr,1))
         call tbhadi(optr,"refleny",IM_LEN(iptr,2))
         call tbhadb(optr,"rvf",rvf)
         call tbhadi(optr,"nser",nser)
         wave = imgetr(iptr,"wave")
         call tbhadr(optr,"wave",wave)
         call tbhadr(optr,"mpu0",mpu0)
         call tbhadr(optr,"dmpu",dmpu)

         # Close the input file

         call imunmap(iptr)

      } # End of if block

      # Ditch workspace

      call sfree(sp)
      call gt_free(gt)
      call gclose(gp)
end


procedure procring(iptr,nring,colour,dtheta,restrict,disp,gp,gt,xc,yc,
   rad,peak,back,sig)

# Parameters

pointer iptr,gp,gt
short colour
real dtheta,restrict,xc,yc,rad,peak,back,sig
bool disp
int nring

# Local variables

pointer sp,iname,wpt1,wpt2,dp,iw,ptr
int npts,wcs_status,nx,ny
real xpos[MAXCPTS],ypos[MAXCPTS]

# Functions

pointer imd_mapframe(),iw_open(),imgs3r()

begin

      # Get a bit of workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)

      # Send a friendly message...

      call printf(" \n")
      call printf("Starting work on file %s, frame %d\n")
         call pargstr(IM_HDRFILE(iptr))
         call pargi(nring)

      # Display the ring and choose points if you want...

      if (disp) {
         call dispring(iptr,nring)
         dp = imd_mapframe(DISPFRAME,READ_WRITE,YES)
         iw = iw_open(dp,DISPFRAME,Memc[iname],SZ_FNAME,wcs_status)
         call curpts(dp,iw,colour,npts,xpos,ypos)
         if (npts < 3) 
            call error(0,"You must define at least 3 points for a circle!")
         call malloc(wpt1,npts*3,TY_REAL)
         call malloc(wpt2,npts,TY_REAL)
         call fitcirc(npts,xpos,ypos,Memr[wpt1],Memr[wpt2],xc,yc,rad)
         call mfree(wpt1,TY_REAL)
         call mfree(wpt2,TY_REAL)
         call drawcirc(dp,iw,xc,yc,rad,colour)
         call iw_close(iw)
         call imunmap(dp)
      }

      # Do profile analysis

      nx = IM_LEN(iptr,1)
      ny = IM_LEN(iptr,2)
      ptr = imgs3r(iptr,1,nx,1,ny,nring,nring)
      call profiles(Memr[ptr],nx,ny,dtheta,restrict,gp,gt,xc,yc,rad,peak,
         back,sig)

      # Ditch workspace

      call sfree(sp)
end


procedure dispring(iptr,ringnum)

# Parameters

pointer iptr
int ringnum

# Local variables

pointer sp,iname,cmd
int ifn

# Functions

int strldx()

begin

      # Get some workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(cmd,SZ_LINE,TY_CHAR)

      # Get the name of the input file and find where extension starts.
      # Add in the image section...

      ifn = strldx(".",IM_HDRFILE(iptr))
      call strcpy(IM_HDRFILE(iptr),Memc[iname],SZ_FNAME)
      call sprintf(Memc[iname+ifn-1],SZ_FNAME,"[*,*,%d:%d]")
         call pargi(ringnum)
         call pargi(ringnum)

      # Create the command string

      call sprintf(Memc[cmd],SZ_LINE,"display %s %d erase+ fill+ zra+ zsc+")
         call pargstr(Memc[iname])
         call pargi(DISPFRAME)

      # Now display the ring...

      call clcmdw(Memc[cmd])

      # Ditch the workspace

      call sfree(sp)
end


procedure drawcirc(dp,iw,xc,yc,rad,colour)

# Parameters

pointer dp,iw
real xc,yc,rad
short colour

# Local variables

real xll,yll,xul,yul,rdisp,xxc,yyc,dtheta,theta,x1,x2,y
int nx,ny,ix,iy

# Functions

pointer imps2s()

begin

      # Start by getting the display limits

      call iw_im2fb(iw,xc-rad,yc-rad,xll,yll)
      call iw_im2fb(iw,xc+rad,yc+rad,xul,yul)
      rdisp = 0.5*abs(xul - xll)
      call iw_im2fb(iw,xc,yc,xxc,yyc)
      nx = IM_LEN(dp,1)
      ny = IM_LEN(dp,2)

      # Get angle increment.

      dtheta = atan(1.0/rdisp)

      # Now draw the circle. Measure angle from -ve Y axis, clockwise...

      theta = 0.0
      while (theta <= PI) {
         y = yyc - rdisp*cos(theta)
         x1 = xxc - rdisp*sin(theta)
         x2 = xxc + rdisp*sin(theta)
         ix = nint(x1)
         iy = nint(y)
         if (ix >= 1 && ix <= nx && iy >= 1 && iy <= ny)
            Mems[imps2s(dp,ix,ix,iy,iy)] = colour
         ix = nint(x2)
         if (ix >= 1 && ix <= nx && iy >= 1 && iy <= ny)
            Mems[imps2s(dp,ix,ix,iy,iy)] = colour
         theta = theta + dtheta
      }
      call imflush(dp)
end
         

procedure curpts(dp,iw,colour,npts,xpos,ypos)

# Parameters

pointer dp,iw
short colour
int npts
real xpos[MAXCPTS],ypos[MAXCPTS]

# Local variables

pointer sp,cmd,col,wptr
int junk,key,irem,i,wcs,i1,i2,j1,j2,memsiz,npix
real cx,cy,rmin,rad,fx,fy

# Functions

int clgcur()
pointer imgs2s(),imps2s()

begin

      # Get a bit of workspace

      call smark(sp)
      call salloc(cmd,SZ_LINE,TY_CHAR)
      call salloc(col,MAXCPTS,TY_INT)
      memsiz = (2*DOTSIZE + 1)**2

      # Do this until you have all the points you need

      npts = 0
      while (npts < MAXCPTS) {

         # First get the cursor stroke...

         junk = clgcur("imagecur",cx,cy,wcs,key,Memc[cmd],SZ_LINE)

         # Now switch for that keystroke

         switch (key) {
            case 'q':    # Get out of loop
               break
            case 'a':    # Add a point at the cursor position
               npts = npts + 1
               xpos[npts] = cx
               ypos[npts] = cy
               call printf("Point #%d added at x = %0.6g, y = %0.6g\n")
                  call pargi(npts)
                  call pargr(cx)
                  call pargr(cy)
               call iw_im2fb(iw,cx,cy,fx,fy)
               call dotlimits(dp,fx,fy,i1,i2,j1,j2,npix)
               call malloc(wptr,memsiz,TY_SHORT)
               Memi[col+npts-1] = wptr
               call amovs(Mems[imgs2s(dp,i1,i2,j1,j2)],Mems[wptr],npix)
               call amovks(colour,Mems[imps2s(dp,i1,i2,j1,j2)],npix)      
               call imflush(dp)
            case 'd':    # Delete a point at the cursor position
               rmin = MAX_REAL
               for (i=1; i <= npts; i = i + 1) {
                  rad = sqrt((xpos[i]-cx)**2 + (ypos[i]-cy)**2)
                  if (rad < rmin) {
                     irem = i
                     rmin = rad
                  }
               }
               call printf("Point #%d removed at x = %0.6g, y = %0.6g\n")
                  call pargi(irem)
                  call pargr(xpos[irem])
                  call pargr(ypos[irem])
               call iw_im2fb(iw,xpos[irem],ypos[irem],fx,fy)
               call dotlimits(dp,fx,fy,i1,i2,j1,j2,npix)
               call amovs(Mems[Memi[col+irem-1]],Mems[imps2s(dp,i1,i2,j1,j2)],
                  npix)
               call mfree(Memi[col+irem-1],TY_SHORT)
               call imflush(dp)
               for (i = irem + 1; i <= npts; i = i + 1) {
                  xpos[i-1] = xpos[i]
                  ypos[i-1] = ypos[i]
                  Memi[col+i-2] = Memi[col+i-1]
               }
               npts = npts - 1
            case 'w':    # Start over again
               call printf("Starting over...\n")
               for (i = 1; i <= npts; i = i + 1) {
                  call iw_im2fb(iw,xpos[i],ypos[i],fx,fy)
                  call dotlimits(dp,fx,fy,i1,i2,j1,j2,npix)
                  call amovs(Mems[Memi[col+i-1]],Mems[imps2s(dp,i1,i2,j1,j2)],
                     npix)
                  call mfree(Memi[col+i-1],TY_SHORT)
               }
               call imflush(dp)
               npts = 0
            case 'I':    # Get out of this whole program immediately
               call fatal(0,"Interrupted!!")
            case '?':    # HELP!!
               call pagefile(HELPFIL,PROMPT)
         }
      }  # end of while block

      # Ditch workspace

      do i = 1,npts 
         call mfree(Memi[col+i-1],TY_SHORT)
      call sfree(sp)
end

procedure fitcirc(npts,xpos,ypos,awork,bwork,xc,yc,rad)

# Parameters

int npts
real xpos[MAXCPTS],ypos[MAXCPTS],xc,yc,rad,awork[npts,3],bwork[npts]

# Local variables

int i,krank
pointer hhh,ggg,ip
real tau,rnorm

begin

      # Start by setting up the condition equations

      for (i=1; i <= npts; i=i+1) {
         awork[i,1] = xpos[i]**2 + ypos[i]**2
         awork[i,2] = xpos[i]
         awork[i,3] = ypos[i]
         bwork[i] = 1.0
      }

      # Allocate some workspace

      call malloc(hhh,npts,TY_REAL)
      call malloc(ggg,npts,TY_REAL)
      call malloc(ip,npts,TY_INT)

      # Do the fit

      call hfti(awork,npts,npts,3,bwork,1,1,tau,krank,rnorm,Memr[hhh],
         Memr[ggg],Memr[ip])

      # Free the workspace

      call mfree(hhh,TY_REAL)
      call mfree(ggg,TY_REAL)
      call mfree(ip,TY_REAL)

      # Solve for the circle's parameters

      xc = -0.5*bwork[2]/bwork[1]
      yc = -0.5*bwork[3]/bwork[1]
      rad = 1.0/bwork[1] + xc*xc + yc*yc
      if (rad < 0) 
         call error(0,"Fitted radius < 0")
      else
         rad = sqrt(rad)

      # Now write out some info...

      call printf(" \n")
      call printf(" \n")
      call printf("Circle fitted with the following parameters:\n")
      call printf("Xc = %0.6g, Yc = %0.6g, R = %0.6g\n")
         call pargr(xc)
         call pargr(yc)
         call pargr(rad)
end

procedure profiles(array,nx,ny,dtheta,restrict,gp,gt,xc,yc,rad,peak,back,sig)

# Parameters

pointer gp,gt
int nx,ny
real array[nx,ny],dtheta,xc,yc,rad,peak,back,sig,restrict

# Local variables

real d[4],rmax,theta,x,y,pnear,rnew,bnew,pnew,snew,cut,tol,pdiff,sig_gau,
  anorm,denom
pointer wpt1,wpt2,wpt3,wpt4,wpt5,wpt6,kptr,nl
int size,ncuts,i,j,ix,iy,n,npeaks,irem,ist,i1,i2,nnn,nfit,addf,adddf,
   itmax,status,iext,nk1,nk,nnx,npar

# Functions

real ahivr(),alovr()
int peaks(),locpr()

extern gau,dgau

begin

      # First sort out the largest size for the work arrays...

      d[1] = sqrt((real(nx)-xc)**2 + (yc-1.0)**2)
      d[2] = sqrt((real(nx)-xc)**2 + (real(ny)-yc)**2)
      d[3] = sqrt((xc-1.0)**2 + (yc-1.0)**2)
      d[4] = sqrt((xc-1.0)**2 + (real(ny)-yc)**2)
      rmax = ahivr(d,4)
      size = nint(rmax)

      # Get some workspace

      call calloc(wpt1,size,TY_REAL)
      call calloc(wpt2,size,TY_REAL)

      # Now make the cuts...

      ncuts = nint(360.0/dtheta) 
      for (i=1; i<=ncuts; i=i+1) {
         theta = DEGTORAD(real(i-1)*dtheta)
         for (j=1; j<=size; j=j+1) {
            x = xc + real(j-1)*cos(theta)
            y = yc + real(j-1)*sin(theta)
            ix = nint(x)
            iy = nint(y)
            if (ix > nx || ix < 1 || iy > ny || iy < 1) break
            Memr[wpt1+j-1] = Memr[wpt1+j-1] + array[ix,iy]
            Memr[wpt2+j-1] = Memr[wpt2+j-1] + 1.0
         }
      }

      # Average the values now...

      n = 0
      for (i=1; i<=size; i=i+1) {
         if (Memr[wpt2+i-1] != 0.0 && Memr[wpt1+i-1] > 0.0) {
            n = n + 1
            Memr[wpt2+n-1] = Memr[wpt1+i-1]/Memr[wpt2+i-1]
            Memr[wpt1+n-1] = real(i-1)
         }
      }

      # Set up a smoothing kernel

      sig_gau = SIGGAU
      iext = nint(sig_gau*2.355)
      nk = 2*iext + 1
      nk1 = nk/2 + 1
      nnx = n + 2*iext
      anorm = 1.0/(sqrt(TWOPI)*sig_gau)
      denom = -0.5/sig_gau**2
      call malloc(kptr,nk,TY_REAL)
      do i = 1,nk 
         Memr[kptr+i-1] = anorm*exp(denom*(real(i-nk1)**2))

      # Move profile to a workspace, extending it at either end...

      call malloc(wpt3,nnx,TY_REAL)
      call calloc(wpt4,nnx,TY_REAL)
      do i = 1,iext
         Memr[wpt3+i-1] = Memr[wpt2]
      do i = n+iext+1,nnx
         Memr[wpt3+i-1] = Memr[wpt2+n-1]
      call amovr(Memr[wpt2],Memr[wpt3+iext],n)

      # Convolve with a Gaussian

      call acnvrr(Memr[wpt3],Memr[wpt4],n,Memr[kptr],nk)
      call mfree(kptr,TY_REAL)
       
      # Now get a bit more workspace...

      call realloc(wpt3,n,TY_REAL)
      call realloc(wpt4,n,TY_REAL)
      call calloc(wpt5,n,TY_REAL)
      call amovr(Memr[wpt1],Memr[wpt3],n)

      # Get the peaks and find the one nearest the old radius.

      npeaks = peaks(Memr[wpt3],Memr[wpt4],Memr[wpt5],n,0.5)
      pnear = abs(Memr[wpt3] - rad)
      irem = 1
      for (i=2; i<=npeaks; i=i+1) {
         pdiff = abs(Memr[wpt3+i-1] - rad)
         if (pdiff < pnear) {
            irem = i
            pnear = pdiff
         }
      }
      rnew = Memr[wpt3+irem-1]
      bnew = alovr(Memr[wpt5],npeaks)
      pnew = Memr[wpt4+irem-1] - bnew

      # Free some workspace

      call mfree(wpt3,TY_REAL)
      call mfree(wpt4,TY_REAL)
      call mfree(wpt5,TY_REAL)
      
      # Get a rough idea of the width

      pnear = abs(rnew - Memr[wpt1])
      irem = 1
      do i = 2,n {
         pdiff = abs(rnew - Memr[wpt1+i-1])
         if (pdiff < pnear) {
            irem = i
            pnear = pdiff
         }
      }      
      ist = irem
      for (i=ist; i >= 1 && (Memr[wpt2+i-1]-bnew) > 0.5*pnew; i = i - 1)
         ;
      i1 = max(i,1)
      ist = irem
      for (i=ist; i <= n && (Memr[wpt2+i-1)-bnew) > 0.5*pnew; i = i + 1)
         ;
      i2 = min(i,n)
      snew = abs(Memr[wpt1+i2-1] - Memr[wpt1+i1-1])/2.355

      # Friendly message...

      call printf("Rough fit gives following parameters:\n")
      call printf("Peak = %0.6g  Radius = %0.6g  Sigma = %0.6g  Back = %0.6g\n")
         call pargr(pnew)
         call pargr(rnew)
         call pargr(snew)
         call pargr(bnew)

      # Save the values

      rad = rnew
      peak = pnew
      sig = snew
      back = bnew

      # Now do a more serious fit.  First set up the data to take into account
      # any restriction the user has specified.

      if (restrict > 0.0) {
         ist = nint(rad)
         cut = rad - restrict*sig
         for (i = ist; Memr[wpt1+i-1] > cut && i >= 1; i = i - 1)
            ;
         i1 = i + 1
         cut = rad + restrict*sig
         for (i = ist; Memr[wpt1+i-1] < cut && i <= n; i = i + 1)
            ;
         i2 = i - 1
         nnn = 0
         for (i = i1; i <= i2; i = i + 1) {
            nnn = nnn + 1
            Memr[wpt1+nnn-1] = Memr[wpt1+i-1]
            Memr[wpt2+nnn-1] = Memr[wpt2+i-1]
         }
         nfit = nnn
      } else {
         nfit = n
      }
     
      # Set up some things for the fitting routine

      call malloc(wpt3,4,TY_REAL)
      call malloc(wpt4,4,TY_REAL)
      call malloc(wpt5,4,TY_INT)
      call malloc(wpt6,nfit,TY_REAL)
      Memr[wpt3] = peak
      Memr[wpt3+1] = rad
      Memr[wpt3+2] = sig
      Memr[wpt3+3] = back
      call amovkr(0.1,Memr[wpt4],4)
      do i = 1,4
         Memi[wpt5+i-1] = i
      call amovkr(1.0,Memr[wpt6],nfit)
      addf = locpr(gau)
      adddf = locpr(dgau)
      tol = 0.001
      itmax = 500

      # Now do the fit...

      call nlinitr(nl,addf,adddf,Memr[wpt3],Memr[wpt4],4,Memi[wpt5],4,tol,
         itmax)
      call nlfitr(nl,Memr[wpt1],Memr[wpt2],Memr[wpt6],nfit,1,WTS_UNIFORM,
         status)
      switch (status) {
         case DONE:
            call nlpgetr(nl,Memr[wpt3],npar)
            peak = Memr[wpt3]
            rad = Memr[wpt3+1]
            sig = Memr[wpt3+2]
            back = Memr[wpt3+3]
            call printf("Final fit with following parameters:\n")
            call printf(
               "Peak = %0.6g  Radius = %0.6g  Sigma = %0.6g  Back = %0.6g\n")
               call pargr(peak)
               call pargr(rad)
               call pargr(sig)
               call pargr(back)
         default:
            call printf("Final fit failed...")
            call printf("Proceding with rough fit parameters\n")
      }

      # Plot the result...

      call greactivate(gp,0)
      call gframe(gp)
      call gascale(gp,Memr[wpt1],nfit,1)
      call gascale(gp,Memr[wpt2],nfit,2)
      call gt_swind(gp,gt)
      call gt_sets(gt,GTTYPE,"histogram")
      call gt_labax(gp,gt)
      call gt_plot(gp,gt,Memr[wpt1],Memr[wpt2],nfit)
      do i = 1,nfit 
         call gau(Memr[wpt1+i-1],1,Memr[wpt3],4,Memr[wpt2+i-1])
      call gt_sets(gt,GTTYPE,"line")
      call gsetr(gp,G_PLWIDTH,3.0)
      call gt_plot(gp,gt,Memr[wpt1],Memr[wpt2],nfit)
      call gsetr(gp,G_PLWIDTH,1.0)
      call gdeactivate(gp,0)

      # Ditch some workspace

      call mfree(wpt1,TY_REAL)
      call mfree(wpt2,TY_REAL)
      call mfree(wpt3,TY_REAL)
      call mfree(wpt4,TY_REAL)
      call mfree(wpt5,TY_INT)
      call mfree(wpt6,TY_REAL)
      call nlfreer(nl)     
end

procedure gau(x,nvars,par,npar,z)

# Parameters

int nvars,npar
real x[nvars],par[npar],z

# Local variables

real arg

begin

      # Evaluate the gaussian...

      arg = (x[1] - par[2])/par[3]
      arg = -0.5*arg*arg
      if (abs(arg) <= 20.0)
         z = par[1]*exp(arg) + par[4]
      else
         z = par[4]
end


procedure dgau(x,nvars,par,dpar,npar,z,dz)

# Parameters

int nvars,npar
real x[nvars],par[npar],dpar[npar],z,dz[npar]

# Local variables

real arg,exparg

begin

      # Evaluate the gaussian...

      arg = (x[1] - par[2])/par[3]
      arg = 0.5*arg*arg
      if (arg < 20.0) 
         exparg = exp(-arg)
      else
         exparg = 0.0
      z = par[1]*exparg + par[4]

      # Now the first derivatives...

      dz[1] = exparg
      dz[2] = par[1]*exparg*((x[1] - par[2])/(par[3]**2))
      dz[3] = par[1]*exparg*(2.0*arg/par[3])
      dz[4] = 1.0
end


procedure dotlimits(dp,x,y,i1,i2,j1,j2,npix)

# Parameters

pointer dp
real x,y
int i1,i2,j1,j2,npix

begin

      # Find x limits

      i1 = nint(x) - DOTSIZE
      i2 = i1 + 2*DOTSIZE
      i1 = min(max(i1,1),IM_LEN(dp,1))
      i2 = min(max(i2,1),IM_LEN(dp,1))

      # Find y limits

      j1 = nint(y) - DOTSIZE
      j2 = j1 + 2*DOTSIZE
      j1 = min(max(j1,1),IM_LEN(dp,1))
      j2 = min(max(j2,1),IM_LEN(dp,1))
      npix = (i2 - i1 + 1)*(j2 - j1 + 1)
end
