include <mach.h>
include <imhdr.h>
include <error.h>

procedure t_resid()

# Local variables

pointer sp,iname,rvname,gname,oname,iptr,rvptr,tgp,wptr,optr,ptr1
int nx,ny,nz,box,window,nrefx,nrefy,mode,dims[2]
real mpu0,dmpu,focus,wave,pixx,fscale,fsr_cen,gap,xcen,ycen,cenphase
bool use_grid

# Functions

pointer immap(),rvopen(),tgio_init(),rt_crfile(),imps2r()
real tbhgtr(),imgetr(),clgetr()
bool tbhgtb()
int tbhgti(),clgeti()

begin

      # Get some stack workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(rvname,SZ_FNAME,TY_CHAR)
      call salloc(gname,SZ_FNAME,TY_CHAR)
      call salloc(oname,SZ_FNAME,TY_CHAR)

      # Get the input cube. Note that this should be ZXY sorted

      call clgstr("inameZXY",Memc[iname],SZ_FNAME)
      iferr (iptr = immap(Memc[iname],READ_ONLY,NULL)) {
         call eprintf("Couldn't open input cube \n")
         call erract(EA_ERROR)
      }
      nx = IM_LEN(iptr,2)
      ny = IM_LEN(iptr,3)
      nz = IM_LEN(iptr,1)
      mpu0 = imgetr(iptr,"mpu0")
      dmpu = imgetr(iptr,"dmpu")

      # Get the smoothing box size and the spectral window

      box = clgeti("box")
      if (mod(box,2) == 0) box = box + 1
      if (box > nx || box > ny)
         call error(0,"Box size too big!")
      call clputi("window.p_max",nz)
      window = clgeti("window")
      focus = clgetr("cam_flen")*1000.0

      # Open the input rvf

      call clgstr("rvname",Memc[rvname],SZ_FNAME)
      rvptr = rvopen(rvname,READ_ONLY)

      # Get some info from the rvf...

      wave = tbhgtr(rvptr,"wave")
      pixx = tbhgtr(rvptr,"pixx")
      fscale = pixx/focus
      use_grid = tbhgtb(rvptr,"use_grid")
      if (use_grid) 
         call tbhgtt(rvptr,"gridname",Memc[gname],SZ_FNAME)
      nrefx = tbhgti(rvptr,"reflenx")
      nrefy = tbhgti(rvptr,"refleny")

      # Open the grid structure...

      tgp = tgio_init(use_grid,1,gname,nrefx,nrefy)

      # Get RVF info...

      fsr_cen = tbhgtr(rvptr,"fsrcen")
      gap = tbhgtr(rvptr,"gap")
      xcen = tbhgtr(rvptr,"xc")
      ycen = tbhgtr(rvptr,"yc")
      cenphase = tbhgtr(rvptr,"cenph")
      mode = tbhgti(rvptr,"mode")

      # Get a little workspace

      call malloc(wptr,nz,TY_REAL)

      # Get output file name and open it...

      call clgstr("resname",Memc[oname],SZ_FNAME)
      dims[1] = nx
      dims[2] = ny
      iferr (optr = rt_crfile(Memc[oname],NEW_FILE,NULL,2,dims,TY_REAL)) {
         call eprintf("Couldn't open new output file\n")
         call erract(EA_ERROR)
      }
      ptr1 = imps2r(optr,1,nx,1,ny)

      # Call the main processing routine...

      call sub_resid(iptr,nx,ny,nz,gap,fsr_cen,wave,fscale,xcen,ycen,
         tgp,cenphase,mpu0,dmpu,mode,box,window,Memr[wptr],Memr[ptr1])

      # Close things up...

      call imunmap(optr)
      call tgio_close(tgp)
      call mfree(wptr,TY_REAL)
      call rvclose(rvptr)
      call imunmap(iptr)
      call sfree(sp)
end

procedure sub_resid(iptr,nx,ny,nz,gap,fsr_cen,wave,fscale,xcen,ycen,tgp,
   cenphase,mpu0,dmpu,mode,box,window,temp,res)

# Parameters

pointer iptr,tgp
int nx,ny,nz,mode,window,box
real gap,fsr_cen,wave,fscale,xcen,ycen,cenphase,mpu0,dmpu,
   res[nx,ny],temp[nz]

# Local variables

int half,jx,jy,jyst,jyend,jxst,jxend,ny2,ix,iy,ifsr,nfsr,index,i,
   idrem,z_look_pix,izst,k,krem,iz,j
pointer gptr,ptr,gptr2,gsptr,gsptr2
real zmax,zmid,beta,fscale2,sum,sumres,xnew,ynew,r2,sec_theta,fsr,dfsr,
   z_pre,z_look,mindiff,diff,tmax,val,result

# Functions

pointer tggetl(),imgs3r()

begin

      # Star by setting up some useful variables

      zmax = mpu0 + real(nz-1)*dmpu
      zmid = mpu0 + (real(nz/2) - 1.0)*dmpu
      beta = float(mode)*2.0*gap*fsr_cen/wave
      fscale2 = fscale*fscale
      half = box/2 + 1

      # Get some workspace for the grid slab

      call malloc(gsptr,box*nx*2,TY_REAL)

      # Loop through the map in steps of the box size

      do jy = half,ny-half+1,box {
         jyst = jy
         jyend = min(ny,jyst+box-1)

         # Get a slab of data...

         ptr = imgs3r(iptr,1,nz,1,nx,jyst,jyend)
         ny2 = jyend - jyst + 1

         # Get a slab of the grid...

         do j = jyst,jyend {
            gptr = tggetl(tgp,j)
            gsptr2 = gsptr + 2*(j-jyst)*nx
            call amovr(Memr[gptr],Memr[gsptr2],2*nx)
         }

         # Loop for X in steps of the box size

         do jx = half,nx-half+1,box {
            jxst = jx
            jxend = min(nx,jxst+box-1)

            # Sum the spectra up...
 
            call res_blk(Memr[ptr],nx,ny2,nz,jxst,jxend,jyst,jyend,temp)

            # Zero some summations

            sum = 0.0
            sumres = 0.0

            # Now loop for each spatial pixel in the box...

            do iy = jyst,jyend {
               do ix = jxst,jxend {
                  
                  # Get the coordinates minus the centre

                  gptr2 = gsptr + 2*(iy-jyst)*nx + 2*(ix-1)
                  xnew = Memr[gptr2] - xcen
                  ynew = Memr[gptr2+1] - ycen

                  # Find the predicted phase for xnew,ynew

                  r2 = xnew*xnew + ynew*ynew
                  sec_theta = sqrt(r2*fscale2 + 1.0)
                  fsr = fsr_cen*sec_theta
                  ifsr = abs(int(fsr/dmpu))
                  nfsr = abs(nint(fsr/dmpu))
                  dfsr = abs(fsr/dmpu) - real(ifsr)
                  z_pre = cenphase + beta*(sec_theta - 1.0)

                  # If the predicted position overruns the spectrum at either
                  # end, then calculate how many FSRs should be subtracted in
                  # order to at least bring it back into the spectral range

                  if (z_pre > zmax) 
                     index = int((z_pre - zmax)/fsr) + 1
                  else if (z_pre < mpu0)
                     index = int((z_pre - mpu0)/fsr) - 1
                  else
                     index = 0
                  z_look = z_pre - fsr*real(index)

                  # Now find out if you are better off looking 1 fsr away
                  
                  mindiff = zmax
                  do i = -1,1 {
                     diff = abs(z_look - float(i)*fsr - zmid)
                     if (diff < mindiff) {
                        idrem = i
                        mindiff = diff
                     }
                  }
                  z_look = z_look - real(idrem)*fsr

                  # Where is this in frame numbers?

                  z_look_pix = nint((z_look - mpu0)/dmpu + 1.0)

                  # Search for the maximum within the defined window...

                  tmax = -MAX_REAL
                  izst = z_look_pix - window - 1
                  do k = 1,2*window+1 {
                     izst = izst + 1
                     if (izst < 1) 
                        iz = izst + nfsr
                     else if (izst > nz)
                        iz = izst - nfsr
                     else
                        iz = izst
                     if (temp[iz] > tmax) {
                        tmax = temp[iz]
                        krem = izst
                     }
                  }

                  # Now sum the weighted residuals where the weight is the
                  # intensity at Z and the residual is the Z axis value at
                  # that frame minus the predicted phase...

                  izst = krem - window - 1
                  do i = 1,2*window+1 {
                     izst = izst + 1
                     if (izst < 1) {
                        iz = izst + ifsr
                        val = temp[iz]*(1.0 - dfsr) + temp[iz+1]*dfsr
                     } else if (izst > nz) {
                        iz = izst - ifsr
                        val = temp[iz]*(1.0 - dfsr) + temp[iz+1]*dfsr
                     } else {
                        iz = izst
                        val = temp[iz]
                     }
                     sumres = sumres + val*(mpu0 + real(izst-1)*dmpu - z_look)
                     sum = sum + val
                  }
               }
            }

            # Find the mean residual and put it in the output frame

            if (sum != 0.0) 
               result = sumres/sum
            else
               result = 0.0
            do iy = jyst,jyend
               do ix = jxst,jxend
                  res[ix,iy] = result
         }
      }

      # Take care of the edges

      do iy = half,ny-half+1 {
         do ix = 1,half-1 
            res[ix,iy] = res[half,iy]
         do ix = nx-half+2,nx
            res[ix,iy] = res[nx-half+1,iy]
      }
      do iy = 1,half-1 
         do ix = half,nx-half+1
            res[ix,iy] = res[ix,half]
      do iy = ny-half+2,ny
         do ix = half,nx-half+1
            res[ix,iy] = res[ix,ny-half+1]

      # Now deal with the corners...

      do iy = 1,half-1
         do ix = 1,half-1
            res[ix,iy] = res[half,half]
      do iy = 1,half-1
         do ix = nx-half+2,nx
            res[ix,iy] = res[nx-half+1,half]
      do iy = ny-half+2,ny
         do ix = 1,half-1
            res[ix,iy] = res[half,ny-half+1]
      do iy = ny-half+2,ny
         do ix = nx-half+2,nx
            res[ix,iy] = res[nx-half+1,ny-half+1]

      # Ditch workspace

      call mfree(gsptr,TY_REAL)
end

procedure res_blk(input,nx,ny,nz,jxst,jxend,jyst,jyend,temp)

# Parameters

real input[nz,nx,ny],temp[nz]
int nx,ny,nz,jxst,jxend,jyst,jyend

# Local variables

int i,j,k

begin

      # Clear out the output buffer

      call aclrr(temp,nz)

      # Now sum all the spectra together...

      do j = 1,ny
         do i = jxst,jxend 
            do k = 1,nz
               temp[k] = temp[k] + input[k,i,j]
      
end
