include <math/nlfit.h>
include <imhdr.h>
include <fset.h>
include <tbset.h>
include "tgio.h"
include "rings.h"

define    NDIV         2501
define    DARG         (25.0/real(NDIV-1))
define    DEFALLOC     4096
define    NPARAM       7

procedure t_rings()

# Local variables

pointer sp,rvname,gname,rvptr,rp,tgp
int comp_fac,weight,itmax,nrefx,nrefy
real tolcent,tolrad,tol,sigs,pixx,pixy
bool circle,restrict,use,rvf

# Functions

pointer rvopen(),tgio_init(),ring_init()
int clgeti(),tbhgti(),fstati()
real clgetr(),imgetr()
bool clgetb(),tbhgtb()

begin

      # Get some workspace

      call smark(sp)
      call salloc(rvname,SZ_FNAME,TY_CHAR)
      call salloc(gname,SZ_FNAME,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Get some parameters. First get the rvf/rvt name and a compression
      # factor.

      call clgstr("rvfile",Memc[rvname],SZ_FNAME)
      comp_fac = clgeti("comp_fac")

      # Tolerance in the error of the position of the centre of the ring
      # and the ring's radius

      tolcent = clgetr("tolcent")
      tolrad = clgetr("tolrad")

      # Sort out weighting scheme and the fit tolerance and maximum 
      # number of iterations

      weight = clgeti("weight")
      tol = clgetr("tolfit")
      itmax = clgeti("itmax")

      # Constrain the ring to have an axial ratio of 1.0?

      circle = clgetb("circle")

      # Restrict the region of interest to just an annulus around the 
      # estimated radius of the ring?

      restrict = clgetb("restrict")
      if (restrict) sigs = clgetr("sigs")

      # Use a grid?

      use = clgetb("use_grid")
      if (use) call clgstr("gridname",Memc[gname],SZ_FNAME)

      # Open the rvf/rvt

      rvptr = rvopen(rvname,READ_WRITE)

      # Is this an rvf or an rvt

      rvf = tbhgtb(rvptr,"rvf")

      # Get the reference length from the rvf/rvt file.  This is the length
      # of a line in the input file, in case no compression is being used.

      nrefx = tbhgti(rvptr,"reflenx")
      nrefy = tbhgti(rvptr,"refleny")

      # Start up the tgrid and ring routines...
 
      rp = ring_init()
      tgp = tgio_init(use,comp_fac,gname,nrefx,nrefy)

      # Now call the main subroutine to do the work...

      call sub_rings(rvptr,rvf,comp_fac,restrict,sigs,tolcent,tolrad,weight,
         circle,tol,itmax,rp,tgp)

      # Add grid info to the rvf/rvt

      call tbhadb(rvptr,"use_grid",use)
      if (use) {
         call tbhadt(rvptr,"gridname",Memc[gname])
         pixx = imgetr(TGIO_GPTR(tgp),"pixx")
         call tbhadr(rvptr,"pixx",pixx)
         pixy = imgetr(TGIO_GPTR(tgp),"pixy")
         call tbhadr(rvptr,"pixy",pixy)
      }

      # Close things up...

      call rvclose(rvptr)
      call tgio_close(tgp)
      call ring_close(rp)
      call sfree(sp)
end


procedure sub_rings(rvptr,rvf,comp_fac,restrict,sigs,tolcent,tolrad,weight,
   circle,tol,itmax,rp,tgp)

# Parameters

pointer rvptr,rp,tgp
bool restrict,circle,rvf
real sigs,tolcent,tolrad,tol
int weight,itmax,comp_fac

# Local variables

pointer exptr,zptr,xptr,iptr,sp,sect,iname,fname,gptr
bool alloc
int run,flag,npar,nfit,status,i,ixerr,iyerr,irerr,errsum,nrows,
   irow
real mpu,xc,yc,rad,sig,peak,back,ell,meanerr,dr,r1,r2,par[NPARAM],
   xxc,yyc,errpar[NPARAM],rms

# Functions

int rvnfrm()
pointer immap(),tgio_full()

# Common block with pointer for exponential lookup table

common /lut/ exptr

begin
  
      # Get some workspace

      call smark(sp)
      call salloc(sect,SZ_FNAME,TY_CHAR)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(fname,SZ_FNAME,TY_CHAR)

      # Set up the exponential lookup table

      call malloc(exptr,NDIV+1,TY_REAL)
      call extable(Memr[exptr])

      # Set up the title flag for the user...

      call printf(" Frame   Max      Xc       Yc       R       Sigma")
      call printf("    Back     Axial\n")

      # Set workspace allocation flag

      alloc = false

      # How many rows are their in the rvt/rvf? Divide by to in order
      # to take account of error rows.

      nrows = rvnfrm(rvptr)

      # Fill the whole grid.  Do this rather than fill it a line at
      # a time since it will be more efficient for multiple frames.

      gptr = tgio_full(tgp)

      # Now loop for each row in the rvf/rvt

      do irow = 1,nrows {

         # Get a row from the input rvf/rvt 

         call rvgetl(rvptr,rp,irow,false,run,mpu,xc,yc,rad,sig,peak,back,
            ell,meanerr,flag)

         # If this ring was flagged for skipping, then skip it...

         if (flag == RING_SKIP) {
            call printf("%4d     not used: Frame was flagged for skipping\n")
            call pargi(irow)
            call printf(" \n")
            next
         }

         # Get the name of the file for analysis

         if (rvf) {
            call tbhgtt(rvptr,"fi0001",Memc[iname],SZ_FNAME)
         } else {
            call sprintf(Memc[sect],SZ_FNAME,"fi%04d")
               call pargi(irow)
            call tbhgtt(rvptr,Memc[sect],Memc[iname],SZ_FNAME)
         }

         # Sort out the data section to be used

         call sprintf(Memc[sect],SZ_FNAME,"[*:%d,*:%d,%d]")
            call pargi(comp_fac)
            call pargi(comp_fac)
            if (rvf) 
               call pargi(irow)
            else 
               call pargi(1)

         # Now open the file

         call strcpy(Memc[iname],Memc[fname],SZ_FNAME)
         call strcat(Memc[sect],Memc[fname],SZ_FNAME)
         iptr = immap(Memc[fname],READ_ONLY,NULL)
            
         # Allocate space for the data to be fit.

         if (alloc) {
            call realloc(xptr,2*DEFALLOC,TY_REAL)
            call realloc(zptr,DEFALLOC,TY_REAL)
         } else {
            call malloc(xptr,2*DEFALLOC,TY_REAL)
            call malloc(zptr,DEFALLOC,TY_REAL)
            alloc = true
         }

         # Calculate the radii for restricted areas...

         if (restrict) {
            dr = sigs*sig
            r1 = rad - dr
            r2 = rad + dr
            call tgx2g(tgp,xc,yc,xxc,yyc)
         }

         # Now fill in the pixels...

         call fill(iptr,gptr,restrict,r1,r2,xxc,yyc,xptr,zptr,nfit)

         # Close the input file...

         call imunmap(iptr)

         # Fill in the initial guesses...

         par[1] = peak
         par[2] = xc
         par[3] = yc
         par[4] = rad
         par[5] = sig
         par[6] = back
         par[7] = ell
         npar = NPARAM
         call amovkr(0.0,errpar,npar)

         # Now do the fit...

         call fitring(nfit,Memr[xptr],Memr[zptr],weight,circle,tol,itmax,
            npar,par,errpar,rms,status)

         # Look for errors

         if (status == RING_BAD) {
            call printf("%4d     not used: 2d fit failed...\n")
               call pargi(irow)
            call printf(" \n")
            call rvputl(rvptr,rp,irow,false,run,mpu,par[2],par[3],par[4],
               par[5],par[1],par[6],par[7],rms,RING_BAD)
            call rvputl(rvptr,rp,irow,true,run,mpu,errpar[2],errpar[3],
               errpar[4],errpar[5],errpar[1],errpar[6],errpar[7],rms,RING_BAD)
            next
         }

         # Write out the results

         call printf("%4d")
            call pargi(irow)
         do i = 1,npar {
            call printf("%9.3f")
               call pargr(par[i])
         }
         call printf(" \n")

         # Reject if the maximum intensity is negative

         if (par[1] < 0.0) {
            call printf("%4d     not used: negative intensity for peak\n")
               call pargi(irow)
            call printf(" \n")
            call rvputl(rvptr,rp,irow,false,run,mpu,par[2],par[3],par[4],
               par[5],par[1],par[6],par[7],rms,RING_BAD)
            call rvputl(rvptr,rp,irow,true,run,mpu,errpar[2],errpar[3],
               errpar[4],errpar[5],errpar[1],errpar[6],errpar[7],rms,RING_BAD)
            next
         }

         # Reject if the radius was negative...

         if (par[4] < 0.0) {
            call printf("%4d     not used: negative radius found\n")
               call pargi(irow)
            call printf(" \n")
            call rvputl(rvptr,rp,irow,false,run,mpu,par[2],par[3],par[4],
               par[5],par[1],par[6],par[7],rms,RING_BAD)
            call rvputl(rvptr,rp,irow,true,run,mpu,errpar[2],errpar[3],
               errpar[4],errpar[5],errpar[1],errpar[6],errpar[7],rms,RING_BAD)
            next
         }

         # Write out errors in parameters

         call printf("%4d")
            call pargi(irow)
         do i = 1,npar {
            call printf("%9.3f")
               call pargr(errpar[i])
         }
         call printf(" \n")

         # Define how much the errors are over the tolerances

         ixerr = int(errpar[2]/tolcent)
         iyerr = int(errpar[3]/tolcent)
         irerr = int(errpar[4]/tolrad)

         # If the errors are more than twice the tolerance for any of the
         # above, then reject the fit

         if (ixerr >= 2 || iyerr >= 2 || irerr >= 2) {
            call printf("%4d     not used: errors in 2d fit are too big\n")
               call pargi(irow)
            call printf(" \n")
            call rvputl(rvptr,rp,irow,false,run,mpu,par[2],par[3],par[4],
               par[5],par[1],par[6],par[7],rms,RING_BAD)
            call rvputl(rvptr,rp,irow,true,run,mpu,errpar[2],errpar[3],
               errpar[4],errpar[5],errpar[1],errpar[6],errpar[7],rms,RING_BAD)
            next
         }

         # Sort out the final flag for the fit

         flag = RING_OK
         errsum = ixerr + iyerr + irerr
         if (errsum > 1) flag = RING_DODGY

         # Now write the result...

         call rvputl(rvptr,rp,irow,false,run,mpu,par[2],par[3],par[4],
            par[5],par[1],par[6],par[7],rms,flag)
         call rvputl(rvptr,rp,irow,true,run,mpu,errpar[2],errpar[3],
            errpar[4],errpar[5],errpar[1],errpar[6],errpar[7],rms,flag)
         call printf(" \n")

      }  # End of the frame loop

      # Get rid of some workspace

      call mfree(exptr,TY_REAL)
      if (alloc) {
         call mfree(xptr,TY_REAL)
         call mfree(zptr,TY_REAL)
      }
      call sfree(sp)
end
            
procedure fill(iptr,gptr,restrict,r1,r2,xc,yc,xptr,zptr,nfit)

# Parameters

pointer iptr,xptr,zptr,gptr
int nfit
real r1,r2,xc,yc
bool restrict

# Local variables

long one_l,v1[IM_MAXDIM],v2[IM_MAXDIM]
int nx,ninc,nalloc,offset,line,i,j
pointer ptr1,ptr
real x,y,rad
bool in,more

# Functions

int imgnlr()

begin

      # Set up a few things...

      one_l = 1
      call amovkl(one_l,v1,IM_MAXDIM)
      call amovkl(one_l,v2,IM_MAXDIM)
      nx = IM_LEN(iptr,1)
      ninc = 0
      nalloc = DEFALLOC
      line = 0

      # Now loop for the input frame

      while (imgnlr(iptr,ptr1,v1) != EOF) {
         line = line + 1

         # If there are no restrictions on the position of the pixel, then
         # just send the info into the workspace...

         if (!restrict) {
            ninc = ninc + nx
            more = false
            while (ninc > nalloc) {
               more = true
               nalloc = nalloc + DEFALLOC
            }
            if (more) {
               call realloc(xptr,2*nalloc,TY_REAL)
               call realloc(zptr,nalloc,TY_REAL)
            }
            offset = ninc - nx
            call amovr(Memr[ptr1],Memr[zptr+offset],nx)
            offset = 2*offset
            call amovr(Memr[gptr+offset],Memr[xptr+offset],2*nx)

         # Otherwise, go through and find which ones to use...

         } else {

            do i = 1,nx {
               ptr = gptr + 2*nx*(line-1) + 2*i - 2
             
               # Calculate where you are in the frame

               x = Memr[ptr]
               y = Memr[ptr+1]
               rad = sqrt((x-xc)**2 + (y-yc)**2)
               in = (rad >= r1 && rad <= r2)

               # If you are within the allowed radii, then include this pixel

               if (in) {
                  ninc = ninc + 1

                  # Increase the workspace size if need be

                  more = false
                  while (ninc > nalloc) {
                     more = true
                     nalloc = nalloc + DEFALLOC
                  }
                  if (more) {
                     call realloc(xptr,2*nalloc,TY_REAL)
                     call realloc(zptr,nalloc,TY_REAL)
                  }

                  # Add in this pixel

                  offset = ninc - 1
                  Memr[zptr+offset] = Memr[ptr1+i-1]
                  offset = 2*offset
                  do j = 0,1
                     Memr[xptr+offset+j] = Memr[ptr+j]
               }
            }
         } # end of restrict condition
      } # end of while loop

      # Now reallocate the workspace so that you only have what you need

      call realloc(xptr,2*ninc,TY_REAL)
      call realloc(zptr,ninc,TY_REAL)
      nfit = ninc
end


procedure fitring(nfit,xx,zz,weight,circle,tol,itmax,npar,par,errpar,rms,
   status)

# Parameters

int nfit,weight,npar,itmax,status
real xx[2,nfit],zz[nfit],par[npar],errpar[npar],rms,tol
bool circle

# Local variables

int rfadd,drfadd,i,nfree,wtflag
pointer dpar,plist,wptr,zfit,nl
real variance,chisqr

# Functions

int locpr()
real fixzero()

extern rfunc,drfunc,fixzero

begin

      # Start by setting up a few things for nlfit to use...

      rfadd = locpr(rfunc)
      drfadd = locpr(drfunc)
      call malloc(dpar,npar,TY_REAL)
      call malloc(plist,npar,TY_INT)
      call amovkr(0.1,Memr[dpar],npar)
      do i = 1,npar
         Memi[plist+i-1] = i
      nfree = npar

      # If you want to constrain the axial ratio to 1, then do so...

      if (circle) {
         Memi[plist+6] = 0
         par[7] = 1.0
         errpar[7] = 0.0
         nfree = nfree - 1
      }

      # Initialise the weighting scheme.  Weight = 1 -->  1/sqrt(n),
      # weight = 2 --> equal weights...

      call malloc(wptr,nfit,TY_REAL)
      switch (weight) {
         case 1: 
            wtflag = WTS_USER
            call asqrr(zz,Memr[wptr],nfit,fixzero)
            call advzr(zz,Memr[wptr],Memr[wptr],nfit,fixzero)
         case 2:
            wtflag = WTS_UNIFORM
         default:
            wtflag = WTS_UNIFORM
      }

      # Now do the fit...

      call nlinitr(nl,rfadd,drfadd,par,Memr[dpar],npar,Memi[plist],nfree,
         tol,itmax)         
      call nlfitr(nl,xx,zz,Memr[wptr],nfit,2,wtflag,status)
      switch (status) {
         case DONE:
            call nlpgetr(nl,par,npar)
            call malloc(zfit,nfit,TY_REAL)
            call nlvectorr(nl,xx,Memr[zfit],nfit,2)
            call nlerrorsr(nl,zz,Memr[zfit],Memr[wptr],nfit,variance,chisqr,
               errpar)
            call mfree(zfit,TY_REAL)
            rms = sqrt(variance)
            status = RING_OK
         default:
            status = RING_BAD
      }
      call nlfreer(nl)

      # Free some workspace

      call mfree(wptr,TY_REAL)
      call mfree(plist,TY_INT)
      call mfree(dpar,TY_REAL)
end

real procedure fixzero(val)

# Parameters

real val

# Local variables

real newval

begin

      # Return a zero...

      newval = 0.0
      return(newval)
end

procedure extable(expon)

# Parameters

real expon[NDIV+1]

# Local variables

int i
real arg

begin

      # Create the table

      do i = 1,NDIV {
         arg = DARG*real(i-1)
         expon[i] = exp(-arg)
      }
      expon[NDIV+1] = 0.0
end


procedure rfunc(xx,nvars,par,nparams,zfit)

# Parameters

int nvars,nparams
real xx[nvars],par[nparams],zfit

# Local variables

real x,y,sig,dx,dy,r,rho,arg,exponential,e1,e2,slope,aint
int nplus,iarg1,iarg2
pointer exptr

common /lut/ exptr

begin

      # Set up Gaussian parameters

      x = xx[1]
      y = xx[2]
      sig = 1.0/par[5]**2
      dx = x - par[2]
      dy = (y - par[3])*par[7]
      r = sqrt(dx*dx + dy*dy)
      rho = r - par[4]
      arg = rho*rho*sig*0.5
      nplus = NDIV + 1

      # Find location in Gaussian look-up table

      iarg1 = min(int(arg/DARG)+1,nplus)
      iarg2 = min(iarg1+1,nplus)

      # Interpolate to get a value of the exponential...

      if (iarg1 == nplus && iarg2 == nplus) {
         exponential = 0.0
      } else {
         e1 = Memr[exptr+iarg1-1]
         e2 = Memr[exptr+iarg2-1]
         slope = (e2 - e1)/DARG
         aint = e2 - slope*real(iarg2-1)*DARG
         exponential = slope*arg + aint
      }

      # Now calculate the model...

      zfit = par[1]*exponential + par[6]
end
    
procedure drfunc(xx,nvars,par,dpar,nparams,zfit,derivs)

# Parameters

int nvars,nparams
real xx[nvars],par[nparams],dpar[nparams],zfit,derivs[nparams]

# Local variables

real x,y,sig,dx,dy,r,rho,arg,exponential,e1,e2,slope,aint,lamda
int nplus,iarg1,iarg2
pointer exptr

common /lut/ exptr

begin

      # Set up Gaussian parameters

      x = xx[1]
      y = xx[2]
      sig = 1.0/par[5]**2
      dx = x - par[2]
      dy = (y - par[3])*par[7]
      r = sqrt(dx*dx + dy*dy)
      rho = r - par[4]
      arg = rho*rho*sig*0.5
      nplus = NDIV + 1

      # Find location in Gaussian look-up table

      iarg1 = min(int(arg/DARG)+1,nplus)
      iarg2 = min(iarg1+1,nplus)

      # Interpolate to get a value of the exponential...

      if (iarg1 == nplus && iarg2 == nplus) {
         exponential = 0.0
         slope = 0.0
      } else {
         e1 = Memr[exptr+iarg1-1]
         e2 = Memr[exptr+iarg2-1]
         slope = (e2 - e1)/DARG
         aint = e2 - slope*real(iarg2-1)*DARG
         exponential = slope*arg + aint
      }

      # Now calculate the model and its derivatives...

      zfit = par[1]*exponential + par[6]
      derivs[1] = exponential
      lamda = -par[1]*slope*rho*sig
      derivs[2] = lamda*dx/r
      derivs[3] = lamda*dy*par[7]/r
      derivs[4] = lamda
      derivs[5] = lamda*rho/par[5]
      derivs[6] = 1.0
      derivs[7] = -lamda*dy*dy/(r*par[7])
end
