include <imhdr.h>
include <fset.h>
include <math/gsurfit.h>
include <mach.h>
include <math.h>
include <error.h>

procedure t_tmask()

# Local variables

pointer sp,iname,data,function,iptr,xc,yc,detname
real maskdist,camflen,colflen,dist,truexpix
int fd,index,junk,nx,ny,np,npalloc,nordmax,nord

define NPINIT 16

# Functions

int fstati(),tau_log(),fscan(),strsearch(),strldx(),sscan(),
   strlen(),clgeti(),stridx()
pointer immap()
real clgetr()
bool streq()

begin

      # Get workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(data,SZ_LINE,TY_CHAR)
      call salloc(function,SZ_FNAME,TY_CHAR)
      call salloc(detname,SZ_FNAME,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Find the latest version of FINDPKS log...

      iferr(fd = tau_log("findpks.log",READ_ONLY))
         call erract(EA_ERROR)

      # Read the file name from the log 

      call strcpy(EOS,Memc[iname],SZ_FNAME)
      while (fscan(fd) != EOF) {
         call gargstr(Memc[data],SZ_LINE)
         if (strsearch(Memc[data],"Input file") != 0) {
            index = strldx(":",Memc[data]) + 1
            junk = sscan(Memc[data+index])
            call gargwrd(Memc[iname],SZ_FNAME)
            break
         }
      }
      if (strlen(Memc[iname]) == 0) 
         call error(0,"Couldn't find input file name in FINDPKS log file")

      # Now open that file

      iferr(iptr=immap(Memc[iname],READ_ONLY,0))
         call error(0,"Couldn't open input matrix mask image")
      nx = IM_LEN(iptr,1)
      ny = IM_LEN(iptr,2)
      call imunmap(iptr)

      # Read the peak information from the findpks log file...

      np = 0
      npalloc = NPINIT
      call malloc(xc,npalloc,TY_REAL)
      call malloc(yc,npalloc,TY_REAL)
      while (fscan(fd) != EOF) {
         call gargstr(Memc[data],SZ_LINE)
         if (stridx("#",Memc[data]) != 1) {
            np = np + 1
            if (np > npalloc) {
               npalloc = npalloc + NPINIT
               call realloc(xc,npalloc,TY_REAL)
               call realloc(yc,npalloc,TY_REAL)
            }
            junk = sscan(Memc[data])
            call gargr(Memr[xc+np-1])
            call gargr(Memr[yc+np-1])
         }
      }
      call realloc(xc,np,TY_REAL)
      call realloc(xc,np,TY_REAL)
      call printf("%d peaks read in from findpks log file\n")
         call pargi(np)
      call close(fd)

      # Get some information from general pset

      dist = clgetr("mask_sep")
      camflen = clgetr("cam_flen")
      colflen = clgetr("col_flen")
    
      # Scale distance by the ratio of the camera and collimator focal-lengths

      maskdist = dist*camflen/colflen

      # Calculate the maximum order of fit allowed given the number of
      # points you have...

      nordmax = int(sqrt(real(np))) - 1
      call clputi("order.p_max",nordmax)

      # Get type of polynomial and order

      call clgstr("function.p_min",Memc[data],SZ_LINE)
      call clgwrd("function",Memc[function],SZ_FNAME,Memc[data])
      nord = clgeti("order")

      # Get detector name.  If it is the IPCS, then get a pixel size.

      call clgstr("det_name",Memc[detname],SZ_LINE)
      if (streq(Memc[detname],"IPCS")) 
         truexpix = clgetr("pixx")
      else
         truexpix = -1.0

      # Now do some work...

      call sub_tmask(np,Memr[xc],Memr[yc],nord,nx,ny,Memc[iname],
         truexpix,dist,camflen,colflen,maskdist,Memc[function])

      # Get rid of the workspace

      call mfree(xc,TY_REAL)
      call mfree(yc,TY_REAL)
      call sfree(sp)
end
      

procedure sub_tmask(np,xc,yc,nord,nx,ny,iname,truexpix,dist,
    camflen,colflen,maskdist,function)

# Parameters

int np,nord,nx,ny
real xc[np],yc[np],maskdist,dist,camflen,colflen,truexpix
char function[ARB],iname[ARB]

# Local variables

pointer sp,data,wpt1,wpt2,sfx,sfy,wpt3,wpt4,wpt5,xcoef,ycoef
int surface,xterms,nunk,ncoefs,ier,ng,i,j,fd,ist,ifn,nsave
real xmin,xmax,ymin,ymax,sigx,sigy,dx,dy,xx,yy,xgrid[100],ygrid[100],
   dxx[100],dxy[100],dyx[100],dyy[100],pixx[100],pixy[100],ellipt[100],
   phi_deg,shrink,xpix

# Functions

bool streq()
int gsgeti(),tau_log()
real gseval()

begin

      # Get stack workspace

      call smark(sp)
      call salloc(data,SZ_LINE,TY_CHAR)

      # Get some workspace and assign a distance to each peak...

      call malloc(wpt1,np,TY_REAL)
      call malloc(wpt2,np,TY_REAL)
      call assign_dist(np,xc,yc,maskdist,Memr[wpt1],Memr[wpt2],
         phi_deg)

      # Set up parameters for fits

      if (streq(function,"legendre"))
         surface = GS_LEGENDRE
      else
         surface = GS_CHEBYSHEV
      xmin = 1.0
      xmax = real(nx)
      ymin = 1.0
      ymax = real(ny)
      xterms = YES

      # Set up two separate fits, one for each X and Y

      nunk = nord + 1
      call gsinit(sfx,surface,nunk,nunk,xterms,xmin,xmax,ymin,ymax)
      call gsinit(sfy,surface,nunk,nunk,xterms,xmin,xmax,ymin,ymax)

      # Do the X fit...

      call malloc(wpt3,np,TY_REAL)
      call malloc(wpt4,np,TY_REAL)
      ncoefs = gsgeti(sfx,GSNCOEFF)  
      call malloc(wpt5,ncoefs,TY_REAL)
      call gsfit(sfx,xc,yc,Memr[wpt1],Memr[wpt3],np,WTS_UNIFORM,ier)
      if (ier != OK) {
         call error(0,"X fit failed...")
      } else {
         do i = 1,np 
            Memr[wpt4+i-1] = gseval(sfx,xc[i],yc[i])
         call gserrors(sfx,Memr[wpt1],Memr[wpt3],Memr[wpt4],sigx,
            Memr[wpt5])
         sigx = sqrt(sigx)
         call printf("X solution fit with rms: %0.6g microns\n")
            call pargr(sigx)
      }

      # Now the Y fit...

      call gsfit(sfy,xc,yc,Memr[wpt2],Memr[wpt3],np,WTS_UNIFORM,ier)
      if (ier != OK) {
         call error(0,"Y fit failed...\n")
      } else {
         do i = 1,np 
            Memr[wpt4+i-1] = gseval(sfy,xc[i],yc[i])
         call gserrors(sfy,Memr[wpt2],Memr[wpt3],Memr[wpt4],sigy,
            Memr[wpt5])
         sigy = sqrt(sigy)
         call printf("Y solution fit with rms: %0.6g microns\n")
            call pargr(sigy)
      }

      # Free some workspace

      call mfree(wpt1,TY_REAL)
      call mfree(wpt2,TY_REAL)
      call mfree(wpt3,TY_REAL)
      call mfree(wpt4,TY_REAL)
      call mfree(wpt5,TY_REAL)

      # Evaluate the shrink factor for the IPCS...

      if (truexpix < 0.0) 
         shrink = 1.0
      else {
         xx = real(nx)*0.5
         yy = real(ny)*0.5
         call gsder(sfx,xx,yy,dxx,1,1,0)
         call gsder(sfy,xx,yy,dyx,1,1,0)
         xpix = sqrt(dxx[1]**2 + dyx[1]**2)
         shrink = truexpix/xpix
      }
     
      # Now evaluate the pixel sizes in a grid across the mask image

      dx = real(nx-1)/9.0
      dy = real(ny-1)/9.0
      ng = 0
      do j = 1,10 {
         yy = real(j-1)*dy + 1.0
         do i = 1,10 {
            xx = real(i-1)*dx + 1.0
            ng = ng + 1
            xgrid[ng] = xx
            ygrid[ng] = yy
         }
      }
      call gsder(sfx,xgrid,ygrid,dxx,100,1,0)
      call gsder(sfx,xgrid,ygrid,dxy,100,0,1)
      call gsder(sfy,xgrid,ygrid,dyx,100,1,0)
      call gsder(sfy,xgrid,ygrid,dyy,100,0,1)
      do i = 1,100 {
         pixx[i] = shrink*sqrt(dxx[i]**2 + dyx[i]**2)
         pixy[i] = shrink*sqrt(dyy[i]**2 + dxy[i]**2)
         ellipt[i] = pixy[i]/pixx[i]
      }

      # Open a log file

      fd = tau_log("tmask.log",NEW_FILE)
      call sysid(Memc[data],SZ_LINE)
      call fprintf(fd,"#%s\n")
         call pargstr(Memc[data])
      call fprintf(fd,"#File used in findpks: %30t %s\n")
         call pargstr(iname)
      call fprintf(fd,"#Pinhole distance: %30t %0.6g microns\n")
         call pargr(dist)
      call fprintf(fd,"#Camera foc-len: %30t %0.6g mm\n")
         call pargr(camflen)
      call fprintf(fd,"#Collimator foc-len: %30t %0.6g mm\n")
         call pargr(colflen)
      call fprintf(fd,"#Proj. pinhole dist: %30t %0.6g microns\n")
         call pargr(maskdist)
      call fprintf(fd,"#Fitting function: %30t %s \n")
         call pargstr(function)
      call fprintf(fd,"#Order of polynomial: %30t %d\n")
         call pargi(nord)
      if (truexpix > 0.0) {
         call fprintf(fd,"#True x pixel size: %30t %0.6g microns\n")
            call pargr(truexpix)
         call fprintf(fd,"#Shrink factor: %30t %0.6g\n")
            call pargr(shrink)
      }

      # Write out a grid of X results

      call fprintf(fd,"# \n")
      call fprintf(fd,"# \n")
      call fprintf(fd,"# %30t x pixel results\n")
      call fprintf(fd,"# %38t x\n")
      call fprintf(fd,"# %14t %6.1f")
         call pargr(1.0)
      do i = 2,10 {
         call fprintf(fd,"%6.1f")
         call pargr(dx*real(i-1)+1.0)
      }
      call fprintf(fd,"\n")
      call fprintf(fd,"#---------------------------------------")
      call fprintf(fd,"----------------------------------\n")
      do j = 1,10 {
         ist = 10*(j-1) + 1
         ifn = ist + 9
         if (j == 5) 
            call fprintf(fd,"#  y  %6.1f |")
         else
            call fprintf(fd,"#     %6.1f |")
         call pargr(dy*real(j-1)+1.0)
         do i = ist,ifn {
            call fprintf(fd,"%6.1f")
            call pargr(pixx[i])
         }
         call fprintf(fd,"\n")
      }

      # Write out a grid of Y results

      call fprintf(fd,"# \n")
      call fprintf(fd,"# \n")
      call fprintf(fd,"# %30t y pixel results\n")
      call fprintf(fd,"# %38t x\n")
      call fprintf(fd,"# %14t %6.1f")
         call pargr(1.0)
      do i = 2,10 {
         call fprintf(fd,"%6.1f")
         call pargr(dx*real(i-1)+1.0)
      }
      call fprintf(fd,"\n")
      call fprintf(fd,"#---------------------------------------")
      call fprintf(fd,"----------------------------------\n")
      do j = 1,10 {
         ist = 10*(j-1) + 1
         ifn = ist + 9
         if (j == 5) 
            call fprintf(fd,"#  y  %6.1f |")
         else
            call fprintf(fd,"#     %6.1f |")
         call pargr(dy*real(j-1)+1.0)
         do i = ist,ifn {
            call fprintf(fd,"%6.1f")
            call pargr(pixy[i])
         }
         call fprintf(fd,"\n")
      }
          
      # Write out a grid of ellipticity results

      call fprintf(fd,"# \n")
      call fprintf(fd,"# \n")
      call fprintf(fd,"# %30t ellipticity results\n")
      call fprintf(fd,"# %38t x\n")
      call fprintf(fd,"# %14t %6.1f")
         call pargr(1.0)
      do i = 2,10 {
         call fprintf(fd,"%6.1f")
         call pargr(dx*real(i-1)+1.0)
      }
      call fprintf(fd,"\n")
      call fprintf(fd,"#---------------------------------------")
      call fprintf(fd,"----------------------------------\n")
      do j = 1,10 {
         ist = 10*(j-1) + 1
         ifn = ist + 9
         if (j == 5) 
            call fprintf(fd,"#  y  %6.1f |")
         else
            call fprintf(fd,"#     %6.1f |")
         call pargr(dy*real(j-1)+1.0)
         do i = ist,ifn {
            call fprintf(fd,"%6.3f")
            call pargr(ellipt[i])
         }
         call fprintf(fd,"\n")
      }

      # Get the fit parameters...

#      nsave = gsstati(sfx,GSNCOEFF)  # Uncomment when gsurfit routine
#                                     # is installed.
      nsave = GS_SAVECOEFF + ncoefs   # Remove when uncommenting previous statement
      call malloc(xcoef,nsave,TY_REAL)
      call malloc(ycoef,nsave,TY_REAL)
      call gssave(sfx,Memr[xcoef])
      call gssave(sfy,Memr[ycoef])

      # Write out the fit information...

      call fprintf(fd,"%s\n")
         call pargstr(function)
      call fprintf(fd,"%d\n")
         call pargi(nord)
      call fprintf(fd,"%d\n")
         call pargi(ncoefs)
      call fprintf(fd,"%d\n")
         call pargi(nsave)
      do i = 1,nsave {
         call fprintf(fd,"%0.6g\n")
            call pargr(Memr[xcoef+i-1])
      }
      do i = 1,nsave {
         call fprintf(fd,"%0.6g\n")
            call pargr(Memr[ycoef+i-1])
      }
      call mfree(xcoef,TY_REAL)
      call mfree(ycoef,TY_REAL)

      # Write out shrink factor

      call fprintf(fd,"%0.6g\n")
         call pargr(shrink)

      # Close up the file and release some space

      call close(fd)
      call gsfree(sfx)
      call gsfree(sfy)
      call sfree(sp)
end

procedure assign_dist(np,x,y,dist,xg,yg,phi_deg)

# Parameters

int np
real x[np],y[np],dist,xg[np],yg[np],phi_deg

# Local variables

real avex,avey,rmin,dx,dy,rad,phi,sumphi,xc,yc,xx,yy,theta,xdis,ydis
int i,irem,jrem,ifound,isign

# Functions

real asumr()

begin

      # Start by finding the peak closest to the middle of the frame...

      avex = asumr(x,np)/real(np)
      avey = asumr(y,np)/real(np)
      rmin = MAX_REAL
      do i = 1,np {
         dx = x[i] - avex
         dy = y[i] - avey
         rad = sqrt(dx*dx + dy*dy)
         if (rad < rmin) {
            irem = i
            rmin = rad
         }
      }

      # Now find the peak to the right of the central one, but in the
      # same grid row...

      rmin = MAX_REAL
      do i = 1,np {
         if (i != irem) {
            dx = x[i] - x[irem]
            dy = y[i] - y[irem]
            if (dx > 0.0 && abs(dx) > abs(dy)) {
               rad = sqrt(dx*dx + dy*dy)
               if (rad < rmin) {
                  jrem = i
                  rmin = rad
               }
            }
         }
      }

      # Calculate a rough rotation from these two peaks

      dx = x[jrem] - x[irem]
      dy = y[jrem] - y[irem]
      phi = atan(dy/dx)

      # Now look at all four points adjacent to the middle one and
      # calculate a getter value of the rotation.  Phi is always defined
      # with respect to the pixel X axis and is always -45 <= phi <= 45

      ifound = 0
      sumphi = 0.0
      do i = 1,np {
         dx = abs(x[i] - x[irem])
         dy = abs(y[i] - y[irem])
         rad = sqrt(dx*dx + dy*dy)
         if (rad >= 0.9*rmin && rad <= 1.1*rmin) {
            if (dx > dy) 
               sumphi = sumphi + atan(dy/dx)
            else
               sumphi = sumphi + atan(dx/dy)
            ifound = ifound + 1
         }
      }

      # Average the measurements of phi and let the user know

      sumphi = sumphi/real(ifound)
      isign = 1
      if (phi < 0) isign = -1
      phi = sumphi*real(isign)
      phi_deg = RADTODEG(phi)
      call printf(" \n")
      call printf("From %d points, rough rotation of mask is: %0.6g degrees\n")
         call pargi(ifound)
         call pargr(phi_deg)

      # Now calculate a grid position for each peak

      xc = x[irem]
      yc = y[irem]
      do i = 1,np {
         if (i == irem) {
            xg[i] = 0.0
            yg[i] = 0.0
         } else {
            xx = x[i]
            yy = y[i]
            dx = xx - xc
            dy = yy - yc
            theta = atan2(dy,dx)
            rad = sqrt(dx*dx + dy*dy)
            xdis = rad*cos(theta-phi)
            ydis = rad*sin(theta-phi)
            xg[i] = real(nint(xdis/rmin))*dist
            yg[i] = real(nint(ydis/rmin))*dist
         }
      }
end
