include <mach.h>                             
include <gset.h>
include <fset.h>
include <pkg/gtools.h>
include <error.h>
include "rings.h"

define HELPFIL     "rgo$taucal/curfils/xrvf.key"
define PROMPT      "xrvf options"

procedure t_xrvf()

# Local variables

pointer sp,rvname,data,rp,rvptr,slims,sfit,slopes,eslopes,inters,
   einters,cps,ecps,wtname,wtdict
real wave,focus,pixx,fscale,fsrcen,errfsr,gap,errgap,xc,errxc,yc,erryc,
   cenph,errcp
int nser,k,i,j,fd,mode,iwts
bool use_grid

# Functions

pointer ring_init(),rvopen()
real tbhgtr(),clgetr()
int tbhgti(),tau_log(),fstati(),clgwrd()
bool tbhgtb()

begin

      # Start by getting the usual workspace

      call smark(sp)
      call salloc(rvname,SZ_FNAME,TY_CHAR)
      call salloc(data,SZ_LINE,TY_CHAR)
      call salloc(wtname,SZ_LINE,TY_CHAR)
      call salloc(wtdict,SZ_LINE,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Open the ring structure

      rp = ring_init()

      # Open the rvf...

      call clgstr("rvfile",Memc[rvname],SZ_FNAME)
      iferr (rvptr = rvopen(rvname,READ_WRITE)) call erract(EA_ERROR)

      # Get some information from the header of the rvf...

      wave = tbhgtr(rvptr,"wave")
      nser = tbhgti(rvptr,"nser")
      
      # Get series limit information

      call salloc(slims,2*nser,TY_INT)
      k = -1
      do i = 1,nser {
         do j = 1,2 {
            call sprintf(Memc[data],SZ_LINE,"ser%d_%d")
               call pargi(i)
               call pargi(j)
            k = k + 1
            Memi[slims+k] = tbhgti(rvptr,Memc[data])
         }
      }

      # Get camera focal length

      focus = clgetr("cam_flen")
      focus = focus*1000.0

      # Get the weighting scheme for initial fit...

      call clgstr("wtscheme.p_min",Memc[wtdict],SZ_LINE)
      iwts = clgwrd("wtscheme",Memc[wtname],SZ_LINE,Memc[wtdict])

      # If a grid was used, then find the pixel size...

      use_grid = tbhgtb(rvptr,"use_grid")
      if (use_grid) {
         pixx = tbhgtr(rvptr,"pixx")

      # Otherwise, get this information from the user...

      } else {
         call clgstr("det_name",Memc[data],SZ_LINE)
         call printf("I need the pixel size for detector %s \n")
            call pargstr(Memc[data])
         pixx = clgetr("pixx")
      }
      fscale = pixx/focus

      # Get some workspace for the results...

      call salloc(sfit,nser,TY_INT)
      call salloc(slopes,nser,TY_REAL)
      call salloc(eslopes,nser,TY_REAL)
      call salloc(inters,nser,TY_REAL)
      call salloc(einters,nser,TY_REAL)
      call salloc(cps,nser,TY_REAL)
      call salloc(ecps,nser,TY_REAL)

      # Do the work...

      call sub_xrvf(rvptr,rp,fscale,wave,nser,iwts,Memi[slims],Memi[sfit],
         Memr[slopes],Memr[eslopes],Memr[inters],Memr[einters],Memr[cps],
         Memr[ecps],fsrcen,errfsr,gap,errgap,xc,errxc,yc,erryc,cenph,
         errcp,mode)

      # Write all these results to a log file...

      fd = tau_log("xrvf.log",NEW_FILE)
      call sysid(Memc[data],SZ_LINE)
      call fprintf(fd,"#%s\n")
         call pargstr(Memc[data])
      call fprintf(fd,"#Program: %30t XRVF\n")
      call fprintf(fd,"#Input RVF: %30t %s\n")
         call pargstr(Memc[rvname])
      call fprintf(fd,"#Wavelength: %30t %0.6g microns\n")
         call pargr(wave)
      call fprintf(fd,"#Number of series: %30t %d\n")
         call pargi(nser)
      do i = 1,nser {
         call fprintf(fd,"#Limits for series %d: %30t %d %d\n")
            call pargi(i)
            call pargi(Memi[slims+2*(i-1)])
            call pargi(Memi[slims+2*(i-1)+1])
      }
      call fprintf(fd,"#Camera focal length: %30t %0.6g mm\n")
         call pargr(focus/1000.0)
      call fprintf(fd,"#Detector pixel size: %30t %0.6g microns\n")
         call pargr(pixx)      
      call fprintf(fd,"#Fits to line R**2 = A + B*Z\n")
      do i = 1,nser {
         if (Memi[sfit+i-1] == 0) {
            call fprintf(fd,"#Series # %d: no fit done\n")
               call pargi(i)
         } else {
            call fprintf(fd,"#Series # %d: A = %0.6g +/- %0.6g")
               call pargi(i)
               call pargr(Memr[inters+i-1])
               call pargr(Memr[einters+i-1])
            call fprintf(fd,"   B = %0.6g +/- %0.6g \n")
               call pargr(Memr[slopes+i-1])
               call pargr(Memr[eslopes+i-1])
         }
      }
      call fprintf(fd,"#Central phase: %30t %0.6g +/- %0.6g MPUs\n")
         call pargr(cenph)
         call pargr(errcp)
      call fprintf(fd,"#On axis FSR: %30t %0.6g +/- %0.6g MPUs\n")
         call pargr(fsrcen)
         call pargr(errfsr)
      call fprintf(fd,"#Gap estimate: %30t %0.6g +/- %0.6g microns\n")
         call pargr(gap)
         call pargr(errgap)
      call fprintf(fd,"#Mode of operation: %30t %d \n")
         call pargi(mode)
      call fprintf(fd,"#Mean centre: %30t %0.6g +/- %0.6g  %0.6g +/- %0.6g \n")
         call pargr(xc)
         call pargr(errxc)
         call pargr(yc)
         call pargr(erryc)
      call close(fd)

      # Now write results to rvf header...

      call tbhadr(rvptr,"fsrcen",fsrcen)
      call tbhadr(rvptr,"errfsr",errfsr)
      call tbhadr(rvptr,"gap",gap)
      call tbhadr(rvptr,"errgap",errgap)
      call tbhadr(rvptr,"xc",xc)
      call tbhadr(rvptr,"errxc",errxc)
      call tbhadr(rvptr,"yc",yc)
      call tbhadr(rvptr,"erryc",erryc)
      call tbhadi(rvptr,"mode",mode)
      call tbhadr(rvptr,"cenph",cenph)
      call tbhadr(rvptr,"errcp",errcp)
      do i = 1,nser {
         call sprintf(Memc[data],SZ_LINE,"inters%d")
            call pargi(i)
         call tbhadr(rvptr,Memc[data],Memr[inters+i-1])
         call sprintf(Memc[data],SZ_LINE,"slopes%d")
            call pargi(i)
         call tbhadr(rvptr,Memc[data],Memr[slopes+i-1])
      }

      # Close things up...

      call rvclose(rvptr)
      call ring_close(rp)
      call sfree(sp)
end



procedure sub_xrvf(rvptr,rp,fscale,wave,nser,iwts,serlims,sfit,slopes,
   eslopes,inters,einters,cps,ecps,fsrcen,errfsr,gap,errgap,xc,errxc,
   yc,erryc,cenph,errcp,mode)

# Parameters

pointer rvptr,rp
real fscale,wave,fsrcen,errfsr,gap,errgap,xc,errxc,yc,erryc,cenph,errcp,
   slopes[nser],eslopes[nser],inters[nser],einters[nser],cps[nser],ecps[nser]
int nser,iwts,serlims[2,nser],mode,sfit[nser]

# Local variables

pointer sp,r2ptr,zptr,woptr,wcptr,gp,gt,xptr,yptr,wptr,cmd,wtname,wtdict
int nframes,key,index,flag1,flag2,iser,nfit,fidser,junk,i,nsfit,nf2,
   ifirst,run,flag,wcs,irem,iwtsc
bool quit,drawfit,draw,drawplot,init
real weight,zmid,diff,cenfirst,sum,sumerr,fsr,sumx,sumxx,sumy,sumyy,
   sumwt,wt,mpu,rad,sig,peak,back,ell,meanerr,cx,cy

# Functions

int rvnfrm(),fndind(),clgeti(),clgcur(),clgwrd()
pointer gopen(),gt_init()
real clgetr(),asumi()

begin

      # Get some workspace 

      call smark(sp)
      call salloc(cmd,SZ_LINE,TY_CHAR)
      call salloc(wtname,SZ_LINE,TY_CHAR)
      call salloc(wtdict,SZ_LINE,TY_CHAR)

      # Now get some workspace for the input RVF info and read it in...

      nframes = rvnfrm(rvptr)
      call malloc(r2ptr,nframes,TY_REAL)
      call malloc(zptr,nframes,TY_REAL)
      call malloc(woptr,3*nframes,TY_REAL)
      call rdrvf(rvptr,rp,nframes,Memr[r2ptr],Memr[zptr],Memr[woptr])

      # Now get a separate copy of the weights...

      call malloc(wcptr,nframes,TY_REAL)
      call amovr(Memr[woptr+nframes*(iwts-1)],Memr[wcptr],nframes)
      iwtsc = iwts

      # Open the plot device and gtools

      gp = gopen("stdgraph",NEW_FILE,STDGRAPH)
      gt = gt_init()
      call gt_sets(gt,GTXLABEL,"Z (mpu)")
      call gt_sets(gt,GTYLABEL,"R**2")
      call gt_sets(gt,GTTITLE,"R**2 vs Z")

      # Set up so that a drawing is done first

      key = 'r'
      quit = false
      drawfit = false
      init = true
      call amovki(0,sfit,nser)

      # Enter main cursor loop

      while (!quit) {

         # Get the next keystroke

         if (! init) junk = clgcur("graphcur",cx,cy,wcs,key,Memc[cmd],SZ_LINE)
         init = false

         # Switch for the various keywords...

         switch (key) {
            case '?':
               call gpagefile(gp,HELPFIL,PROMPT)
               draw = false
            case 'q':
               quit = true
               draw = false
            case 'I':
               call fatal(0,"Immediate interrupt")
            case 'b':
               iwtsc = iwts
               call amovr(Memr[woptr+nframes*(iwts-1)],Memr[wcptr],nframes)
               draw = true
               drawfit = false
               drawplot = true
            case 'n':
               call clgstr("wtscheme.p_min",Memc[wtdict],SZ_LINE)
               iwtsc = clgwrd("wtscheme",Memc[wtname],SZ_LINE,Memc[wtdict])
               call amovr(Memr[woptr+nframes*(iwtsc-1)],Memr[wcptr],nframes)
            case 'w':
               index = fndind(cx,cy,nframes,Memr[zptr],Memr[r2ptr])
               call printf("Current weight = %0.6g  ")
                  call pargr(Memr[wcptr+index-1])
               weight = clgetr("weight")
               flag1 = 1
               if (Memr[wcptr+index-1] < EPSILONR) flag1 = 0
               flag2 = 1
               if (weight < EPSILONR) flag2 = 0
               if (flag1 != flag2) {
                  call erasept(gp,Memr[zptr+index-1],Memr[r2ptr+index-1],flag1)
                  call plotpt(gp,Memr[zptr+index-1],Memr[r2ptr+index-1],flag2)
                  call gflush(gp)
               }
               Memr[wcptr+index-1] = weight
               draw = false
            case 'f':
               do iser = 1,nser {
                  nfit = serlims[2,iser] - serlims[1,iser] + 1
                  call malloc(xptr,nfit,TY_REAL)
                  call malloc(yptr,nfit,TY_REAL)
                  call malloc(wptr,nfit,TY_REAL)
                  call collect(nframes,Memr[r2ptr],Memr[zptr],Memr[wcptr],
                     serlims[1,iser],serlims[2,iser],nfit,Memr[yptr],
                     Memr[xptr],Memr[wptr])
                  if (nfit >= 2) {
                     call fitline(Memr[xptr],Memr[yptr],Memr[wptr],nfit,
                        inters[iser],slopes[iser],einters[iser],eslopes[iser])
                     sfit[iser] = 1
                  } else {
                     sfit[iser] = 0
                  }
                  call mfree(xptr,TY_REAL)
                  call mfree(yptr,TY_REAL)
                  call mfree(wptr,TY_REAL)
               }
               draw = true
               drawfit = true
               drawplot = false
            case 'r':
               draw = true
               drawplot = true
            case 's':
               call clputi("fidser.p_max",nser)
               fidser = clgeti("fidser")
               nfit = serlims[2,fidser] - serlims[1,fidser] + 1
               call malloc(xptr,nfit,TY_REAL)
               call malloc(yptr,nfit,TY_REAL)
               call malloc(wptr,nfit,TY_REAL)
               call collect(nframes,Memr[r2ptr],Memr[zptr],Memr[wcptr],
                  serlims[1,fidser],serlims[2,fidser],nfit,Memr[yptr],
                  Memr[xptr],Memr[wptr])
               if (nfit < 2) 
                  call error(0,"Too few points for a series to be fit!")
               call fitline(Memr[xptr],Memr[yptr],Memr[wptr],nfit,
                  inters[fidser],slopes[fidser],einters[fidser],eslopes[fidser])
               sfit[fidser] = 1
               call mfree(xptr,TY_REAL)
               call mfree(yptr,TY_REAL)
               call mfree(wptr,TY_REAL)
               do iser = 1,nser {
                  if (iser != fidser) {
                     nfit = serlims[2,iser] - serlims[1,iser] + 1
                     call malloc(xptr,nfit,TY_REAL)
                     call malloc(yptr,nfit,TY_REAL)
                     call malloc(wptr,nfit,TY_REAL)
                     call collect(nframes,Memr[r2ptr],Memr[zptr],
                        Memr[wcptr],serlims[1,iser],serlims[2,iser],nfit,
                        Memr[yptr],Memr[xptr],Memr[wptr])
                     if (nfit >= 2) {
                        call fitsame(Memr[xptr],Memr[yptr],Memr[wptr],nfit,
                           slopes[fidser],eslopes[fidser],inters[iser],
                           slopes[iser],einters[iser],eslopes[iser])
                        sfit[iser] = 1
                     } else {
                        sfit[iser] = 0
                     }
                     call mfree(xptr,TY_REAL)
                     call mfree(yptr,TY_REAL)
                     call mfree(wptr,TY_REAL)
                  }
               }
               draw = true
               drawfit = true
               drawplot = false
         } # end of switch for key

         # If you have been asked to do so, then draw the points and
         # the fits

         if (draw) {
            call replot(gp,gt,nframes,Memr[zptr],Memr[r2ptr],Memr[wcptr],
               nser,serlims,inters,slopes,drawplot,drawfit)
            draw = false
         }
      }

      # Close the graphics things 

      call gt_free(gt)
      call gclose(gp)

      # See how many series have been fit...Warn accordingly

      nsfit = nint(asumi(sfit,nser))
      if (nsfit == 0) call error(0,"No series have been fit")

      # Now, give the user a few answers...

      call printf(" \n")
      call printf(" \n")
      call printf("Fits to line R**2 = A + B*Z\n")
      call printf(" \n")
      do i = 1,nser {
         if (sfit[i] == 1) {
            call printf("Series # %d  A = %0.6g +/- %0.6g")
               call pargi(i)
               call pargr(inters[i])
               call pargr(einters[i])
            call printf("   B = %0.6g +/- %0.6g \n")
               call pargr(slopes[i])
               call pargr(eslopes[i])
         }
      }
               
      # Calculate the central phase of each series

      nf2 = nframes/2
      zmid = Memr[zptr+nf2-1]
      diff = MAX_REAL
      do i = 1,nser {
         if (sfit[i] == 1) {
            cps[i] = -inters[i]/slopes[i]
            ecps[i] = cps[i]*sqrt((eslopes[i]/slopes[i])**2 +
               (einters[i]/inters[i])**2)
            if (abs(cps[i] - zmid) < diff ) {
               diff = abs(cps[i] - zmid)
               cenph = cps[i]
               errcp = ecps[i]
               irem = i
            }
         }
      }

      # Which is the first series with a good fit?

      for (i=1; i <= nser && sfit[i] == 0; i = i + 1)
         ;
      cenfirst = cps[i]
      ifirst = i

      # If only one series was fit, then prompt for an estimate of the
      # fsr which can be used later...

      if (nsfit == 1) {
         call printf(" \n")
         call printf("***Only one series has been fit, ergo the FSR cannot")
         call printf(" be calculated.  Please give me an estimate of the")
         call printf("FSR in MPUs***\n")
         fsrcen = clgetr("fsrcen")
         errfsr = 0.0

       # Otherwise find the mean FSR...

       } else {
          sum = 0.0
          sumerr = 0.0
          do i = ifirst+1,nser {
             if (sfit[i] == 0) next
             fsr = (cps[i] - cenfirst)/real(i-ifirst)
             sum = sum + fsr
             sumerr = sumerr + ecps[i]**2
          }
         fsrcen = sum/real(nsfit-1)
         errfsr = sqrt(sumerr/real(nsfit-1))
      }
      call printf(" \n")
      call printf("On axis FSR is %0.6g +/- %0.6g   MPUs\n")
         call pargr(fsrcen)
         call pargr(errfsr)

      # Now find the mean gap and mode of operation

      mode = nint(slopes[ifirst]/abs(slopes[ifirst]))
      gap = wave/(fscale*fscale*abs(slopes[irem])*fsrcen)
      errgap = gap*sqrt((eslopes[irem]/slopes[irem])**2 + (errfsr/fsrcen)**2)
      call printf(" \n")
      call printf("Etalon gap from series %d is: %0.6g +/- %0.6g   microns\n")
         call pargi(irem)
         call pargr(gap)
         call pargr(errgap)
      call printf(" \n")
      call printf("Mode of operation variable is %d\n")
         call pargi(mode)
      call printf(" \n")

      # Calculate the mean centre...First read a line of the RVF

      sumx = 0.0
      sumxx = 0.0
      sumy = 0.0
      sumyy = 0.0
      sumwt = 0.0
      do i = 1,nframes {
         wt = Memr[wcptr+i-1]
         if (wt < EPSILONR) next
         call rvgetl(rvptr,rp,i,false,run,mpu,xc,yc,rad,sig,peak,back,
            ell,meanerr,flag)
         sumx = sumx + xc*wt
         sumxx = sumxx + xc*xc*wt
         sumy = sumy + yc*wt
         sumyy = sumyy + yc*yc*wt
         sumwt = sumwt + wt
      }
      xc = sumx/sumwt
      errxc = sqrt(sumxx/sumwt - xc*xc)
      yc = sumy/sumwt
      erryc = sqrt(sumyy/sumwt - yc*yc)
      call printf("Mean centre is: %0.6g +/- %0.6g  %0.6g +/- %0.6g \n")
         call pargr(xc)
         call pargr(errxc)
         call pargr(yc)
         call pargr(erryc)

      # Ditch some workspace...

      call mfree(r2ptr,TY_REAL)
      call mfree(zptr,TY_REAL)
      call mfree(woptr,TY_REAL)
      call mfree(wcptr,TY_REAL)
      call sfree(sp)
end

int procedure fndind(cx,cy,nframes,z,r2)

# Parameters

real cx,cy,z[nframes],r2[nframes]
int nframes

# Local variables

real rmin,rad
int i,irem

begin

      # Initialise the minimum radius

      rmin = MAX_REAL

      # Loop for each entry...

      do i = 1,nframes {
         rad = abs(cx - z[i])
         if (rad < rmin) {
            rmin = rad
            irem = i
         }
      }

      # Return the index of the minimum radius...

      return (irem)
end

procedure replot(gp,gt,nf,z,r2,wts,nser,serlims,inters,slopes,drawplot,
   drawfit)

# Parameters

pointer gp,gt
int nf,nser,serlims[2,nser]
real z[nf],r2[nf],wts[nf],inters[nser],slopes[nser]
bool drawplot,drawfit

# Local variables

int i,flag,j
real xp[2],yp[2],linewidth

# Functions

real gstatr()

begin

      # Start by redrawing the plot, if that's what you want...

      if (drawplot) {
         call gframe(gp)
         call gascale(gp,z,nf,1)
         call gascale(gp,r2,nf,2)
         call gt_swind(gp,gt)
         call gt_labax(gp,gt)
         do i = 1,nf {
            flag = 1
            if (wts[i] < EPSILONR) flag = 0
            call plotpt(gp,z[i],r2[i],flag)
         }
         call gflush(gp)
      }

      # Now draw the fits...

      if (drawfit) {
         linewidth = gstatr(gp,G_PLWIDTH)
         call gsetr(gp,G_PLWIDTH,2.0*linewidth)
         do i = 1,nser {
            do j = 1,2 {
               xp[j] = z[serlims[j,i]]
               yp[j] = inters[i] + slopes[i]*xp[j]
            }
            call gpline(gp,xp,yp,2)
         }
         call gsetr(gp,G_PLWIDTH,linewidth)
         call gflush(gp)
      }
end

procedure plotpt(gp,x,y,flag)

# Parameters

pointer gp
real x,y
int flag
                  
# Local variables

int fill

begin

      # Do you want an open or a filled marker?

      if (flag == 0)
         fill = GM_CROSS
      else
         fill = GM_CIRCLE
      
      # Plot the point

      call gmark(gp,x,y,fill,2.0,2.0)
end

procedure erasept(gp,x,y,flag)

# Parameters

pointer gp
real x,y
int flag

# Local variables

int pmltold

# Functions

int gstati()

begin

      # First find out what the old colour is and set the new colour to
      # the background value...

      pmltold = gstati(gp,G_PMLTYPE)
      call gseti(gp,G_PMLTYPE,GL_CLEAR)

      # Now 'erase' the marker and reset the colour

      call plotpt(gp,x,y,flag)
      call gseti(gp,G_PMLTYPE,pmltold)
end


procedure fitline(x,y,wt,n,aint,slope,errain,errsl)

# Parameters

int n
real x[n],y[n],wt[n],aint,slope,errain,errsl

# Local variables

real sx,sy,st2,ss,sxoss,t,sig
int i

begin

      # Start by zeroing a few counters

      sx = 0.0
      sy = 0.0
      st2 = 0.0
      slope = 0.0
      ss = 0.0

      # Do the required sums

      do i = 1,n {
         if (wt[i] > 0.0) {
            ss = ss + wt[i]
            sx = sx + x[i]*wt[i]
            sy = sy + y[i]*wt[i]
         }
      }
      sxoss = sx/ss
      do i = 1,n {
         if (wt[i] > 0.0) {
            sig = 1.0/sqrt(wt[i])
            t = (x[i] - sxoss)/sig
            st2 = st2 + t*t
            slope = slope + t*y[i]/sig
         }
      }

      # Now solve for slope, intercept and their variances

      slope = slope/st2
      aint = (sy - sx*slope)/ss
      errain = sqrt((1.0 + (sx*sx)/(ss*st2))/ss)
      errsl = sqrt(1.0/st2)
end

procedure fitsame(x,y,wt,n,slopef,errslf,aint,slope,errain,errsl)
      
# Parameters

int n
real x[n],y[n],wt[n],slopef,errslf,aint,slope,errain,errsl

# Local variables

int i
real sum,sumsq,sumwt

begin

      # Zero a few things...

      sum = 0.0
      sumsq = 0.0
      sumwt = 0.0

      # Set slope and its error to that of the fiducial series

      slope = slopef
      errsl = errslf

      # Do some sums for the intercept

      do i = 1,n {
         if (wt[i] > 0.0) {
            aint = y[i] - slope*x[i]
            sum = sum + aint*wt[i]
            sumsq = sumsq + aint*aint*wt[i]
            sumwt = sumwt + wt[i]
         }
      }

      # Now solve for the intercept...

      aint = sum/sumwt
      errain = sqrt(sumsq/sumwt - aint*aint)
end

procedure rdrvf(rvptr,rp,nframes,r2orig,zorig,wtorig)

# Parameters

pointer rvptr,rp
int nframes
real r2orig[nframes],zorig[nframes],wtorig[nframes,3]

# Local variables

int i,run,flag,j
real mpu,xc,yc,rad,sig,peak,back,ell,meanerr,err,errrad,weight[3]

begin

      # Loop for each ring...

      weight[1] = 1.0
      do i = 1,nframes {

         # First read the data

         call rvgetl(rvptr,rp,i,false,run,mpu,xc,yc,rad,sig,peak,back,ell,
            meanerr,flag)
         r2orig[i] = rad**2
         zorig[i] = mpu
         if (meanerr > EPSILONR)
            weight[3] = 1.0/(meanerr**2)
         else
            weight[3] = 0.0
         call rvgetl(rvptr,rp,i,true,run,mpu,xc,yc,errrad,sig,peak,back,ell,
            meanerr,flag)
         err = 4.0*rad*rad*errrad*errrad
         if (err > EPSILONR) 
            err = 1.0/err
         else
            err = 0.0
         weight[2] = err
         
         # Now sort out the weights...

         do j = 1,3 {
            switch (flag) {
               case RING_OK:
                  wtorig[i,j] = 1.0*weight[j]
               case RING_DODGY:
                  wtorig[i,j] = 0.5*weight[j]
               case RING_PRE, RING_SKIP, RING_BAD:
                  wtorig[i,j] = 0.0
            }
         }
      }
end

procedure collect(nframes,r2orig,zorig,wtorig,iser1,iser2,nout,r2out,zout,
   wtout)

# Parameters

int nframes,iser1,iser2,nout
real r2orig[nframes],zorig[nframes],wtorig[nframes],r2out[nout],zout[nout],
   wtout[nout]

# Local variables

int i

begin

      # Move the appropriate bits over...

      do i = iser1,iser2 {
         r2out[i-iser1+1] = r2orig[i]
         zout[i-iser1+1] = zorig[i]
         wtout[i-iser1+1] = wtorig[i]
      }
end
                                 
                            
