include "tgio.h"
include <imhdr.h>

pointer procedure tgio_init(use,comp_fac,gname,nrefx,nrefy)

# Parameters 

int comp_fac,nrefx,nrefy
pointer gname
bool use

# Local variables

pointer sp,iname,sect,gptr,msg,tgp
int nx,ny

# Functions

pointer immap()

begin

      # Allocate the space for the structure

      call malloc(tgp,TGIOSZ,TY_STRUCT)

      # Get stack workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(sect,SZ_FNAME,TY_CHAR)
      call salloc(msg,SZ_LINE,TY_CHAR)

      # Set some variables

      TGIO_COMP(tgp) = comp_fac
      TGIO_ALLOC(tgp) = false
      TGIO_WSP(tgp) = NULL
      TGIO_USE(tgp) = use
      TGIO_NREFX(tgp) = nrefx
      TGIO_NREFY(tgp) = nrefy

      # If a grid frame is being used, then do the following

      if (use) {

         # Open input file and get image dimensions. Save them.

         iferr (gptr = immap(Memc[gname],READ_ONLY,NULL)) {
            call sprintf(Memc[msg],SZ_LINE,"Can't open grid file: %s")
               call pargstr(Memc[gname])
            call error(0,Memc[msg])
         }
         nx = IM_LEN(gptr,2)
         ny = IM_LEN(gptr,3)
         TGIO_NX(tgp) = nx
         TGIO_NY(tgp) = ny
         call imunmap(gptr)
         call strcpy(Memc[gname],TGIO_GNAME(tgp),SZ_FNAME)

         # Construct image section

         call sprintf(Memc[sect],SZ_FNAME,"[*,*:%d,*:%d]")
            call pargi(comp_fac)
            call pargi(comp_fac)

         # Construct input file name

         call strcpy(Memc[gname],Memc[iname],SZ_FNAME)
         call strcat(Memc[sect],Memc[iname],SZ_FNAME)

         # Open this file

         TGIO_GPTR(tgp) = immap(Memc[iname],READ_ONLY,NULL) 

      # Otherwise fill the remaining objects with null values

      } else {
         TGIO_GPTR(tgp) = NULL
         TGIO_NX(tgp) = 0
         TGIO_NY(tgp) = 0
         TGIO_GNAME(tgp) = EOS
      }

      # Ditch workspace

      call sfree(sp)

      # Return output pointer

      return (tgp)
end

procedure tgio_close(tgp)

# Parameters

pointer tgp

begin

      # Delete any workspace associated with grids
      
      if (TGIO_ALLOC(tgp)) call mfree(TGIO_WSP(tgp),TY_REAL)

      # Close grid file, if you used one

      if (TGIO_USE(tgp)) call imunmap(TGIO_GPTR(tgp))

      # Ditch workspace

      call mfree(tgp,TY_STRUCT)
end

pointer procedure tggetl(tgp,line)

# Parameters

pointer tgp
int line

# Local variables

int nx,i,k,poff,nalloc
real xyt[2]
pointer sptr,optr

# Functions

pointer imgs3s()

begin

      # If a grid is being used, then do the following...

      if (TGIO_USE(tgp)) {

         # Start by getting a section from the grid file...
 
         nx = IM_LEN(TGIO_GPTR(tgp),2)
         if (line > IM_LEN(TGIO_GPTR(tgp),3))
            call error(0,"Requested line in grid file doesn't exist")
         sptr = imgs3s(TGIO_GPTR(tgp),1,2,1,nx,line,line)

         # Now get a workspace for the grid.  If an old workspace has
         # been allocated, then reallocate it instead...

         if (TGIO_ALLOC(tgp)) {
            call realloc(TGIO_WSP(tgp),2*nx,TY_REAL)
         } else {
            call malloc(TGIO_WSP(tgp),2*nx,TY_REAL)
            TGIO_ALLOC(tgp) = true
         }
         optr = TGIO_WSP(tgp)

         # Now convert the input grid info into real positions...

         poff = -1
         xyt[2] = real((line-1)*TGIO_COMP(tgp) + 1)
         do i = 1,TGIO_NX(tgp),TGIO_COMP(tgp) {
            xyt[1] = real(i)
            do k = 1,2 {
               poff = poff + 1
               Memr[optr+poff] = 0.01*real(Mems[sptr+poff]) + xyt[k]
            }
         }

      # Otherwise create a uniform grid...

      } else {
         
         # Get a workspace for the grid.  If an old workspace has
         # been allocated, then reallocate it instead...

         nx = TGIO_NREFX(tgp)
         nalloc = nx/TGIO_COMP(tgp)
         if (mod(nx,TGIO_COMP(tgp)) != 0) nalloc = nalloc + 1
         if (TGIO_ALLOC(tgp)) {
            call realloc(TGIO_WSP(tgp),2*nalloc,TY_REAL)
         } else {
            call malloc(TGIO_WSP(tgp),2*nalloc,TY_REAL)
            TGIO_ALLOC(tgp) = true
         }
         optr = TGIO_WSP(tgp)

         # Now create the grid...

         poff = -1
         xyt[2] = real((line-1)*TGIO_COMP(tgp) + 1)
         do i = 1,nx,TGIO_COMP(tgp) {
            xyt[1] = real(i)
            do k = 1,2 {
               poff = poff + 1
               Memr[optr+poff] = xyt[k]
            }
         }
      }

      # Return the output pointer

      return (optr)
end

procedure tgx2g(tgp,x,y,gx,gy)

# Parameters

pointer tgp
real x,y,gx,gy

# Local variables

int ix,iy,nx,ny,ix1,ix2,iy1,iy2
pointer sptr,gptr

# Functions

pointer imgs3s(),immap()

begin

      # If a grid is being used, then do the following...

      if (TGIO_USE(tgp)) {

         # Open the original grid file.  This is necessary since the
         # grid which is known to tgio may have been compressed and the
         # particular pixel you want may not be included in the compressed
         # grid.

         gptr = immap(TGIO_GNAME(tgp),READ_ONLY,NULL)

         # Check that coords are ok

         ix = nint(x)
         iy = nint(y)
         if (ix < 1 || iy < 1 || ix > IM_LEN(gptr,2) || iy > IM_LEN(gptr,3)) {
            call eprintf("ix = %4d, iy = %4d\n")
               call pargi(ix)
               call pargi(iy)
            call error(0,"Requested pixel doesn't exist in grid file\n")
         }

         # Find area of grid file to search
         
         ix1 = max(1,int(x))
         ix2 = min(ix1+1,IM_LEN(gptr,2))
         iy1 = max(1,int(y))
         iy2 = min(iy1+1,IM_LEN(gptr,3))
         nx = ix2 - ix1 + 1
         ny = iy2 - iy1 + 1

         # Get x section and convert the x coord

         sptr = imgs3s(gptr,1,1,ix1,ix2,iy1,iy2)
         call tgio_pix(Mems[sptr],ix1,ix2,nx,ny,x,y,gx)

         # Do the same for the y now...

         sptr = imgs3s(gptr,2,2,ix1,ix2,iy1,iy2)
         call tgio_pix(Mems[sptr],iy1,iy2,nx,ny,x,y,gy)

         # Close the grid

         call imunmap(gptr)

      # If a grid isn't being used, then just return the old coordinates

      } else {
         gx = x
         gy = y
      }
end

procedure tgio_pix(grid,ix1,ix2,nx,ny,x,y,gx)

# Parameters

int nx,ny,ix1,ix2
short grid[nx*ny]
real x,y,gx

# Local variables

real w[4],xx[4]
int nused,i

begin

      # Sort out the weights 

      w[1] = real(int(x)+1) - x
      w[2] = 1.0 - w[1]
      w[3] = real(int(y)+1) - y
      w[4] = 1.0 - w[3]         

      # Convert the corners...

      xx[1] = 0.01*real(grid[1]) + real(ix1)
      nused = nx*ny
      if (nused == 1) {
         do i = 2,4
            xx[i] = xx[1]
      } else if (nused == 2) {
         if (nx == 2) {
            xx[2] = 0.01*real(grid[2]) + real(ix2)
            do i = 3,4
               xx[i] = xx[i-2]
         } else {
            xx[3] = 0.01*real(grid[2]) + real(ix1)
            do i = 2,4,2
               xx[i] = xx[i-1]
         }
      } else if (nused == 4) {
         xx[2] = 0.01*real(grid[2]) + real(ix2)
         xx[3] = 0.01*real(grid[3]) + real(ix1)
         xx[4] = 0.01*real(grid[4]) + real(ix2)
      }

      # Now do the interpolation

      gx = 0.0
      do i = 1,2
         gx = gx + w[3]*w[i]*xx[i] + w[4]*w[i]*xx[i+2]
end

pointer procedure tgio_full(tgp)

# Parameters

pointer tgp

# Local variables

int nalloc,i,nx,ny,nxp,nyp,line
pointer optr,gptr,ptr

# Functions

pointer tggetl()

begin

      # Start by getting the workspace needed

      if (TGIO_USE(tgp)) {
         nalloc = 1
         do i = 1,3
            nalloc = nalloc*IM_LEN(TGIO_GPTR(tgp),i)
         nx = IM_LEN(TGIO_GPTR(tgp),2)
         ny = IM_LEN(TGIO_GPTR(tgp),3)
      } else {
         nxp = TGIO_NREFX(tgp)/TGIO_COMP(tgp)
         if (mod(TGIO_NREFX(tgp),TGIO_COMP(tgp)) != 0) nxp = nxp + 1
         nyp = TGIO_NREFY(tgp)/TGIO_COMP(tgp)
         if (mod(TGIO_NREFY(tgp),TGIO_COMP(tgp)) != 0) nyp = nyp + 1
         nalloc = 2*nxp*nyp
         nx = nxp
         ny = nyp
      }
      call malloc(optr,nalloc,TY_REAL)

      # Now fill this workspace...

      do line = 1,ny {
         gptr = tggetl(tgp,line)
         ptr = optr + 2*nx*(line-1)
         call amovr(Memr[gptr],Memr[ptr],2*nx)
      }

      # Now flag the workspace

      if (TGIO_ALLOC(tgp)) call mfree(TGIO_WSP(tgp),TY_REAL)
      TGIO_ALLOC(tgp) = true
      TGIO_WSP(tgp) = optr

      # Return the workspace pointer

      return (optr)
end
                 
