include "tgrid.h"
include <math/gsurfit.h>
include <fset.h>

pointer procedure tg_init()

# Local variables

pointer sp,data,tgp
int fd,junk,i

# Functions

int tau_log(),fscan(),stridx(),gsgeti()
real gsgetr()

begin

      # Get a bit of workspace

      call smark(sp)
      call salloc(data,SZ_LINE,TY_CHAR)

      # Allocate the space for the structure

      call malloc(tgp,LEN_STR,TY_STRUCT)

      # Get the name of the mask log file to use...

      fd = tau_log("tmask.log",READ_ONLY)
      call fstats(fd,F_FILENAME,TG_MFIL(tgp),LEN_MF)

      # Go through the file and find the first line with at a hash 
      # at the beginning

      while (fscan(fd) != EOF) {
         call gargstr(Memc[data],SZ_LINE)
         if (stridx("#",Memc[data]) != 1) break
      }

      # Now get the order info and the number of saved params

      if (fscan(fd) != EOF) 
         call gargi(TG_NORD(tgp))
      else
         call error(0,"Premature end of mask log file")
      junk = fscan(fd)
      if (junk != EOF && fscan(fd) != EOF)
         call gargi(TG_NSAVE(tgp))
      else
         call error(0,"Premature end of mask log file")

      # Get workspace for the saved parameters

      call malloc(TG_GPX(tgp),TG_NSAVE(tgp),TY_REAL)
      call malloc(TG_GPY(tgp),TG_NSAVE(tgp),TY_REAL)

      # Now read in these parameters

      for (i = 1; i <= TG_NSAVE(tgp); i=i+1) {
         if (fscan(fd) == EOF) call error(0,"Premature end of mask log file")
         call gargr(Memr[TG_GPX(tgp)+i-1])
      }
      for (i = 1; i <= TG_NSAVE(tgp); i=i+1) {
         if (fscan(fd) == EOF) call error(0,"Premature end of mask log file")
         call gargr(Memr[TG_GPY(tgp)+i-1])
      }

      # Now reopen GSURFIT with these parameters

      call gsrestore(TG_GSFX(tgp),Memr[TG_GPX(tgp)])
      call gsrestore(TG_GSFY(tgp),Memr[TG_GPY(tgp)])
      
      # Get the function type

      TG_FUNC(tgp) = gsgeti(TG_GSFX(tgp),GSTYPE)  

      # Get fit limits

      TG_XMIN(tgp) = gsgetr(TG_GSFX(tgp),GSXMIN)
      TG_XMAX(tgp) = gsgetr(TG_GSFX(tgp),GSXMAX)
      TG_YMIN(tgp) = gsgetr(TG_GSFX(tgp),GSYMIN)
      TG_YMAX(tgp) = gsgetr(TG_GSFX(tgp),GSYMAX)

      # Get the shrink factor

      if (fscan(fd) == EOF) call error(0,"Premature end of mask log file")
         call gargr(TG_SHRINK(tgp))

      # Get the central pixel size

      call tg_size(tgp,TG_PIXX(tgp),TG_PIXY(tgp),TG_SHRINK(tgp))

      # Release some workspace and get out of here.

      call close(fd)
      call sfree(sp)
      return(tgp)
end


procedure tg_size(tgp,pixx,pixy,shrink)

# Parameters

pointer tgp
real pixx,pixy,shrink

# Local variables

real xc,yc,dxx,dxy,dyy,dyx

begin

      # Get the central X,Y position...

      xc = (TG_XMIN(tgp) + TG_XMAX(tgp))*0.5
      yc = (TG_YMIN(tgp) + TG_YMAX(tgp))*0.5

      # Get the necessary derivatives

      call gsder(TG_GSFX(tgp),xc,yc,dxx,1,1,0)
      call gsder(TG_GSFX(tgp),xc,yc,dxy,1,0,1)
      call gsder(TG_GSFY(tgp),xc,yc,dyx,1,1,0)
      call gsder(TG_GSFY(tgp),xc,yc,dyy,1,0,1)

      # And the answer is....

      pixx = shrink*sqrt(dxx**2 + dyx**2)
      pixy = shrink*sqrt(dyy**2 + dxy**2)
end

procedure tg_close(tgp)

# Parameters

pointer tgp

begin

      # First free the gsurfit slots

      call gsfree(TG_GSFX(tgp))
      call gsfree(TG_GSFY(tgp))

      # Now ditch the workspace holding the gsurfit parameters...

      call mfree(TG_GPX(tgp),TY_REAL)
      call mfree(TG_GPY(tgp),TY_REAL)

      # Finally ditch the workspace for the tgrid structure

      call mfree(tgp)
end
                    
