/* Table support routines for APM analysis of IRAF images. */
#include <stdio.h>
#include <xtables.h>
#include "ap.h"
#include "aptabdef.h"

int row;

void tabInit(IRAFPointer intab)
{
	int i;
	char plab[4];

	/* Table was tbtopn'd in main program. */
	tab = intab;

	/* Define columns for APM output: */
	c_tbcdef1(tab, &col[X], "x", "pixel", "%7.1f", IRAF_REAL, 1);
	c_tbcdef1(tab, &col[Y], "y", "pixel", "%7.1f", IRAF_REAL, 1);
	c_tbcdef1(tab, &col[TOTAL], "total", "adu", "%12.1f", IRAF_REAL, 1);
	c_tbcdef1(tab, &col[AREA], "area", "pix**2", "%10d", IRAF_INT, 1);
	c_tbcdef1(tab, &col[PEAK], "peak", "adu", "%9d", IRAF_INT, 1);
	c_tbcdef1(tab, &col[XX], "x**2", "", "%9.1f", IRAF_REAL, 1);
	c_tbcdef1(tab, &col[XY], "x*y", "", "%9.1f", IRAF_REAL, 1);
	c_tbcdef1(tab, &col[YY], "y**2", "", "%9.1f", IRAF_REAL, 1);
	for(i=0; i<8; i++)
	{
		(void)sprintf(plab, "p%d", i);
		c_tbcdef1(tab, &col[PROF+i], plab, "pixel**2",
				"%9d", IRAF_INT, 1);
	}

	/* Create the table: */
	c_tbtcre(tab);

	row = 0;
}

/* Store analysis parameters out of ap into the table: */
void tabAddHead(ap_t *ap)
{
	c_tbhadi(tab, "SKYMED", ap->background);
	c_tbhadr(tab, "SKYSIG", ap->sigma);
	c_tbhadi(tab, "THRESH", ap->thresh);
}

/* Store the output of the `moments' calculation into the table: */
void tabAdd(apmCat_t cat)
{
	int i;

	row++;
	c_tbeptr(tab, col[X], row, cat.x);
	c_tbeptr(tab, col[Y], row, cat.y);
	c_tbeptr(tab, col[TOTAL], row, cat.total);
	c_tbepti(tab, col[AREA], row, cat.area);
	c_tbepti(tab, col[PEAK], row, cat.peak);
	c_tbeptr(tab, col[XX], row, cat.xx);
	c_tbeptr(tab, col[XY], row, cat.xy);
	c_tbeptr(tab, col[YY], row, cat.yy);
	for (i=0; i<8; i++)
	{
		c_tbepti(tab, col[PROF+i], row, cat.areal[i]);
	}
}
