#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <c_iraf.h>
#include <ximio.h>
#include <apm.h>

/* This routine to support qsort: */
static int fcompare(float *x, float *y)
{
	if(*x > *y) return(1);
	if(*x < *y) return(-1);
	return(0);
}
void background(IRAFPointer in, IRAFPointer out, int *bak, ap_t *ap)
{
	int dim1, dim2;
	int n,m,inc;
	int i,j,joff,ii,jj,ih,iq;
	short int hist[65536];
	int *map = NULL;
	int count, imap;
	int imax, jmax, isq, jsq, harea, qarea;
	int xmap[128];
	int rail1[66], rail2[66];
	int mchange, nchange;
	int mnext, nnext;
	int area, joffset;
	int dx, dy[66];
	int b1, b2;
	int sky;
	int *subr, *ssubr, val;
	/* int fd; */
	float *sigma;
	int isig;

	/* Attempt to make backround bins 64x64 raw pixels; but not less
	 * than 2x2 for the whole background map. */
	inc = 64;
	dim1 = c_imglen(in, 1);
	dim2 = c_imglen(in, 2);
	n = (dim1 + inc - 1)/inc;
	if (n < 2)
	{
		n = 2;
		inc = (dim1 + 1)/2;
	}
	m = (dim2 + inc - 1)/inc;
	c_implen(out, 1, n);
	c_implen(out, 2, m);
	map = c_imps2i(out, 1, n, 1, m);
	sigma = malloc(n*m*sizeof(*sigma));
	isig = 0;
	imap = 0;
	sky = 0;
	
	/* Step thru image in blocks of inc x inc: */
	for(j = 1; j <= dim2; j+=inc)
	{
		jmax = j + inc - 1;
		if(jmax > dim2)
		{
			jmax = dim2;
			jsq = jmax % inc;
		}
		else
		{
			jsq = inc;
		}
		subr = c_imgs2i(in, 1, dim1, j, jmax);
		for(i = 1; i <= dim1; i+=inc)
		{
			/* Zero the histogram: */
			(void)memset(hist, 0, sizeof(hist));

			/* build hist from the little square: */
			imax = i + inc - 1;
			if(imax > dim1)
			{
				/* bit left over at the end: */
				imax = dim1;
				isq = imax % inc;
			}
			else
			{
				isq = inc;
			}
			area = isq*jsq;
			harea = (area+1)/2;
			qarea = (area+3)/4;
			ssubr = subr;
			for(jj=jsq; jj != 0; jj--)
			{
				for (ii=0; ii < isq; ii++)
				{
					val = ssubr[ii];
					val = ((val<0 ? 0 : val) > 65535
								? 65535 :val);
					hist[val] += 1;
				}
				ssubr += dim1;
			}
			subr += isq;

			/* Search up hist to find median: */
			count = 0;
			ih = 0;
			while(count < qarea)
				count += hist[ih++];
			iq = ih;
			while(count < harea)
				count += hist[ih++];
			map[imap] = ih - 1;
			/* the 1.48 converts MAD to rms equivalent */
			sigma[isig++] = 1.48*(float)(ih - iq);
			sky += map[imap++];
		}
	}
	sky /= n*m;
	qsort( (void *)sigma, isig, sizeof(*sigma), fcompare);
	ap->sigma = sigma[isig/2];
	free(sigma);

	/* Now do interpolation: */
	area = inc*inc;
	rail2[0] = map[0];
	for(i=1; i < n+1; i++) rail2[i] = map[i-1];
	rail2[n+1] = map[n-1];
	mchange = -inc/2;
	mnext = 0;
	joffset = 0;
	for(j = 0; j < dim2; j++)
	{
		if( j >= mchange)
		{
			for(i=0; i < n+2; i++)
				rail1[i] = rail2[i];
			rail2[0] = map[mnext];
			rail2[n+1] = map[mnext+n-1];
			for(i=0; i < n; i++)
				rail2[i+1] = map[i+mnext];
			for(i=0; i < n+2; i++)
			{
				dy[i] = rail2[i] - rail1[i];
				rail1[i] *= inc;
			}
			mchange += inc;
			if(mnext < m*n) mnext += n;
		}
		else
			for(i=0; i<n+2; i++)
				rail1[i] += dy[i];

		b2 = rail1[0];
		nchange = -inc/2;
		nnext = 1;
		for(i=0; i<dim1; i++)
		{
			if(i >= nchange)
			{
				b1 = b2;
				b2 = rail1[nnext];
				dx = b2 - b1;
				b1 *= inc;
				nchange += inc;
				nnext++;
			}
			else
				b1 = b1 + dx;
			bak[i + joffset] = b1/area;
		}
		joffset += dim1;
	}
	ap->background = sky;
	/* Don't write full backmap out.
	fd = open("/tmp/backmap",O_CREAT|O_WRONLY,0664);
	if(fd == -1)
		perror("backmap");
	write(fd,bak, dim1*dim2*sizeof(*bak));
	*/
}
