/* get_stars is a procedure to open the private binary copy of GSC, and
 * given ra, dec limits to extract the list of catalogue stars in that region.
 * To be used as a procedure in Jim's overall WCS prog.
 * v1.1 1998 Aug 31 Peter Bunclark
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "wfcwcs.h"

void get_stars(cLimits_t *cLimit, cat_t **cat, int *n, int *status)
{
	char name[132];
	char *GSC_DATA;
	struct catbuf { int id; float ra; float dec; float mag; } saobuf ;
	int fd;
	ssize_t nr;
	off_t fillen, seek, half;
	int i;

	*n = 0;
	if(*status) return;

	GSC_DATA = getenv("GSC_DATA");
	if(GSC_DATA == NULL)
	{
		*status = 1;
		printf("Environment variable GSC_DATA not set.\n");
		return;
	}
	(void)strcpy(name, GSC_DATA);

	if((fd = open(name, O_RDONLY)) == -1)
	{
		perror(name);
		*status = 2;
		return;
	}

	fillen = lseek(fd, 0, SEEK_END);
	half = fillen/2;
	seek = half;
	while(half > 1)
	{
		nr = pread(fd, &saobuf, sizeof(saobuf), seek);
		if(nr == -1)
		{
			perror("pread");
			*status = 3;
			return;
		}
		half = (half/2);
		half = (half/16)*16;
		if (saobuf.ra > cLimit->ra1)
			seek = seek - half;
			else
			seek = seek + half;
	}
	(void)lseek(fd, seek, SEEK_SET);
	while ( saobuf.ra <= cLimit->ra2)
	{
		nr = read(fd, &saobuf, sizeof(saobuf));
		if(saobuf.dec >= cLimit->dec1 &&
			saobuf.dec <= cLimit->dec2)
			(*n)++;
	}
	*cat = calloc((size_t)*n, sizeof(cat_t));
	/* Now go back and stuff 'em in array: */
	(void)lseek(fd, seek, SEEK_SET);
	nr = read(fd, &saobuf, sizeof(saobuf));
	i = -1;
	while ( saobuf.ra <= cLimit->ra2)
	{
		nr = read(fd, &saobuf, sizeof(saobuf));
		if(saobuf.dec >= cLimit->dec1 &&
			saobuf.dec <= cLimit->dec2)
		{
			i++;
			(*cat + i)->ra = saobuf.ra*RAD2DEG;
			(*cat + i)->dec = saobuf.dec*RAD2DEG;
			(*cat + i)->mag = saobuf.mag;
		}
	}
}




