#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <c_iraf.h>
#include <xclio.h>
#include <ximio.h>
#include <xtables.h>
#include <xmwcs.h>
#include <xvops.h>
#include "aptabdef.h"
#include "wfcwcs.h"

#define THRESH 60.0
#define MAXRAD THRESH

     
void iraferr() {
        printf("IRAF error code %d\nIRAF message: %s\n",
                c_iraferr(),c_iraferrmsg());
        exit(0);
}

int main(int argc, char **argv) {
        IRAFPointer iptr,tab,mw,xytoradec,radectoxy,xptr,yptr;
        int nx,ny,i,j,m,nrows,ns,status,n,rah,ram,dd,dm,iminus;
        float ds,ras;
        char tabname[BUFSIZ],fname[BUFSIZ],col[BUFSIZ];
        double x,y,a[4],d[4],rad,ra,dec,xx,yy;
        FILE *fd;
        cat_t *ipete = NULL;
        cLimits_t cLimits;
        void radec();

        /* must always remember to call c_irafinit */

        c_irafinit(argc, argv);

        /* install the error handler */

        c_pusherr(iraferr);
        printf("IRAF WCS program\n");

        /* Get the input file name.  Assume the table is the same */

        c_clgstr("Gimme an input file please",fname,BUFSIZ);
        c_clpstr("file",fname);
        (void)strcpy(tabname,fname);
        (void)strcat(tabname,".tab");

        /* Map both files... */

        tab = c_tbtopn(tabname,IRAF_READ_WRITE,0);
        iptr = c_immap(fname,IRAF_READ_WRITE,0);

        /* Get the rough WCS from the input file header. */

        mw = c_mw_openim(iptr);
        xytoradec = c_mw_sctran(mw,"logical","world",3);
        radectoxy = c_mw_sctran(mw,"world","logical",3);

	/* Get min and max ra and dec from the corners of the chip */
    
        nx = c_imglen(iptr,1);
        ny = c_imglen(iptr,2);
        m = -1;
        for (j = 1; j <= ny; j = j + ny - 1) {
            for (i = 1; i <= nx; i = i + nx - 1) {
                x = (double)i;
                y = (double)j;
                m++;
                c_mw_c2trand(xytoradec,x,y,&a[m],&d[m]);
	    }
	}
        n = 4;
        cLimits.ra1 = (c_alovd(a,&n)*M_PI)/180.0;
        cLimits.ra2 = (c_ahivd(a,&n)*M_PI)/180.0;
        cLimits.dec1 = (c_alovd(d,&n)*M_PI)/180.0;
        cLimits.dec2 = (c_ahivd(d,&n)*M_PI)/180.0;

        /* Get info from Pete's program */

        /* ipete = testit(); */
        status = 0;
	(void)get_stars(&cLimits,&ipete,&ns,&status);
	printf("%d stars found\n",ns);

        /* Take Pete's Ra and Decs and turn them into x,y */

        for (i = 0; i < ns; i++) {
            ra =  (ipete+i)->ra;
            dec = (ipete+i)->dec;
            c_mw_c2trand(radectoxy,ra,dec,&x,&y);
            (ipete+i)->x = x;
            (ipete+i)->y = y;
            (ipete+i)->rad = MAXRAD;
            (ipete+i)->match = 0;
        }

        /* Ok, read the X,Y of the objects in the table.  Compare
           with the X,Y of the standard stars.  If they are inside
           the error circle, then take note of them.  Let the match
           be the closest candidate */

        nrows = c_tbpsta(tab,TBL_NROWS);
        c_tbcfnd1(tab,"x",&xptr);
        c_tbcfnd1(tab,"y",&yptr);
        for (i = 1; i <= ns; i++) {
            c_tbegtd(tab,xptr,i,&x);
            c_tbegtd(tab,yptr,i,&y);
            for (j = 0; j < ns; j++) {
                xx = x - (ipete+j)->x;
                yy = y - (ipete+j)->y;
	        rad = sqrt(pow(xx,2) + pow(yy,2));
                if (rad < THRESH && rad < (ipete+j)->rad) {
                    (ipete+j)->match = i;
                    (ipete+j)->rad = rad;
                    (ipete+j)->xm = x;
                    (ipete+j)->ym = y;
                }
            }
        }

        /* Right, now write this stuff out */

        fd = fopen("ccmap.in","w");
        for (i = 0; i < ns; i++) {
            if ((ipete+i)->match > 0) {
                radec((ipete+i)->ra,(ipete+i)->dec,&rah,&ram,&ras,&iminus,&dd,&dm,&ds);
                if (iminus == -1) {
                    fprintf(fd,"%02d:%02d:%05.2f -%d:%02d:%04.1f %g %g\n",rah,ram,ras,dd,dm,ds,(ipete+i)->xm,(ipete+i)->ym);
                } else {
                    fprintf(fd,"%02d:%02d:%05.2f %d:%0d:%04.1f %g %g\n",rah,ram,ras,dd,dm,ds,(ipete+i)->xm,(ipete+i)->ym);
                }
            }
        }
        close(fd);

        /* Free up workspace */
  
        free(ipete);
  
        /* Close up the WCS */

        c_mw_ctfree(&xytoradec);
        c_mw_ctfree(&radectoxy);
        c_mw_close(&mw);
        printf("Error code %d\n",c_iraferr());

        /* Close up files */

        c_imunmap(iptr);
        c_tbtclo(tab);
        printf("Error code %d\n",c_iraferr());

        return 0;
}

void radec(double ra, double dec, int *rah, int *ram, float *ras, int *iminus, int *dd, int *dm, float *ds) {

    ra /= 15.0;
    *iminus = (dec < 0.0) ? -1 : 1;
    
    *rah = (int)ra;
    *ram = (int)((ra - (double)*rah)*60.0);
    *ras = (float)((ra - (double)*rah - ((double)*ram)/60.0)*3600.0);

    dec = (double)*iminus*dec;
    *dd = (int)dec;
    *dm = (int)((dec - (double)*dd)*60.0);
    *ds = (float)((dec - (double)*dd - ((double)*dm)/60.0)*3600.0);

    return;
}
