include <error.h>
include <math.h>

define   PIXSCA   0.3334
define   RATYPE   "RA--TAN"
define   DECTYPE  "DEC--TAN"

procedure t_wfcwcs()

# Local variables

real xoff[4],yoff[4],tpa,tpd,dr,newrot,xapoff,yapoff,xapnom,yapnom
real rotskypa,pix36,aa,bb,cc,dd,ee,ff,ra,dec
pointer sp,fname,inlist,iptr
int list,daschan,i

# Functions

int imtopen(),imtgetim(),imgeti(),imaccf()
real imgetr()
pointer immap()

# Data statements

data xoff /-284.0,3225.0,3931.0,1828.0/
data yoff /3042.0,1732.0,2998.0,3029.0/

begin

      # Get workspace

      call smark(sp)
      call salloc(inlist,SZ_LINE,TY_CHAR)
      call salloc(fname,SZ_FNAME,TY_CHAR)

      # Initialise a few things...

      pix36 = PIXSCA/3600.0

      # Get a list of files to deal with...Open the list structure.

      call clgstr("inlist",Memc[inlist],SZ_LINE)
      list = imtopen(Memc[inlist])

      # Ok loop for each file...

      while (imtgetim(list,Memc[fname],SZ_FNAME) != EOF) {

          # Check that you can open the file...

          iferr (iptr = immap(Memc[fname],READ_WRITE,0)) {
              call eprintf("%s: Error opening file\n")
                  call pargstr(Memc[fname])
              next
          }

          # If you've already done this file, then don't bother to do it
          # again since it will now probably be wrong.

          if (imaccf(iptr,"WFCWCS") == YES) {
              call imunmap(iptr)
              next
          }

          # Right if you have opened the file, make sure you can now
          # get all of the astrometric information you need...

          iferr {
              ra = imgetr(iptr,"RA")
              dec = imgetr(iptr,"DEC")
              daschan = imgeti(iptr,"DASCHAN")
              xapoff = imgetr(iptr,"XAPOFF")
              yapoff = imgetr(iptr,"YAPOFF")
              xapnom = imgetr(iptr,"XAPNOM")
              yapnom = imgetr(iptr,"YAPNOM")
              rotskypa = imgetr(iptr,"ROTSKYPA")
          } then {
              call eprintf("%s: Astrometric information missing from header\n")
                  call pargstr(Memc[fname])
              call erract(EA_WARN)
              call imunmap(iptr)
              next
          }

          # Now assuming you've got all the info you need, then diddle some
          # of the values...

          tpa = ra*15.0
          tpd = dec
          if (daschan == 5) daschan = 2
          newrot = -1.0
          for (i = 0; i <= 360; i = i + 90) {
              dr = real(i)
              if (abs(rotskypa - dr) < 5.0) {
                  newrot = dr
                  break
              }
          }
          if (newrot == -1.0) {
              call eprintf("%s: Funny rotation angle: %g\n")
                  call pargstr(Memc[fname])
                  call pargr(rotskypa)
              call imunmap(iptr)
              next
          } else if (newrot == 360.0) {
              newrot = 0.0
          }
          rotskypa = newrot

          # For the transformation matrix.  If this is the second chip
          # it's on its side...

          if (daschan == 2) {
              aa = pix36
              bb = 0.0
              dd = bb
              ee = -aa    
          } else {
              aa = 0.0
              bb = -pix36
              dd = bb
              ee = aa    
          }

          # Optical axis/rotator centre location in virtual pixel space on CCD

          cc = xoff[daschan]
          ff = yoff[daschan]

          # Tangent points of optical axis/rotator centre

          tpa = tpa - yapoff
          tpd = tpd + xapoff

          # Now write all this to the header.

          iferr {
              call imastr(iptr,"CTYPE1",RATYPE)               
              call imastr(iptr,"CTYPE2",DECTYPE)
              call imaddr(iptr,"CRVAL1",tpa)
              call imaddr(iptr,"CRVAL2",tpd)
              call imaddr(iptr,"CD1_1",aa)
              call imaddr(iptr,"CD2_2",ee)
              call imaddr(iptr,"CRPIX1",cc)
              call imaddr(iptr,"CRPIX2",ff)
              call imaddr(iptr,"CD1_2",bb)
              call imaddr(iptr,"CD2_1",dd)
              call imastr(iptr,"WFCWCS","Done")
          } then {
              call eprintf("%s: Error updating header\n")
                  call pargstr(Memc[fname])
              call erract(EA_WARN)
              call imunmap(iptr)
              next
          }          

          # Close up and go on to the next one.

          call imunmap(iptr)
      }

      # Free some workspace

      call sfree(sp)
end
