procedure mag1count(input,output)

file    input                 {prompt="Input file with standard star spectrum"}
file    output                {prompt="Output text file"}
bool    autonm = yes          {prompt="Sort out star name automatically?"}
file    rootdir = "onedstds$" {prompt="Root directory for calibration data"}
bool    ans                   {prompt="Is this ok?",mode="q"}
string  dname                 {prompt="Path for calibration file",mode="q"}
string  starname              {prompt="Star name",mode="q"}

struct  *fd
begin

      file in,temp,dirname,fname,stname,extfile,out
      string lponame,othername,egname,junkstr,msg
      int junk,nw,junki,npix
      real w1,dw,w2,junkr,ww1,ww2,cenwave,abmag,dwave,dwave2,exptime
      real mean_star,mean_ext,abmag2,dm,mag1,airmass,gain
      bool ans2
      struct data

      # Get input file name and make sure it's wavelength calibrated

      in = input
      out = output
      hselect(in,"dc-flag",yes) | scan(junkstr)
      if (nscan() == 0)
         error(0,"Input file isn't wavelength calibrated")

      # Find the spectral range of the standard star spectrum

      temp = mktemp("tmp$iraf")
      slist(in,apertures="",long_header=no,>>temp)
      fd = temp
      junk = fscan(fd,junkstr,junki,junki,junki,junki,w1,dw,nw)
      w2 = w1 + real(nw - 1)*dw
      delete(temp,verify=no)

      # If using the auto name facility then find out what the name of the
      # object is from the header.

      if (autonm) {
         hselect(in,"cat-name",yes) | scan(lponame)
         othername = ""; egname = ""
         stdname(lponame) | scan(othername,egname)

         # Find out which directory in the root directory has the best
         # calibration data for your star

         stddir(othername,egname,temp,rootdir,w1,w2)
         if (access(temp)) {
            print("\nResults of directory search:")
            type(temp,map_cc=yes,device="terminal")
            print("\n")
            fd = temp
            junk = fscan(fd,dirname,fname)
            printf("I'll use %s, %s\n",dirname,fname)
            ans2 = ans
            if (!ans2)
               dirname = dname
            delete(temp,verify=no)
         } else
            error(0,"No calibration data found for your star")
         stname = substr(fname,1,strlen(fname)-4)

      # If you're not using the automatic name cross-reference, then you have
      # to include a complete directory name and a star name.  The star name 
      # needs to be the same format as the standard star file names. (e.g.
      # G191-B2B has a calibration file g191b2b.dat, all lower case with no
      # spaces, minus signs etc...)

      } else {
         dirname = dname
         print(starname) | lcase | scan(stname)
         fname = stname//".dat"
         if (!access(dirname//fname)) {
            printf("Can't find file %s",dirname//fname) | scan(msg)
            error(0,msg)
         }
      }

      # Get the LPO extinction file and make it into a spectrum. Multiply
      # it by the airmass in the star spectrum

      extfile = mktemp("tmp$iraf")
      sinterp("rgo$flux/lpoext.tab","STDIN",temp,x1=w1,x2=w2,
         order=5,dx=1.0,npts=0,curve_gen=yes,make_image=yes,tbl_size=1024,
         interp_mode="line")
      hselect(in,"airmass",yes) | scan(airmass)
      sarith(temp,"*",airmass,extfile,w1=INDEF,w2=INDEF,apertures="",bands="",
         beams="",apmodulus=0,reverse=no,ignoreaps=no,format="multispec",
         renumber=no,offset=0,clobber=no,merge=no,rebin=no,errval=0.0,
         verbose=no)
      imdelete(temp,verify=no)
      sarith(extfile,"lum","",temp,w1=INDEF,w2=INDEF,apertures="",bands="",
         beams="",apmodulus=0,reverse=no,ignoreaps=no,format="multispec",
         renumber=no,offset=0,clobber=no,merge=no,rebin=no,errval=0.0,
         verbose=no)
      imdelete(extfile,verify=no)
      imrename(temp,extfile,verbose=no)

      # Get the exposure time and gain

      hselect(in,"exposed,gain",yes) | scan(exptime,gain)

      # Ok, open the AB magnitude file and start reading

      fd = dirname//fname
      while (fscan(fd,data) != EOF) {
         if (substr(data,1,1) == "#")
            next
         print(data) | scan(cenwave,abmag,dwave)
         dwave2 = 0.5*dwave
         ww1 = cenwave - dwave2
         ww2 = cenwave + dwave2
         if ((ww1 < w1) || (ww2 > w2))
            next

         # Copy the relevant bit of the spectrum over. Do stats to find
         # mean counts.  Divide by the dispersion per pixel.
         # Finally divide by the exposure time.

         scopy(in,temp,w1=ww1,w2=ww2,apertures="",bands="",beams="",
            apmodulus=0,format="multispec",renumber=no,offset=0,
            clobber=no,merge=no,rebin=no,verbose=no)
         slist(temp,apertures="",long_header=no) | scan(junkstr,junk,junk,junk,
            junk,junkr,dw)
         imstat(temp,fields="mean",lower=INDEF,upper=INDEF,binwidth=0.1,
            format=no) | scan(mean_star)
         mean_star = mean_star*gain/(dw*exptime)
         imdelete(temp,verify=no)

         # Do the same for the extinction.  It's been binned to 1 A, so
         # we shouldn't need to do the arithmetic at the end of the
         # previous calculation. Remember this is in luminosity units, so
         # needs to be converted back to a magnitude after the stats.

         scopy(extfile,temp,w1=ww1,w2=ww2,apertures="",bands="",beams="",
            apmodulus=0,format="multispec",renumber=no,offset=0,
            clobber=no,merge=no,rebin=no,verbose=no)
         imstat(temp,fields="mean",lower=INDEF,upper=INDEF,binwidth=0.1,
            format=no) | scan(mean_ext)
         mean_ext = -2.5*log10(mean_ext)
         imdelete(temp,verify=no)

         # Add the mean extinction to the AB magnitude.  Calculate the
         # difference in magnitude required to reduce (hopefully) the
         # observed mean counts in this band to 1 per second per Angstrom.

         abmag2 = abmag + mean_ext
         dm = 2.5*log10(mean_star)
         mag1 = abmag2 + dm

         # Write out the result.

         print(cenwave," ",mag1,>>out)
      }
      imdelete(extfile,verify=no)
      fd = ""
end
