include <error.h>

procedure t_arcstat()

# Local variables

pointer sp,iname,rootname,dbname,line,slist,junkstr,dt,sptr,nptr
int fd,apnum,ispec,record,nfeat,ntot,i,iwt,junkint,nalloc,n,ncoef
real sum,sumsq,junkr,w1,w2,resid,mean,sig,emed

# Functions

int open(),fscan(),dtlocate(),dtgeti(),dtscan()
real ensmed()
                     
begin

      # Get some workspace

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(rootname,SZ_FNAME,TY_CHAR)
      call salloc(dbname,SZ_FNAME,TY_CHAR)
      call salloc(line,SZ_LINE,TY_CHAR)
      call salloc(slist,SZ_FNAME,TY_CHAR)
      call salloc(junkstr,SZ_LINE,TY_CHAR)

      # Get input file name and database name. Strip any extra rubbish
      # off the input file name. 

      call clgstr("input",Memc[iname],SZ_FNAME)
      call xt_imroot(Memc[iname],Memc[rootname],SZ_FNAME)
      call sprintf(Memc[iname],SZ_LINE,"id%s")
         call pargstr(Memc[rootname])
      call clgstr("database",Memc[dbname],SZ_FNAME)

      # Open the database file...

      dt = NULL
      iferr (call dtremap(dt,Memc[dbname],Memc[iname],READ_ONLY)) {
         call eprintf("Can't map file %s in %s/\n")
            call pargstr(Memc[iname])
            call pargstr(Memc[dbname])
         call erract(EA_ERROR)
      }

      # Open the slist file...

      call clgstr("slistf",Memc[slist],SZ_FNAME)
      iferr (fd = open(Memc[slist],READ_ONLY,TEXT_FILE)) {
         call eprintf("Can't find file %s")
            call pargstr(Memc[slist])
         call erract(EA_ERROR)
      }

      # Get an initial allocation of workspace 

      nalloc = 64
      call malloc(sptr,nalloc,TY_REAL)
      call malloc(nptr,3*nalloc,TY_INT)
      n = 0

      # Now loop for each entry in the slist file...

      while (fscan(fd) != EOF) {
         
         # Check that workspace allocation is sufficient

         n = n + 1
         if (n > nalloc) {
            nalloc = nalloc + 64
            call realloc(sptr,nalloc,TY_REAL)
            call realloc(nptr,3*nalloc,TY_INT)
         }

         # Get the spectrum number and the aperture number

         call gargwrd(Memc[junkstr],SZ_LINE)
         call gargi(ispec)
         call gargi(apnum)

         # Get database record for aperture and find out how many
         # features were identified

         call sprintf(Memc[line],SZ_LINE,"identify %s - Ap %d")
            call pargstr(Memc[rootname])
            call pargi(apnum)
         iferr (record = dtlocate(dt,Memc[line])) {
            call eprintf("Can't find database record for aperture %d\n")
               call pargi(apnum)
            Memr[sptr+n-1] = INDEF
            Memi[nptr+3*(n-1)] = ispec
            Memi[nptr+3*(n-1)+1] = apnum
            Memi[nptr+3*(n-1)+2] = 0
            next
         }
         nfeat = dtgeti(dt,record,"features")

         # Now read the feature information and do the required summations
         
         sum = 0.0
         sumsq = 0.0
         ntot = 0
         do i = 1,nfeat {
            junkint = dtscan(dt)
            call gargr(junkr)
            call gargr(w1)
            call gargr(w2)
            call gargr(junkr)
            call gargi(junkint)
            call gargi(iwt)
            if (IS_INDEF(w1) || IS_INDEF(w2)) iwt = 0
            if (iwt > 0) {
               resid = w1 - w2
               sum = sum + resid
               sumsq = sumsq + resid**2
               ntot = ntot + 1
            }
         }

         # How many coefficients were fit?  If the number of coef is less
         # than or equal to the number of features, then the sigma has
         # no meaning.  Hence, set it to INDEF.

         ncoef = dtgeti(dt,record,"coefficients") - 4

         # Calculate rms.  Deal with error situations...

         if (ntot <= ncoef || ntot <= 2)
            sig = INDEF
         else {
            mean = sum/real(ntot)
            sig = sumsq/real(ntot) - mean*mean
            if (sig < 0.0) 
               sig = INDEF
            else
               sig = sqrt(sig)
         }

         # Write the result to the workspace

         Memr[sptr+n-1] = sig
         Memi[nptr+3*(n-1)] = ispec
         Memi[nptr+3*(n-1)+1] = apnum
         Memi[nptr+3*(n-1)+2] = ntot
      }

      # Reallocate the workspace so that it's only just what you want

      call realloc(sptr,n,TY_REAL)
      call realloc(nptr,3*n,TY_INT)

      # Get the ensemble median sigma. 

      emed = ensmed(n,Memr[sptr])

      # Now report the situation...

      do i = 1,n {
         call printf(
            "Spectrum: %d,  Aperture: %d,  No. lines: %d,  RMS error: %0.6g\n")
            call pargi(Memi[nptr+3*(i-1)])
            call pargi(Memi[nptr+3*(i-1)+1])
            call pargi(Memi[nptr+3*(i-1)+2])
            call pargr(Memr[sptr+i-1])
      }
      call printf(" \n")
      call printf("Ensemble median: %0.6g\n")
         call pargr(emed)

      # Free workspace and close things up

      call dtunmap(dt)
      call close(fd)
      call mfree(nptr,TY_INT)
      call mfree(sptr,TY_REAL)
      call sfree(sp)
end

real procedure ensmed(n,x)

# Parameters

int n
real x[ARB],emed

# Local variables

pointer xptr,sp
int i,nmed

# Functions

real amedr()

begin

      # Get some workspace
  
      call smark(sp)
      call salloc(xptr,n,TY_REAL)

      # Copy over the array, ignoring the INDEF values.

      nmed = 0
      do i = 1,n {
         if (!IS_INDEFR(x[i])) {
            nmed = nmed + 1
            Memr[xptr+nmed-1] = x[i]
         }
      }

      # Get the median of the distribution

      if (nmed == 0)
         emed = INDEF
      else
         emed = amedr(Memr[xptr],nmed)

      # Tidy and exit

      call sfree(sp)
      return(emed)
end
