include <imhdr.h>

define TARGET "TARGET"
define ARC    "ARC"
define SKY    "SKY"

procedure t_collect()

# Local variables

pointer sp,imtype,fname,obstype,filenmo,filenma,filenms,list,iptr,line
bool skyobj,dispcor
int fdo,fds,fda,nfo,nfs,nfa,fd
real val

# Functions

bool clgetb(),streq()
int open(),imtgetim(),chkfname(),countem(),access()
pointer imtopenp(),immap(),imtopen()
real imgetr()

errchk immap,imgstr,imgetr(),imputr()

begin

      # Get workspace

      call smark(sp)
      call salloc(imtype,SZ_FNAME,TY_CHAR)
      call salloc(fname,SZ_FNAME,TY_CHAR)
      call salloc(obstype,SZ_FNAME,TY_CHAR)
      call salloc(filenmo,SZ_FNAME,TY_CHAR)
      call salloc(filenma,SZ_FNAME,TY_CHAR)
      call salloc(filenms,SZ_FNAME,TY_CHAR)
      call salloc(line,SZ_LINE,TY_CHAR)

      # Get image type

      call envgets("imtype",Memc[imtype],SZ_FNAME)

      # Are we treating skys as objects? Dispersion correcting?

      skyobj = clgetb("skyobj")
      dispcor = clgetb("dispcor")

      # Get the names of the output files

      call clgstr("objfile",Memc[filenmo],SZ_FNAME)
      fdo = open(Memc[filenmo],NEW_FILE,TEXT_FILE)
      call clgstr("skyfile",Memc[filenms],SZ_FNAME)
      fds = open(Memc[filenms],NEW_FILE,TEXT_FILE)
      call clgstr("arcfile",Memc[filenma],SZ_FNAME)
      fda = open(Memc[filenma],NEW_FILE,TEXT_FILE)

      # Get name of a file for chip info

      call clgstr("output",Memc[fname],SZ_FNAME)
      if (access(Memc[fname],0,0) == YES) 
         call delete(Memc[fname])
      fd = open(Memc[fname],NEW_FILE,TEXT_FILE)

      # Open the input list

      list = imtopenp("inlist")
      while (imtgetim(list,Memc[fname],SZ_FNAME) != EOF) {

         # First check that the name passes all known criteria

         if (chkfname(Memc[fname],Memc[imtype]) == NO)
            next

         # If you can't map it or the observation type is missing, then
         # skip it

         iferr {
            iptr = immap(Memc[fname],READ_ONLY,0)
            call imgstr(iptr,"OBSTYPE",Memc[obstype],SZ_LINE)     
            call imunmap(iptr)
         } then {
            next
         }

         # Ok, now see what type of file you have here...

         if (streq(Memc[obstype],TARGET)) {
            call fprintf(fdo,"%s\n")
               call pargstr(Memc[fname])
         } else if (streq(Memc[obstype],ARC)) {
            call fprintf(fda,"%s\n")
               call pargstr(Memc[fname])
         } else if (streq(Memc[obstype],SKY)) {
            call fprintf(fds,"%s\n")
               call pargstr(Memc[fname])
            if (skyobj) {
               call fprintf(fdo,"%s\n")
                  call pargstr(Memc[fname])
            }
         }
      }
      call imtclose(list)

      # Close the output lists

      call close(fdo)
      call close(fda)
      call close(fds)

      # Count how many files there are in each list

      nfo = countem(Memc[filenmo])
      nfa = countem(Memc[filenma])
      nfs = countem(Memc[filenms])

      # If there aren't any objects in the list, then there isn't much
      # point in continuing...

      if (nfo <= 0) 
         call error(0,"No object files specified")

      # If there aren't any offset skies, try searching the current directory

      if (nfs == 0 || (nfa == 0 && dispcor)) {
         fds = open(Memc[filenms],APPEND,TEXT_FILE)
         fda = open(Memc[filenma],APPEND,TEXT_FILE)
         call sprintf(Memc[fname],SZ_FNAME,"*.%s")
            call pargstr(Memc[imtype])
         list = imtopen(Memc[fname])
         while (imtgetim(list,Memc[fname],SZ_FNAME) != EOF) {

            # First check that the name passes all known criteria

            if (chkfname(Memc[fname],Memc[imtype]) == NO)
               next

            # If you can't map it or the observation type is missing, then
            # skip it

            iferr {
               iptr = immap(Memc[fname],READ_ONLY,0)
               call imgstr(iptr,"OBSTYPE",Memc[obstype],SZ_LINE)     
               call imunmap(iptr)
            } then {
               next
            }

            # Is it an offset sky?

            if (nfs == 0 && streq(Memc[obstype],SKY)) {
               call fprintf(fds,"%s\n")
                  call pargstr(Memc[fname])
               if (skyobj) {
                  call fprintf(fdo,"%s\n")
                     call pargstr(Memc[fname])
               }
            } else if ((nfa == 0 && dispcor) && streq(Memc[obstype],ARC)) {
               call fprintf(fda,"%s\n")
                  call pargstr(Memc[fname])
            }
         }
         call imtclose(list)
         call close(fds)
         call close(fda)

         # Now count them up again

         nfs = countem(Memc[filenms])
         nfa = countem(Memc[filenma])
      }

      # If no offset skies are available then we can't continue

      if (nfs == 0)
         call error(0,"No offset sky fields available")

      # If no arc files are available and you want to dispersion correct
      # send out a warning message that you won't be able to.

      if (nfa == 0 && dispcor) {
         call eprintf(
            "Warning: No arc files specified -- can't dispersion correct\n")
         dispcor = false
      }
      call fprintf(fd,"%b ")
         call pargb(dispcor)

      # Open the object list and look for the chip name

      call sprintf(Memc[fname],SZ_FNAME,"@%s\n")
         call pargstr(Memc[filenmo])
      list = imtopen(Memc[fname])
      Memc[line] = EOS
      while (imtgetim(list,Memc[fname],SZ_FNAME) != EOF) {
         ifnoerr {
            iptr = immap(Memc[fname],READ_ONLY,0)
            call imgstr(iptr,"CCDCHIP",Memc[line],SZ_LINE)
            call imunmap(iptr)
         } then {
            if (Memc[line] != EOS) 
               break
            else
               next
         } else
            next
      }
      if (Memc[line] == EOS) 
         call error(0,"No chip name in headers")
      else {
         call fprintf(fd," %.3s\n")
            call pargstr(Memc[line])
      }
      call imtclose(list)
      call close(fd)

      # Finally add the dispersion keyword to the header of the arc files

      call sprintf(Memc[fname],SZ_FNAME,"@%s")
         call pargstr(Memc[filenma])
      list = imtopen(Memc[fname])
      call clgstr("disp",Memc[line],SZ_LINE)
      while (imtgetim(list,Memc[fname],SZ_FNAME) != EOF) {
         iferr {
            iptr = immap(Memc[fname],READ_WRITE,0)
            val = imgetr(iptr,"DISPERSI")*imgetr(iptr,"CCDYPIXE")*0.001
            call imaddr(iptr,Memc[line],val)
            call imunmap(iptr)
         } then
            next
      }
      call imtclose(list)

      # Get out of here

      call sfree(sp)
end

int procedure countem(filename)

# Parameters

char filename[ARB]
int n

# Local variables

pointer list,sp,fn

# Functions

int fntopnb(),fntlenb()

begin

      # Get workspace

      call smark(sp)
      call salloc(fn,SZ_FNAME,TY_CHAR)

      # Open the list and count the number of files in it

      call sprintf(Memc[fn],SZ_FNAME,"@%s")
         call pargstr(filename)
      list = fntopnb(Memc[fn],NO)
      n = fntlenb(list)
      call fntclsb(list)

      # Return number 

      call sfree(sp)
      return (n)
end



int procedure chkfname(fname,extn)

# Parameters

char fname[ARB],extn[ARB]
int ans

# Local variables

pointer sp,out,iptr
int n

# Functions

int strlen(),strsearch()
pointer immap()
bool streq()

define  exit_    99

begin

      # Get workspace

      call smark(sp)
      call salloc(out,SZ_FNAME,TY_CHAR)

      # Strip off the image type if need be.

      call strpextn(fname,extn,Memc[out],SZ_FNAME)

      # Check for ".ms" extension

      ans = YES
      n = strlen(Memc[out])
      if (streq(Memc[out+n-3],".ms")) {
         ans = NO
         goto exit_
      }

      # Check for existence of "noscat"

      if (strsearch(Memc[out],"noscat") != 0) {
         ans = NO
         goto exit_
      }

      # Check to make sure this isn't a 1d sky spectrum.

      if (strsearch(Memc[out],"sky") == 4) {
         iferr(iptr = immap(Memc[out],READ_ONLY,0)) {
            ans = NO
            goto exit_
         }
         if (IM_NDIM(iptr) == 1 || (IM_NDIM(iptr) > 1 && IM_LEN(iptr,2) == 1))
            ans = NO
         call imunmap(iptr)
      }

      # Exit point

exit_
      if (ans == YES) 
         call strcpy(Memc[out],fname,SZ_FNAME)
      call sfree(sp)
      return(ans)
end
