include <imhdr.h>
include <error.h>
include <fset.h>
include "hddb.h"

procedure t_fibinfo()

# Local variables

pointer sp,iname,fibrange,iptr,feed,stdfile,name,ra,dec,mag,comment,rp,
   pptr
int nr,nfib,fd,fibx,pex,fiby,pey,stat,wt,nalloc,n,nfibtot,npiv,junk
real rval
bool compressed,full,explain

# Functions

pointer immap(),rng_open()
int rng_index(),imaccf(),open(),fstati(),fscan(),strlen()
bool clgetb()

begin

      # Get workspace first...

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(fibrange,SZ_LINE,TY_CHAR)
      call salloc(feed,SZ_LINE,TY_CHAR)
      call salloc(stdfile,SZ_FNAME,TY_CHAR)
      call salloc(name,SZ_LINE,TY_CHAR)
      call salloc(ra,SZ_LINE,TY_CHAR)
      call salloc(dec,SZ_LINE,TY_CHAR)
      call salloc(mag,SZ_LINE,TY_CHAR)
      call salloc(comment,SZ_LINE,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Open the input file

      call clgstr("input",Memc[iname],SZ_FNAME)
      iferr (iptr = immap(Memc[iname],READ_ONLY,0)) {
         call eprintf("Can't find file %s \n")
            call pargstr(Memc[iname])
         call erract(EA_ERROR)
      }         

      # How much output does the user want?  If full == yes then
      # write out as much information as there is available...

      full = clgetb("full")

      # Open the correct spectrum to pivot lookup table

      iferr (call imgstr(iptr,"WYFFEED",Memc[feed],SZ_LINE)) {
         call eprintf("Can't find feed name in %s\n")
            call pargstr(Memc[iname])
         call erract(EA_ERROR)
      }
      call strcpy("wyffos$lib/cross",Memc[stdfile],SZ_FNAME)
      call strcat(Memc[feed],Memc[stdfile],SZ_FNAME)
      iferr (fd = open(Memc[stdfile],READ_ONLY,TEXT_FILE)) {
         call eprintf("Can't find standard cross-reference file %s\n")
            call pargstr(Memc[stdfile])
         call erract(EA_ERROR)
      }

      # Read the lookup table input memory

      nalloc = 64
      call malloc(pptr,nalloc,TY_INT)
      n = 0
      junk = fscan(fd)
      while (fscan(fd) != EOF) {
         n = n + 1
         if (n > nalloc) {
            nalloc = nalloc + 64
            call realloc(pptr,nalloc,TY_INT)
         }
         call gargi(nfib)
         call gargi(Memi[pptr+n-1])
      }
      call realloc(pptr,n,TY_INT)
      nfibtot = n
      call close(fd)

      # Get the range of fibres to be considered and open up a range structure

      call clgstr("fib_rng",Memc[fibrange],SZ_LINE)
      rp = rng_open(Memc[fibrange],1.0,real(nfibtot),INDEF)

      # Find out whether the header has been compressed or not

      compressed = (imaccf(iptr,"F001DAT1") == YES)

      # Write an explanitory header if you want it.

      explain = clgetb("explain")
      if (explain) {
         call printf("\nFib Piv Object                  RA           DEC      Magn\n")
         if (full) {
            call printf("      Fibre X            Fibre Y       Status   Weight\n")
            call printf("Comment (optional)\n")
         }
      }

      # Now loop until there are no more entries in the range

      nr = 1
      while (rng_index(rp,nr,rval) != EOF) {
         nfib = nint(rval)

         # Find the corresponding pivot number

         if (nfib < 1 || nfib > nfibtot) {
            call eprintf("Fibre %d not specified in %s \n")
               call pargi(nfib)
               call pargstr(Memc[stdfile])
            next
         }
         npiv = Memi[pptr+nfib-1]
         
         # Read the compressed header
         
         if (compressed) {
            call rdcomp(iptr,npiv,Memc[name],Memc[ra],Memc[dec],
               Memc[mag],fibx,pex,fiby,pey,stat,wt,Memc[comment])
        
         # Read an uncompressed header

         } else {
            call rducomp(iptr,npiv,Memc[name],Memc[ra],Memc[dec],
               Memc[mag],fibx,pex,fiby,pey,stat,wt,Memc[comment])
         }

         # Now write the info to the terminal.  First the standard info.

         call printf("%3d %3d %-18s  %11s  %11s  %-9s\n")
            call pargi(nfib)
            call pargi(npiv)
            call pargstr(Memc[name])
            call pargstr(Memc[ra])
            call pargstr(Memc[dec])
            call pargstr(Memc[mag])

         # If the user wants more info, then give it to him!

         if (full) {
            call printf("%7d +/- %6d  %7d +/- %6d   %1d   %6d\n")
               call pargi(fibx)
               call pargi(pex)
               call pargi(fiby)
               call pargi(pey)
               call pargi(stat)
               call pargi(wt)
            if (strlen(Memc[comment]) > 0) {
               call printf("%s\n")
                  call pargstr(Memc[comment])
            }
            call printf(" \n")
         }

         # Go to next aperture in range
 
         nr = nr + 1
       }

       # Close up the input file, free the range structure and free all
       # allocated workspace

       call imunmap(iptr)
       call rng_close(rp)
       call mfree(pptr,TY_INT)
       call sfree(sp)
end


procedure rdcomp(iptr,npiv,name,ra,dec,mag,fibx,pex,fiby,pey,stat,wt,
   comment)

# Parameters

pointer iptr
int npiv,fibx,pex,fiby,pey,stat,wt
char name[ARB],ra[ARB],dec[ARB],mag[ARB],comment[ARB]

# Local variables

pointer sp,data
int junk,i,idummy
char headitem[8]

# Functions

int sscan(),imaccf()

begin

       # Get some workspace

       call smark(sp)
       call salloc(data,SZ_LINE,TY_CHAR)

       # Read the first header data line

       call sprintf(headitem,8,"F%03dDAT1")
          call pargi(npiv)
       call imgstr(iptr,headitem,Memc[data],SZ_LINE)
       call strcpy(Memc[data],name,18)
       call strcpy(Memc[data+19],ra,11)
       call strcpy(Memc[data+31],dec,11)
       call strcpy(Memc[data+46],mag,9)

       # Now the second
               
       call sprintf(headitem,8,"F%03dDAT2")
          call pargi(npiv)
       call imgstr(iptr,headitem,Memc[data],SZ_LINE)
       junk = sscan(Memc[data])
          call gargi(fibx)
          call gargi(pex)
          call gargi(fiby)
          call gargi(pey)
          do i = 1,3
             call gargi(idummy)
          call gargi(stat)
          call gargi(wt)

       # Finally read the third line if it exists

       call sprintf(headitem,8,"F%03dDAT3")
          call pargi(npiv)
       if (imaccf(iptr,headitem) == YES) 
          call imgstr(iptr,headitem,comment,SZ_LINE)
       else
          call strcpy(EOS,comment,SZ_LINE)

       # Ditch workspace

       call sfree(sp)
end

procedure rducomp(iptr,npiv,name,ra,dec,mag,fibx,pex,fiby,pey,stat,wt,
   comment)

# Parameters

pointer iptr
int npiv,fibx,pex,fiby,pey,stat,wt
char name[ARB],ra[ARB],dec[ARB],mag[ARB],comment[ARB]

# Local variables

pointer sp,data,hp
int junk,rah,ram,dd,dm
real realdat,ras,ds
char sign[1]

# Functions

pointer hdopen()
int sscan()
real hdgetr()

begin

      # Get some workspace

      call smark(sp)
      call salloc(data,SZ_LINE,TY_CHAR)

      # Open header structure

      hp = hdopen(iptr)
      call hdgfib(hp,npiv)

      # Get the name and format it in...

      call hdgstr(hp,"NAME",Memc[data],SZ_LINE)
      if (Memc[data] != EOS) {
         call sprintf(name,19,"%-.18s")
            call pargstr(Memc[data])
      } else 
         call strcpy("                  ",name,19)

      # Get RA.  Format it as a sexigesimal real.

      call hdgstr(hp,"RA",Memc[data],SZ_LINE)
      if (Memc[data] != EOS) {
         junk = sscan(Memc[data])
            call gargi(rah)
            call gargi(ram)
            call gargr(ras)
         call sprintf(ra,12,"%02d:%02d:%05.2f")
            call pargi(rah)
            call pargi(ram)
            call pargr(ras)
      } else
         call strcpy("           ",ra,12)

      # Get DEC.  Format it as a sexigesimal real including a sign.

      call hdgstr(hp,"DEC",Memc[data],SZ_LINE)
      if (Memc[data] != EOS) {
         if (Memc[data] == '-')
            sign[1] = '-'
         else
            sign[1] = '+'
         junk = sscan(Memc[data])
            call gargi(dd)
            call gargi(dm)
            call gargr(ds)
         dd = abs(dd)
         call sprintf(dec,12,"%1s%02d:%02d:%04.1f")
            call pargstr(sign[1])
            call pargi(dd)
            call pargi(dm)
            call pargr(ds)
      } else
         call strcpy("           ",dec,12)

      # Get the magnitude. 
         
      call hdgstr(hp,"MAGN",Memc[data],SZ_LINE)
      if (Memc[data] != EOS) {
         call sprintf(mag,10,"%.9s")
            call pargstr(Memc[data])
      } else
         call strcpy("         ",mag,10)

      # Get the fibre X position and error.

      realdat = hdgetr(hp,"FIBX")
      if (IS_INDEF(realdat))
         fibx = 0
      else {
         if (abs(realdat) >= 1.0e6)
            fibx = 0
         else
            fibx = nint(realdat)
      }
      realdat = hdgetr(hp,"PEX")
      if (IS_INDEF(realdat))
         pex = 0
      else {
         if (abs(realdat) >= 1.0e5)
            pex = 0
         else
            pex = nint(realdat)
      }

      # Get the fibre Y position and error.

      realdat = hdgetr(hp,"FIBY")
      if (IS_INDEF(realdat))
         fiby = 0
      else {
         if (abs(realdat) >= 1.0e6)
            fiby = 0
         else
            fiby = nint(realdat)
      }
      realdat = hdgetr(hp,"PEY")
      if (IS_INDEF(realdat))
         pey = 0
      else {
         if (abs(realdat) >= 1.0e5)
            pey = 0
         else
            pey = nint(realdat)
      }

      # Get the status code and fibre weight

      realdat = hdgetr(hp,"STAT")
      if (IS_INDEF(realdat))
         stat = 0
      else {
         if (realdat >= 10.0 || realdat < 0.0)
            stat = 0
         else
            stat = nint(realdat)
      }
      realdat = hdgetr(hp,"WT")
      if (IS_INDEF(realdat))
         wt = 0
      else {
         if (realdat >= 1.0e6 || realdat < 0.0)
            wt = 0
         else
            wt = nint(realdat)
      }

      # Get the comment. 

      call hdgstr(hp,"CMNT",Memc[data],SZ_LINE)
      if (Memc[data] != EOS)
         call strcpy(Memc[data],comment,SZ_LINE)
      else
         comment[1] = EOS

      # Ditch workspace

      call hdclose(hp)
      call sfree(sp)
end
                     
