include <imhdr.h>                                                
include <error.h>
include <fset.h>
include "hddb.h"

define   HDSIZE   63

procedure t_fixhead()

# Local variables

pointer sp,iname,line1[NHEAD],line2[NHEAD],line3[NHEAD],data,iptr
pointer name,ra,dec,type,mag,stdtype,typedict,hp
int i,junk,rah,ram,dd,dm,id,fibx,pex,fiby,pey,fibt,fpvx,fpvy,
   stat,wt
bool comment,verbose
real ras,ds,realdat
char sign[1]

# Functions

pointer immap(),hdopen()
int sscan(),fstati(),imaccf()
real hdgetr()
bool clgetb(),streq()

define exit_ 10

begin

      # Get workspace first...

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
      call salloc(data,SZ_LINE,TY_CHAR)
      call salloc(name,19,TY_CHAR)
      call salloc(ra,12,TY_CHAR)
      call salloc(dec,12,TY_CHAR)
      call salloc(type,3,TY_CHAR)
      call salloc(mag,10,TY_CHAR)
      call salloc(stdtype,3,TY_CHAR)
      call salloc(typedict,SZ_LINE,TY_CHAR)

      # Flush buffered output immediately

      if (fstati(STDOUT,F_REDIR) == NO) call fseti(STDOUT,F_FLUSHNL,YES)

      # Get input file first...

      call clgstr("input",Memc[iname],SZ_FNAME)
      iferr(iptr = immap(Memc[iname],READ_WRITE,0)) {
         call eprintf("Can't find file %s\n")
            call pargstr(Memc[iname])
         call erract(EA_ERROR)
      }
      verbose = clgetb("verbose")

      # If the file already has a compressed header, then get out of here

      if (imaccf(iptr,"F001DAT1") == YES) {
         call printf("File %s already has a compressed header\n")
           call pargstr(Memc[iname])
         goto exit_
      } else {
         call printf("File %s will have its header compressed\n")
            call pargstr(Memc[iname])
      }
 
      # Get a standard type in case the type parameter is blank.

      call clgstr("stdtype.p_min",Memc[typedict],SZ_LINE)
      call clgwrd("stdtype",Memc[stdtype],3,Memc[typedict])

      # Helpful opening line...

      if (verbose) {
         call printf("Compressing headers for file: %s \n")
            call pargstr(Memc[iname])
      }

      # Go through the file fibre by fibre and sort out the
      # header...

      hp = hdopen(iptr)
      do i = 1,NHEAD {
         call malloc(line1[i],HDSIZE,TY_CHAR)
         call malloc(line2[i],HDSIZE,TY_CHAR)
         call malloc(line3[i],HDSIZE,TY_CHAR)
         call hdgfib(hp,i)

         # Get the name and format it in...

         call hdgstr(hp,"NAME",Memc[data],SZ_LINE)
         if (Memc[data] != EOS) {
            call sprintf(Memc[name],19,"%-.18s")
               call pargstr(Memc[data])
         } else 
            call strcpy("                  ",Memc[name],19)

         # Get RA.  Format it as a sexigesimal real.

         call hdgstr(hp,"RA",Memc[data],SZ_LINE)
         if (Memc[data] != EOS) {
            junk = sscan(Memc[data])
               call gargi(rah)
               call gargi(ram)
               call gargr(ras)
            call sprintf(Memc[ra],12,"%02d:%02d:%05.2f")
               call pargi(rah)
               call pargi(ram)
               call pargr(ras)
         } else
            call strcpy("           ",Memc[ra],12)

         # Get DEC.  Format it as a sexigesimal real including a sign.

         call hdgstr(hp,"DEC",Memc[data],SZ_LINE)
         if (Memc[data] != EOS) {
            if (Memc[data] == '-')
               sign[1] = '-'
            else
               sign[1] = '+'
            junk = sscan(Memc[data])
               call gargi(dd)
               call gargi(dm)
               call gargr(ds)
            dd = abs(dd)
            call sprintf(Memc[dec],12,"%1s%02d:%02d:%04.1f")
               call pargstr(sign[1])
               call pargi(dd)
               call pargi(dm)
               call pargr(ds)
         } else
            call strcpy("           ",Memc[dec],12)

         # Get the TYPE.  If it is blank, give it a standard value. Also only
         # test the first character in case the data are shifted which
         # has happened in the past.

         call hdgstr(hp,"TYPE",Memc[data],SZ_LINE)
         call sprintf(Memc[type],3,"%.2s")
         if (Memc[data] != EOS) {
            switch (Memc[data]) {
               case 'F':
                  call pargstr("FI")
               case 'U':
                  call pargstr("UN")
               case 'P':
                  call pargstr("PR")
               case 'S':
                  call pargstr("SK")
               default:
                  call pargstr(Memc[stdtype])
            }
         } else {
            if (streq(Memc[name],"PARKED") || streq(Memc[name],"DISABLED")
               || streq(Memc[name],"MISSING"))
               call pargstr("UN")
            else
               call pargstr(Memc[stdtype])
         }

         # Get the magnitude. 
         
         call hdgstr(hp,"MAGN",Memc[data],SZ_LINE)
         if (Memc[data] != EOS) {
            call sprintf(Memc[mag],10,"%.9s")
               call pargstr(Memc[data])
         } else
            call strcpy("         ",Memc[mag],10)

         # Get the object identification code.  Make sure it's sensible.

         realdat = hdgetr(hp,"ID")
         if (IS_INDEF(realdat)) {
            id = 0
         } else {
            if (realdat >= 1.0e6 || realdat < 0.0)
               id = 0
            else
               id = nint(realdat)
         }

         # Now write all this out to the header line

         call sprintf(Memc[line1[i]],HDSIZE,"%-18s %11s %11s %2s %-9s %6d")
            call pargstr(Memc[name])
            call pargstr(Memc[ra])
            call pargstr(Memc[dec])
            call pargstr(Memc[type])
            call pargstr(Memc[mag])
            call pargi(id)

         # Get the fibre X position and error.

         realdat = hdgetr(hp,"FIBX")
         if (IS_INDEF(realdat))
            fibx = 0
         else {
            if (abs(realdat) >= 1.0e6)
               fibx = 0
            else
               fibx = nint(realdat)
         }
         realdat = hdgetr(hp,"PEX")
         if (IS_INDEF(realdat))
            pex = 0
         else {
            if (abs(realdat) >= 1.0e5)
               pex = 0
            else
               pex = nint(realdat)
         }

         # Get the fibre Y position and error.

         realdat = hdgetr(hp,"FIBY")
         if (IS_INDEF(realdat))
            fiby = 0
         else {
            if (abs(realdat) >= 1.0e6)
               fiby = 0
            else
               fiby = nint(realdat)
         }
         realdat = hdgetr(hp,"PEY")
         if (IS_INDEF(realdat))
            pey = 0
         else {
            if (abs(realdat) >= 1.0e5)
               pey = 0
            else
               pey = nint(realdat)
         }

         # Get the pivot position angle and X,Y position.

         realdat = hdgetr(hp,"FIBT")
         if (IS_INDEF(realdat))
            fibt = 0
         else {
            if (realdat >= 1.0e5 || realdat < 0.0)
               fibt = 0
            else
               fibt = nint(realdat)
         }
         realdat = hdgetr(hp,"FPVX")
         if (IS_INDEF(realdat))
            fpvx = 0
         else {
            if (abs(realdat) >= 1.0e6)
               fpvx = 0
            else
               fpvx = nint(realdat)
         }
         realdat = hdgetr(hp,"FPVY")
         if (IS_INDEF(realdat))
            fpvy = 0
         else {
            if (abs(realdat) >= 1.0e6)
               fpvy = 0
            else
               fpvy = nint(realdat)
         }

         # Get the status code and fibre weight

         realdat = hdgetr(hp,"STAT")
         if (IS_INDEF(realdat))
            stat = 0
         else {
            if (realdat >= 10.0 || realdat < 0.0)
               stat = 0
            else
               stat = nint(realdat)
         }
         realdat = hdgetr(hp,"WT")
         if (IS_INDEF(realdat))
            wt = 0
         else {
            if (realdat >= 1.0e6 || realdat < 0.0)
               wt = 0
            else
               wt = nint(realdat)
         }

         # Now write out second line...

         call sprintf(Memc[line2[i]],HDSIZE,
            "%7d %6d %7d %6d %5d %7d %7d %1d %6d")
            call pargi(fibx)
            call pargi(pex)
            call pargi(fiby)
            call pargi(pey)
            call pargi(fibt)
            call pargi(fpvx)
            call pargi(fpvy)
            call pargi(stat)
            call pargi(wt)

         # Get the comment.  If there is a comment, then print it to the 
         # third header line...

         call hdgstr(hp,"CMNT",Memc[data],SZ_LINE)
         if (Memc[data] != EOS) {
            comment = true
            call strcpy(Memc[data],Memc[line3[i]],HDSIZE)
         } else {
            comment = false
            Memc[line3[i]] = EOS
         }

         # Helpful verbose output, if required...

         if (verbose) {
            if (mod(i,10) == 0) {
               call printf("%3d fibre records done...\n")
                  call pargi(i)
            }
         }
      } # end of NHEAD loop

      # Write out the new headers and free the workspaces.

      call hddelf(hp)
      do i = 1,NHEAD {
         call puthdc(iptr,i,"DAT1",Memc[line1[i]])
         call puthdc(iptr,i,"DAT2",Memc[line2[i]])
         if (Memc[line3[i]] != EOS)
            call puthdc(iptr,i,"DAT3",Memc[line3[i]])
      }
      do i = 1,NHEAD {
         call mfree(line1[i],TY_CHAR)
         call mfree(line2[i],TY_CHAR)
         call mfree(line3[i],TY_CHAR)
      }
      call hdclose(hp)

      # Close up the file and exit out of here

exit_
      call imunmap(iptr)
      call sfree(sp)
end

procedure puthdc(iptr,fibnum,item,value)

# Parameters

pointer iptr
int fibnum
char item[ARB],value[ARB]

# Local variables

char headitem[SZKEY]

begin

      # Create header item name

      call sprintf(headitem,SZKEY,"F%03d%s")
         call pargi(fibnum)
         call pargstr(item)

      # Now write it to the header

      call imastr(iptr,headitem,value)
 end




