include <imhdr.h>
include <pkg/center1d.h>
include <math/nlfit.h>

procedure t_w_limits()

# Local variables

pointer sp,iname,iptr,igptr,ptr1,ptr2,xptr,wptr,wpt2,nl
real lowcut,highcut,mean,sigma,pkthresh,contrast,pkwidth,w1,w2,peak,diff,
   crad,cthresh,pkpos[2],width[2],tol,initsig,dpar[4],par[4],maxdiff
int ndim,nx,ngpix,sep,edge,nmax,type,npeaks,addf,adddf,itmax,i,plist[4],
   ipos,status,low,high,npar,j,j1,j2,nj,irem,i1
bool debug

# Functions

int awvgr(),find_peaks(),locpr()
real center1d()
pointer immap(),imgl1r()

extern gau,dgau

begin

      # Get the usual workspace...

      call smark(sp)
      call salloc(iname,SZ_FNAME,TY_CHAR)
 
      # Open input data file...

      call clgstr("input",Memc[iname],SZ_FNAME)
      iferr(iptr = immap(Memc[iname],READ_ONLY,0)) 
         call error(0,"Input gradient file unavailable")
      ndim = IM_NDIM(iptr)
      if (ndim != 1) 
         call error(0,"Input gradient file must be 1d")
      nx = IM_LEN(iptr,1)
      ptr1 = imgl1r(iptr)

      # Open the input gradient file...

      call clgstr("ingrad",Memc[iname],SZ_FNAME)
      iferr(igptr = immap(Memc[iname],READ_ONLY,0)) 
         call error(0,"Input gradient file unavailable")
      ndim = IM_NDIM(igptr)
      if (ndim != 1) 
         call error(0,"Input gradient file must be 1d")
      if (IM_LEN(iptr,1) != IM_LEN(igptr,1))
         call error(0,"Input and gradient files are not same length!")
      ptr2 = imgl1r(igptr)

      # Get some workspace for the peak finding routine.  Move
      # the input data array into one of them

      call salloc(xptr,nx,TY_REAL)
      call salloc(wptr,nx,TY_REAL)
      call salloc(wpt2,nx,TY_REAL)
      call amovr(Memr[ptr2],Memr[wptr],nx)
      call amovkr(1.0,Memr[wpt2],nx)

      # First do some stats on the input array.  Find a clipping threshold and 
      # then redo the stats

      call aavgr(Memr[wptr],nx,mean,sigma)
      lowcut = mean - 3.0*sigma
      highcut = mean + 3.0*sigma
      ngpix = awvgr(Memr[wptr],nx,mean,sigma,lowcut,highcut)
      if (IS_INDEF(sigma)) {
         call eprintf("Warning:  Estimate of low and high cuts too severe\n")
         sigma = 0.0
      }
      pkthresh = 5.0*sigma

      # Set up some things for the peak finding routine...

      sep = 2
      edge = 0
      nmax = 10
      debug = false
      contrast = 0.0

      # Set up some things for centring routine

      pkwidth = 4.0
      type = EMISSION
      crad = 5.0
      cthresh = 10.0

      # See if there is a peak...Note that because a 0 gradient transform
      # is performed and the jump (if it exists) is positive, then this
      # should be the left-hand limit

      npeaks = find_peaks(Memr[wptr],Memr[xptr],nx,contrast,sep,edge,nmax,
         pkthresh,debug)

      # If no peaks were found then skip the next bit...

      if (npeaks == 0)
         pkpos[1] = INDEF

      # If peaks were found start by trying to centre on them.  If you can't
      # do a centroid, then skip to the next one...

      else {
         maxdiff = 0.0
         irem = 0
         do i = 1,npeaks {
            Memr[xptr+i-1] = center1d(Memr[xptr+i-1],Memr[wptr],nx,pkwidth,
               type,crad,cthresh)

            # If the centroiding worked, do some stats on the graident data.
            # See if the peak value is greater than 3 times the sigma over
            # the mean value in the neighbourhood of the peak.  Also, see if
            # the area immediately to the left of the peak has an average
            # value of zero (within one sigma).

            if (!IS_INDEF(Memr[xptr+i-1])) {
               j = nint(Memr[xptr+i-1])
               j1 = min(1,j-25)
               j2 = max(nx,j+25)
               nj = j2 - j1 + 1
               call aavgr(Memr[wptr+j1-1],nj,mean,sigma)
               highcut = mean + 3.0*sigma
               lowcut = mean - 10.0*sigma
               ngpix = awvgr(Memr[wptr+j1-1],nj,mean,sigma,lowcut,highcut)
               i1 = int(Memr[xptr+i-1])
               w1 = 1.0 - (Memr[xptr+i-1] - real(i1))
               w2 = 1.0 - w1
               peak = Memr[wptr+i1-1]*w1 + Memr[wptr+i1]*w2
               diff = peak - (mean + 3.0*sigma)
               nj = j - j1 + 1
               call aavgr(Memr[ptr1+j1-1],nj,mean,sigma)
               highcut = mean + 3.0*sigma
               lowcut = mean - 10.0*sigma
               ngpix = awvgr(Memr[ptr1+j1-1],nj,mean,sigma,lowcut,highcut)
               if (abs(mean) < sigma) {
                  if (diff <= 0.0)
                     Memr[xptr+i-1] = INDEF
                  else {
                     if (diff > maxdiff) {
                        maxdiff = diff
                        irem = i
                     }
                  }
               } else {
                  Memr[xptr+i-1] = INDEF
               }
            }
         }
         if (irem != 0) 
            pkpos[1] = Memr[xptr+irem-1]
         else
            pkpos[1] = INDEF
      }

      # Now flip the spectrum over and find the negative peak...Flip it
      # back when you're finished

      call anegr(Memr[wptr],Memr[wptr],nx)      
      npeaks = find_peaks(Memr[wptr],Memr[xptr],nx,contrast,sep,edge,nmax,
         pkthresh,debug)

      # If no peaks were found then skip the next bit...

      if (npeaks == 0)
         pkpos[2] = INDEF

      # If peaks were found start by trying to centre on them.  If you can't
      # do a centroid, then skip to the next one...

      else {
         maxdiff = 0.0
         irem = 0
         do i = 1,npeaks {
            Memr[xptr+i-1] = center1d(Memr[xptr+i-1],Memr[wptr],nx,pkwidth,
               type,crad,cthresh)

            # If the centroiding worked, do some stats on the graident data.
            # See if the peak value is greater than 3 times the sigma over
            # the mean value in the neighbourhood of the peak.  Also, see if
            # the area immediately to the right of the peak has an average
            # value of zero (within one sigma).

            if (!IS_INDEF(Memr[xptr+i-1])) {
               j = nint(Memr[xptr+i-1])
               j1 = min(1,j-25)
               j2 = max(nx,j+25)
               nj = j2 - j1 + 1
               call aavgr(Memr[wptr+j1-1],nj,mean,sigma)
               highcut = mean + 3.0*sigma
               lowcut = mean - 10.0*sigma
               ngpix = awvgr(Memr[wptr+j1-1],nj,mean,sigma,lowcut,highcut)
               i1 = int(Memr[xptr+i-1])
               w1 = 1.0 - (Memr[xptr+i-1] - real(i1))
               w2 = 1.0 - w1
               peak = Memr[wptr+i1-1]*w1 + Memr[wptr+i1]*w2
               diff = peak - (mean + 3.0*sigma)
               nj = j2 - j + 1
               call aavgr(Memr[ptr1+j-1],nj,mean,sigma)
               highcut = mean + 3.0*sigma
               lowcut = mean - 10.0*sigma
               ngpix = awvgr(Memr[ptr1+j-1],nj,mean,sigma,lowcut,highcut)
               if (abs(mean) < sigma) {
                  if (diff <= 0.0)
                     Memr[xptr+i-1] = INDEF
                  else {
                     if (diff > maxdiff) {
                        maxdiff = diff
                        irem = i
                     }
                  }
               } else {
                   Memr[xptr+i-1] = INDEF
               }
            }
         }
         if (irem != 0) 
            pkpos[2] = Memr[xptr+irem-1]
         else
            pkpos[2] = INDEF
      }
      call anegr(Memr[wptr],Memr[wptr],nx)      

      # Routine addresses and fit parameters

      addf = locpr(gau)
      adddf = locpr(dgau)
      tol = 0.1
      itmax = 500
      initsig = 5.0
      do i = 1,4 {
         plist[i] = i
         dpar[i] = 0.1
      }
      do i = 1,nx 
         Memr[xptr+i-1] = real(i)

      # Now loop for the two peaks and fit gaussians to them

      do i = 1,2 {

         # If this peak exists, then do the gauss fit...

         if (!IS_INDEF(pkpos[i])) {
            ipos = nint(pkpos[i])
            par[1] = Memr[wptr+ipos-1]
            par[2] = pkpos[i]
            par[3] = initsig
            par[4] = 0.0
            call nlinitr(nl,addf,adddf,par,dpar,4,plist,4,tol,itmax)
            call nlfitr(nl,Memr[xptr],Memr[wptr],Memr[wpt2],nx,1,WTS_UNIFORM,
               status)
            switch (status) {
               case DONE:
                  call nlpgetr(nl,par,npar)
                  width[i] = par[3]
               default:
                  width[i] = INDEF
            }
            call nlfreer(nl)
         } else {
            width[i] = INDEF
         }
         call anegr(Memr[wptr],Memr[wptr],nx)      
      }

      # Decide about the final limits now depending upon whether you got
      # reasonable results above...

      do i = 1,2 {

         # If the peak position was not found then assume there is no peak
         # and the spectrum end point is the appropriate extreme

         if (IS_INDEF(pkpos[i])) {
            if (i == 1) 
               low = 1
            else
               high = nx

         # If the peak position exists but the sigma didn't fit very well, then
         # use the default width above...

         } else if (IS_INDEF(width[i])) {
            if (i == 1)
               low = nint(pkpos[i] + 1.178*initsig)
            else
               high = nint(pkpos[i] - 1.178*initsig)
         
         # Otherwise, set the limits such that they are 1 FWHM away from the
         # peak.

         } else {
            if (i == 1)
               low = nint(pkpos[i] + 1.178*width[i])
            else
               high = nint(pkpos[i] - 1.178*width[i])
         }
      }

      # Write out the results

      call printf("%d %d \n")
         call pargi(low)
         call pargi(high)

      # Bad result...

      if (low > high) 
         call error(0,"Nonsense results for spectral limits")

      # Close files and ditch workspaces

      call imunmap(iptr)
      call imunmap(igptr)
      call sfree(sp)

end



procedure gau(x,nvars,par,npar,z)

# Parameters

int nvars,npar
real x[nvars],par[npar],z

# Local variables

real arg

begin

      # Evaluate the gaussian...

      arg = (x[1] - par[2])/par[3]
      arg = -0.5*arg*arg
      if (abs(arg) <= 20.0)
         z = par[1]*exp(arg) + par[4]
      else
         z = par[4]
end


procedure dgau(x,nvars,par,dpar,npar,z,dz)

# Parameters

int nvars,npar
real x[nvars],par[npar],dpar[npar],z,dz[npar]

# Local variables

real arg,exparg

begin

      # Evaluate the gaussian...

      arg = (x[1] - par[2])/par[3]
      arg = 0.5*arg*arg
      if (arg < 20.0) 
         exparg = exp(-arg)
      else
         exparg = 0.0
      z = par[1]*exparg + par[4]

      # Now the first derivatives...

      dz[1] = exparg
      dz[2] = par[1]*exparg*((x[1] - par[2])/(par[3]**2))
      dz[3] = par[1]*exparg*(2.0*arg/par[3])
      dz[4] = 1.0
end
