procedure transm(input)

file    input             {prompt="Name of input file"}
bool    scattered = yes   {prompt="Do scattered light correction?"}
bool    interactive = yes {prompt="Do some of the work interactively?"}

struct  *fd
struct  *fd2

begin

      string log1,log2,feed,chip,tempname,extn,aps,sample,junkstr,otype
      int nfibres,maxshift,wyflim1,wyflim2,sam1,sam2,spnum,apnum,pivnum,anum
      int junkint,i,j
      real minsep,maxsep,width,lower,upper,ishift,meanval,maxval,junkr,value
      real val1,val2
      file apidtable,crlist,inp,output,temp,slfile,shifts
      bool inter,scat
      struct line

      # Get the input file name and hidden parameters

      inp = input
      inter = interactive
      scat = scattered

      # Set up the logging...

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose)
         log2 = "STDOUT"
      else
         log2 = "dev$null"

      # Get the name of the spectrum vs. pivot number cross references list

      hselect(inp,"WYFFEED",yes) | scan(feed)
      if (nscan() == 0) 
         error(0,"No feed information in input file header")
      crlist = "wyffos$lib/cross"//feed
      if (!access(crlist)) 
         error(0,"Can't access cross reference for feed "//feed)

      # If the ccdred backup prefix is a directory, then create it if it
      # doesn't exist

      if (stridx("/$]",backup) != 0) 
         if (!access(backup)) mkdir(backup)

      # Get the chip name

      chip = ""
      hselect(inp,"CCDCHIP",yes) | scan(chip)
      if (chip == "")
         error(0,"No chip name in file header")
      chip = substr(chip,1,3)

      # Set instrument info 

      print("Doing CCD processing on input frame") | tee (log1, >log2)
      w_preccd("","",chip) 

      # CCD process (bias subtract) the input frame

      ccdproc(inp,zero="") | tee(log1, >log2)

      # Compress the input file header

      print("Compressing headers in input frame") | tee (log1, >log2)
      w_hdcomp(inp,log1,log2,"",no)

      # Get fibre information and create an apidtable. Change the
      # name of AF2FLDNM in the header to avoid possible clashes with
      # already existing apidtables. This is needed because AF2FLDNM may
      # not have been changed from a previous setup.

      
      print("Creating aperture id table") | tee (log1, >log2)
      tempname = mktemp("STD")
      hedit(inp,"AF2FLDNM",tempname,add=yes,delete=no,verify=no,
         show=no,update=yes)
      headid(inp,inp) | scan(nfibres,minsep,maxsep,width,lower,upper,
         apidtable,shifts)
      wyfpars.sfile = shifts
      wyfpars.lower = lower
      wyfpars.upper = upper

      # If this is a sky or a flat, then edit the aperture reference so that
      # all of the parked fibres are extracted...

      otype = ""
      hselect(inp,"OBSTYPE",yes) | scan(otype)
      if (otype == "SKY" || otype == "FLAT") {
         temp = mktemp("tmp$iraf")
         fd = apidtable
         while (fscan(fd,i,j,line) != EOF) {
            if (substr(line,1,8) != "DISABLED" && 
               substr(line,1,7) != "MISSING") 
               j = 1
            print(i,j,line,>>temp)
         }
         fd = ""
         delete(apidtable,verify=no)
         rename(temp,apidtable,field="all")
      }

      # Create multispec file extension

      extn = ".ms."//envget("imtype")

      # Set up some apscript variables

      apscript.readnoise = wyfpars.readnoise
      apscript.gain = wyfpars.gain
      apscript.nfind = nfibres
      apscript.width = width
      apscript.t_width = width
      apscript.minsep = minsep
      apscript.maxsep = maxsep
      apscript.radius = minsep
      apscript.clean = no
      apscript.saturation = INDEF
      apscript.apidtable = apidtable
      apscript.references = ""
      apscript.ansfind = "YES"
      apscript.ansresize = "NO"
      apscript.ansrecenter = "NO"
      apscript.anstrace = "YES"
      if (scat) {
         apscript.ansscat = "YES"
         apscript.anssmooth = "YES"
      } else {
         apscript.ansscat = "NO"
         apscript.anssmooth = "NO"
      }
      if (scat && inter) {
         apscript.ansfitscatter = "yes"
         apscript.ansfitsmooth = "yes"
      } else {
         apscript.ansfitscatter = "NO"
         apscript.ansfitsmooth = "NO"
      }
      apscript.ansextract = "YES"
      apscript.ansreview = "NO" 
      if (inter) {
         apscript.ansedit = "yes"
         apscript.ansfittrace = "no"
      } else {
         apscript.ansedit = "NO"
         apscript.ansfittrace = "NO"
      }

      # If you want to do scattered light subtraction, do that now...

      if (scat) {
         print("Subtracting scattered light") | tee (log1, >log2)
         apscript(inp,output=inp,ansextract="NO")
         apscript.ansscat = "NO"
         apscript.anssmooth = "NO"
         apscript.ansedit = "NO"
         apscript.ansfind = "NO"
         apscript.anstrace = "NO"
      }
      
      # Ok, now extract the frame...

      print("Extracting spectra") | tee (log1, >log2)
      output = inp//extn
      apscript(inp,output=output)

      # Do sawtooth correction and locate the ends.

      if (wyfpars.dosaw) {
         maxshift = 0
         fd = wyfpars.sfile
         while (fscan(fd,ishift,aps) != EOF) {
            if (abs(ishift) > maxshift)
               maxshift = nint(abs(ishift))
            if (ishift != 0.0) 
               specshift(output,ishift,apertures=aps,verbose=no)
         }
         fd = ""
      }
      if (wyfpars.fends) 
         w_ends(inp,inter=inter)
      else
         w_qends(inp)

      # Find the limits of the spectra 

      hselect(inp,"WYFLIM*",yes) | scan(wyflim1,wyflim2)
      if (nscan() < 2)
         error(0,"Must have spectral limits in header")

      # Find the sample limits for stats.  This is necessary so that when
      # you are normalising the flats you are only comparing the spectra
      # in the regions where the overlap.

      if (wyfpars.dosaw) {
         sam1 = wyflim1 + maxshift
         sam2 = wyflim2 - maxshift
         sample = sam1//":"//sam2
      } else {
         sample = "*"
         sam1 = INDEF
         sam2 = INDEF
      }

      # Make some temporary files...

      temp = mktemp("tmp$iraf")
      slfile = mktemp("tmp$iraf")

      # Attempt to get rid of CR hits and night sky emission lines.

      sfit(output,temp,lines="*",bands="1",type="data",
         replace=yes,wavescale=yes,logscale=no,override=no,
         listonly=no,logfile="",interac=no,
         sample=sample,naverage=1,function="legendre",
         order=10,low_rej=4.0,high_rej=4.0,niterate=5,grow=0)

      # Now do the averaging.  This should create a file with the spectra
      # replaced by constant values which reflect the average value of each
      # of the original spectra.

      hedit(temp,"SFIT","",add=no,delete=yes,verify=no,show=no,
         update=yes)
      hedit(temp,"SFITB","",add=no,delete=yes,verify=no,show=no,
         update=yes)
      sfit(temp,temp,lines="*",bands="1",type="fit",
         replace=no,wavescale=yes,logscale=no,override=no,
         listonly=no,logfile="",interac=no,
         sample=sample,naverage=1,function="legendre",
         order=1,low_rej=3.0,high_rej=3.0,niterate=0,grow=0)
                                                   
      # Find the mean and the maximum values of the averages...

      imstatistics(temp//"[1,*]",fields="mean,max",lower=INDEF,upper=INDEF,
         binwidth=0.1,format=no) | scan(meanval,maxval)

      # Get a list of the apertures...

      slist(temp,apertures="",long_header=no,>slfile)

      # Ok...scroll through each spectrum.  For each, write out the
      # spectrum number, its corresponding pivot number, the throughput
      # normalised by the mean and the throughput normalised by the max.

      print("\n\nFibre transmission results for file: ",inp)
      print("T(mean) scaled to a mean of: ",meanval)
      print("T(max) scaled to a maximum of: ",maxval)
      print("\n")
      print(" Ap  Piv   T(mean)   T(max)")
      fd = slfile
      fd2 = crlist
      junkint = fscan(fd2)
      while (fscan(fd,junkstr,spnum,apnum) != EOF) {
         while (fscan(fd2,anum,pivnum) != EOF && anum < apnum) 
            ;
         listpix(temp//"[1:1,"//spnum//"]",wcs="logical",formats="",
            verbose=no) | scan(junkr,value)
         val1 = value/meanval
         val2 = value/maxval
         printf("%3d  %3d   %6.2f   %6.2f\n",apnum,pivnum,val1,val2) | tee(log1)
      }
      fd = ""; fd2 = ""

      # Now clean up a bit

      delete(slfile,verify=no)
      delete(apidtable,verify=no)
      imdelete(temp,verify=no)
      imdelete(output,verify=no)
      imdelete(inp,verify=no)
      imrename(backup//inp,inp,verbose=no)
end
