# W_ARCREFS -- Determine dispersion relation for reference arcs.

procedure w_arcrefs(arcref1,arcref1ms,apidtable,response,log1,log2,log3)

file    arcref1
file    arcref1ms
file    apidtable
file    response
file    log1
file    log2
file    log3

struct  *fd

begin
      string   imtype,aps,arcref,arcrefms,temp,str1
      int      wyflim1,wyflim2
      real     ishift

      # Get image type and a temporary file name

      imtype = "." // envget ("imtype")
      temp = mktemp ("tmp$iraf")

      # Get wyffos spectral limits...

      hselect(apscript.references,"WYFLIM*",yes) | scan(wyflim1,wyflim2)
      if (nscan() < 2)
         error(0,"WYFFOS spectral limits not in apref header")

      # Extract the primary arc reference spectrum. Do sawtooth correction

      arcref = arcref1
      arcrefms = arcref1ms
      if (!access (arcrefms//imtype)) {
         print("Extract arc reference image ", arcref) | tee (log1, > log3)
         apscript(arcref,output=arcrefms,ansrecenter="NO",
            ansresize="NO",ansedit="NO",anstrace="NO",
            nsubaps=wyfpars.nsubaps,background="none",clean=no,
            weights="none")
         sapertures(arcrefms,apertures="",apidtable=apidtable,
            wcsreset=no,verbose=no,beam=INDEF,dtype=INDEF,w1=INDEF,
            dw=INDEF,z=INDEF,aplow=INDEF,aphigh=INDEF,title=INDEF)
         if (wyfpars.dosaw) {
            fd = wyfpars.sfile
            while (fscan(fd,ishift,aps) != EOF) 
               specshift(arcrefms,ishift,apertures=aps,verbose=no)
            fd = ""
         }
         w_mksafe(arcrefms,wyflim1,wyflim2)

         # Flat field the arc spectra.

         if (response != "") {
            if (wyfpars.nsubaps == 1)
               sarith (arcrefms, "/", response, arcrefms, w1=INDEF,
                  w2=INDEF, apertures="", bands="", beams="", apmodulus=0,
                  reverse=no, ignoreaps=no, format="multispec",
                  renumber=no, offset=0, clobber=yes, merge=no,
                  errval=0, verbose=no)
            else {
               blkrep (response, temp, 1, wyfpars.nsubaps)
               sarith (arcrefms, "/", temp, arcrefms, w1=INDEF, w2=INDEF,
                  apertures="", bands="", beams="", apmodulus=0,
                  reverse=no, ignoreaps=yes, format="multispec",
                  renumber=no, offset=0, clobber=yes, merge=no,
                  errval=0, verbose=no)
               imdelete (temp, verify=no)
            }
         }
      }

      # Check to see if this file has already been dispersion corrected.
      # If it hasn't then call a routine appropriate to the level of
      # automation requested by the user

      hselect (arcrefms, "dclog1", yes) | scan (str1)
      if (nscan () != 1) {
         print ("Determine dispersion solution for ", arcref) | 
            tee (log1, >log3)
         delete (database//"/id"//arcrefms//"*", verify=no)
         if (wyf_red.autoid == 1) 
            w_arcref1(arcrefms)
         else if (wyf_red.autoid == 2)
            w_arcref2(arcrefms)
         else if (wyf_red.autoid == 3)
            w_arcref3(arcrefms)

         # Get a listing of the goodness of the arc fits

         slist(arcrefms,apertures="",long_header=no, > temp)
         arcstat(arcrefms,temp) | tee (log1, > log3)
         delete(temp,verify=no)

         # Dispersion correct the reference arc.  This step is required to
         # use the confirm option of MSDISPCOR to set the wavelength scale
         # for all further spectra.  Set the newdisp flag.

         print ("Dispersion correct ", arcref) | tee (log1, > log3)
         dispcor.log = wyfpars.log
         dispcor(arcrefms,"",linearize=wyfpars.linearize,
            database=database,table="",w1=INDEF,w2=INDEF,dw=INDEF,
            nw=INDEF,flux=wyfpars.flux,samedisp=no,global=no, 
            ignoreaps=yes,confirm=no,listonly=no, 
            verbose=verbose,logfile=logfile)
         if (wyfpars.nsubaps > 1) {
            imrename (arcrefms, temp, verbose=no)
            scopy (temp, arcrefms, w1=INDEF, w2=INDEF, apertures="-999",
               bands="", beams="", apmodulus=0, offset=0,
               format="multispec", clobber=no, merge=no, renumber=no,
               verbose=no)
            blkavg (temp, temp, 1, wyfpars.nsubaps, option="sum")
            imcopy (temp, arcrefms//"[*,*]", verbose=no)
            imdelete (temp, verify=no)
         }
         w_proc.newdisp = yes
      }
end
                        
