procedure w_doarcs(spec,response,arcref1,extn,apidtable,reextract,logfile,
   batch)

file    spec
file    response
file    arcref1
string  extn
file    apidtable
bool    reextract
file    logfile
bool    batch

struct  *fd

begin
      string    imtype,log1,log2,log3,aps
      int       j,k,wyflim1,wyflim2
      file      temp,arc1,arc2,str1,apref,arcms
      real      ishift

      # Get image type and set up temporary file name

      imtype = "." // envget ("imtype")
      temp = mktemp ("tmp$iraf")

      # Set up logging.
   
      tee.append = yes
      if (logfile == "")
          log1 = "dev$null"
      else
          log1 = logfile
      if (verbose && !batch)
          log2 = "STDOUT"
      else
          log2 = "dev$null"
      if (batch)
         log3 = "dev$null"
      else
         log3 = "STDOUT"

       
      # Get WYFFOS spectral limits from aperture reference header...

      hselect(apscript.references,"WYFLIM*",yes) | scan(wyflim1,wyflim2)
      if (nscan() < 2)
         error(0,"WYFFOS spectral limits not in apref header")

      # Loop for each reference spectrum which might be listed in the
      # object file header.  The reference spectrum initially refers
      # to the 2d spectrum.  We'll change this later.

      for (j=1; j<=2; j+=1) {
         hselect (spec, "refspec"//j, yes, > temp)
         fd = temp
         k = fscan (fd, arc1, str1)
         fd = ""; delete (temp, verify=no)
         if (k < 1)
            break

         # Set extraction output and aperture reference depending on whether
         # the arcs are to be rextracted using recentered or retraced object
         # apertures.

         arc2 = arc1 // extn
         apref = apscript.references
         if (reextract && access (arc2//imtype)) {
            if (arc2 != arcref1 // extn)
               imdelete (arc2, verify=no)
         }

         # Extract and determine dispersion function if necessary.

         if (!access (arc2//imtype)) {
            delete (database//"/id"//arc2//"*", verify = no)
            print ("Extract and reidentify arc spectrum ", arc1) |
               tee (log1, > log3)
            apscript (arc1, output=arc2, references=apref,
               ansrecenter="NO", ansresize="NO", ansedit="NO",
               anstrace="NO", nsubaps=wyfpars.nsubaps, background="none",
               clean=no, weights="none", verbose=verbose)
            sapertures (arc2, apertures="", apidtable=apidtable,
               wcsreset=no, verbose=no, beam=INDEF, dtype=INDEF, w1=INDEF,
               dw=INDEF, z=INDEF, aplow=INDEF, aphigh=INDEF, title=INDEF)
            if (wyfpars.dosaw) {
               fd = wyfpars.sfile
               while (fscan(fd,ishift,aps) != EOF) 
                  if (ishift != 0.0)
                     specshift(arc2,ishift,apertures=aps,verbose=no)
               fd = ""
            }
            w_mksafe(arc2,wyflim1,wyflim2)
            if (response != "") {
               if (wyfpars.nsubaps == 1)
                  sarith (arc2, "/", response, arc2, w1=INDEF, w2=INDEF,
                     apertures="", bands="", beams="", apmodulus=0,
                     reverse=no, ignoreaps=no, format="multispec",
                     renumber=no, offset=0, clobber=yes, merge=no,
                     errval=0, verbose=no)
               else {
                  blkrep (response, temp, 1, wyfpars.nsubaps)
                  sarith (arc2, "/", temp, arc2, w1=INDEF, w2=INDEF,
                     apertures="", bands="", beams="", apmodulus=0,
                     reverse=no, ignoreaps=yes, format="multispec",
                     renumber=no, offset=0, clobber=yes, merge=no,
                     errval=0, verbose=no)
                  imdelete (temp, verify=no)
               }
            }

            # Identify the arc lines

            reidentify (arcref1//extn, arc2,
               interactive=no, section="middle line",
               shift=0., step=1, nsum=1, cradius=wyfpars.cradius,
               threshold=wyfpars.cthresh, nlost=100,
               refit=wyfpars.refit, trace=no, override=no,
               addfeatures=wyfpars.addfeatures,
               coordlist=wyf_red.coordlist, match=wyfpars.match,
               maxfeatures=50, minsep=2., database=database,
               plotfile=plotfile, logfiles=logfile,
               verbose=verbose,aidpars="")

            # Get a listing of the goodness of the arc fits
 
            slist(arc2,apertures="",long_header=no, > temp)
            arcstat(arc2,temp) | tee (log1, > log3)
            delete(temp,verify=no)
         }
    
         # Set the REFSPEC parameters for multispec spectrum.

         if (k == 1)
            hedit (spec//".ms", "refspec"//j, arc2, add=yes,
               verify=no, show=no, update=yes)
         else
            hedit (spec//".ms", "refspec"//j, arc2//" "//str1,
               add=yes, verify=no, show=no, update=yes)
      }
end
