procedure w_ends(apreference)

# Find where the spectra stop on the dispersion axis.

file     apreference   {prompt="Aperture reference spectrum"}
bool     inter         {prompt="Interactive input?"}
bool     automatic     {prompt="Use automatic limit routine?", mode="q"}
real     userlow       {prompt="Input estimate for lower limit", mode="q"}
real     userhigh      {prompt="Input estimate for upper limit", mode="q"}

begin

      file     temp1,temp2,log1,log2,aprefms,apref
      real     crval1,crpix1,cd1_1
      string   mstype,imtype
      int      low,high
      bool     auto

      # Temporary files...

      temp1 = mktemp("tmp$iraf")
      temp2 = mktemp("tmp$iraf")

      # Sort out file extensions

      apref = apreference
      imtype = "."//envget("imtype")
      mstype = ".ms"//imtype      
      aprefms = apref//mstype

      # Set logging

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      log2 = "dev$null"

      # Combine all the spectra in the aperture reference from pixel 1 to
      # where-ever.  NB: if a sawtooth correction has been done, then 
      # pixel 1 is not necessarily the first pixel in the spectrum.

      print("Find the spectral limits from file ",apref) | tee (log1, >log2)
      scombine.log = no
      scombine(apref//mstype,temp1,noutput="",logfile="STDOUT",
         apertures="",group="all",combine="average",reject="none",
         first=no,w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,
         scale="none",zero="none",weight="none",sample="",
         lthreshold=INDEF,hthreshold=INDEF,nlow=1,nhigh=1,
         nkeep=1,mclip=yes,lsigma=3.0,hsigma=3.0,rdnoise=0.0,
         gain=1.0,snoise=0.0,pclip=-0.5,grow=0,blank=0.0) |
         tee (log1, > log2)

      # Do you want this to be automatic?  Only ask if in interactive mode.

      if (inter)
         auto = automatic
      else
         auto = yes

      # If find the endpoints automatically

      if (auto) {

         # Do a gradient transform of the combined spectrum.  Smooth the
         # resulting transform.

         gradient(temp1,temp2,0,boundary="nearest",constant=0.0)

         # Locate the peaks and find the limits...

         w_limits(temp1,temp2) | scan(low,high)
         hselect(temp1,"CRVAL1",yes) | scan(crval1)
         hselect(temp1,"CRPIX1",yes) | scan(crpix1)
         hselect(temp1,"CD1_1",yes) | scan(cd1_1)
         low = (low - crpix1)*cd1_1 + crval1
         high = (high - crpix1)*cd1_1 + crval1
         print("Spectral limits found from file ",apref," are ", low, high) | 
            tee (log1, >log2)

         # Write the limits to the header of the aperture reference file.

         hedit(apref//imtype,"wyflim1",low,add=yes,delete=no,verify=no,
            show=no,update=yes)
         hedit(apref//imtype,"wyflim2",high,add=yes,delete=no,verify=no,
            show=no,update=yes)
 
         # Ditch the temporary files...

         imdelete(temp1,verify=no)
         imdelete(temp2,verify=no)

      # Find the endpoints by hand...

      } else {
         print("Locate the the endpoints using the cursor on this graph")
         splot(temp1,line=1,band=1,units="",options="auto wreset",xmin=INDEF,
            xmax=INDEF,ymin=INDEF,ymax=INDEF,save_file="",graphics="stdgraph",
            cursor="",sigma0=INDEF,invgain=INDEF,function="spline3",order=1,
            low_reject=3,high_reject=3,niterate=1,grow=0,markrej=yes)
         low = userlow
         high = userhigh
         print("Spectral limits defined by user from file ",apref,
            " are ", low, high) | tee (log1, >log2)
         if (low > high) 
            error(0,"low > high: Nonsense limits")

         # Write the limits to the header of the aperture reference file.

         hedit(apref//imtype,"wyflim1",low,add=yes,delete=no,verify=no,
            show=no,update=yes)
         hedit(apref//imtype,"wyflim2",high,add=yes,delete=no,verify=no,
            show=no,update=yes)
 
      }
end
