# LISTONLY -- List processing to be done.
#
# This follows pretty much the same logic as the full procedure but doesn't
# do anything but list the operations.

procedure w_listonly (objects,apidtable,apref,flat,throughput,arcs1,
   scattered,arcscat,dispcor,skysubtract,tippex,redo,update)

string  objects = ""            {prompt="List of object spectra"}
file    apidtable = ""          {prompt="Aperture ID table"}
file    apref = ""              {prompt="Aperture reference spectrum"}
file    flat = ""               {prompt="Flat field spectrum"}
file    throughput = ""         {prompt="Throughput file or image"}
string  arcs1 = ""              {prompt="List of arc spectra"}

bool    scattered               {prompt="Subtract scattered light?"}
bool    arcscat                 {prompt="Subtract scattered light for arcs?"}
bool    dispcor                 {prompt="Dispersion correct spectra?"}
bool    skysubtract             {prompt="Subtract sky?"}
bool    tippex                  {prompt="Tippex out sky line residuals?"}
bool    redo = no               {prompt="Redo operations if previously done?"}
bool    update = yes            {prompt="Update spectra if cal data changes?"}

struct  *fd

begin
      string imtype,mstype,extn,spec,arcref1,specms,arcref1ms,response,objs
      string str
      file temp,temp1
      bool reextract,newaps,newresp,newdisp,scat,extract,disp,sky,tipp
      int i,dc

      # Get image type and multispec extension.  Also define an extension
      # to be used with calibration images

      imtype = "." // envget ("imtype")
      mstype = ".ms" // imtype
      extn = apidtable//".ms"

      # Create some temporary file names

      temp = mktemp("tmp$iraf")
      temp1 = mktemp("tmp$iraf")

      # Get some query parameters

      objs = objects

      # If the update switch is used changes in the calibration data
      # can cause images to be reprocessed (if they are in the object
      # list).  Possible changes are in the aperture definitions,
      # response function, dispersion solution, and sensitivity
      # function.

      newaps = no
      newresp = no
      newdisp = no

      # If you are reextracting or you can't access the aperture
      # database file for the aperture reference, then locate and
      # trace the apertures.

      reextract = redo
      if (reextract || !access (database//"/ap"//apref//extn)) {
         print ("Set reference aperture for ", apref)
         newaps = yes
      }

      # Process the offset sky exposure.  Start off by checking whether
      # to subtract out the scattered light and whether that has already
      # been done.

      spec = throughput
      specms = spec//mstype
      if (scattered) {
         scat = no
         if (redo && access(spec//"noscat"//imtype))
            hselect(spec//"noscat","apscatte",yes) | scan (str)
         else
            hselect(spec,"apscatte",yes) | scan (str)
         if (nscan() == 0)
            scat = yes
      }
      if (scat)
         print("Subtract scattered light from ", spec)

      # Create the response function now.

      response = flat//spec//extn
      reextract = redo || (update && newaps)
      if (reextract || !access(response//imtype) || (redo && scat)) {
         print ("Create response function ", response)
         newresp = yes
      }

      # Do this section if dispersion correcting.  Get the first arc file
      # out of the list.  Decide whether it needs extracting.

     if (dispcor) {
         sections(arcs1,option="fullname",>temp)
         if (arcscat) {
            fd = temp
            while (fscan(fd,arcref1) != EOF) {
               scat = no
               if (redo && access(arcref1//"noscat"//imtype))
                  hselect(arcref1//"noscat","apscatte",yes) | scan (str)
               else
                  hselect(arcref1,"apscatte",yes) | scan (str)
               if (nscan() == 0)
                  scat = yes
               if (scat) {
                  print ("Subtract scattered light from ", arcref1) | 
                     tee (log1, > log3)
               }
            }
         }
         fd = temp
         i = fscan(fd,arcref1)
         fd = ""
         delete(temp,verify=no)
         arcref1ms = arcref1//extn
         reextract = redo || (update && newaps)
         if (reextract || !access(arcref1ms//imtype)) {
            print("Extract arc reference image ", arcref1)
            print("Determine dispersion solution for ", arcref1)
            newdisp = yes
         } else {
            hselect(arcref1ms,"dclog1",yes, > temp)
            fd = temp
            dc = -1
            i = fscan(fd, dc)
            fd = ""; delete(temp,verify=no)
            if (i < 1) {
               print("Determine dispersion solution for ", arcref1)
               newdisp = yes
            }
         }
      }

      # Right, now process the object spectra! First check whether a
      # reextraction is needed.  Then decide exactly what needs doing.

      reextract = redo || (update && (newaps || newresp || newdisp))
      spec = objs
      specms = spec//mstype
      scat = no
      extract = no
      disp = no
      sky = no
      tipp = no
      if (scattered) {
         if (redo && access(spec//"noscat"//imtype))
            hselect(spec//"noscat","apscatte",yes) | scan (str)
         else
            hselect (spec,"apscatte",yes) | scan(str)
         if (nscan() == 0)
            scat = yes
      }
      if (reextract || !access(specms) || (redo && scat))
         extract = yes
      else {
         hselect(specms,"dclog1",yes) | scan(str)
         if (nscan() == 0)
            disp = yes
         else
            extract = update && newdisp
         hselect(specms,"skysub",yes) | scan(str)
         if (nscan() == 0)
            sky = skysubtract
         hselect(specms,"tippex",yes) | scan(str1)
         if (nscan() == 0)
            tipp = tippex
      }

      # If an extraction needs to be done then get the rest of the processing
      # switches directly from the input parameters

      if (extract) {
         disp = dispcor
         sky = skysubtract
         tipp = tippex
      }

      # Right, if you're going to extract the object frame, then do that
      # now.  Check and see if the scattered light wants removing too.

      if (scat)
         print("Subtract scattered light from ", spec)
      if (extract)
         print("Extract object spectrum ", spec)
      if (disp)
         print("Dispersion correct ", spec)
      if (sky)
         print("Sky subtract ", spec)
      if (tipp)
         print("Tippexing ",spec)
end
