procedure w_preccd(mbias,mflat,chip,listonly)

file    mbias    {prompt="Mean bias frame"}
file    mflat    {prompt="Mean flat frame"}
string  chip     {prompt="Chip type"}
bool    listonly {prompt="Don't actually do any processing"}

struct  *fd
begin

      string    imtype,instr,tempfile,temp2,log1,log2
      file      input,meanbias,meanflat
      int       n,i,nf

      # Set up the logging...

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose)
         log2 = "STDOUT"
      else
         log2 = "dev$null"

      # Get temporary file name

      tempfile = mktemp("tmp$iraf")
      temp2 = mktemp("tmp$iraf")
 
      # Get the image type...

      imtype = "."//envget("imtype")
      n = strlen(imtype)

      # Decide which files to look at by removing any multispec files...

      sections("*"//imtype,option="root", > tempfile)
      fd = tempfile
      while (fscan(fd,input) != EOF) {
         i = strlen(input)
         if (i > n && substr(input,i-n+1,i) == imtype)
            input = substr(input,1,i-n)
         i = strlen(input)
         if (i > 3 && substr(input,i-2,i) == ".ms") next
         print(input,>>temp2)
      }
      delete (tempfile, verify=no)

      # Strip extension from mean file names
   
      meanbias = mbias
      i = strlen(meanbias)
      if (i > n && substr(meanbias,i-n+1,i) == imtype)
         meanbias = substr(meanbias,1,i-n)
      meanflat = mflat
      i = strlen(meanflat)
      if (i > n && substr(meanflat,i-n+1,i) == imtype)
         meanflat = substr(meanflat,1,i-n)

      # Decide which chip you are using and set the instrument for ccdred.
      # note that this also sets up a lot of the hidden parameters in ccdproc.

      if (chip == "TEK") {
         instr = "wyf_tek"
      } else if (chip == "EEV") {
         instr = "wyf_eev"
      } else {
         print("Input chip type:", chip)
         error(0,"Unknown chip type")
      }
      setinstrument(instr,site="lpo",directory="rgo$ccddb/",rev-)

      # Set up backup directory and logfile names in ccdred 

      ccdred.backup = backup
      ccdred.logfile = logfile

      # If a mean bias frame is not specified, then assume that you are
      # only subtracting a constant bias

      if (meanbias == "")
         ccdproc.zerocor = no
      else 
         ccdproc.zerocor = yes

      # If you are processing bias frames, then do that here.  If the
      # mean frame already exists or there is no mean frame specified
      # then get out of here.  If there is a mean frame specified, but
      # but no bias frames available, then change the switch to subtract
      # bias frames off.  Otherwise, combine all the available bias frames.
      # Change the OBSTYPE of the mean frame to keep it from being reused
      # as a single bias frame in future runs.

      if (access(meanbias//imtype)) {
         print("Mean bias file ",meanbias," already exists") | 
            tee (log1, >log2)
      } else if (meanbias != "") {
         hselect("@"//temp2,"$I","OBSTYPE == 'BIAS'",>tempfile)
         count(tempfile) | scan(nf)
         if (nf == 0) {
            print("No bias frames available -- subtracting off constant") |
               tee (log1, >log2)
            ccdproc.zerocor = no
         } else if (!listonly) {
            zerocombine("@"//tempfile,output=meanbias) | tee (log1, > log2)
            hedit(meanbias,"OBSTYPE","MEANBIAS",add=yes,delete=no,verify=no,
               show=no,update=yes)
         }
         if (access(tempfile))
            delete(tempfile,verify=no)
      }

      # If you are processing flat frames, then do that here.  If the
      # mean frame already exists then get out of here.  If no flat
      # frames are available, then signal an error. Otherwise, combine all 
      # the available flat frames. Change the OBSTYPE of the mean frame to 
      # keep it from being reused as a single flat frame in future runs.

      if (access(meanflat//imtype)) {
         print("Mean flat file ",meanflat," already exists") | 
            tee (log1, >log2)
      } else if (meanflat != "") {
         hselect("@"//temp2,"$I","OBSTYPE == 'FLAT'",>tempfile)
         count(tempfile) | scan(nf)
         if (nf == 0) {
            delete(tempfile, verify=no)
            error(0,"No flat frames available")
         } else if (!listonly) {
            ccdproc("@"//tempfile,zero=meanbias)
            flatcombine("@"//tempfile,output=meanflat) | tee (log1, > log2)
            hedit(meanflat,"OBSTYPE","MEANFLAT",add=yes,delete=no,verify=no,
               show=no,update=yes)
         }
         if (access(tempfile))
            delete(tempfile,verify=no)
      }
      delete(temp2,verify=no)
end
