procedure w_proc (objects,apref,flat,throughput,arcs1,fibers,
   apidtable,scattered,arcscat,fitflat,clean,dispcor,skysubtract,skyedit,
   saveskys,tippex,tfile,splot,redo,update,listonly)

string  objects                 {prompt="List of object spectra"}
file    apref                   {prompt="Aperture reference spectrum"}
file    flat                    {prompt="Flat field spectrum"}
file    throughput              {prompt="Throughput file or image (optional)"}
string  arcs1                   {prompt="List of arc spectra"}
int     fibers                  {prompt="Number of fibers"}
file    apidtable               {prompt="Aperture identifications"}
bool    scattered               {prompt="Subtract scattered light?"}
bool    arcscat                 {prompt="Subtract scattered light from arcs?"}
bool    fitflat                 {prompt="Fit and ratio flat field spectrum?"}
bool    clean                   {prompt="Detect and replace bad pixels?"}
bool    dispcor                 {prompt="Dispersion correct spectra?"}
bool    skysubtract             {prompt="Subtract sky?"}
bool    skyedit                 {prompt="Edit the sky spectra?"}
bool    saveskys                {prompt="Save sky spectra?"}
bool    tippex                  {prompt="Tippex out sky line residuals?"}
file    tfile                   {prompt="Name of tippex file"}
bool    splot                   {prompt="Plot the final spectrum?"}
bool    redo                    {prompt="Redo operations if previously done?"}
bool    update                  {prompt="Update spectra if cal data changes?"}
bool    listonly                {prompt="List steps but don't process?\n"}
real    datamax = INDEF         {prompt="Max data value cosmic ray threshold"}

bool    newaps, newresp, newdisp, newarcs, batch

struct  *fd,*fd2

begin

      string imtype,mstype,extn,objs,arcrefs,aps,spec,specms,str1,str2
      string str3,str4,response,arcref1,arcref1ms
      file temp,temp1,log1,log2,log3,arc
      bool reextract,newv,scat,finter,extract,disp,sky,tipp,disperr
      real ishift
      int wyflim1,wyflim2,i

      # Call a separate task to do the listing to minimize the size of
      # this script and improve it's readability.

      if (listonly) {
         w_listonly(objects,apidtable,apref,flat,throughput,arcs1, 
            scattered,arcscat,dispcor,skysubtract,tippex,redo,update)
         bye
      }

      # Get image type and multispec extension.  Also define an extension
      # to be used with calibration images

      imtype = "." // envget ("imtype")
      mstype = ".ms" // imtype
      extn = apidtable//".ms"

      # Create some temporary file names

      temp = mktemp("tmp$iraf")
      temp1 = mktemp("tmp$iraf")

      # Get some query parameters

      objs = objects
      arcrefs = arcs1

      # Set up the logging.  Use TEE to pipe to "dev$null" rather than
      # to have lots of switches for verbose flag

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose && !batch)
         log2 = "STDOUT"
      else
         log2 = "dev$null"
      if (batch)
         log3 = "dev$null"
      else
         log3 = "STDOUT"

      # If the update switch is used changes in the calibration data
      # can cause images to be reprocessed (if they are in the object
      # list).  Possible changes are in the aperture definitions,
      # response function, dispersion solution, and sensitivity
      # function.  The newarcs flag is used to only go through the arc
      # image headers once setting the reference spectrum, airmass, and
      # UT.

      newaps = no
      newresp = no
      newdisp = no
      newarcs = yes

      # We can't continue if we don't have an aperture reference.

      if (!access(apref//imtype))
         error(1,"No aperture reference!")

      # Initialize apscript

      apscript.saturation = INDEF
      apscript.references = apref
      apscript.profiles = ""
      apscript.apidtable = apidtable
      apscript.nfind = fibers
      apscript.clean = clean
      apscript.ansrecenter="YES"
      apscript.ansresize="NO"
      reextract = redo

      # If you are reextracting or you can't access the aperture
      # database file for the aperture reference, then locate and
      # trace the apertures.

      if (reextract || !access(database//"/ap"//apref//extn)) {
         print("Set reference apertures for ", apref) | tee (log1, >log3)
         if (access(database//"/ap"//apref))
            delete(database//"/ap"//apref,verify=no)
         if (access (database//"/ap"//apref//extn))
            delete (database//"/ap"//apref//extn,verify=no)
         if (wyf_red.w_interact) {
            apscript.ansedit = "yes"
            apscript.ansfittrace = "no"
         } else {
            apscript.ansedit = "NO"
            apscript.ansfittrace = "NO"
         }
         if (wyfpars.fends) {
            apscript(apref,references="",ansfind="YES",ansrecenter="NO", 
               anstrace="YES",ansextract="YES",weights="none",clean=no, > log3)
            newaps = yes
            copy(database//"/ap"//apref,database//"/ap"//apref//extn,
               verbose=no)
            if (wyfpars.dosaw) {
               fd = wyfpars.sfile
               while (fscan(fd,ishift,aps) != EOF) 
                  if (ishift != 0.0) 
                     specshift(apref//mstype,ishift,apertures=aps,verbose=no)
               fd = ""
            }
            w_ends(apref,inter=wyf_red.w_interact)
            imdelete(apref//mstype,verify=no)
         } else {
            apscript(apref,references="",ansfind="YES",ansrecenter="NO", 
               anstrace="YES",ansextract="NO", > log3)
            newaps = yes
            copy(database//"/ap"//apref,database//"/ap"//apref//extn,
               verbose=no)
            w_qends(apref)
         }

      # If you're not reextracting make sure you can get the spectral limits.
      # If you can't, then extract the aperture reference and find them

      } else {
         if (access (database//"/ap"//apref))
            delete(database //"/ap"//apref,verify=no)
         copy(database//"/ap"//apref//extn,database//"/ap"//apref,
            verbose=no)
         hselect(apref,"WYFLIM*",yes) | scan(wyflim1,wyflim2)
         newv = no
         if (nscan() < 2) {
            if (wyfpars.fends) {
               if (!access(apref//mstype)) {
                  apscript(apref,references="",ansfind="NO",ansrecenter="NO",
                     anstrace="NO",ansextract="YES",weights="none",clean=no,
                     > log3)
                  if (wyfpars.dosaw) {
                     fd = wyfpars.sfile
                     while (fscan(fd,ishift,aps) != EOF)
                        if (ishift != 0.0)  
                           specshift(apref//mstype,ishift,apertures=aps,
                              verbose=no)
                     fd = ""
                  }
                  newv = yes
               }
               w_ends(apref,inter=wyf_red.w_interact)
               if (newv) imdelete(apref//mstype,verify=no)
            } else {
               w_qends(apref)
            }
         }
      }
      hselect(apref,"WYFLIM*",yes) | scan(wyflim1,wyflim2)

      # Initialise a few more things in apscript

      apscript.ansedit = "NO"
      apscript.anstrace = "NO"
      apscript.ansfittrace = "NO"
      apscript.ansextract = "YES"
      apscript.ansreview = "NO"
      if (scattered && wyf_red.w_interact) {
         apscript.ansfitscatter = "yes"
         apscript.ansfitsmooth = "yes"
      } else {
         apscript.ansfitscatter = "NO"
         apscript.ansfitsmooth = "NO"
      }

      # Process the offset sky exposure.  Start off by checking whether
      # to subtract out the scattered light and whether that has already
      # been done.

      spec = throughput
      specms = spec//mstype
      scat = no
      if (scattered) {
         if (redo && access(spec//"noscat"//imtype)) {
            imdelete(spec,verify=no)
            imrename(spec//"noscat",spec)
         }
         hselect (spec,"apscatte",yes) | scan (str1)
         if (nscan() == 0)
            scat = yes
      }
      if (scat) {
         print ("Subtract scattered light from ", spec) | 
            tee (log1, > log3)
         imrename (spec,spec//"noscat")
         apscript (spec//"noscat",output=spec,ansextract="NO",
            ansscat="YES",anssmooth="YES")
      }

      # Create the response function now.

      response = flat//spec//extn
      reextract = redo || (update && newaps)
      if (reextract || !access(response//imtype) || (redo && scat)) {
         print ("Create response function ", response) | tee (log1, > log3)
         if (access(response//imtype))
            imdelete(response,verify=no)
         if (access(flat//mstype))
            imdelete(flat//mstype,verify=no)
         if (access(specms))
            imdelete(specms,verify=no)
         finter = (wyf_red.w_interact && wyfpars.f_interactive)
         w_fibresponse(flat,spec,apref,response,apidtable=apidtable,
            clean=clean,fitflat=fitflat,interactive=finter,
            function=wyfpars.f_function,order=wyfpars.f_order,batch=batch)
         newresp = yes
      }

      # Do this section if dispersion correcting.  If doing scattered light
      # correction for arcs then do that here.  Then get the first arc file
      # out of the list.  Decide whether it needs extracting.

      if (dispcor) {
         sections(arcs1,option="fullname",>temp)
         if (arcscat) {
            fd = temp
            while (fscan(fd,arcref1) != EOF) {
               scat = no
               if (redo && access(arcref1//"noscat"//imtype)) {
                  imdelete(arcref1,verify=no)
                  imrename(arcref1//"noscat",arcref1)
               }
               hselect (arcref1,"apscatte",yes) | scan (str1)
               if (nscan() == 0)
                  scat = yes
               if (scat) {
                  print ("Subtract scattered light from ", arcref1) | 
                     tee (log1, > log3)
                  imrename (arcref1,arcref1//"noscat")
                  apscript (arcref1//"noscat",output=arcref1,ansextract="NO",
                     ansscat="YES",anssmooth="YES")
               }
            }
         }
         fd = temp
         i = fscan(fd,arcref1)
         fd = ""
         delete(temp,verify=no)
         arcref1ms = arcref1//extn
         reextract = redo || (update && newaps)
         if (reextract && access(arcref1ms//imtype))
            imdelete(arcref1ms,verify=no)
         w_arcrefs(arcref1,arcref1ms,apidtable,response,log1,log2,log3)
      }

      # Right, now process the object spectra! First check whether a
      # reextraction is needed.  Then decide exactly what needs doing.

      reextract = redo || (update && (newaps || newresp || newdisp))
      spec = objs
      specms = spec//mstype
      scat = no
      extract = no
      disp = no
      sky = no
      tipp = no
      if (scattered) {
         if (redo && access(spec//"noscat"//imtype)) {
            imdelete(spec,verify=no)
            imrename(spec//"noscat",spec)
         }
         hselect (spec,"apscatte",yes) | scan (str1)
         if (nscan() == 0)
            scat = yes
      }
      if (reextract || !access (specms) || (redo && scat))
         extract = yes
      else {
         hselect (specms, "dclog1", yes) | scan (str1)
         if (nscan () == 1) {
            extract = update && newdisp
         } else {
            disp = dispcor
            hselect (specms, "skysub", yes) | scan (str1)
            if (nscan() == 0)
               sky = skysubtract
            hselect (specms, "tippex", yes) | scan (str1)
            if (nscan() == 0)
               tipp = tippex
         }
      }

      # If an extraction needs to be done then get the rest of the processing
      # switches directly from the input parameters

      if (extract) {
         disp = dispcor
         sky = skysubtract
         tipp = tippex
      }

      # Right, if you're going to extract the object frame, then do that
      # now.  Check and see if the scattered light wants removing too.

      if (extract) {
         if (access(specms))
            imdelete(specms,verify=no)
         if (scat) {
            print("Subtract scattered light from ",spec) | 
               tee (log1, > log3)
            imrename(spec,spec//"noscat")
            apscript(spec//"noscat",output=spec,ansextract="NO",
               ansscat="YES",anssmooth="YES")
         }

         # Set the Julian date and the airmass for later use and then
         # extract the spectra. Do the sawtooth correction if requested.

         print("Extract object spectrum ",spec) | tee(log1,>log3)
         hselect(spec,"date-obs,utobs,exposed",yes,> temp)
         hselect(spec,"ra,dec,epoch,st",yes, >> temp)
         fd = temp
         if (fscan(fd,str1,str2,str3) == 3) {
            setjd(spec,observatory=observatory,date="date-obs",
               time="utobs",exposure="exposed",jd="jd",hjd="",
               ljd="ljd",utdate=yes,uttime=yes,listonly=no,
               >> log1)
            if (fscan(fd,str1,str2,str3,str4) == 4)
               setairmass(spec,intype="beginning",
                  outtype="effective",exposure="exposed",
                  observatory=observatory,show=no,update=yes,
                  override=yes,>> log1)
         }
         fd = ""; delete (temp,verify=no)
         apscript(spec,nsubaps=wyfpars.nsubaps,saturation=datamax)
         sapertures(specms,apertures="",apidtable=apidtable,
            wcsreset=no,verbose=no,beam=INDEF,dtype=INDEF,w1=INDEF,
            dw=INDEF,z=INDEF,aplow=INDEF,aphigh=INDEF,title=INDEF)
         if (wyfpars.dosaw) {
            fd = wyfpars.sfile
            while (fscan(fd,ishift,aps) != EOF)
               if (ishift != 0.0) 
                  specshift(specms,ishift,apertures=aps,verbose=no)
            fd = ""
         }
         w_mksafe(specms,wyflim1,wyflim2)

         # Correct the spectra by dividing by the response function

         if (wyfpars.nsubaps == 1)
            sarith (specms, "/", response, specms, w1=INDEF,
               w2=INDEF, apertures="", bands="", beams="",
               apmodulus=0, reverse=no, ignoreaps=no,
               format="multispec", renumber=no, offset=0,
               clobber=yes, merge=yes, errval=0, verbose=no)
         else {
            blkrep (response, temp, 1, wyfpars.nsubaps)
            sarith (specms, "/", temp, specms, w1=INDEF,
               w2=INDEF, apertures="", bands="", beams="",
               apmodulus=0, reverse=no, ignoreaps=yes,
               format="multispec", renumber=no, offset=0,
               clobber=yes, merge=yes, errval=0, verbose=no)
            imdelete (temp, verify=no)
         }
      }

      # Do this section if doing dispersion correction.  Start by
      # updating the JD and airmass

      disperr = no
      if (disp) {
         if (newarcs) {
            sections(arcs1,option="fullname",>temp)
            fd = temp
            while (fscan(fd,arc) != EOF) {
               hselect(arc,"date-obs,utobs,exposed",yes, > temp1)
               hselect(arc,"ra,dec,epoch,st",yes, >> temp1)
               fd2 = temp1
               if (fscan(fd2, str1, str2, str3) == 3) {
                  setjd(arc,observatory=observatory, 
                     date="date-obs",time="utobs", 
                     exposure="exposed",jd="jd",hjd="",
                     ljd="ljd",utdate=yes,uttime=yes,listonly=no,
                     >> log1)
                  if (fscan(fd2,str1,str2,str3,str4) == 4)
                     setairmass(arc,intype="beginning",
                        outtype="effective",exposure="exposed",
                        observatory=observatory,show=no,update=yes,
                        override=yes, >> log1)
               }
               fd2 = ""; delete(temp1,verify=no)
               hedit(arc,"refspec1",arc,add=yes,verify=no,
                  show=no,update=yes)
            }
            fd = ""; delete(temp,verify=no)
            newarcs = no
         }   

         # Assign the reference spectra

         print("Assign arc spectra for ", spec) | tee (log1, > log3)
         refspectra(spec,references=arcrefs,apertures="",refaps="",
            ignoreaps=no,select=wyfpars.select,sort=wyfpars.sort,
            group=wyfpars.group,time=wyfpars.time,timewrap=wyfpars.timewrap,
            override=yes,confirm=no,assign=yes,logfiles="STDOUT",
            verbose=no) | tee (log1, > log2)

         # Identify the arcs referenced in the object header

         w_doarcs(spec,response,arcref1,extn,apidtable,reextract,
            logfile,batch)

         # See if any arcs were referenced.  If not then you have a 
         # dispersion error.  If there is no dispersion error, then
         # dispersion correct the spectra.

         hselect(specms,"refspec1",yes, > temp)
         fd= temp
         i = fscan (fd, arc)
         fd= ""; delete(temp,verify=no)
         if (i < 1) {
            print("No arc reference assigned for ",spec) | 
                tee (log1, > log3)
            disperr = yes
         } else {
            print("Dispersion correct ",spec) | tee (log1, > log3)
            dispcor.log = wyfpars.log
            dispcor(specms,"",linearize=wyfpars.linearize,
               database=database,table=arcref1ms,w1=INDEF,
               w2=INDEF,dw=INDEF,nw=INDEF, 
               flux=wyfpars.flux,samedisp=no,global=no,
               ignoreaps=yes,confirm=no,listonly=no,
               verbose=verbose,logfile=logfile)
            if (wyfpars.nsubaps > 1) {
               imrename(specms,temp,verbose=no)
               scopy(temp,specms,w1=INDEF,w2=INDEF,
                  apertures="-999",bands="",beams="",apmodulus=0,
                  offset=0,format="multispec",clobber=no,merge=no,
                  renumber=no,verbose=no)
               blkavg(temp,temp,1,wyfpars.nsubaps,option="sum")
               imcopy(temp, specms//"[*,*]",verbose=no)
               imdelete(temp,verify=no)
            }
         }
      }

      # Sky subtract if the user wants and if there weren't any errors
      # in the dispersion correction.

      if (sky && !disperr) {
         print("Sky subtract ",spec) | tee (log1, > log3)
         w_skysub.reject = wyfpars.reject
         w_skysub(specms,response,skyedit=skyedit,combine=wyfpars.combine,
            scale=wyfpars.scale,saveskys=saveskys,logfile=logfile)
         wyfpars.reject = w_skysub.reject
      }

      # Do tippexing if the user wants and if there weren't any errors
      # in the dispersion correction.

      if (tipp && !disperr) {
         print ("Tippexing ", spec) | tee (log1, > log3)
         w_tippex(tfile,specms,temp) | tee (log1, > log3)
      }

      # Does the user want to splot?

      if (!disperr && (extract || disp || sky)) {
         if (splot)
            splot(specms)
      }

      # Exit

      bye
end
