procedure w_red1lo()

string    input = ""        {prompt="List of input program images"}
file      apref = ""        {prompt="Aperture reference image"}
bool      batch = no        {prompt="Do processing in batch?"}
file      mbias = ""        {prompt="Mean bias frame"}
file      mflat = ""        {prompt="Mean flat frame"}
bool      skyobj = no       {prompt="Reduce offset skys as objects as well?"}
bool      override = no     {prompt="Override automatic aperture info?"}
bool      scattered = yes   {prompt="Do scattered light correction?"}
bool      arcscat = no      {prompt="Do scattered light correction for arcs?"}
bool      fitflat = yes     {prompt="Fit and ratio flat field spectrum?"}
bool      clean = yes       {prompt="Detect and replace bad pixels?"}
bool      dispcor = yes     {prompt="Dispersion correct spectra?"}
bool      skysubtract = yes {prompt="Subtract sky?"}
bool      skyedit = yes     {prompt="Edit the sky spectra?"}
bool      saveskys = yes    {prompt="Save sky spectra?"}
bool      tippex = yes      {prompt="Tippex out sky line residuals?"}
file      tfile = ""        {prompt="Name of tippex file"}
bool      splot = no        {prompt="Plot the final spectrum?"}
bool      redo = no         {prompt="Redo operations if previously done?"}
bool      update = yes      {prompt="Update spectra if cal data changes?"}
bool      listonly = yes    {prompt="List steps but don't process?"}

pset      wyfpars = ""      {prompt="Algorithm parameters"}

struct   *fd
begin

      string   inp,chip,offsky,object,log3
      struct   fieldname
      real     minsep,maxsep,width,lower,upper
      file     meanbias,meanflat,aprefer,objfils,skyfils,arcfils,temp1
      file     apidtable,shifts
      int      nafiles,nfibres
      bool     dispcorr
    
      # Get all the query parameters...

      inp = input
      aprefer = apref
      meanbias = mbias
      meanflat = mflat
      dispcorr = dispcor

      # Set up the logging...

      if (batch)
         log3 = "dev$null"
      else
         log3 = "STDOUT"

      # Create some temporary files...

      objfils = mktemp("tmp$iraf")
      skyfils = mktemp("tmp$iraf")
      arcfils = mktemp("tmp$iraf")
      temp1 = mktemp("tmp$iraf")

      # Collect all the info on the files you have specified

      collect(skyobj,dispcorr,objfils,skyfils,arcfils,inp,temp1)
      count(arcfils) | scan(nafiles)

      # Read the temporary file and delete

      type(temp1,map_cc=yes,device="terminal") | scan(dispcorr,chip)
      delete(temp1,verify=no)

      # Set instrument info and process bias and flats if you want...

      print("Doing CCD processing on input frames", >log3)
      w_preccd(mbias,mflat,chip,yes)

      # Compress headers in object, sky and arcfiles

      print("Compressing headers in input frames", >log3)
      w_hdcomp("@"//objfils,log3,"dev$null","dev$null",yes)
      w_hdcomp("@"//skyfils,log3,"dev$null","dev$null",yes)
      w_hdcomp("@"//arcfils,log3,"dev$null","dev$null",yes)
      w_hdcomp(aprefer,log3,"dev$null","dev$null",yes)

      # Now loop for each object file...

      fd = objfils
      while (fscan(fd,object) != EOF) {

         # Find the offset sky file which goes with this object file

         hselect(object,"AF2FLDNM",yes) | scan(fieldname)
         if (nscan() == 0) {
            print("No field name in ",object," (AF2FLDNM)",> log3)
            next
         }
         matchem(temp1,object,"@"//skyfils)
         type(temp1,map_cc=yes,device="terminal") | scan(offsky)
         if (nscan() == 0) {
            print("No offset sky for field ",fieldname)
            next
         }
         print("Object frame ",object," paired with offset sky frame ",
            offsky, > log3) 
            
         # Look in file header to get information about fibres and to
         # create an apidtable (fudged for listonly)

         nfibres = wyfpars.nfibres
         minsep = wyfpars.minsep
         maxsep = wyfpars.maxsep
         width = wyfpars.width
         lower = wyfpars.lower
         upper = wyfpars.upper
         apidtable = wyfpars.apidtable
         shifts = wyfpars.sfile

         # Process the data...

         wyf_dofibers.batch = batch
         wyf_dofibers(objects=object,apref=aprefer,flat=meanflat,
            throughput=offsky,arcs1="@"//arcfils,readnoise=wyfpars.readnoise,
            gain=wyfpars.gain,datamax=INDEF,fibers=nfibres,width=width,
            minsep=minsep,maxsep=maxsep,apidtable=apidtable,
            scattered=scattered,arcscat=arcscat,
            fitflat=fitflat,clean=clean,dispcor=dispcorr,
            skysubtract=skysubtract,skyedit=skyedit,saveskys=saveskys,
            tippex=tippex,tfile=tfile,splot=splot,redo=redo,update=update,
            listonly=yes)
      }
      fd = ""

      # Ditch temporary files...

      delete(objfils,verify=no)               
      delete(skyfils,verify=no)               
      delete(arcfils,verify=no)               
end
