# SKYSUB -- Sky subtract fiber spectra.
#   Subtract the selected sky apertures from the selected object apertures.
#   The sky apertures are combined to a single master sky which is subtracted
#   from each selected object aperture.  The subtracted sky may be
#   saved in an image with the prefix "sky" and the same output name.  Note
#   that existing output images are clobbered.

procedure w_skysub (input,response)

string  input = ""              {prompt="Input spectra to sky subtract"}
string  response = ""           {prompt="Input response frame"}

bool    skyedit = yes           {prompt="Edit the sky spectra?"}
string  combine = "average"     {prompt="Combining option",
                                 enum="average|median"}
string  reject = "avsigclip"    {prompt="Sky rejection option",
                                 enum="none|minmax|avsigclip"}
string  scale = "none"          {prompt="Sky scaling option",
                                 enum="none|mode|median|mean"}
bool    saveskys = yes          {prompt="Save sky spectra?"}
file    logfile = ""            {prompt="Logfile"}

begin

      file log,in,out,resp,sky,goodaps,temp
      string imtype,aps

      # Get input and output file names

      in = input
      resp = response
      out = in

      # Define log file and image type.

      if (logfile == "")
         log = "dev$null"
      else
         log = logfile
      imtype = "."//envget("imtype")

      # Does the output sky file exist?

      sky = "sky"//out
      if (access(sky//imtype))
         imdelete(sky,verify=no)

      # If you want to edit the sky apertures then do that now.  If you
      # are not editing, then use all of the sky apertures available.

      if (skyedit) {
         goodaps = mktemp("tmp$iraf")
         w_skyedit(in,resp,0.2,goodaps,log=log)
         aps = "@"//goodaps
      } else {
         aps = ""
      }
   
      # Combine all of the non-rejected sky beams into one sky spectrum

      temp = mktemp("tmp$iraf")
      scopy(in,temp,w1=INDEF,w2=INDEF,apertures=aps,bands="",beams="0",
         apmodulus=0,offset=0,clobber=yes,format="multispec",merge=yes,
         renumber=no,verbose=yes, >> log)

      # If no sky beams exist, then signal an error.  If they do,
      # then combine them into one row and subtract it off the input
      # frame.  

      if (!access(temp//imtype)) {
         print("No sky beams in ",in,". Skipping sky subtraction")
      } else {
         scombine(temp,sky,noutput="",logfile=logfile,apertures=aps,
            group="all",combine=combine,reject=reject,first=no,
            w1=INDEF,w2=INDEF,dw=INDEF,nw=INDEF,scale=scale,zero="none",
            weight="none",sample="",lthreshold=INDEF,hthreshold=INDEF,
            nlow=1,nhigh=1,nkeep=1,mclip=yes,lsigma=3.0,hsigma=2.0,
            rdnoise="0",gain="1",snoise="0.0",sigscale=0.0,pclip=-0.5,
            grow=0,blank=0.0)
         imdelete(temp,verify=no)
         sarith(in,"-",sky,out,w1=INDEF,w2=INDEF,apertures="",bands="",
            beams="0,1",reverse=no,ignoreaps=yes,format="multispec",
            renumber=no,offset=0,clobber=yes,merge=no,errval=0.0,
            verbose=yes, >> log)
         hedit(in,"skysub",yes,add=yes,show=no,verify=no,update=yes)
         if (!saveskys)
            imdelete(sky,verify=no)
      }

      # Clean up and exist

      if (access(goodaps))
         delete(goodaps,verify=no, >>& "dev$null")
end
         
