procedure wyf_red1()

string    input = ""        {prompt="List of input program images"}
file      apref = ""        {prompt="Aperture reference image"}
bool      batch = no        {prompt="Do processing in batch?"}
file      mbias = ""        {prompt="Mean bias frame"}
file      mflat = ""        {prompt="Mean flat frame"}
bool      skyobj = no       {prompt="Reduce offset skys as objects as well?"}
bool      override = no     {prompt="Override automatic aperture info?"}
bool      scattered = yes   {prompt="Do scattered light correction?"}
bool      arcscat = no      {prompt="Do scattered light correction on arcs?"}
bool      fitflat = yes     {prompt="Fit and ratio flat field spectrum?"}
bool      clean = yes       {prompt="Detect and replace bad pixels?"}
bool      dispcor = yes     {prompt="Dispersion correct spectra?"}
bool      skysubtract = yes {prompt="Subtract sky?"}
bool      skyedit = yes     {prompt="Edit the sky spectra?"}
bool      saveskys = yes    {prompt="Save sky spectra?"}
bool      tippex = yes      {prompt="Tippex out sky line residuals?"}
file      tfile = ""        {prompt="Name of tippex file"}
bool      splot = no        {prompt="Plot the final spectrum?"}
bool      redo = no         {prompt="Redo operations if previously done?"}
bool      update = yes      {prompt="Update spectra if cal data changes?"}
bool      listonly = no     {prompt="List steps but don't process?"}

pset      wyfpars = ""      {prompt="Algorithm parameters"}

struct   *fd
begin

      string   inp,chip,offsky,object,log1,log2,log3
      struct   fieldname
      real     minsep,maxsep,width,lower,upper
      file     meanbias,meanflat,aprefer,objfils,skyfils,arcfils,temp1
      file     apidtable,shifts
      int      nafiles,nfibres
      bool     dispcorr
    
      # Get all the query parameters...

      inp = input
      aprefer = apref
      meanbias = mbias
      meanflat = mflat
      dispcorr = dispcor

      # Set up the logging...

      tee.append = yes
      if (logfile == "")
         log1 = "dev$null"
      else
         log1 = logfile
      if (verbose && !batch)
         log2 = "STDOUT"
      else
         log2 = "dev$null"
      if (batch)
         log3 = "dev$null"
      else
         log3 = "STDOUT"

      # Create some temporary files...

      objfils = mktemp("tmp$iraf")
      skyfils = mktemp("tmp$iraf")
      arcfils = mktemp("tmp$iraf")
      temp1 = mktemp("tmp$iraf")

      # Collect all the info on the files you have specified

      collect(skyobj,dispcorr,objfils,skyfils,arcfils,inp,temp1)
      count(arcfils) | scan(nafiles)

      # Read the temporary file and delete.

      type(temp1,map_cc=yes,device="terminal") | scan(dispcorr,chip)
      delete(temp1,verify=no)

      # If doing dispersion correction and you can't find the arcline
      # file, then signal an error and escape

      if (dispcorr) {
         if (!access(wyf_red.coordlist)) {
            print("Can't access arc linelist ",wyf_red.coordlist) | 
               tee(log1,>log3)
            error(0,"Check linelist and try again")
         }
      }

      # If the ccdred backup prefix is a directory, then create it if it
      # doesn't exist

      if (stridx("/$]",backup) != 0) 
         if (!access(backup)) mkdir(backup)

      # Set instrument info and process bias and flats if you want...

      print("Doing CCD processing on input frames") | tee (log1, >log3)
      w_preccd(meanbias,meanflat,chip,no)

      # Now process any object, arc or sky frames...(Just debias, really...)
      # Make sure the aperture reference frame is done as well...

      ccdproc("@"//objfils,zero=meanbias) | tee (log1, > log2)
      ccdproc("@"//skyfils,zero=meanbias) | tee (log1, > log2)
      if (nafiles != 0)
         ccdproc("@"//arcfils,zero=meanbias) | tee (log1, > log2)
      ccdproc(aprefer,zero=meanbias) | tee(log1, >log2)

      # Compress headers in object, sky and arcfiles

      print("Compressing headers in input frames") | tee (log1, >log3)
      w_hdcomp("@"//objfils,log1,log2,log3,no)
      w_hdcomp("@"//skyfils,log1,log2,log3,no)
      w_hdcomp("@"//arcfils,log1,log2,log3,no)
      w_hdcomp(aprefer,log1,log2,log3,no)

      # Set Julian dates for object and offset sky frames

      setjd.jd = "jd"
      setjd("@"//objfils,observatory=observatory,date="date-obs",
         time="utobs",exposure="exposed",hjd="",ljd="ljd",utdate=yes,
         uttime=yes, listonly=no, >> log1)
      setjd("@"//skyfils,observatory=observatory,date="date-obs",
         time="utobs",exposure="exposed",hjd="",ljd="ljd",utdate=yes,
         uttime=yes, listonly=no, >> log1)

      # Now loop for each object file...

      fd = objfils
      while (fscan(fd,object) != EOF) {

         # Find the offset sky file which goes with this object file

         hselect(object,"AF2FLDNM",yes) | scan(fieldname)
         if (nscan() == 0) {
            print("No field name in ",object," (AF2FLDNM)") | tee(log1, > log3)
            next
         }
         matchem(temp1,object,"@"//skyfils)
         type(temp1,map_cc=yes,device="terminal") | scan(offsky)
         if (nscan() == 0) {
            print("No offset sky for field ",fieldname)
            next
         }
         print("Object frame ",object," paired with offset sky frame ",
            offsky) | tee(log1, > log2)   

         # Look in file header to get information about fibres and to
         # create an apidtable

         if (!override) {
            headid(object,aprefer) | scan(nfibres,minsep,maxsep,width,
               lower,upper,apidtable,shifts)
            wyfpars.sfile = shifts
            wyfpars.lower = lower
            wyfpars.upper = upper
         } else {
            nfibres = wyfpars.nfibres
            minsep = wyfpars.minsep
            maxsep = wyfpars.maxsep
            width = wyfpars.width
            apidtable = wyfpars.apidtable
         }

         # Process the data...

         wyf_dofibers.batch = batch
         wyf_dofibers(objects=object,apref=aprefer,flat=meanflat,
            throughput=offsky,arcs1="@"//arcfils,readnoise=wyfpars.readnoise,
            gain=wyfpars.gain,datamax=INDEF,fibers=nfibres,width=width,
            minsep=minsep,maxsep=maxsep,apidtable=apidtable,
            scattered=scattered,arcscat=arcscat,fitflat=fitflat,
            clean=clean,dispcor=dispcorr,skysubtract=skysubtract,
            skyedit=skyedit,saveskys=saveskys,tippex=tippex,tfile=tfile,
            splot=splot,redo=redo,update=update,listonly=listonly)
      }
      fd = ""

      # Ditch temporary files...

      delete(objfils,verify=no)               
      delete(skyfils,verify=no)               
      delete(arcfils,verify=no)               
end
